-- Function: update_time_updated_column()

-- DROP FUNCTION update_time_updated_column();

CREATE OR REPLACE FUNCTION update_time_updated_column()
  RETURNS trigger AS
$BODY$
	BEGIN
	   NEW.time_updated = now(); 
	   RETURN NEW;
	END;
	$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100;
ALTER FUNCTION update_time_updated_column() OWNER TO "mailtrace";

-- Table: config

-- DROP TABLE config;

CREATE TABLE config
(
  "key" character varying(1024) NOT NULL,
  "value" text,
  time_created timestamp with time zone NOT NULL DEFAULT now(),
  time_updated timestamp with time zone NOT NULL DEFAULT now(),
  CONSTRAINT pri_config_key PRIMARY KEY (key)
)
WITH (
  OIDS=FALSE
);
ALTER TABLE config OWNER TO mailtrace;

-- Trigger: update_config_modtime on config

-- DROP TRIGGER update_config_modtime ON config;

CREATE TRIGGER update_config_modtime
  BEFORE UPDATE
  ON config
  FOR EACH ROW
  EXECUTE PROCEDURE update_time_updated_column();

-- Table: roles

-- DROP TABLE roles;

CREATE TABLE roles
(
  id serial NOT NULL,
  role_name character varying(50) NOT NULL,
  time_created timestamp with time zone NOT NULL DEFAULT now(),
  time_updated timestamp with time zone NOT NULL DEFAULT now(),
  CONSTRAINT pri_roles_id PRIMARY KEY (id),
  CONSTRAINT unique_role_name UNIQUE (role_name)
)
WITH (
  OIDS=FALSE
);
ALTER TABLE roles OWNER TO mailtrace;

-- Trigger: update_roles_modtime on roles

-- DROP TRIGGER update_roles_modtime ON roles;

CREATE TRIGGER update_roles_modtime
  BEFORE UPDATE
  ON roles
  FOR EACH ROW
  EXECUTE PROCEDURE update_time_updated_column();

  
  
-- Table: rule_ids

-- DROP TABLE rule_ids;

CREATE TABLE rule_ids
(
  id serial NOT NULL,
  rule_name character varying(50) NOT NULL,
  description text,
  time_created timestamp with time zone NOT NULL DEFAULT now(),
  time_updated timestamp with time zone NOT NULL DEFAULT now(),
  CONSTRAINT pri_id_rules PRIMARY KEY (id),
  CONSTRAINT unique_rule_id UNIQUE (rule_name)
)
WITH (
  OIDS=FALSE
);
ALTER TABLE rule_ids OWNER TO mailtrace;

-- Trigger: update_rule_ids_modtime on rule_ids

-- DROP TRIGGER update_rule_ids_modtime ON rule_ids;

CREATE TRIGGER update_rule_ids_modtime
  BEFORE UPDATE
  ON rule_ids
  FOR EACH ROW
  EXECUTE PROCEDURE update_time_updated_column();

  
  
-- Table: regexrule

-- DROP TABLE regexrule;

CREATE TABLE regexrule
(
  id serial NOT NULL,
  regex character varying(1024) NOT NULL,
  rule_id integer,
  deactivated boolean,
  description text,
  param_map character varying(512),
  priority integer NOT NULL DEFAULT 0,
  time_created timestamp with time zone NOT NULL DEFAULT now(),
  time_updated timestamp with time zone NOT NULL DEFAULT now(),
  CONSTRAINT pri_id PRIMARY KEY (id),
  CONSTRAINT ref_regex_ruleid FOREIGN KEY (rule_id)
      REFERENCES rule_ids (id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL,
  CONSTRAINT unique_regex UNIQUE (regex)
)
WITH (
  OIDS=FALSE
);
ALTER TABLE regexrule OWNER TO mailtrace;

-- Trigger: update_regexrule_modtime on regexrule

-- DROP TRIGGER update_regexrule_modtime ON regexrule;

CREATE TRIGGER update_regexrule_modtime
  BEFORE UPDATE
  ON regexrule
  FOR EACH ROW
  EXECUTE PROCEDURE update_time_updated_column();

-- Table: reasons_de_de

-- DROP TABLE reasons_de_de;

CREATE TABLE reasons_de_de
(
  id serial NOT NULL,
  role_id integer NOT NULL,
  rule_id integer NOT NULL,
  short_desc text NOT NULL,
  long_desc text,
  tag_desc character varying(50),
  time_created timestamp with time zone NOT NULL DEFAULT now(),
  time_updated timestamp with time zone NOT NULL DEFAULT now(),
  CONSTRAINT prid_id_reasons_de_de PRIMARY KEY (id),
  CONSTRAINT ref_reasons_de_de_roleid FOREIGN KEY (role_id)
      REFERENCES roles (id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL,
  CONSTRAINT ref_reasons_de_de_ruleid FOREIGN KEY (rule_id)
      REFERENCES rule_ids (id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL,
  CONSTRAINT unique_ruleid_role UNIQUE (rule_id, role_id)
)
WITH (
  OIDS=FALSE
);
ALTER TABLE reasons_de_de OWNER TO mailtrace;

-- Trigger: update_reasons_de_de_modtime on reasons_de_de

-- DROP TRIGGER update_reasons_de_de_modtime ON reasons_de_de;

CREATE TRIGGER update_reasons_de_de_modtime
  BEFORE UPDATE
  ON reasons_de_de
  FOR EACH ROW
  EXECUTE PROCEDURE update_time_updated_column();

-- Table: reasons_en_en

-- DROP TABLE reasons_en_en;

CREATE TABLE reasons_en_en
(
  id serial NOT NULL,
  role_id integer NOT NULL,
  rule_id integer NOT NULL,
  short_desc text NOT NULL,
  long_desc text,
  tag_desc character varying(50),
  time_created timestamp with time zone NOT NULL DEFAULT now(),
  time_updated timestamp with time zone NOT NULL DEFAULT now(),
  CONSTRAINT prid_id_reasons_en_en PRIMARY KEY (id),
  CONSTRAINT ref_reasons_en_en_roleid FOREIGN KEY (role_id)
      REFERENCES roles (id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL,
  CONSTRAINT ref_reasons_en_en_ruleid FOREIGN KEY (rule_id)
      REFERENCES rule_ids (id) MATCH SIMPLE
      ON UPDATE NO ACTION ON DELETE SET NULL,
  CONSTRAINT unique_ruleid_role_en_en UNIQUE (rule_id, role_id)
)
WITH (
  OIDS=FALSE
);
ALTER TABLE reasons_en_en OWNER TO mailtrace;

-- Trigger: update_reasons_en_en_modtime on reasons_en_en

-- DROP TRIGGER update_reasons_en_en_modtime ON reasons_en_en;

CREATE TRIGGER update_reasons_en_en_modtime
  BEFORE UPDATE
  ON reasons_en_en
  FOR EACH ROW
  EXECUTE PROCEDURE update_time_updated_column();


