<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created   : Mar 8, 2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2012-10-24 10:33:44 +0200 (Mi, 24 Okt 2012) $
 * last changed by: $LastChangedBy: d.mohn $
 * revision       : $LastChangedRevision: 9795 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.frontend/application/modules/auth/controllers/LoginController.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 */
/***/
require_once 'Hpls/Controller/Action.php';
require_once 'Hpls/Acl.php';
require_once 'Hpls/User/Object.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package HPLS
 * @subpackage Login
 * @version $LastChangedRevision: 9795 $
 * @since version
 *
 * tags
 */
class Auth_LoginController extends Hpls_Controller_Action {
	
	protected static $ALLOWED_MAINTENANCE_ROLE = array(
			Hpls_Consts0::ROLE_HELPDESK,
			Hpls_Consts0::ROLE_ADMIN );

	public function preDispatch() {
		if (!Hpls_Application::i()->isStandAlone()) {
			$this->_redirect('/admin');
			return;
		}
		if (Hpls_Auth::i()->isLoggedIn()) {
			$this->_redirect($this->getFrontController()->getBaseUrl().'/overview');
			return;
		}
	}

	/**
	 * The default action - show the home page
	 */
	public function indexAction() { // TODO Auto-generated LoginController::indexAction() default action
		isDebug('auth') && $this->log()->debug(__METHOD__ . ' START');
		if ($this->getRequest()->isPost()) {
			//do login with post data
			$this->_forward('login');
		} else {
			//display
			$this->_forward('show');
		}
	}

	public function loginAction() {
		//		try {
		isDebug('auth') && $this->log()->debug(__METHOD__ . ' START');
		
		if ($this->_request->isPost()) {
			/**
			 * @var Zend_Auth
			 */
			$forwardUrl = $this->getForwardUrl();
			if (Hpls_Auth::i()->isLoggedIn()) {
				isDebug('auth') && $this->log()->debug(__METHOD__ . " have an identity -> redirect to " . $this->_redirectUrl);
				$this->_redirect($forwardUrl);
				return;
			}
			
			$username = $this->getRequest()->getParam('login_username', '');
			$password = $this->getRequest()->getParam('login_password', '');
			
			if (empty($username)) {
				//FEHLER: kein anmeldename
				Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'login_admin', 'ac_auth_login_failed_invalid_username_error', -1, array(
						'username' => $username ), false);
				$this->_forward('show');
				return;
			}
			
			//formular
			$_authResult = Hpls_Auth::i()->login($username, $password);
			if ($_authResult !== false) {
				if ($_authResult->getCode() === Zend_Auth_Result::SUCCESS) {
					
					//gegen ein Source war die Authentifizierung erfolgreich und es ein User mit mindestens einer geforderten Rolle
					isDebug('auth') && $this->log()->debug(__METHOD__ . ": identity" . Hpls_Debug::print_r($_authResult->getIdentity()));
					isDebug('auth') && $this->log()->debug(__METHOD__ . ": forward to: " . $forwardUrl);
					$this->_redirect($forwardUrl);
					return;
				} else {
					switch ($_authResult->getCode()) {
						case Zend_Auth_Result::FAILURE_CREDENTIAL_INVALID :
							Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'login_admin', 'ac_auth_login_failed_invalid_credential_error');
							break;
						case Zend_Auth_Result::FAILURE_IDENTITY_AMBIGUOUS :
							Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'login_admin', 'ac_auth_login_failed_identity_ambiguous_error');
							break;
						case Zend_Auth_Result::FAILURE_IDENTITY_NOT_FOUND :
							Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'login_admin', 'ac_auth_login_failed_identity_not_found_error');
							break;
						case Zend_Auth_Result::FAILURE_UNCATEGORIZED :
						case Zend_Auth_Result::FAILURE :
							$this->log()->debug(Hpls_Debug::print_r($_authResult->getMessages(), __METHOD__));
							Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'login_admin', 'ac_auth_login_admin_failed_error');
							Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'login_admin', $_authResult->getMessages());
							break;
					}
				} //if SUCCESS
			} else { // if false
				Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'configuration', 'ac_configuration_no_authsource_error');
			
			}
		}
		
		//		} catch (Exception $ex) {
		//			$this->log()->err(__METHOD__ . ": login FAILED -> retry. Exception: $ex");
		//			$redirect = new Zend_Controller_Action_Helper_Redirector();
		//			$redirect->gotoSimpleAndExit('index', 'login', 'auth');
		//		}
		

		//		$this->_redirect('/auth/login');
		$this->_forward('show');
	}

	public function adminAction() {
		isDebug('auth') && $this->log()->debug(__METHOD__ . ' START');
		
		if ($this->_request->isPost()) {
			/**
			 * @var Zend_Auth
			 */
			$forwardUrl = $this->getForwardUrl();
			if (Hpls_Auth::i()->isLoggedIn()) {
				isDebug('auth') && $this->log()->debug(__METHOD__ . " have an identity -> redirect to " . $this->_redirectUrl);
				$this->_redirect($forwardUrl);
				return;
			}
			
			$username = $this->getRequest()->getParam('login_username', '');
			$password = $this->getRequest()->getParam('login_password', '');
			
			if (empty($username)) {
				//FEHLER: kein anmeldename
				Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'login_admin', 'ac_auth_login_failed_invalid_username_error', -1, array(
						'username' => $username ), false);
			} else {
				
				//formular
				$_authResult = Hpls_Auth::i()->login($username, $password);
				if ($_authResult !== false) {
					if ($_authResult->getCode() === Zend_Auth_Result::SUCCESS) {
						
						if (hasRole(self::$ALLOWED_MAINTENANCE_ROLE, null, true)) {
							//gegen ein Source war die Authentifizierung erfolgreich und es ein User mit mindestens einer geforderten Rolle
							isDebug('auth') && $this->log()->debug(__METHOD__ . ": identity" . Hpls_Debug::print_r($_authResult->getIdentity()));
							isDebug('auth') && $this->log()->debug(__METHOD__ . ": forward to: " . $forwardUrl);
							$this->_redirect($forwardUrl);
							return;
						} else {
							Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'login_admin', 'ac_auth_login_admin_no_sufficient_rights_error_msg', Hpls_Consts0::ERR_AUTH_NO_RIGHTS);
						}
					} else {
						switch ($_authResult->getCode()) {
							case Zend_Auth_Result::FAILURE_CREDENTIAL_INVALID :
								Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'login_admin', 'ac_auth_login_failed_invalid_credential_error');
								break;
							case Zend_Auth_Result::FAILURE_IDENTITY_AMBIGUOUS :
								Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'login_admin', 'ac_auth_login_failed_identity_ambiguous_error');
								break;
							case Zend_Auth_Result::FAILURE_IDENTITY_NOT_FOUND :
								Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'login_admin', 'ac_auth_login_failed_identity_not_found_error');
								break;
							case Zend_Auth_Result::FAILURE_UNCATEGORIZED :
							case Zend_Auth_Result::FAILURE :
								$this->log()->debug(Hpls_Debug::print_r($_authResult->getMessages(), __METHOD__));
								Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'login_admin', 'ac_auth_login_admin_failed_error');
								Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'login_admin', $_authResult->getMessages());
								break;
						}
					} //if SUCCESS
				} else { // if false
					Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'configuration', 'ac_configuration_no_authsource_error');
				
				}
			} // END: if empty($username)
		}
		
		$this->view->assign(HVConsts::V_PAGE_TITLE, 'ac_login_admin');
		$sess = Hpls_Application::i()->getSession(Hpls_Consts::SESSION_NS_AUTH);
		$sess->authToken = uniqid('auth');
		$this->view->assign('authToken', $sess->authToken);
		$this->view->maintenanceMode = false;
		Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_WARN, 'login_admin', 'ac_login_admin_warning_msg');
	}

	public function showAction() {
		isDebug('auth') && $this->log()->debug(__METHOD__ . ' START');
		Hpls_Msgs::i()->clearGlobalMsgs('auth');
		$this->view->assign(HVConsts::V_PAGE_TITLE, 'ac_login');
		$sess = Hpls_Application::i()->getSession(Hpls_Consts::SESSION_NS_AUTH);
		$sess->authToken = uniqid('auth');
		$this->view->assign('authToken', $sess->authToken);
		$offline = false;
		$cfgLocal = Hpls_Settings::i()->getConfig('ac.maintenance.local');
		$text = '';
		if (is_null($cfgLocal) || !$cfgLocal->offline) {
			try {
				$cfgGlobal = Hpls_Settings::i()->getConfig('ac.maintenance.global');
				if (!is_null($cfgGlobal)) {
					$offline = Hpls_Utils::toBool($cfgGlobal->offline);
					$text = $cfgGlobal->text;
				} else {
					$offline = false;
				}
			} catch (Exception $ex) {
			
			}
		} else {
			$offline = true;
			$text = $cfgLocal->text;
		}
		if ($offline) {
			$this->view->maintenanceText = $text;
		}
		$this->view->license = $this->checkLicense();
		$this->view->maintenanceMode = $offline;
	}

	public function rolerequiredAction() {
		isDebug('auth') && $this->log()->debug(__METHOD__ . ' START');
		$msg = Hpls_Msgs::i()->getGlobalMsgs('auth');
		
		$this->view->assign('requiredRole', $this->getRequest()->getParam('requiredRole', 'admin'));
	}

	protected function getForwardUrl() {
		$sess = Hpls_Application::i()->getSession(Hpls_Consts::SESSION_NS_AUTH);
		if (isset($sess->forwardUrl)) {
			$urlArray = $sess->forwardUrl;
			$url = array(
					$urlArray['module'],
					$urlArray['controller'],
					$urlArray['action'],
					implode('/', $urlArray['params']) );
			isDebug('auth') && $this->log()->debug(__METHOD__ . ": forward in session1: " . Hpls_Debug::print_r($url));
			$url = implode('/', $url);
			isDebug('auth') && $this->log()->debug(__METHOD__ . ": forward in session: " . $url);
			return $url;
		}
		return '/';
	}
	function checkLicense(){
		if(is_file('/opt/heinlein/mailtrace/frontend/license.hmt')){
			return '';
		} else {
			return 'Bitte erwerben sie eine <a href="mailto:support@heinlein-elements.de?subject=Mailtrace Lizenz&body=Bitte machen Sie mir ein Angebot.">Lizenz</a>';
		}
	}
	//	public function __call($methodName, array $args) {
//		$this->_forward('index');
//	}
}

