<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 01.08.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Mailtrace/Consts.php';
require_once dirname(dirname(dirname(realpath(__FILE__)))) . '/mailtrace/controllers/SearchController.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision$
 * @since 1.0.9
 */
class HmtReason_SearchtoolController extends Mailtrace_SearchController {

	public function init() {
		$this->contexts[] = array(
				'query2' => array(
						'json' ) );
		parent::init();
	}

	public function indexAction() {
		$this->_forward('search');
	}

	public function query2Action() {
		if ($this->isConstraintError()) {
			$this->view->assign('__error__', $this->getRequest()->getParam(Hpls_Constraint_Consts::REQ_KEY_RESULTS, null));
			return;
		}
		$this->prepareRequest();
		if (is_null($this->_query_token)) {
			require_once 'Hpls/Mailtrace/Exception/Search.php';
			throw new Hpls_Mailtrace_Exception_Search('Query-ID not present.', 3);
		}
		
		require_once 'Hmt/Db/Query/NomatchSearch.php';
		
		$sesToken = new Zend_Session_Namespace('hmt_query_' . $this->_query_token);
		
		$queryClause = $this->requestToClause($this->_searchArgs);
		$this->log()->debug(__METHOD__ . '## queryClause: ' . $queryClause);
		$log = Hmt_Log::i();
		$constraints = $this->requestToConstraints();
		$qry = $log->connect(new Hmt_Db_Query_NomatchSearch($queryClause, $constraints));
		$part = $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_PARTITION];
		$limit = $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_LIMIT];
		$result = array();
		
		do {
			$this->log()->debug(__METHOD__ . "## TRYING\nPART: $part\nOffset: ".$constraints->offset."\nLimit: $limit");
			//Partition setzen
			$this->_searched_ivs[] = $part;
			$qry->setPartition($part);
			//Abfrage nach DBIDs ausführen
			$qry->execute();
			/**
			 * @var array:Hmt_Log_DeliverEntry
			 */
			$res = $qry->getResults();
			if ($res != 0) {
				$all = $this->_getParam('r_noregex', false);
				foreach ($res as $deliver) {
					$reason = Hmt_Reason::i()->matchRule($deliver);
					
					if (empty($reason) || $all !== false) {
						//das, was wir suchen -> kein Reason-Match
						$result[] = $deliver;
					}
				}
			} else {
				//keine ergebnisse gefunden -> nächstes interval untersuchen
				if ($part == null) {
					$this->view->rlimit = $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_LIMIT];
					$this->view->roffset = $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_OFFSET];
					$this->view->reftime = $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_SEARCH_REF_TIME];
					$this->view->noresults = true;
				}
			}
			if (count($result) < $limit) {
				if (count($res) < $limit) {
					//partition erschöpft > nächste
					$part = Hmt_Log::i()->getPreviousInterval($part);
				} else {
					//nächste results in selber partition
					$constraints->offset += $limit; 
				}
			}
		} while (count($result) < $limit && $part != null);
		
		$this->view->delivers = $result;
		$this->view->roffset = $constraints->offset+$limit;
		$this->view->reftime = $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_SEARCH_REF_TIME];
		if (count($result) < $limit) {
			$this->view->finishSearch = true;
		}
		$this->reassignView();
		$this->log()->debug(__METHOD__ . '  COMPLETELY FINISHED ' . Hpls_Debug::print_r($this->_searched_ivs, 'SEARCHED INTERVALS'));
	}

	public function searchAction() {
		parent::searchAction();
		
	}

	protected function assignViewDefaults() {
		parent::assignViewDefaults();
		$this->view->searchControllerZF = 'hmtreason/searchtool';
		$ivs = Hmt_Log::i()->getIntervals();
		rsort($ivs);
		$this->view->intervals = $ivs;
	}

	protected function reassignView() {
		parent::reassignView();
		$all = $this->_getParam('r_noregex', false);
		$this->view->r_noregex = $all === false ? false : true;
	}

}
