<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 01.08.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/**
 */
require_once 'Hpls/Mailtrace/Consts.php';
require_once 'Hpls/Mailtrace/Controller/Action.php';
require_once 'Hmt/Log.php';

/**
 *
 * @todo comment
 * @author Daniel Mohn <d.mohn@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision$
 * @since 1.0.9
 */
class Hmtstats_MailController extends Hpls_Mailtrace_Controller_Action {

	// Actions
        public $reject_types = array(
                    0 => 'clean', 
                    1 => 'Spam',
                    2 => 'Greylisted',
                    3 => 'Policyd-Weight',
                    4 => 'Unknown Domain',
                    5 => 'RBL',
                    6 => 'Amavis Spam',
                    7 => 'Amavis Virus',
                    8 => 'Amavis Attachment',
                    9 => 'Gesamt',
                );
    
	public function indexAction() {
		$this->_forward('dindex');
	}
	
	// User Actions
	
	public function uindexAction(){
		$today = 'connectlog_' . date('Ymd') . '000000';
		$alltables = Hmt_Log::i()->getConnectTables();
		if(in_array($today, $alltables)){
			// gibt daten von heute
			$this->view->mailstats_user = $this->buildUserDailyStats($today);
		} else {
			// no data :/
			$this->view->mailstats_user = array();
		}
	}
	
	public function udomainAction(){
		$today = 'connectlog_' . date('Ymd') . '000000';
		$alltables = Hmt_Log::i()->getConnectTables();
		if(in_array($today, $alltables)){
			// gibt daten von heute
			$this->view->mailstats_domain = $this->buildUserDomainDailyStats($today);
		} else {
			// no data :/
			$this->view->mailstats_domain = array();
		}
	}
	
	// Connect Actions
	
	public function cindexAction() {
		$today = 'connectlog_' . date('Ymd') . '000000';
		$alltables = Hmt_Log::i()->getConnectTables();
	
		if(in_array($today, $alltables)){
			// gibt daten von heute
			$this->view->mailstats = $this->buildConnectDailyStats($today);
		} else {
			// no data :/
			$this->view->mailstats = array();
		}
	}
	
	public function ctypeAction(){
		$getStats = $this->buildConnectStatsByType();
		$this->view->mailstats = $getStats;
	}
	
	public function ctimeAction(){
		$getStats = $this->buildConnectStatsByDay();
		$this->view->mailstats = $getStats;
		// $this->view->mailstats_total = $this->countThem($getStats);
	}
	
	public function cserverAction(){
		$this->view->mailstats = $this->buildConnectStatsByServer();
	}
	
	
	public function buildUserDailyStats($day){
		$stats = Hmt_Log::i()->getStatistics();
		return $stats->getUserDailyStats($day);
	}
	
	public function buildUserDomainDailyStats($day){
		$stats = Hmt_Log::i()->getStatistics();
		return $stats->getUserDomainDailyStats($day);
	}
	
	
	public function buildConnectStatsByDay(){
		$alltables = Hmt_Log::i()->getConnectTables();
		
		$stats = Hmt_Log::i()->getStatistics();
		$stats_array = array();
		
		foreach ($alltables AS $table){
			$dummy = explode('_', $table);
			$date = substr($dummy[1], 0, 8);
			$newdate = substr($date, 0, 4) . "" . substr($date, 4, 2) . "" . substr($date, 6, 2);
			$stats_array[$newdate] = $stats->getConnectDailyStats($table);
		}
		ksort($stats_array);
		return $stats_array;
	}
	
	public function buildConnectStatsByServer(){
		$server = Hmt_Log::i()->getServers();
		$stats = Hmt_Log::i()->getStatistics();
		
		$server_stats = array();
		foreach($server AS $myserver){
			$mystats = $stats->getConnectServerStats($myserver->id);
			$server_stats[$myserver->s_name] = $mystats;
		}
		return $server_stats;
	}
	
	public function buildConnectDailyStats($day){
		$stats = Hmt_Log::i()->getStatistics();
		return $stats->getConnectTimedStats($day);
	}
	
        // Reject Actions
        
        public function stindexAction() {
                $this->view->reject_types = $this->reject_types;
                $this->view->mailstats = $this->buildRejectDailyStats();
        }
        
        public function sttimelineAction(){
                $this->view->reject_types = $this->reject_types;
                $this->view->mailstats = $this->buildRejectTimelineStats();
        }
        
        public function buildRejectTimelineStats(){
                $stats = Hmt_Log::i()->getStatistics();
                return $stats->getRejectTimelineStats();
        }
        
        public function buildRejectDailyStats(){
                $stats = Hmt_Log::i()->getStatistics();
                return $stats->getRejectDailyStats();
	}
        
	// Deliver Actions
	
	public function dindexAction() {
		$today = 'deliverlog_' . date('Ymd') . '000000';
		$alltables = Hmt_Log::i()->getDeliverTables();
		
		if(in_array($today, $alltables)){
			// gibt daten von heute
			$this->view->mailstats = $this->buildDeliveryDailyStats($today);
		} else {
			// no data :/
			$this->view->mailstats = array();
		}
	}

	public function dtypeAction(){
		$getStats = $this->buildDeliveryStatsByType();
		$this->view->mailstats = $getStats;
	}

	public function dtimeAction(){
		$getStats = $this->buildDeliveryStatsByDay();
		$this->view->mailstats = $getStats;
		$this->view->mailstats_total = $this->countThem($getStats);
	}
	
	public function dserverAction(){
		$this->view->mailstats = $this->buildDeliveryStatsByServer();
	}
	
	public function buildDeliveryStatsByServer(){
		$server = Hmt_Log::i()->getServers();
		$stats = Hmt_Log::i()->getStatistics();
		
		$server_stats = array();
		foreach($server AS $myserver){
			$mystats = $stats->getDeliveryServerStats($myserver->id);
			$server_stats[$myserver->s_name] = $mystats;
		}
		return $server_stats;
	}
	
	public function buildDeliveryStatsByType($day){		
		$getStats = $this->buildDeliveryStatsByDay();
		$stats_array = array(
							'total' => 0,
							'sent' => 0,
							'rejected' => 0,
							'deferred' => 0,
							'bounced' => 0
						);
		
		foreach($getStats AS $stats){
			foreach($stats AS $key => $value){
				$stats_array[$key] += $value;
			}
		}
		return $stats_array;
	}
	
	
	public function buildDeliveryDailyStats($day){
		$stats = Hmt_Log::i()->getStatistics();
		return $stats->getDeliveryTimedStats($day);
	}
	
	public function buildDeliveryStatsByDay(){
		$alltables = Hmt_Log::i()->getDeliverTables();
		
		$stats = Hmt_Log::i()->getStatistics();
		$stats_array = array();
		
		foreach ($alltables AS $table){
			$dummy = explode('_', $table);
			$date = substr($dummy[1], 0, 8);
			$newdate = substr($date, 0, 4) . "" . substr($date, 4, 2) . "" . substr($date, 6, 2);
			$stats_array[$newdate] = $stats->getDeliveryDailyStats($table);
		}
		ksort($stats_array);
		return $stats_array;
	}
	
	public function countThem($statsArray){
		$totalStats = array();
		foreach($statsArray AS $dailys){
			foreach ($dailys as $id=>$value) {
    			$totalStats[$id] += $value;
  			}
		}
		return $totalStats;
	}
}