<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created   : Mar 25, 2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2013-07-01 14:23:29 +0200 (Mo, 01 Jul 2013) $
 * last changed by: $LastChangedBy: d.mohn $
 * revision       : $LastChangedRevision: 10248 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/HMT/GUI/Core/trunk/hpls.hmt.gui.frontend/application/modules/mailtrace/controllers/ConfigController.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 */
/**
 */
require_once 'Hpls/Controller/Config.php';
require_once 'Hpls/Mailtrace/Consts.php';
require_once 'Zend/Config/Ini.php';
require_once 'Hpls/Settings.php';

/**
 *
 * @todo comment
 *      
 * @author author
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *           
 * @package HMT
 * @subpackage Konfiguration
 * @version $LastChangedRevision: 10248 $
 * @since 1.0.6
 */
class Mailtrace_ConfigController extends Hpls_Controller_Config {
	
	/**
	 * Mapping der Formularfeldnamen nach DB-spaltennamen für die LOG-DB.
	 *
	 * @var array:string->string
	 */
	protected static $FIELDS_LOGDB_MAP = array(
			'hmt_logdb_username' => Hpls_Utils_Db::KEY_USERNAME,
			"hmt_logdb_passwd" => Hpls_Utils_Db::KEY_PASSWORD,
			"hmt_logdb_tableprefix" => Hpls_Utils_Db::KEY_TABLEPREFIX,
			"hmt_logdb_dbname" => Hpls_Utils_Db::KEY_DBNAME,
			"hmt_logdb_port" => Hpls_Utils_Db::KEY_PORT,
			"hmt_logdb_hostip" => Hpls_Utils_Db::KEY_HOST,
			"hmt_logdb_type" => Hpls_Utils_Db::KEY_TYPE );
	/**
	 * Mapping der Formularfeldnamen nach DB-spaltennamen für die REASON-DB.
	 *
	 * @var array:string->string
	 */
	protected static $FIELDS_RSNDB_MAP = array(
			'username' => Hpls_Utils_Db::KEY_USERNAME,
			"passwd" => Hpls_Utils_Db::KEY_PASSWORD,
			"tableprefix" => Hpls_Utils_Db::KEY_TABLEPREFIX,
			"dbname" => Hpls_Utils_Db::KEY_DBNAME,
			"port" => Hpls_Utils_Db::KEY_PORT,
			"hostip" => Hpls_Utils_Db::KEY_HOST,
			"type" => Hpls_Utils_Db::KEY_TYPE );

	public function preDispatch() {
		if (!Hpls_Acl::i()->hasRole(Hpls_Acl::ROLE_ADMIN)) {
			$this->_forward('rolerequired', 'login', 'auth');
			$this->getRequest()->setParam('requiredRole', Hpls_Acl::ROLE_ADMIN);
			return;
		}
	}

	public function indexAction() {
		$this->_forward('daemon');
	}

	public function daemonAction() {
		$onoffValue = array();
		$lDaemonGlobal = Hpls_Settings::i()->getConfig('hmt.daemon.global');
		try {
			$daemonCfg = $lDaemonGlobal;
			if (is_null($daemonCfg)) {
				$daemonCfg = new Zend_Config(array(), true);
			}
			
			if ($this->getRequest()->isPost()) {
				$newValue = $this->getRequest()->getParam('hmt_config_daemon_onoff', Hpls_Mailtrace_Consts::DAEMON_DEFAULT_ONOFF);
				// nur bei wirklicher Änderung des Wertes etwas tun
				$daemonCfg->{Hpls_Mailtrace_Consts::DAEMON_DEFAULT_CONF_KEY} = $this->getRequest()->getParam('hmt_config_daemon_onoff', Hpls_Mailtrace_Consts::DAEMON_DEFAULT_ONOFF);
				
				try {
					Hpls_Settings::i()->setConfig('hmt.daemon.global', $daemonCfg)->save();
					$lDaemonGlobal = Hpls_Settings::i()->getConfig('hmt.daemon.global');
				} catch (Hpls_Exception_Configuration $ex) {
					$this->log()->err(__METHOD__ . " $ex");
					$this->addValueError($onoffValue, $ex);
				}
			}
		
		} catch (Hpls_Exception_Configuration $ex) {
			$this->log()->err(__METHOD__ . " $ex");
			$this->addValueError($onoffValue, $ex);
		}
		isDebug() && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($daemonCfg));
		$lDaemonGlobal = Hpls_Settings::i()->getConfig('hmt.daemon.global');
		
		$onoffValue['value'] = $this->_getParam('hmt_config_daemon_onoff', isset($lDaemonGlobal->{Hpls_Mailtrace_Consts::DAEMON_DEFAULT_CONF_KEY}) ? $lDaemonGlobal->{Hpls_Mailtrace_Consts::DAEMON_DEFAULT_CONF_KEY} : Hpls_Mailtrace_Consts::DAEMON_DEFAULT_ONOFF);
		$this->view->hmt_config_daemon_onoff = $onoffValue;
		$this->log()->debug(Hpls_Debug::print_r($this->getRequest()->getParams(), 'PARAMS'));
	}

	public function logdatabaseAction() {
		$dbCfg = Hpls_Settings::i()->getConfig('hmt.mtlog.storage.db', true, true);
		$postValues = array_intersect_key($this->_getAllParams(), self::$FIELDS_LOGDB_MAP);
		$getPosts = $this->_getAllParams();
		$ses = new Zend_Session_Namespace('hmt_wizard');
		if($getPosts['wizard_skip'] == 'Skip'){
			$ses->skip = 1;
		}
		$this->log()->debug(Hpls_Debug::print_r($postValues, __METHOD__));
		if ($this->getRequest()->isPost() && count($postValues) > 0) {
			$v = array();
			foreach ($postValues as $key => $value) {
				$dbCfg->{self::$FIELDS_LOGDB_MAP[$key]} = trim($value);
			}
			try {
				Hpls_Settings::i()->setConfig('hmt.mtlog.storage.db', $dbCfg, true)->clearCache();
			
			} catch (Exception $e) {
				Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'logdb', "Einstellungen für die Logdatenbank konnten nicht gespeichert werden.", 1);
			}
			
			if ($this->_getParam('hmt_logdb_create', 'no') !== 'no') {
				$this->log()->debug(__METHOD__ . ' CREATING DB');
				try {
					require_once 'Hmt/Log/Manage.php';
					Hmt_Log_Manage::i()->createStructure();
				} catch (Exception $ex) {
					$this->log()->err($ex->getMessage());
					Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'hmt_logdb_create', $ex->getMessage(), $ex->getCode());
				}
			}
			
			$this->log()->debug(Hpls_Debug::print_r($dbCfg, __METHOD__));
		}
		
		try {
			require_once 'Hmt/Log.php';
			Hmt_Log::i()->refreshConfig();
			Hmt_Log::i()->getDb()->query('SELECT 1;');
		} catch (Exception $e) {
			$this->log()->warn("Einstellungen für die Datenbank sind fehlerhaft oder die Datenbank ist momentan nicht erreichbar. Fehlermeldung: " . $e->getMessage());
			Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_WARN, 'logdb', "Einstellungen für die Datenbank sind fehlerhaft oder die Datenbank ist momentan nicht erreichbar.\\nFehlermeldung: " . $e->getMessage(), 2);
			if($ses->skip != 1){
				$this->_helper->redirector('wizard');
			}
			// Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_INFO, 'logdb', "Falls sie Mailtrace neu installiert haben, können sie auch den Wizard benutzen um den User und die Datenbanken anzulegen. <br />\\n<a style='margin-left:10px;' href='" . Zend_Controller_Front::getInstance()->getBaseUrl() . "/mailtrace/config/wizard'><b>Zum Wizard</b></a>\\n\\n");
		}
		
		foreach (self::$FIELDS_LOGDB_MAP as $iniKey) {
			if (($iniKey == Hpls_Utils_Db::KEY_PASSWORD) && (!Hpls_Application::i()->isStandAlone())) {
				$dbCfg->{$iniKey} = '';
			} else if (!isset($dbCfg->{$iniKey}))
				$dbCfg->{$iniKey} = '';
		}
		
		$this->view->hmt_logdbObj = $dbCfg;
		$this->view->hmt_logdb_types = array(
				'mysql' => 'MySQL',
				'pgsql' => 'PostgreSQL');
	}
	
	public function wizardAction(){
		$error = false;
		$postValues = $this->_getAllParams();
		$user = 'mailtrace';
		$this->view->wizardry = 0;
		$randpw = md5(uniqid('', true));
		if (
					!empty($postValues['hmt_wizard_type']) &&
					!empty($postValues['hmt_wizard_hostip']) &&
					!empty($postValues['hmt_wizard_port']) &&
					!empty($postValues['hmt_wizard_username']) &&
					!empty($postValues['hmt_wizard_password'])
			){
			
			$type = $postValues['hmt_wizard_type'];

			
			if ($type == 'pgsql'){
				$constr = "host=" . $postValues['hmt_wizard_hostip'] . " user=" . $postValues['hmt_wizard_username'] . " password=" . $postValues['hmt_wizard_password'] . " dbname=postgres";
				$connection = pg_connect($constr);
				
				if($connection !== FALSE){
					$randpw = 'dummy';
					$template1 = array();
					$template1[] = "CREATE USER $user WITH SUPERUSER CREATEDB LOGIN PASSWORD '$randpw';";
					$template1[] = "CREATE DATABASE mailtrace_logdata OWNER $user;";
					$template1[] = "CREATE DATABASE mailtrace_rules_vendor OWNER $user;";
					$template1[] = "CREATE DATABASE mailtrace_rules_customer OWNER $user;";

					foreach($template1 AS $sql){
						$result = pg_query($connection, $sql);
						if (!$result) {
							Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'logdb', "Error in SQL query: " . pg_last_error());
						}
					}
					pg_free_result($result);
					pg_close($connection);
				}				
				
			} else if ($type == 'mysql'){
				// try connection
				$connection = mysql_connect($postValues['hmt_wizard_hostip'],
											$postValues['hmt_wizard_username'],
											$postValues['hmt_wizard_password']);
				if($connection !== FALSE){
					// create tables
					
					$sqlDbs = array("CREATE DATABASE IF NOT EXISTS mailtrace_logdata;",
									"CREATE DATABASE IF NOT EXISTS mailtrace_rules_vendor;",
									"CREATE DATABASE IF NOT EXISTS mailtrace_rules_customer;");
					foreach($sqlDbs AS $sql){
						mysql_query($sql);
					}
					// create user
					
					$randpw = md5(uniqid());
					$host = $postValues['hmt_wizard_hostip'];

					$sqlUser[] = "CREATE USER '$user'@'$host' IDENTIFIED BY '$randpw';";
					$sqlUser[] = "GRANT SUPER, USAGE ON *.* TO '$user'@'$host' IDENTIFIED BY '$randpw' WITH MAX_QUERIES_PER_HOUR 0 MAX_CONNECTIONS_PER_HOUR 0 MAX_UPDATES_PER_HOUR 0 MAX_USER_CONNECTIONS 0;";
					$sqlUser[] = "GRANT ALL PRIVILEGES ON mailtrace_logdata.* TO '$user'@'$host';";
					$sqlUser[] = "GRANT ALL PRIVILEGES ON mailtrace_rules_vendor.* TO '$user'@'$host';";
					$sqlUser[] = "GRANT ALL PRIVILEGES ON mailtrace_rules_customer.* TO '$user'@'$host';";
					
					foreach ($sqlUser AS $sql){
						mysql_query($sql);
					}
					mysql_close($connection);
				}
			}
			$dbCfg = Hpls_Settings::i()->getConfig('hmt.mtlog.storage.db', true, true);
			$dbCfg->host = $postValues['hmt_wizard_hostip'];
			$dbCfg->port = $postValues['hmt_wizard_port'];
			$dbCfg->type = $postValues['hmt_wizard_type'];
			$dbCfg->dbname = 'mailtrace_logdata';
			$dbCfg->username = $user;
			$dbCfg->password = $randpw;
			
			try {
				Hpls_Settings::i()->setConfig('hmt.mtlog.storage.db', $dbCfg, true)->clearCache();					
			} catch (Exception $e) {
				$error = true;
				Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'logdb', "Einstellungen für die Logdatenbank konnten nicht gespeichert werden.", 1);
			}
			
			try {
				require_once 'Hmt/Log/Manage.php';
				Hmt_Log_Manage::i()->createStructure();
			} catch (Exception $ex) {
				$error = true;
				$this->log()->err($ex->getMessage());
				Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'hmt_logdb_create', $ex->getMessage(), $ex->getCode());
			}
			
			
			$db = Hpls_Settings::i()->getConfig('hmt.reason.storage', true, true);
			$bothDb = array('vendor', 'customer');
			foreach ($bothDb AS $mydb){
				$db->$mydb->host = $postValues['hmt_wizard_hostip'];
				$db->$mydb->port = $postValues['hmt_wizard_port'];
				$db->$mydb->type = $postValues['hmt_wizard_type'];
				$db->$mydb->dbname = 'mailtrace_rules_' . $mydb;
				$db->$mydb->username = $user;
				$db->$mydb->password = $randpw;
				$db->$mydb->create = TRUE;
			}
			try {
				Hpls_Settings::i()->setConfig('hmt.reason.storage', $db, true)->clearCache();
				$db = Hpls_Settings::i()->getConfig('hmt.reason.storage');
			} catch (Exception $e) {
				$error = true;
				Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'logdb', "Einstellungen für die Logdatenbank konnten nicht gespeichert werden.\\nFehlermeldung: " . $e->getMessage(), 1);
			}

			try {
				require_once 'Hmt/Reason/Manage.php';
				Hmt_Reason_Manage::i()->createStructure($bothDb);
			} catch (Exception $ex) {
				$error = true;
				$this->log()->err($ex->getMessage());
				Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'hmt_rsndb_create', $ex->getMessage(), $ex->getCode());
			}
			try {
				require_once 'Hmt/Reason.php';
				Hmt_Reason::i()->refreshConfig();
				Hmt_Reason::i()->getDb()->query('SELECT 1;');
				Hmt_Reason::i()->clearCache();				
			} catch (Exception $e) {
				$error = true;
				$this->log()->warn("Einstellungen für die Datenbank sind fehlerhaft oder die Datenbank ist momentan nicht erreichbar. Fehlermeldung: $e");
				Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_WARN, 'logdb', "Einstellungen für die Datenbank sind fehlerhaft oder die Datenbank ist momentan nicht erreichbar.\\nFehlermeldung: " . $e->getMessage(), 2);
			}
			if($error == false){
				Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_INFO, 'wizard', "User und Datenbanken wurden angelegt.\\n<b>User:</b> $user \\n<b>Passwort:</b> $randpw" );
				$this->_helper->actionStack('update','db','hmtreason', array('hmtr_update' => 'yipyip'));
				$this->view->wizardry = 1;
			}
			
		} else {
			if ($this->getRequest()->isPost() && count($postValues) > 0){
				$this->log()->err('Es wurden nicht alle Felder ausgefüllt!');
				Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'wizard', "Es wurden nicht alle Felder ausgefüllt!");
			}
		}
		$this->view->hmt_db_types = array(
				'mysql' => 'MySQL',
				'pgsql' => 'PostgreSQL');		
	}
	

	public function reasondatabaseAction() {
		$db = Hpls_Settings::i()->getConfig('hmt.reason.storage', true, true);
		// $this->log()->debug(Hpls_Debug::print_r($db, __METHOD__));
		// $this->log()->debug(Hpls_Debug::print_r($this->_getAllParams(),
		// __METHOD__." POSTVALUES"));
		// $this->log()->debug(Hpls_Debug::print_r($this->_getAllParams(),
		// __METHOD__." POSTVALUES"));
		$postValues = $this->_getParam('hmt_rsndb', null);
		$this->log()->debug(Hpls_Debug::print_r($postValues, __METHOD__));
		if ($this->getRequest()->isPost() && $postValues !== null) {
			$this->log()->debug(Hpls_Debug::print_r($postValues, __METHOD__));
			foreach ($postValues as $key => $spec) {
				if (isset($db->{$key})) {
					foreach ($spec as $k => $v) {
						if (array_key_exists($k, self::$FIELDS_RSNDB_MAP))
							$db->{$key}->{self::$FIELDS_RSNDB_MAP[$k]} = trim($v);
					}
				}
			}
			try {
				Hpls_Settings::i()->setConfig('hmt.reason.storage', $db, true)->clearCache();
				$db = Hpls_Settings::i()->getConfig('hmt.reason.storage');
			} catch (Exception $e) {
				Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'logdb', "Einstellungen für die Logdatenbank konnten nicht gespeichert werden.\\nFehlermeldung: " . $e->getMessage(), 1);
			}

			$this->log()->debug(Hpls_Debug::print_r($db, __METHOD__.' db TOCREATE'));
			$this->log()->debug(Hpls_Debug::print_r($postValues, __METHOD__.' specs TOCREATE'));
				
			// zu erzeugende DBs sammeln, damit sie in einem rutsch erstellt
			// werden und der cache nur 1x aufgefrischt wird
			$toCreate = array();
			foreach ($postValues as $key => $spec) {
				if (isset($db->{$key})) {
			$this->log()->debug(Hpls_Debug::print_r($spec, __METHOD__.' spec TOCREATE'));
					if (isset($spec['create']) && $spec['create'] == 'true') {
						$toCreate[] = $key;
					}
				}
			}
			$this->log()->debug(Hpls_Debug::print_r($toCreate, __METHOD__.' TOCREATE'));
			// wenn's was zu erzeugen gibt, dann los...
			if (!empty($toCreate)) {
				$this->log()->debug(__METHOD__ . ' CREATING DB');
				try {
					require_once 'Hmt/Reason/Manage.php';
					Hmt_Reason_Manage::i()->createStructure($toCreate);
				} catch (Exception $ex) {
					$this->log()->err($ex->getMessage());
					Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'hmt_rsndb_create', $ex->getMessage(), $ex->getCode());
				}
			
			}
		
		}
		
		try {
			require_once 'Hmt/Reason.php';
			Hmt_Reason::i()->refreshConfig();
			Hmt_Reason::i()->getDb()->query('SELECT 1;');
			Hmt_Reason::i()->clearCache();
		} catch (Exception $e) {
			$this->log()->warn("Einstellungen für die Datenbank sind fehlerhaft oder die Datenbank ist momentan nicht erreichbar. Fehlermeldung: $e");
			Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_WARN, 'logdb', "Einstellungen für die Datenbank sind fehlerhaft oder die Datenbank ist momentan nicht erreichbar.\\nFehlermeldung: " . $e->getMessage(), 2);
		}
		$this->log()->debug(Hpls_Debug::print_r($db, __METHOD__));
		
		if (FALSE === Hpls_Acl::i()->isAllowed('hmt.reason.db.cache', 'edit')) {
			unset($db->{Hpls_Mailtrace_Consts::CONFIG_KEY_CACHEDB});
		}
		// if (FALSE ===
		// Hpls_Acl::i()->isAllowed('hmt.reason.db.vendor','edit')) {
		// unset($db[Hpls_Mailtrace_Consts::CONFIG_KEY_VENDORDB]);
		// }
		if (!Hpls_Application::i()->isStandAlone()) {
			// auf Appliance passwörter nicht anzeigen
			foreach ($db as $dbCfg) {
				if (isset($dbCfg->{Hpls_Utils_Db::KEY_PASSWORD})) {
					$dbCfg->{Hpls_Utils_Db::KEY_PASSWORD} = '';
				}
			}
		}
		
		$this->view->hmt_reasonDbs = $db;
		$this->view->hmt_rsndb_types = array(
				'pgsql' => 'PostgreSQL',
				'mysql' => 'MySQL',
				'sqlite' => 'SQLite' );
	}

	public function logdbrotateAction() {
		$value = array();
		$ini = null;
		try {
			$cfg = Hpls_Settings::i()->getConfig('hmt.daemon.global', true);
			if (is_null($cfg))
				$cfg = new Zend_Config(array(), true);
			
			if ($this->getRequest()->isPost()) {
				isDebug() && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($this->getRequest()->getParams()));
				// save
				$newValue = $this->getRequest()->getParam('hmt_config_logdbrotate_interval', Hpls_Mailtrace_Consts::DAEMON_DEFAULT_ROTATE);
				
				// check
				if (0 < preg_match('/^\s*(\d+)\s*d?\s*$/i', $newValue, $matches)) {
				
				} else {
				}
				
				$matches = array();
				if (0 < preg_match('/^\s*(\d+)\s*d?\s*$/i', $newValue, $matches)) {
					if (isset($cfg->database)) {
						$ini2 = $cfg->database;
					} else {
						$ini2 = new Zend_Config(array(), true);
						$cfg->database = $ini2;
					}
					$ini2->db_max_age = $matches[1];
					Hpls_Settings::i()->setConfig('hmt.daemon.global', $cfg)->save();
					$cfg = Hpls_Settings::i()->getConfig('hmt.daemon.global', true); // neu
						                                                                 // einlesen
						                                                                 // vom
						                                                                 // store
				} else { // preg_match
				         // error
				         // $this->log()->err(Hpls_Debug::print_r($newValue,
				         // 'Formulardaten wurden gefälscht.'));
				         // FEHLER
					Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'validate', 'Es dürfen nur Zahlen angeben werden.');
				}
			}
		
		} catch (Hpls_Exception_Configuration $ex) {
			Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'Es liegt ein Fehler im Konfigurationsverarbeitungssystem vor: ' . $ex->getMessage(), $ex->getCode());
		}
		
		// view
		if (isset($cfg->database)) {
			$ini2 = $cfg->database;
			if (isset($ini2->db_max_age)) {
				$value = $ini2->db_max_age;
			}
		} else {
			$value = 14;
		}
		// $value = $value['value'] . 'd';
		// $this->view->hmt_config_logdbrotate_interval_options = $intervals;
		$this->view->hmt_config_logdbrotate_interval = $value;
	}

	public function groupingAction() {
		require_once 'Hmt/Log.php';
		$states = Hmt_Log::i()->getDeliverStates();
		$cfg = Hpls_Settings::i()->getConfig('hmt.webui.result.grouping');
		if (is_null($cfg)) {
			$cfg = new Zend_Config(array(), true);
		}
		foreach ($states as $state) {
			if (!isset($cfg->{$state})) {
				$cfg->{$state} = new Zend_Config(array(
						'count' => 5 ), true);
			}
		}
		
		if ($this->getRequest()->isPost()) {
			$this->log()->debug(Hpls_Debug::print_r($this->getRequest()->getParams(), __METHOD__));
			foreach ($this->getRequest()->getParams() as $key => $value) {
				if (stripos($key, 'hmt_config_result_grouping_count_') === 0) {
					$this->log()->debug(Hpls_Debug::print_r($key, __METHOD__));
					$state = substr($key, strlen('hmt_config_result_grouping_count_'));
					if (in_array($state, $states)) {
						$cfg->{$state}->count = intval($value);
					}
				}
			}
			try {
				Hpls_Settings::i()->setConfig('hmt.webui.result.grouping', $cfg)->save();
				$cfg = Hpls_Settings::i()->getConfig('hmt.webui.result.grouping', true);
			} catch (Exception $ex) {
				$this->log()->err($ex);
				Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'hmt.webui.result.grouping', $ex->getMessage());
			}
		}
		
		$this->view->hmt_config_result_grouping = $cfg->toArray();
	}

	/**
	 *
	 * @todo comment
	 */
	public function checkAction() {
	
	}

	/**
	 * Enter description here .
	 *
	 *
	 *
	 *
	 * ..
	 *
	 * @return Hpls_Configuration_Mailtrace_ConfigInterface
	 */
	protected function getMailtraceConfigService() {
		return $this->getConfigService('Hpls_Configuration_Mailtrace_ConfigInterface');
	}

}