<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : Jan 29, 2010
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2013-06-20 14:02:22 +0200 (Do, 20 Jun 2013) $
 * last changed by: $LastChangedBy: d.mohn $
 * revision       : $LastChangedRevision: 10190 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/HMT/GUI/Core/trunk/hpls.hmt.gui.frontend/application/modules/mailtrace/controllers/SearchController.php $
 *
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/**
 */
require_once 'Hpls/Mailtrace/Controller/Action.php';
require_once 'Hpls/Mailtrace/Consts.php';
require_once 'Hpls/Request/Utils.php';
require_once 'Hmt/Log.php';
require_once 'Hmt/Reason.php';
require_once 'Hpls/Var/Clause.php';
require_once 'Hmt/Log/TransportJson.php';
require_once 'Hpls/Utils/Time.php';
require_once 'Hpls/Utils/Db.php';

/**
 *
 * @todo comment
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 * @package HMT
 * @subpackage Suchsystem
 * @version $LastChangedRevision: 10190 $
 * @since alpha
 */
class Mailtrace_SearchController extends Hpls_Mailtrace_Controller_Action implements 
	Hpls_Mailtrace_Consts {
	
	/**
	 * Wird gelesen von: $this->_helper->contextSwitch()->initContext()
	 * 
	 * @var array:string:array
	 */
	public $contexts = array(
			'query' => array(
					'json' ),
			'query2' => array(
					'json' ),
			'getconnects' => array(
					'json' ),
			'getdelivers' => array(
					'json' ),
			'update' => array(
					'json' ),
			'getintervals' => array(
					'json' ) );
	
	/**
	 *
	 * @todo comment
	 * @var unknown_type
	 */
	protected static $SEARCH_V_DATE = array(
			'v_date_from',
			'v_date_to' );
	
	/**
	 *
	 * @todo comment
	 * @var unknown_type
	 */
	protected static $SEARCH_FIELDS = array(
			'c_mail_from' => Hpls_Mailtrace_Consts::SRCH_MAILFROM,
			'c_msgid' => Hpls_Mailtrace_Consts::SRCH_MSG_ID,
			'v_date_from' => Hpls_Mailtrace_Consts::SRCH_C_TIME,
			'v_date_to' => Hpls_Mailtrace_Consts::SRCH_C_TIME,
			'v_queue_id' => Hpls_Mailtrace_Consts::SRCH_C_QUEUE_ID,
			'c_queue_id' => Hpls_Mailtrace_Consts::SRCH_C_QUEUE_ID,
			'd_queue_id' => Hpls_Mailtrace_Consts::SRCH_D_QUEUE_ID,
			'v_host' => Hpls_Mailtrace_Consts::SRCH_CLIENT,
			'c_client' => Hpls_Mailtrace_Consts::SRCH_CLIENT,
			'd_relay' => Hpls_Mailtrace_Consts::SRCH_RELAY_HOST,
			'c_mxname' => Hpls_Mailtrace_Consts::SRCH_MX,
			'd_status' => Hpls_Mailtrace_Consts::SRCH_STATUS,
			'd_reason' => Hpls_Mailtrace_Consts::SRCH_REASON,
			'd_mail_to' => Hpls_Mailtrace_Consts::SRCH_MAILTO );
	/**
	 *
	 * @todo comment
	 * @var unknown_type
	 */
	protected static $RESULT_OPT_FIELDS = array(
			Hpls_Mailtrace_Consts::RESOPT_LIMIT => Hpls_Mailtrace_Consts::RESOPT_LIMIT,
			Hpls_Mailtrace_Consts::RESOPT_OFFSET => Hpls_Mailtrace_Consts::RESOPT_OFFSET,
			Hpls_Mailtrace_Consts::RESOPT_DIRECTION => Hpls_Mailtrace_Consts::RESOPT_DIRECTION,
			Hpls_Mailtrace_Consts::RESOPT_TIME_JUMP => Hpls_Mailtrace_Consts::RESOPT_TIME_JUMP,
			Hpls_Mailtrace_Consts::RESOPT_TIME_JUMP_REF => Hpls_Mailtrace_Consts::RESOPT_TIME_JUMP_REF,
			Hpls_Mailtrace_Consts::RESOPT_SEARCH_START_TIME => Hpls_Mailtrace_Consts::RESOPT_SEARCH_START_TIME,
			Hpls_Mailtrace_Consts::RESOPT_PARTITION => Hpls_Mailtrace_Consts::RESOPT_PARTITION );
	
	protected $_searchArgs = null;
	protected $_resultOptions = null;
	protected $_query_token = null;
	protected $_searched_ivs = array();

	public function init() {
		$this->_helper->contextSwitch()->setAutoDisableLayout(true);
		$this->_helper->contextSwitch()->initContext();
		
		// $contextSwitch = $this->_helper->getHelper('contextSwitch');
		// $contextSwitch->addActionContext('getconnects', 'json')
		// ->initContext()->;
	}

	public function preDispatch() {
		isDebug('log_request') && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($this->getRequest(), ' REQUEST'));
	}

	public function indexAction() {
		$this->_forward('form');
	}

	public function formAction() {
		$this->prepareRequest(); // muss drin sein, damit die suchparameter geladen werden.
		// 		$this->_query_token = $this->_getParam('hmt_query_token');
		

		try {
			$this->view->hmt_servers = Hmt_Log::i()->getServers();
		} catch (Zend_Db_Exception $ex) {
			
			switch (Hpls_Utils_Db::getErrorCategory(Hmt_Log::i()->getDb(), $ex->getCode())) {
				case Hpls_Utils_Db::ERR_CAT_STRUCTURE :
				case Hpls_Utils_Db::ERR_CAT_CONNECTION :
				case Hpls_Utils_Db::ERR_CAT_UNKNOWN :
				default :
					Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'search', $ex->getMessage(), $ex->getCode(), null, false, true);
					$this->_redirect($this->getFrontController()->getBaseUrl() . '/config/logdatabase');
					return;
			}
		}
		// $this->reassignView();
		if (!hasRole(Hpls_Acl_Role::POSTMASTER)) {
			Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_INFO, 'userinfo', 'Die Suche ist auf Ihre eigenen E-Mails-Adressen beschränkt.');
		}
		$groupSettings = Hpls_Settings::i()->getConfig('hmt.webui.result.grouping');
		
		if (FALSE === is_null($groupSettings)) {
			$ret = array();
			foreach ($groupSettings->toArray() as $k => $v) {
				if (!isset($v['count'])) {
					$ret[$k] = array(
							'count' => 5 );
				} else if ($v['count'] <= 0) {
					$ret[$k] = array(
							'count' => 1000000 );
				} else {
					$ret[$k] = $v;
				}
			
			}
			$this->view->hmt_group_limits = $ret;
		} else {
			$states = Hmt_Log::i()->getDeliverStates();
			$ret = array();
			foreach ($states as $state) {
				if (!isset($cfg->{$state})) {
					$ret[$state] = array(
							'count' => 5 );
				}
			}
			
			$this->view->hmt_group_limits = $ret;
		}
		$this->assignViewDefaults();
		$this->reassignView();
		if (!isset($this->view->hmt_srch)) {
			$x = array();
			foreach (self::$SEARCH_FIELDS as $k => $v) {
				$x[$k] = '';
			}
			$this->view->hmt_srch = $x;
		}
		$today = mktime(0, 0, 0, date("m", time()), date("d",time()), date("Y",time()));
		$yesterday = mktime(0, 0, 0, date("m"), date("d")-1, date("Y"));
		$daybeforeyesterday = mktime(0, 0, 0, date("m"), date("d")-2, date("Y"));
		
		$days = array();
		foreach (Hmt_Log::i()->getIntervals() AS $day){
			setlocale(LC_ALL, 'de_DE.UTF8', 'de_DE', 'de', 'DE');
			$newdate = substr($day, 0, 4) . '-' . substr($day, 4,2) . '-' . substr($day, 6,2);
			$readabledate = substr($day, 6,2) . '.' . substr($day, 4,2) . '.' . substr($day, 0, 4);
			$checkdate = str_replace('-', '', $newdate);
			
			if($checkdate == date('Ymd',$today)){
				$weekday = 'heute';
			} else if($checkdate == date('Ymd',$yesterday)){
				$weekday = 'gestern';
			} else if($checkdate == date('Ymd',$daybeforeyesterday)){
				$weekday = 'vorgestern';
			} else {
				$weekday = strftime("%A", strtotime($newdate));
			}
			$days[$newdate] = $readabledate . ' (' . $weekday . ')';
		}
		$days = array_reverse($days);
		$this->view->hmt_days = $days;
	}

	public function searchAction() {
		$this->prepareRequest();
		if (!$this->getRequest()->has('hmt_query_token')) {
			$this->_query_token = uniqid();
			$this->getRequest()->setParam('hmt_query_token', $this->_query_token);
			// Anfrage
			// Zend_Session::
		}
		$ses = new Zend_Session_Namespace('hmt_query_' . $this->_query_token);
		$t = Hmt_Log::i()->getServerTime();
		$ses->searchStartTime = '0.'. str_pad(($t % 1000),3,'0',STR_PAD_LEFT).'00000 '. ((int)($t/1000));
// 		$ses->searchStartTime = microtime();
		isDebug('hmt_search_values') && $this->log()->debug(__METHOD__ . ': setting search time : ' . $ses->searchStartTime);
		if ($this->isConstraintError()) {
			$this->getRequest()->setParam('has_errors', true);
			$this->_forward('form');
			return;
		}
		if (!isset($ses->oldest)) {
			$ses->oldest = 0;
			$ses->newest = 0;
		}
// 		$ses->searchStartTime = microtime();
		$this->_forward('form');
	}

	/**
	 * Komplette Anfrage für AJAX.
	 * Request-Parameter:
	 * <dl>
	 * <dt>rjump</dt><dd></dd>
	 * <dt>rpart</dt><dd></dd>
	 * <dt>rdir</dt><dd></dd>
	 * <dt>roffset</dt><dd></dd>
	 * <dt>rlimit</dt><dd></dd>
	 * <dt>rtime</dt><dd>Anker der Suche, wenn <0, dann neues aktuelles rtime
	 * setzen.</dd>
	 * </dl>
	 */
	public function queryAction() {
		
		if ($this->isConstraintError()) {
			$this->view->assign('__error__', $this->getRequest()->getParam(Hpls_Constraint_Consts::REQ_KEY_RESULTS, null));
			return;
		}
		$this->prepareRequest();

		if (is_null($this->_query_token)) {
			require_once 'Hpls/Mailtrace/Exception/Search.php';
			throw new Hpls_Mailtrace_Exception_Search('Query-ID not present.', 3);
		}
		
		require_once 'Hmt/Db/Query/LogSearch.php';
		
		$sesToken = new Zend_Session_Namespace('hmt_query_' . $this->_query_token);
		
		$queryClause = $this->requestToClause($this->_searchArgs);
		isDebug('hmt_search_values') && $this->log()->debug(__METHOD__ . '## queryClause: ' . $queryClause);
		$log = Hmt_Log::i();
		$constraints = $this->requestToConstraints();
		$qry = $log->connect(new Hmt_Db_Query_LogSearch($queryClause, $constraints));
		$part = $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_PARTITION];
		
		do {
			$this->log()->debug(__METHOD__ . "## TRYING PART: $part");
			// Partition setzen
			$this->_searched_ivs[] = $part;
			$qry->setPartition($part);
			// Abfrage nach DBIDs ausführen
			$qry->execute();
			$res = $qry->getResults();
			if ($res != 0) {
				$this->processQueryResults($log, $res);
			
			} else {
				// keine ergebnisse gefunden -> nächstes interval untersuchen
				$part = Hmt_Log::i()->getPreviousInterval($part);
				if ($part == null) {
					$this->view->rlimit = $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_LIMIT];
					$this->view->roffset = $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_OFFSET];
					$this->view->reftime = $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_SEARCH_REF_TIME];
					$this->view->noresults = true;
				}
			
			}
		} while ($res == 0 && $part != null);
		$this->reassignView();
		$this->log()->debug(__METHOD__ . '  COMPLETELY FINISHED' . Hpls_Debug::print_r($this->_searched_ivs, 'SEARCHED INTERVALS'));
	
	}

	/**
	 *
	 * @todo comment
	 * @param log
	 * @param res
	 */
	protected function processQueryResults($log, $res) {
        $sesToken = new Zend_Session_Namespace('hmt_query_' . $this->_query_token);
		$sesToken->oldest = $res['oldest'];
		$sesToken->newest = $res['newest'];
		
		$this->view->oldest = $res['oldest'];
		$this->view->newest = $res['newest'];
		
		$qryCon = $log->connect($res['connect']);
		$qryCon->execute();
		$resCon = $qryCon->getResults();
		$this->log()->debug(__METHOD__ . '  FINISHED SEARCH > to view');
		$this->view->rlimit = $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_LIMIT];
		$this->view->roffset = $res['offset'];
		$this->view->reftime = $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_SEARCH_REF_TIME];
		$this->view->connects = array_values($qryCon->getResults());
	}

	/**
	 * Anfrage nur nach connectlog-einträgen
	 */
	public function getconnectsAction() {
		require_once 'Hmt/Db/Query/Connects.php';
		
		$log = Hmt_Log::i();
		$qry = $log->connect(new Hmt_Db_Query_Connects(array(
				69,
				70 )));
		$qry->setPartition($this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_PARTITION]);
		$qry->execute();
		$this->view->connects = $qry->getResults();
	
	}

	/**
	 * Anfrage nur nach deliverlog-Einträgen
	 */
	public function getdeliversAction() {
		$cids = $this->_getParam('cids', null);
		if (is_null($cids)) {
			$this->view->error = "No cids";
			$this->forward('error');
			return;
		}
		
		$cids = explode(',', $cids);
		$reqCIDs = array();
		foreach ($cids as $cid) {
			$s = explode('c', $cid);
			if (count($s) == 2) {
				if (!isset($reqCIDs[$s[0]])) {
					$reqCIDs[$s[0]] = array();
				}
				// duplikate gleich mit ausschließen
				$reqCIDs[$s[0]][$s[1]] = $s[1];
			}
		}
		$this->log()->debug(__METHOD__ . Hpls_Debug::print_r($reqCIDs, 'requested CIDs'));
		require_once 'Hmt/Db/Query/Delivers.php';
		
		$ret = array();
		foreach ($reqCIDs as $part => $cids) {
			$log = Hmt_Log::i();
			if ($log->isValidInterval($part)) {
				
				$clause = new Hpls_Var_Clause(Hpls_Var_Clause::CL_IN);
				$clause->addValue(Hpls_Mailtrace_Consts::SRCH_D_DBID, array_values($cids));
				// @todo utime nutzen
				$qry = $log->connect(new Hmt_Db_Query_Delivers($clause));
				$qry->setPartition($part);
				$qry->execute();
				$res = $qry->getResults();
				
				$this->log()->debug('####### !!! ####### -- About to use REASON for DELIVERS');
				
				if (Hmt_Reason::i()->isReadyToUse()) {
					
					$this->log()->debug('####### !!! ####### -- REASON is ready to use');
					
					$n=0;
					foreach ($res as $deliver) {
						$n++;
						$this->log()->debug('####### !!! ####### -- cycling delivers, Element '.$n);
						
						$reason = Hmt_Reason::i()->matchRule($deliver);
						if (!is_null($reason)) {
							
							$this->log()->debug('####### !!! ####### -- MATCHRULE-Result ist NICHT null :)');
							$this->log()->debug(print_r($reason, TRUE));
							
							// die Connect-Eintragsdaten werden erst im Browser
							// per JS ergänzt!
							// Weil sonst zusätzliche Zeit zum Laden der
							// Connects drauf geht und das nicht zu knapp.
							
							$deliver->setShortExplain($reason->getShortExplanation());
							$deliver->setReasonTag($reason->getReasonTag());
							
							$this->log()->debug('####### !!! ####### -- setShortExplain("'.$reason->getShortExplanation().'")');
							$this->log()->debug('####### !!! ####### -- setReasonTag("'.$reason->getReasonTag().'")');
						
						} else {
							$this->log()->debug('####### !!! ####### -- ERROR! MATCHRULE-Result ist NULL!');
						}
						#$deliver->tevt = 'THIS IS A TEST!';
					}
				} else {
					$this->log()->debug('####### !!! ####### -- ERROR! REASON is NOT ready to use!');
				}
				
				$this->log()->debug('####### !!! ####### -- merging return arrays...');
				$ret = array_merge($ret, array_values($res));
				
				// @todo reason ersetzung
			} else {
				// @todo Error handling
			}
		}
		$this->log()->debug(__METHOD__ . Hpls_Debug::print_r($ret, 'results'));
		$this->view->delivers = new Hmt_Log_TransportJson($ret);
		$this->reassignView();
	}

	public function getintervalsAction() {
		$this->view->intervals = Hmt_Log::i()->getIntervals();
	}

	public function updateAction() {
		$cids = $this->_getParam('cids', null);
		if (is_null($cids)) {
			$this->view->error = "No cids";
			$this->forward('error');
			return;
		}
		
		$cids = explode(',', $cids);
		$reqCIDs = array();
		foreach ($cids as $cid) {
			$s = explode('c', $cid);
			if (count($s) == 2) {
				if (!isset($reqCIDs[$s[0]])) {
					$reqCIDs[$s[0]] = array();
				}
				// duplikate gleich mit ausschließen
				$reqCIDs[$s[0]][$s[1]] = $s[1];
			}
		}
		$this->log()->debug(__METHOD__ . Hpls_Debug::print_r($reqCIDs, 'requested CIDs'));
		require_once 'Hmt/Db/Query/Delivers.php';
		
		$ret = array();
		foreach ($reqCIDs as $part => $cids) {
			$log = Hmt_Log::i();
			if ($log->isValidInterval($part)) {

				/*
				 * $qryCon = $log->connect($res['connect']); $qryDel =
				 * $log->connect($res['deliver']); $qryCon->execute();
				 * $qryDel->execute(); $resCon = $qryCon->getResults(); $resDel
				 * = $qryDel->getResults(); foreach ($resDel as $deliver) {
				 * $reason = Hmt_Reason::i()->matchRule($deliver); if
				 * (!is_null($reason)) { if (isset($resCon[$deliver->d_dbid])) {
				 * $reason->setConnect($resCon[$deliver->d_dbid]); }
				 * $deliver->setShortExplain($reason->getShortExplanation()); }
				 * } $this->log()->debug(__METHOD__ . ' FINISHED SEARCH > to
				 * view'); $this->view->rlimit =
				 * $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_LIMIT];
				 * $this->view->roffset = $res['offset']; $this->view->connects
				 * = array_values($qryCon->getResults()); $this->view->delivers
				 * = new Hmt_Log_TransportJson($qryDel->getResults());
				 * $this->view->qar = $this->_getParam('qar', 0);
				 */
				
				$clause = new Hpls_Var_Clause(Hpls_Var_Clause::CL_IN);
				$clause->addValue(Hpls_Mailtrace_Consts::SRCH_D_DBID, array_values($cids));
				// @todo utime nutzen
				$qry = $log->connect(new Hmt_Db_Query_Delivers($clause));
				$qry->setPartition($part);
				$qry->execute();
				$res = $qry->getResults();
				$ret = array_merge($ret, array_values($res));
				
				// @todo reason ersetzung
			} else {
				// @todo Error handling
			}
		}
		$this->log()->debug(__METHOD__ . Hpls_Debug::print_r($ret, 'results'));
		$this->view->delivers = new Hmt_Log_TransportJson($ret);
		$this->reassignView();
	}
	
	protected function filterFromAndToValues(){
		if((isset($this->_searchArgs['c_mail_from']) && $this->_searchArgs['c_mail_from'] != '') && $this->_searchArgs['d_mail_to'] != ''){
			$emails = Hpls_Auth::i()->getUserObject()->getEmails();
			// $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($emails, '$emails'));
			$all_emails = array();
			if (is_array($emails)) {
				foreach ($emails as $email) {
					$all_emails[] = strtolower($email);
				}
			} else {
				$this->log()->warn(__METHOD__ . " UserObject has invalid email data: " . Hpls_Debug::print_r($emails));
			}
			if (hasRole(Hpls_Acl_Role::POSTMASTER)) {
				return true;
			}
			$from = strtolower($this->_searchArgs['c_mail_from']);
			$to = strtolower($this->_searchArgs['d_mail_to']);
			if(!in_array($from, $all_emails) && !in_array($to, $all_emails)){
				$this->_searchArgs['c_mail_from'] = $all_emails[0];
			}
		} else {
            $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($this->_searchArgs, '_searchArgs'));
        }
	}
	
	protected function prepareRequest() {
		
		$args = array_change_key_case($this->getRequest()->getParams(), CASE_LOWER);
		
		$this->_query_token = $this->_getParam('hmt_query_token', null);
		
		$this->_searchArgs = array_intersect_key($args, self::$SEARCH_FIELDS);
		
		$this->log()->debug(__METHOD__ . Hpls_Debug::print_r($this->_searchArgs, '_searchArgs before'));
		
		// HMT-375: Filtern wenn User was anderes als seine mails durchsuchen will
		$this->filterFromAndToValues();
		
		// $keyArray = array_combine(self::$RESULT_OPT_FIELDS,
		// self::$RESULT_OPT_FIELDS);
		$this->_resultOptions = array_intersect_key($args, self::$RESULT_OPT_FIELDS);
		
		isDebug('hmt_search_values') && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($this->_resultOptions, 'ResultOptions before'));
		
		if (FALSE == isset($this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_DIRECTION])) {
			$this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_DIRECTION] = Hpls_Mailtrace_Consts::RESOPT_DIRECTION_DEFAULT;
		}
		$this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_DIRECTION] = strtolower($this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_DIRECTION]);
		if (FALSE == isset($this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_LIMIT])) {
			$this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_LIMIT] = Hpls_Mailtrace_Consts::RESOPT_LIMIT_DEFAULT;
		}
		$this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_LIMIT] = intval($this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_LIMIT]);
		// wenn OFFSET nicht gesetzt oder JUMPZEIT angegeben => OFFSET == 0
		if (FALSE == isset($this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_OFFSET]) || TRUE == isset($this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_TIME_JUMP])) {
			$this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_OFFSET] = 0;
		}
		$this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_OFFSET] = intval($this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_OFFSET]);
		
		if (isset($this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_TIME_JUMP])) {
			$this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_TIME_JUMP] = Hpls_Utils_Time::timediffToSeconds($this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_TIME_JUMP]);
		}
		
		if (isset($this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_TIME_JUMP_REF])) {
			$t = strtotime($this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_TIME_JUMP_REF]);
			if (false == $t) {
				require_once 'Hpls/Mailtrace/Exception/Search.php';
				throw new Hpls_Mailtrace_Exception_Search('Jump time ref date is not a valid date: "' . $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_TIME_JUMP_REF], 2);
			}
			$this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_TIME_JUMP_REF] = $t;
		}
		
		if (isset($this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_SEARCH_REF_TIME])) {
			$t = strtotime($this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_SEARCH_REF_TIME]);
			if (false == $t) {
				require_once 'Hpls/Mailtrace/Exception/Search.php';
				throw new Hpls_Mailtrace_Exception_Search('Jump time ref date is not a valid date: "' . $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_TIME_JUMP_REF], 2);
			}
			$this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_SEARCH_REF_TIME] = $t;
		} else {
			$sesToken = new Zend_Session_Namespace('hmt_query_' . $this->_query_token);
			$this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_SEARCH_REF_TIME] = $sesToken->searchStartTime;
		}
		
		// resultoptions normalisieren und defaults setzen, falls nicht
		// vorhanden
		

		$this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_PARTITION] = $this->calcPartition();
		
		isDebug('hmt_search_values') && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($this->_searchArgs, 'SearchArgs'));
		isDebug('hmt_search_values') && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($this->_resultOptions, 'ResultOptions'));
	}

	/**
	 * Enter description here .
	 * ..
	 * 
	 * @return array:Hpls_Var_Clause null
	 */
	protected function requestToClause($searchArgs) {
		$ret = new Hpls_Var_Clause(Hpls_Var_Clause::CL_AND);
		$date_c_clause = new Hpls_Var_Clause(Hpls_Var_Clause::CL_AND);
		$date_d_clause = new Hpls_Var_Clause(Hpls_Var_Clause::CL_AND);
		foreach ($searchArgs as $k => $v) {
			if (!empty($v)) {
				switch ($k) {
					case 'v_date_from' :
						$date_c_clause->addValue(Hpls_Mailtrace_Consts::SRCH_C_TIME, $v, Hpls_Var_Clause::OP_GE);
						// $date_d_clause->addValue(Hpls_Mailtrace_Consts::SRCH_D_TIME,
						// $v, Hpls_Var_Clause::OP_GE);
						break;
					case 'v_date_to' :
						$date_c_clause->addValue(Hpls_Mailtrace_Consts::SRCH_C_TIME, $v, Hpls_Var_Clause::OP_LE);
						// $date_d_clause->addValue(Hpls_Mailtrace_Consts::SRCH_D_TIME,
						// $v, Hpls_Var_Clause::OP_LE);
						break;
					case 'v_host' :
						// zusammenfassung von client && host
						$ret->addValue(new Hpls_Var_Clause(Hpls_Var_Clause::CL_OR, array(
								Hpls_Mailtrace_Consts::SRCH_CLIENT => $v,
								Hpls_Mailtrace_Consts::SRCH_RELAY_HOST => $v )));
						break;
					case 'v_queue_id' :
						$ret->addValue(new Hpls_Var_Clause(Hpls_Var_Clause::CL_OR, array(
								Hpls_Mailtrace_Consts::SRCH_C_QUEUE_ID => $v,
								Hpls_Mailtrace_Consts::SRCH_D_QUEUE_ID => $v )));
						break;
					case 'c_mxname' :
						if ($v != -1) {
							$ret->addValue(self::$SEARCH_FIELDS[$k], $v);
						}
						break;
					
					default :
						// Wenn Values verODERt sind mit '|'
						$split = preg_split('#\s*(?<!\\\)\\|\s*#', $v);
						if (count($split) > 1) {
							$tmpCl = new Hpls_Var_Clause(Hpls_Var_Clause::CL_OR);
							foreach ($split as $v1) {
								$tmpCl->addValue($k, $v1);
							}
							$ret->addValue($tmpCl);
						} else {
							$ret->addValue(self::$SEARCH_FIELDS[$k], $v);
						}
						break;
				}
			}
		}
		// @todo Reference-Zeitpunkt einfügen
		

		// if ($this->isNewerDirection()) {
		// //suche nach neueren EMails
		// $newUtime = microtime();
		// $oldTime =
		// $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_SEARCH_REF_TIME];
		// $cl = new Hpls_Var_Clause();
		// $cl->addValue(Hpls_Mailtrace_Consts::SRCH_C_UTIME, $newUtime,
		// Hpls_Var_Clause::OP_LT);
		// $cl->addValue(Hpls_Mailtrace_Consts::SRCH_C_UTIME, $oldTime,
		// Hpls_Var_Clause::OP_GE);
		// $ret->addValue($cl);
		// //ORDER BY muss noch umgekehrt werden
		// } else {
		// //suche nach ältere EMails
		// $ret->addValue(Hpls_Mailtrace_Consts::SRCH_C_UTIME,
		// $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_SEARCH_REF_TIME],
		// Hpls_Var_Clause::OP_LT);
		// }
		

		if (!$date_c_clause->isEmpty()) {
			if (!$date_d_clause->isEmpty()) {
				$x = new Hpls_Var_Clause();
				$x->addValue($date_c_clause);
				$x->addValue($date_d_clause);
				$ret->addValue($x);
			} else {
				$ret->addValue($date_c_clause);
			}
		}
		$this->log()->debug(__METHOD__ . "## " . $ret);
		return $ret;
	}

	protected function requestToConstraints() {
		$ret = new Hmt_Db_Query_Constraints($this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_LIMIT], $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_OFFSET]);
		$ret->addOrderBy(Hpls_Mailtrace_Consts::SRCH_C_TIME, $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_DIRECTION] == Hpls_Mailtrace_Consts::RESOPT_DIRECTION_OLDER);
		$ret->direction = $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_DIRECTION];
		$ret->startSearchTime = $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_SEARCH_REF_TIME];
		
		return $ret;
	}

	protected function reassignView() {
		// if (FALSE == is_null($this->_searchArgs)) {
		$this->assignViewDefaults();
		$x = array();
		if (!empty($this->_searchArgs)) {
			$x = $this->_searchArgs;
		}
		foreach (self::$SEARCH_FIELDS as $k => $v) {
			if (!isset($x[$k]))
				$x[$k] = '';
		}
		$t_time = $f_time = $s_day = '';
		
		if(isset($x['v_date_to']) && $x['v_date_to'] != ''){
			$t_time = date('H:i:s' , (int)$x['v_date_to']);
			$s_day = date('Y-m-d' , (int)$x['v_date_to']);
		}
		if(isset($x['v_date_from']) && $x['v_date_from'] != ''){
			$f_time = date('H:i:s' , (int)$x['v_date_from']);
		}
		
		if ($t_time != ''){
			$x['v_date_to_dummy'] = $t_time;
		} else {
			$x['v_date_to_dummy'] = '23:59:59';
		}
		if ($f_time != ''){
			$x['v_date_from_dummy'] = $f_time;
		} else {
			$x['v_date_from_dummy'] = '00:00:00';
		}		
		$x['v_day'] = $s_day;
		
		$this->view->hmt_srch = $x;
		
		$this->view->hmt_result = $this->_resultOptions;
		// }
		$this->view->hmt_query_token = $this->_query_token;
		$this->view->qar = $this->_getParam('qar', 0);
		$this->view->searched_ivs = $this->_searched_ivs;
		$this->view->mtSearchExtendState = $this->_getParam('mtSearchExtendState', '0');
	}

	protected function assignViewDefaults() {
		$this->view->searchControllerZF = 'mailtrace/search';
	}

	/**
	 * Berechnet die Partition, in der gesucht werden soll.
	 * 
	 * @return NULL
	 */
	protected function calcPartition() {
		
		// Prio1: Suche per 'Überspringe X minuten...'
		if (isset($this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_TIME_JUMP]) && isset($this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_TIME_JUMP_REF])) {
			$time = 0;
			if ($this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_DIRECTION] == Hpls_Mailtrace_Consts::RESOPT_DIRECTION_NEWER) {
				// plus jumptime -> neuere zeit
				$time = $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_TIME_JUMP_REF] + $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_TIME_JUMP];
			} else {
				// minus jumptime -< ältere zeit
				$time = $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_TIME_JUMP_REF] - $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_TIME_JUMP];
			}
			$this->log()->debug(Hpls_Debug::print_r($time, __METHOD__));
			return Hmt_Log::i()->getContainingInterval($time);
		}
		// Prio2: Angabe der Partition per 'rpart'
		if (TRUE == isset($this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_PARTITION])) {
			if (isset($this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_PARTITION]) && FALSE == in_array($this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_PARTITION], Hmt_Log::i()->getIntervals())) {
				require_once 'Hpls/Mailtrace/Exception/Search.php';
				throw new Hpls_Mailtrace_Exception_Search('The specified partition "' . $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_PARTITION] . '" does not exists in log database.', 1);
			}
			$this->log()->debug(__METHOD__ . " RESOPT_PART INTERVAL");
			
			return $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_PARTITION];
		}
		
		// Prio3: Suchparameter
		

		if (array_key_exists('v_date_to', $this->_searchArgs) && !empty($this->_searchArgs['v_date_to'])) {
			return Hmt_Log::i()->getContainingInterval($this->_searchArgs['v_date_to']);
		} elseif (array_key_exists('v_date_to', $this->_searchArgs) && !empty($this->_searchArgs['v_date_to'])) {
			return Hmt_Log::i()->getContainingInterval($this->_searchArgs['v_date_to']);
		}
		$this->log()->debug(__METHOD__ . " GET NEWEST INTERVAL: " . Hmt_Log::i()->getNewestInterval());
		
		return Hmt_Log::i()->getNewestInterval();
	}

	protected function isNewerDirection() {
		return $this->_resultOptions[Hpls_Mailtrace_Consts::RESOPT_DIRECTION] === Hpls_Mailtrace_Consts::RESOPT_DIRECTION_NEWER;
	}

}

