<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 30.08.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hmt/Reason/Edit.php';
require_once 'Hmt/Reason/Entry.php';

class Hmt_Reason_Updater {
    
    const OPTION_KEY_CONFIG = 'config';
    
    protected static $_instance;
    
    protected $_entryList = array();
    protected $_langs = array();
    protected $_config = null;
    protected $_files = array();
    protected $_archive = null;
    protected $_archiveVersion = null;
    protected $_createTime = null;
    protected $_extractMethod = null;
    protected $_availableExtracts = array('zip');
    /**
	 * @var Hmt_Reason_Db
	 */
    protected $_db = null;
    protected $_dbSource = null;
    protected $_dbVersion = null;
    protected $_filename = null;
    
	protected $_tmpdir = null;
	protected $_processSuccess = true;	
	
	protected $_isInitialized = false;
	protected $_isFirstEntry = true;
	
	/**
	 * @return Hmt_Reason_Exporter
	 */
	public static function i() {
		if (is_null(self::$_instance)) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}
	/**
     * Setzen der Options aus der GUI. Wenn Options NULL ist werden Default-Werte verwendet.
     * Optionen:
     * exportdir = Exportpfad (String)
     * langs = Sprachen, der zu exportierenden TBLs (Array)
     * database = Name des Datenbank-Objektes
     * @param Array $options
     */
    public function setOptions($options) {
        $this->_config = $options[self::OPTION_KEY_CONFIG];
        $this->_dbSource = $options['database'];
        $this->_createTime = intval(microtime(true));
	    $this->_tmpdir = $options[self::OPTION_KEY_CONFIG]->dir . DIRECTORY_SEPARATOR . $this->_createTime . '_' . md5('export_hmtr_');
	    $this->_extractMethod = strtolower($options['extractmethod']);
	    $this->_archive = $options['file'];
	    
        $this->getDbO();
    }
    /**
     * @return Boolean
     */
    public function hasProcessSuccess() {
        return $this->_processSuccess;
    }
    
    public function getVersion() {
        return $this->_dbVersion;
    }
    
	public function run() {
	    $configVersion = $this->_config->archiveversion;
	    $this->extractFile($this->_archive);
	    $this->readConfig();
	    
	    if ($configVersion == $this->_archiveVersion) {
	        $this->updateData();
	        Hmt_Reason_Edit::i()->clearCache();    
	    } else {
	        Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'hmt_reason_update', "Update kann nicht erfolgen. Grund:<br />Die Archiv-Version der Updatedatei ist nicht gleich der Einstellung in der Config.<br />Archiv-Version: " . $this->_archiveVersion . "<br />Config: " . $configVersion);
	        $this->_processSuccess = false;
	    }
	}
	
	protected function extractFile($filename) {
	    list($dirname, $type) = explode('.', $filename);
        switch ($this->_extractMethod) {
            case 'zip':
                $this->extractZipArchive($filename, $dirname);
                break;
                
            default:
                Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'hmt_reason_update', "Ungültige Extrahierungs-Methode: '" . $this->_extractMethod . "'.");
                $this->_processSuccess = false;
                break;
        }
        $this->_tmpdir = $dirname;
	}
	
	protected function extractZipArchive($filename, $destiantion) {
	    try {
    	    if (FALSE === class_exists('ZipArchive')) {
    	        throw new Exception("Ziparchiv $filename konnte nicht erstellt werden.");
    	    }
    	    
        	$zip = new ZipArchive();
            
            if (TRUE !== $zip->open($filename)) {
                throw new Exception("Ziparchiv $filename konnte nicht erstellt werden.");
            }
             
            if ($zip->numFiles < 1) {
                throw new Exception("Es wurden keine Dateien im Archive $filename gefunden.");
            }
            
            if (FALSE === $zip->extractTo($destiantion)) {
                throw new Exception("Datei $filename konnte nicht nach $destiantion entpackt werden.");
            }   
            $zip->close();
            unlink($filename);
    	    
	    } catch (Exception $e) {
	        Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'hmt_reason_export', $e->getMessage());
	        $this->_processSuccess = false;
	        return;
	    }
	}
	
	protected function readConfig() {
	    $config = new Zend_Config_Ini($this->_tmpdir. '/controller.ini');
	    $config = $config->settings;
	    $this->_langs = explode(',', $config->languages);
	    $this->_dbVersion = $config->version;
	    $this->_config = $config->import;
	    $this->_archiveVersion = $config->archive_version;
	}
	
	protected function updateData() {
	    $files = explode(',', $this->_config->insertsort);
	    $filelist = scandir($this->_tmpdir);
	    
	    foreach ($files as $prefix) {
	        foreach ($filelist as $tmpfile) {
	            if (FALSE !== strpos($tmpfile, $prefix) && preg_match('/\.json$/i', $tmpfile)) {
	                $filepath = $this->_tmpdir . DIRECTORY_SEPARATOR . $tmpfile;
	                $this->_isFirstEntry = true;
                    try {
    	                $lines = file($filepath);
    	                foreach ($lines as $json) {
//    	                    echo "<b>$filepath</b><br>$json<hr>";
    	                    $entry = Hmt_Reason_Entry::parseJsonStr($json);
                            $this->processInsert($entry);
    	                }
    	            } catch (Exception $ex) {
    	                Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_WARN, 'hmt_reason_update', "Meldung: " . $ex->getMessage() . ".");
    	                $this->_processSuccess = false;
    	            }
	            }
	        }
	    }
	}
	
	/**
	 * BEFORE INSERT / WITHOUT INDEX (disable)
	 * @param Hmt_Reason_Entry $entry
	 */
	protected function processPrepareInsert(Hmt_Reason_Entry $entry) {
	    // TODO: CREATE
	    $files = explode(',', $this->_config->createsort);
	    $filelist = scandir($this->_tmpdir);
	    $doPrepareInsert = true;
	    
	    switch (get_class($this->_db->getDb())) {
	        case 'Zend_Db_Adapter_Pdo_Mysql':
	            $sql = 'mysql';
	            break;
	            
	        case 'Zend_Db_Adapter_Pdo_Postgres':
	            $sql = 'pgsql';
	            break;
	            
	        default:
	            Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_WARN, 'hmt_reason_update', "Der Datenbank-Adapter ist nicht definiert: ".get_class($this->_db->getDb()));
	            $doPrepareInsert = false;
	    }
	    
	    if (TRUE === $doPrepareInsert) {
            if (FALSE === $this->_isInitialized) {
                // einmalige Ausführung 
                $this->_isInitialized = true;
                
                foreach ($files as $prefix) {   
        	        foreach ($filelist as $tmpfile) {	            
        	            if (FALSE !== strpos($tmpfile, $prefix.$sql)) {
    	                    $query = trim(file_get_contents($this->_tmpdir . DIRECTORY_SEPARATOR . $tmpfile));
    	                    
    	                    if (!empty($query)) {
    	                        $this->_db->getDb()->query($query);    	                        
    	                    } 
        	            }
        	        }
        	    }
            }
	        
    	    if (TRUE === $this->_isFirstEntry) {
    	        // Ausführung befor der erste Eintrag in die Datenbank geschrieben wird
    	        $this->_isFirstEntry = false;    	        
    	        $this->_db->getDb()->query("TRUNCATE " . $entry->getTablename() . ";");    
    	    }   
	    }	    
	}
	
	protected function processInsert(Hmt_Reason_Entry $entry) {
	    
	    // BEFORE INSERT
	    $this->processPrepareInsert($entry);
	    
	    $db = $this->_db->getDb();
	    
	    $data = array();
	    $table = $entry->getTablename();
	    try {
            $result = $db->describeTable($table);
    	    foreach ($entry->getKeys() as $key) {
    	        $data[$key] = $entry->getValue($key); 
    	    }
    	    $db->insert($entry->getTablename(), $data);   
        } catch (Exception $e) {
            Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_WARN, 'hmt_reason_update', "Meldung: ".$e->getMessage());
        }
	    
        // AFTER INSERT
//	    $this->processPostInsert($entry);
	}

	/**
	 * AFTER INSERT / CREATE INDEX
	 */
	protected function processPostInsert(Hmt_Reason_Entry $entry) {
	    
	}
	
	/**
     * Löscht ein Verzeichnisbaum, den der Apacheuser auch löschen darf. (hier für TMP-Dir)
     * @param String $dir
     * @return void
     */
	protected function deleteTree($dir, $isRoot = true) {
	    $files = glob( $dir . '*', GLOB_MARK );
        foreach( $files as $file ){
            if( substr( $file, -1 ) == '/' ) {
                self::deleteTree($file, false);
            } else {
                if (FALSE === unlink( $file )) {
                    Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_WARN, 'hmt_reason_update', "System konnte die Tmp-Datei '$file' nicht löschen.");   
                }
            }
        }
   
        if (is_dir($dir)) { 
            if( FALSE === rmdir( $dir )) {
                Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_WARN, 'hmt_reason_update', "System konnte den Tmp-Ordner '$dir' nicht löschen.");
            } 
        } 
	}
	/**
	 * @return Hmt_Reason_Db
	 */
	protected function getDbO() {
	    $this->_db = Hmt_Reason_Edit::i()->getDbO($this->_dbSource);
		return $this->_db;
	}	
}
//----------------------------------------------------------------------------------------------
//require_once 'Hmt/Reason/Edit.php';
//
//class Hmt_Reason_Updater {
//	
//	protected $_options = null;
//	protected $_config = null;
//	protected $_templates = array();
//	protected $_sqlkey = null;
//	protected $_tmpdir = null;
//	protected $_langs = array();
//	protected $_version = null;
//	protected $_dbAdapter = null;
//	protected $_files = array();
//	protected $_processSuccess = true;
//	
//	public static function factory($options) {
//	    $instance = null;
//	    switch ($options['type']) {
//	        case 'mysql':
//                require_once 'Hmt/Reason/Updater/Mysql.php';
//	            $instance = Hmt_Reason_Updater_Mysql::i();
//	            break;
//	        case 'pgsql':
//	            require_once 'Hmt/Reason/Updater/Postgres.php';
//	            $instance = Hmt_Reason_Updater_Postgres::i();
//	            
//	            break;
//        }
//        $instance->setOptions($options);
//	    return $instance;
//	}
//	/**
//	 * @var Hmt_Reason_Db
//	 */
//	protected $_db = null;
//	protected $_dbSource = null;
//	protected $_filename = null;
//
//	protected function __construct() {
//	    
//	}
//	/**
//     * Setzen der Options aus der GUI. Wenn Options NULL ist werden Default-Werte verwendet.
//     * Optionen:
//     * exportdir = Exportpfad (String)
//     * langs = Sprachen, der zu exportierenden TBLs (Array)
//     * database = Name des Datenbank-Objektes
//     * @param Array $options
//     */
//    public function setOptions($options) {
//        $this->_options = $options;
//        $this->_dbSource = $options['database'];
//        $this->_sqlkey = $options['type'];
//        $this->getDbO();
//    }
//    /**
//     * @return Boolean
//     */
//    public function hasProcessSuccess() {
//        return $this->_processSuccess;
//    }
//    
//	public function run() {
//        $filename = $this->_options['file'];
//	    
//	    $this->extractFile($filename);
//	    $this->readConfig();
//	    $this->updateData();
//	    
//	    $this->deleteTree($this->_tmpdir);
//	    
//	    Hmt_Reason_Edit::i()->clearCache();
//	}
//	
//	protected function extractFile($filename) {
//	    list($dirname, $type) = explode('.', $filename);
//        switch ($this->_options['conf']->extractmethod) {
//            case 'zip':
//                $this->extractZipArchive($filename, $dirname);
//                break;
//        }
//        $this->_tmpdir = $dirname;
//	}
//	
//	protected function extractZipArchive($filename, $destiantion) {
//	    $zip = new ZipArchive();
//	    $zip->open($filename);
//	    $zip->extractTo($destiantion);
//        $zip->close();
//	}
//	
//	protected function readConfig() {
//	    $config = new Zend_Config_Ini($this->_tmpdir . '/controller.ini');
//	    $config = $config->settings;
//	    $this->_langs = explode(',', $config->languages);
//	    $this->_version = $config->version;
//	    $this->_config = $config->import;
//	}
//	
//	protected function updateData() {
//	    $this->processPrepareInserts();
//	    $this->processInsert();
//	    $this->processPostInserts();
//	}
//	
//	/**
//	 * BEFORE INSERT / WITHOUT INDEX (disable)
//	 * OVERIDE
//	 */
//	protected function processPrepareInserts() {
//	    $cfg = $this->_config->before;
//	    
//	    // TODO: CREATE
//	    // TODO: ALTER TABLE
//	    
//	    if ($cfg->disable_trigger == 1) {
//	        // Trigger abschalten
//	    }
//	}
//	
//	protected function processInsert() {
//	    $files = explode(',', $this->_config->insertsort);
//	    $filelist = scandir($this->_tmpdir);
//	    
//	    foreach ($files as $prefix) {
//	        $prefix .= $this->_sqlkey;
//	        foreach ($filelist as $tmpfile) {
//	            if (FALSE !== strpos($tmpfile, $prefix)) {
//	                $filepath = $this->_tmpdir . DIRECTORY_SEPARATOR . $tmpfile;
//	                
//	                $tmp = file_get_contents($filepath);
//	                try {
//	                    $lines = file($filepath);
//	                    foreach ($lines as $tmp) {
//	                        $this->_db->getDb()->query($tmp);
//                            
//	                    }
//	                } catch (Exception $ex) {
//	                    Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'hmt_reason_update', "Update fehlgeschlagen. Meldung: '" . $ex->getMessage() . "'.");
//	                    $this->_processSuccess = false;
//	                }
//	                break;
//	            }
//	        }
//	    }
//	}
//
//	/**
//	 * AFTER INSERT / CREATE INDEX
//	 */
//	protected function processPostInserts() {
//	    $cfg = $this->_config->before;
//	    if ($cfg->disable_trigger == 1) {
//	        // Trigger anschalten
//	    }
//	}
//	
//	/**
//     * Löscht ein Verzeichnisbaum, den der Apacheuser auch löschen darf. (hier für TMP-Dir)
//     * @param String $dir
//     * @return void
//     */
//	protected function deleteTree($dir, $isRoot = true) {
//	    $files = glob( $dir . '*', GLOB_MARK );
//        foreach( $files as $file ){
//            if( substr( $file, -1 ) == '/' ) {
//                self::deleteTree($file, false);
//            } else {
//                if (FALSE === unlink( $file )) {
//                    Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_WARN, 'hmt_reason_update', "System konnte die Tmp-Datei '$file' nicht löschen.");   
//                }
//            }
//        }
//   
//        if (is_dir($dir)) { 
//            if( FALSE === rmdir( $dir )) {
//                Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_WARN, 'hmt_reason_update', "System konnte den Tmp-Ordner '$dir' nicht löschen.");
//            } 
//        } 
//	}
//	/**
//	 * @return Hmt_Reason_Db
//	 */
//	protected function getDbO() {
//	    $this->_db = Hmt_Reason_Edit::i()->getDbO($this->_dbSource);
//		return $this->_db;
//	}
//}