<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 * 
 * created        : 12.10.2010
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2012-02-22 16:19:52 +0100 (Mi, 22 Feb 2012) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9234 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/HMT/GUI/Core/trunk/hpls.hmt.gui.frontend/libs/Hpls/Application/Mailtrace.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Application.php';
require_once 'Hpls/Debug.php';
require_once 'Hpls/Configuration/Factory.php';
require_once 'Hpls/Mailtrace/Consts.php';
require_once 'Hpls/Settings.php';
require_once 'Hpls/Settings/Handler/Db.php';
require_once 'Hpls/Mailtrace/Err.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 9234 $
 * @since 1.0.5
 */
class Hpls_Application_Mailtrace extends Hpls_Application {
	
	protected $_isStandAlone = null;
	protected $_version = null;

	public static function initApplicationObject() {
		if (is_null(self::$_instance)) {
			self::$_instance = new Hpls_Application_Mailtrace();
		}
	}

	public function runInits() {
		$this->initDebug();
		$this->initConfigs();
		$this->initRequestId();
		$this->initLogger();
		$this->log()->DEBUG("#######################################################################################");
		$this->log()->DEBUG("+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++");
		$this->initFrontcontroller();
		$this->initSubModules();
		$this->initHplsSettings();
		$this->initConfigurationServices();
		if (!$this->isStandAlone()) {
			$this->log()->info(__METHOD__.'Running in Appliance Mode');
		}
		$this->_initDone = true;
	}

	public function isStandAlone() {
		if (null === $this->_isStandAlone) {
			$cfg = Hpls_Settings::i()->getConfig('application.product');
			$this->_isStandAlone = isset($cfg->standalone) ? Hpls_Utils::toBool($cfg->standalone) : true;
			$this->log()->debug(Hpls_Debug::print_r($cfg,__METHOD__));
//			$prods = $this->getInstalledHPLSProducts();
//			$this->_isStandAlone = count($prods) === 2 && in_array('HMT', $prods) && in_array('AC', $prods);
		}
		return $this->_isStandAlone;
	}

	protected function initIncludePaths() {
		$this->_configDir = realpath(APP_CONFIGS_PATH);
	}

	/**
	 * 
	 */
	protected function initFrontcontroller() {
		$front = $this->getFrontController();
		$config = $this->getConfig('frontcontroller');
		
		//		$front->setControllerDirectory(self::$conrollerDirs);
		$front->setParam('useModules', true);
		$front->addModuleDirectory(APPLICATION_PATH . "/modules");
		$front->setDefaultModule('default');
		$front->setDefaultControllerName('index');
		if (isset($config['baseUrl'])) {
			$front->setBaseUrl($config['baseUrl']);
		}
	}

	/**
	 * 
	 */
	protected function initConfigs() {
		
		$ini = $this->readXmlCascaded(Hpls_Consts0::FILE_CONFIG_AUTH);
		Zend_Registry::set(Hpls_Consts0::REGK_AUTH, $ini);
	}

	protected function initHplsSettings() {
		parent::initHplsSettings();
		Hpls_Settings::i()->registerHandler('hmt', new Hpls_Settings_Handler_Component(Hpls_Mailtrace_Consts::COMP_NAME));
		$dbConfig = Hpls_Settings::i()->getConfig('hmt.mtlog.storage.db')->toArray();
		
		$dbConfig['keycol'] = 'configuration_key';
		
		Hpls_Settings::i()->registerHandler('ac.maintenance.global', new Hpls_Settings_Handler_Db($dbConfig, 'ac.maintenance.global'));
		Hpls_Settings::i()->registerHandler('hmt.daemon.global', new Hpls_Settings_Handler_Db($dbConfig, 'hmt.daemon.global'));
		
		$ini = $this->readXmlCascaded(Hpls_Consts0::FILE_CONFIG_AUTH);
		Hpls_Settings::i()->registerHandler('ac.auth', new Hpls_Settings_Handler_Config($ini, 'ac.auth'));
	}

	/**
	 * Zwingend nach dem Init der Submodule, da hier auf die Component-Konfiguration von HMT zugegriffen wird.
	 */
	protected function initConfigurationServices() {
		//		require_once 'Hpls/Configuration/Mailtrace/Config/Frontend.php';
	//		Hpls_Configuration_Factory::i()->registerService('Hpls_Configuration_Mailtrace_ConfigInterface', new Hpls_Configuration_Mailtrace_Config_Frontend());
	}

	/* (non-PHPdoc)
	 * @see Hpls_Application::getVersion()
	 */
	public function getVersion() {
		// TODO Auto-generated method stub
		if (is_null($this->_version)) {
			$v = $this->scanRecursive(APPLIANCE_PATH);
			if (isDebug('version_filedate') && $v !== false) {
				$t = date_default_timezone_get();
				date_default_timezone_set('Europe/Berlin');
				$this->_version = date('Y-m-d H:i:s', $v['ntime']);
				date_default_timezone_set($t);
			} else {
				$cfg = Hpls_Settings::i()->getConfig('application.product');
				if (!is_null($cfg) && isset($cfg->version)) {
					$this->_version = (string) $cfg->version;
				} else {
					$this->_version = '(2.0)';
				}
			}
		}
		return $this->_version;
	}

	/**
	 * @todo comment
	 * @param unknown_type $basedir
	 * @return boolean|multitype:number string <i>false</i>, wenn ein dirscan gescheitert ist 
	 */
	function scanRecursive($basedir) {
		$dirs = @scandir($basedir);
		if ($dirs === false) {
			return false;
		}
		$ret = array(
				'ntime' => 0,
				'otime' => 10000000000 );
		foreach ($dirs as $dir) {
			if ($dir == '.' || $dir == '..')
				continue;
			$filename = $basedir . DIRECTORY_SEPARATOR . $dir;
			if (stripos($filename, 'frontend/data') !== false) {
				continue;
			}
			if (is_dir($filename)) {
				$r = $this->scanRecursive($basedir . DIRECTORY_SEPARATOR . $dir);
				if ($r !== false) {
					if ($ret['ntime'] < $r['ntime']) {
						$ret['ntime'] = $r['ntime'];
						$ret['nfile'] = $r['nfile'];
					}
					if ($ret['otime'] > $r['otime']) {
						$ret['otime'] = $r['otime'];
						$ret['ofile'] = $r['ofile'];
					}
				}
			}
			$time = @filemtime($filename);
			if ($time !== false) {
				if ($ret['ntime'] < $time) {
					$ret['ntime'] = $time;
					$ret['nfile'] = $filename;
				}
				if ($ret['otime'] > $time) {
					$ret['otime'] = $time;
					$ret['ofile'] = $filename;
				}
			}
		
		}
		return $ret;
	}

	/* (non-PHPdoc)
	 * @see Hpls_Application0::initPaths()
	 */
	protected function initPaths() {
		// TODO Auto-generated method stub
	

	}
/* (non-PHPdoc)
	 * @see Hpls_Application0::initACL()
	 */
	protected function initACL() {
		// TODO Auto-generated method stub
		
	}


}