/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 * 
 * created : 2011-08-16 created by : Christoph Graupner
 * <c.graupner@heinlein-support.de>
 * 
 * last changed : $$LastChangedDate$$ last changed by: $$LastChangedBy$$
 * revision : $$LastChangedRevision$$ SVN head URL : $$HeadURL$$
 * 
 * author : Christoph Graupner <c.graupner@heinlein-support.de>
 */

var hpls = {
  _data : {
	  jobid : 0
  },
  parseDate : parseDate,
  toIdValue : toIdValue,
  dateFromInterval : dateFromInterval,
  sortObjByDateAsc : sortObjByDateAsc,
  sortObjByDateAsc : sortObjByDateAsc,
  toTimeStr : toTimeStr,
  padInteger : padInteger,
  toISOTimeString : toISOTimeString,
  htmlEntities : htmlEntities,
  sleep : sleep,
  execJob : function(job, hplsprogress) {
	  if (null == job) {
		  dbgc.error('No job given.');
		  throw new IllegalArgumentException('No job given.');
	  }
	  if (null == job.id) {
		  job.id = nextJobID();
	  }
	  return job.id;
  },
  nextJobID : function() {
	  return this._data.jobid++;
  }
};

/**
 * @param value
 *          string
 * @returns
 */
function toIdValue(value) {
    value = value.toLowerCase().replace(/(@|-|\.)/g, '__');
	return value;
}

function sortObjByDateAsc(a, b) {
	return ((a.time > b.time) ? 1 : ((a.time < b.time) ? -1 : 0));
}
function sortObjByDateDesc(a, b) {
	return ((a.time < b.time) ? 1 : ((a.time > b.time) ? -1 : 0));
}

function sleep(ms) {
	var zeit = (new Date()).getTime();
	var stoppZeit = zeit + ms;
	while ((new Date()).getTime() < stoppZeit) {
	}
}
function parseDate(isoDate) {
	var Ausdruck = /(\d{4,4})-(\d{1,2})-(\d{1,2})(T|\s)(\d{1,2}):(\d{1,2}):(\d{1,2})(\.(\d{1,4}))?(\+(\d{2,4}))/;
	var rets = Ausdruck.exec(isoDate);
	if (rets) {
		// 2011-02-09T33:55:00(.00)+0200
		// return new Date(Jahr, Monat, Tag, Stunden, Minuten, Sekunden);
		var t = new Date(rets[1], rets[2] - 1, rets[3], rets[5], rets[6], rets[7]);
		// sind millisekunden angegeben?
		if (rets[9]) {
			t.setMilliseconds(rets[9]);
		}
		return t;
	}
	Ausdruck = /0\.(\d{1,3})\d*\s+(\d+)/;
	rets = Ausdruck.exec(isoDate);
	if (rets) {
		var z = rets[2] * 1000;
		if (rets[1]) {
			z += rets[1] * 1; // *1 ==> cast to int
		}
		return new Date(z);
	}
	return null;
}
function dateFromInterval(iv) {
	if (iv.length < 14) {
		while (iv.length < 14) {
			iv += '0';
		}
	}
	return new Date(parseInt(iv.substring(0, 4)), parseInt(iv.substring(4, 6)) - 1, parseInt(iv
	    .substring(6, 8)), parseInt(iv.substring(8, 10)), iv.substring(10, 12), iv.substring(12, 14));
}

function dateToInterval(t) {
	return '' + t.getFullYear() + padInteger((t.getMonth() + 1), 2) + padInteger(t.getDate(), 2)
	    + padInteger(t.getHours(), 2) + padInteger(t.getMinutes(), 2) + padInteger(t.getSeconds(), 2);
}

function padInteger(value, digits) {
	var ret = new String(value);
	while (ret.length < digits) {
		ret = "0" + ret;
	}
	return ret;
}

/**
 * 
 * @param t
 * @returns {String}
 */
function toTimeStr(t) {
	// var t = new Date();
	return padInteger(t.getDate(), 2) + '.' + padInteger((t.getMonth() + 1), 2) + '.'
	    + t.getFullYear() + ' ' + padInteger(t.getHours(), 2) + ':' + padInteger(t.getMinutes(), 2)
	    + ':' + padInteger(t.getSeconds(), 2);
}

function toTimeDiffStr(ms) {
	if (ms == 0) {
		return '0s';
	}
	var ret = '';
	if (ms >= 604800000) {// woche 1000*60*60*24*7
		ret += Math.floor(ms / 604800000) + 'w';
		ms = ms % 604800000;
	}
	if (ms >= 86400000) { // tag 1000*60*60*24
		ret += ' ' + Math.floor(ms / 86400000) + 'd';
		ms = ms % 86400000;

	}
	if (ms >= 3600000) { // stunde 1000*60*60
		ret += ' ' + Math.floor(ms / 3600000) + 'h';
		ms = ms % 3600000;

	}
	if (ms >= 60000) { // minute 1000*60
		ret += ' ' + Math.floor(ms / 60000) + 'm';
		ms = ms % 60000;
	}
	if (ms >= 1000) { // sekunde
		ret += ' ' + Math.floor(ms / 1000) + 's';
		ms = ms % 1000;
	}
	ret += ' ' + ms + 'ms';
	return ret;
}

function toISOTimeString(t) {
	if (t == null || typeof t != 'object')
		return null;
	return t.getFullYear() + '-' + padInteger((t.getMonth() + 1), 2) + '-'
	    + padInteger(t.getDate(), 2) + 'T' + padInteger(t.getHours(), 2) + ':'
	    + padInteger(t.getMinutes(), 2) + ':' + padInteger(t.getSeconds(), 2) + '.'
	    + padInteger(t.getMilliseconds(), 3) + '+0200';
}

function htmlEntities(s) {
	return $("<div/>").text(s).html();
}

// ###############################################################################################
// ## Progress Panel

function HplsProgressPanel(config) {
	this._config = config;
	this._jobs = {};
}

HplsProgressPanel.prototype = {
  TEMPLATES : {
    job : '<div class="hpls_search_progress_msg_bg"><div id="hpls_search_progress_msg"/></div><div id="hpls_search_progress_bar"/>',
    panel : '<div id="hpls_search_progress_panel"></div>'
  },
  addWidget : function() {
	  if ($('#hpls_search_progress_panel').length == 0) {
	  	var TPL = $(this.TEMPLATES.panel).html(this.TEMPLATES.job);
		  $('body').append(TPL);
		  dbgc.info('ADDED PROGRESS PANEL');
	  }
  },
  addJob : function(job) {
	  if (job.id == null) {
		  dbgc.error('Job object has no job id', job);
		  throw new IllegalArgumentException();
	  }
	  this._jobs[job.id] = job;
	  $('#hpls_search_progress_panel').append(this.TEMPLATES.job);
  },
  set : function(value, msg, jobid) {
	  this.setValue(value);
	  this.setMessage(msg);
  },
  setValue : function(value, jobid) {
	  $('#hpls_search_progress_bar').progressbar({
		  'value' : value
	  });
	  dbgc.debug('PROGRESS: ',value);
  },

  getValue : function(jobid) {
	  return 0;
  },

  hide : function(mseconds) {
	  if (mseconds == null)
		  mseconds = 3000;
	  $('#hpls_search_progress_panel').show().fadeOut(mseconds);
	  dbgc.info('HIDING PROGRESS PANEL');
  },
  show : function(mseconds) {
	  if (mseconds == null)
		  mseconds = 3000;
	  $('#hpls_search_progress_panel').hide().fadeIn(mseconds);
	  dbgc.info('SHOWING PROGRESS PANEL');
  },

  setMessage : function(msg, jobid) {
	  $('#hpls_search_progress_msg').text(msg);
  }
};

function HplsMsgs(config) {
	this._globalID = 'hplsmsgs_global';
	if (config == null) {
		config = {};
	}
	this._config = config;
	this.TEMPLATES = {
		global : {
		  frame : '<div class="global_error" id="'+this._globalID+'"></div>',
		  category : '',
		  item : ''
		}
	};
};

HplsMsgs.prototype = {
  globalError : function(msg, code) {

  },
  fieldError : function(field, msg, code) {

  },
  addGlobalMsg : function(level, category, msg, code) {
	  var msg_area = $('#' + this._globalID);
	  dbgc.debug('INFOpanelID: ',this._globalID);
	  if (msg_area.length == 0) {
		  if (this._config.placeIn != null) {
			  $(this._config.placeIn).first().before(this.TEMPLATES.global.frame);
		  }
	  }
	  alert(msg);
  },
  addFieldMsg : function(field, level, category, msg, code) {

  }
};

var hplsmsgs = new HplsMsgs();
