<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 * 
 * created        : 28.10.2010
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2013-06-20 11:49:14 +0200 (Do, 20 Jun 2013) $
 * last changed by: $LastChangedBy: d.mohn $
 * revision       : $LastChangedRevision: 10186 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/HMT/GUI/Core/trunk/hpls.hmt.gui.libs/Hmt/Db/Query.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Zend/Db.php';
require_once 'Hmt/Db/Consts.php';
require_once 'Hpls/Mailtrace/Consts.php';
require_once 'Hpls/Var/Clause.php';

abstract class Hmt_Db_Query implements 
	Hmt_Db_Consts {
	/**
	 * @var Hpls_Var_Clause
	 */
	protected $_clause = null;
	/**
	 * @var Hpls_Var_Clause
	 */
	protected $_originalClause = null;
	/**
	 * @todo comment
	 * @var Hmt_Db_Query_Constraints
	 */
	protected $_resultOptions = array();
	protected $_preparedValues = null;
	protected $_fieldValueCount = 0;
	protected $_tables = array();
	/**
	 * Enter description here ...
	 * @var Zend_Db_Statement
	 */
	protected $_currentStatement = null;
	/**
	 * Enter description here ...
	 * @var Zend_Db_Adapter_Abstract
	 */
	protected $_db = null;
	
	protected $_resultsFetched = false;
	
	protected $_currentResults = null;

	/**
	 * Enter description here ...
	 * @param null|int|Hpls_Var_Clause $searchArgs
	 * @param array $constraints
	 */
	public function __construct($searchArgs = null, Hmt_Db_Query_Constraints $constraints = null) {
		
		$clause = null;
		if (empty($searchArgs)) {
			$this->_originalClause = new Hpls_Var_Clause();
		} elseif (is_array($searchArgs)) {
			$this->_originalClause = new Hpls_Var_Clause(Hpls_Var_Clause::CL_IN, array(
					$this->getTableIDField() => $searchArgs ));
		} elseif (is_int($searchArgs)) {
			$this->_originalClause = new Hpls_Var_Clause(Hpls_Var_Clause::CL_AND, array(
					$this->getTableIDField() => $searchArgs ));
		} else if ($searchArgs instanceof Hpls_Var_Clause) {
			$this->_originalClause = $searchArgs;
		} else {
			throw new InvalidArgumentException('$searchArgs is of wrong type.');
		}
		$this->_resultOptions = is_null($constraints) ? new Hmt_Db_Query_Constraints() : $constraints;
		//Clause um referenz-zeitpunkt ergänzen und normalisieren
		$cloneClause = clone $this->_originalClause;
		$startTime = Hmt_Log::i()->toDbTimestamp($this->_resultOptions->startSearchTime);
		if ($this->_resultOptions->startSearchTime != null) {
			//@todo searchDirection mit in Operator einfliessen lassen.
			$cloneClause->addValue(Hmt_Db_Consts::F_C_CREATE_TIME, $startTime, Hpls_Var_Clause::OP_LE);
		}
		$this->_clause = $this->rewriteClause($cloneClause);
		
		isDebug('hmt_search_values', 'hmt_model_sql') && $this->log()->debug(__METHOD__ . "  VAR CLAUSE: " . $this->_clause);
		isDebug('hmt_search_values', 'hmt_model_sql') && $this->log()->debug(__METHOD__ . "  CONSTRAINTS: " . Hpls_Debug::print_r($this->_resultOptions));
	}

	/**
	 * Für die Serialisierung, sollte die DB-Verbindung gelöscht werden und nicht mit serialisiert werden. Das wird hier erledigt.
	 * Nach dem Deserialisieren muss eine neue Verbindung zur Datenbank gesetzt werden
	 * @return multitype:string 
	 */
	public function __sleep() {
		$this->_db = null;
		$this->_currentStatement = null;
		return array(
				'_clause',
				'_resultOptions',
				'_preparedValues',
				'_tables' );
	}

	/**
	 * Setzt das Limit für die Ergebnismenge
	 * @param int $limit
	 * @param int $offset
	 */
	public function setLimit($limit, $offset = 0) {
		$this->_resultOptions->limit = $limit;
		$this->_resultOptions->offset = $offset;
	}

	public function setDate($date) {
		$this->_resultOptions['date'] = $date;
	}

	/**
	 * Bindet die Query an eine definierte Datenbankverbindung.
	 * @param Zend_Db_Adapter_Abstract $db
	 */
	public function setDb(Zend_Db_Adapter_Abstract $db) {
		$this->_db = $db;
	}

	/**
	 * Enter description here ...
	 * @return Zend_Db_Adapter_Abstract
	 */
	public function getDb() {
		return $this->_db;
	}

	/**
	 * Führt die Anfrage aus, speichert aber nur das Anfragehandle. Daten werden erst mit getResults() tatsächlich gelesen.
	 */
	public function execute() {
		$sql = $this->getSelect();
		$this->applyLimit($sql);
		$this->applyOrder($sql);
		
		$bind = $this->getPreparedValues();
		
		isDebug('hmt_model_sql') && $this->log()->info(Hpls_Debug::print_r($bind, get_class($this) . '::' . __FUNCTION__ . "## SQL: " . $sql->assemble()));
		
		$this->_currentStatement = $this->getDb()->query($sql, $bind);
	}

	public function getResults() {
		if (is_null($this->_currentResults)) {
			$this->_currentResults = array();
			while ($row = $this->_currentStatement->fetch()) {
				$res = $this->transformResult($row);
				$this->_currentResults[$res->getUniqueKey()] = $res;
			}
		}
		return $this->_currentResults;
	}

	public function getCount() {
	
	}

	/**
	 * @return Zend_Db_Select $select
	 */
	abstract public function getSelect();

	protected function applyLimit(Zend_Db_Select $select) {
		if (!is_null($this->_resultOptions->limit) && $this->_resultOptions->limit > 0) {
			$select->limit($this->_resultOptions->limit, $this->_resultOptions->offset);
		}
	}

	/**
	 * @todo comment
	 * @param Zend_Db_Select $select
	 */
	protected function applyOrder(Zend_Db_Select $select) {
		//@todo implement
		$orders = array();
		$cols = $select->getPart(Zend_Db_Select::COLUMNS);
		$colNew = array();
		//		foreach ($cols as $colDesc) {
		//			if (empty($colDesc[2])) {
		//				$colNew[] = (empty($colDesc[0]) ? '' : $colDesc[0] . '.') . $colDesc[1];
		//			} else {
		//				$colNew[$colDesc[2]] = (empty($colDesc[0]) ? '' : $colDesc[0] . '.') . $colDesc[1];
		//			}
		//		}
		foreach ($this->_resultOptions->order_by as $order => $desc) {
			$name = explode('.', $this->sqlFieldName($order));
			$orders[] = $this->sqlFieldName($order) . ($desc ? ' DESC' : '');
			if ($select->getPart(Zend_Db_Select::DISTINCT)) {
				/* $cols:
				 * Array
(
    [0] => Array
        (
            [0] => c
            [1] => dbid
            [2] => 
        )

    [1] => Array
        (
            [0] => c
            [1] => c_utime
            [2] => 
        )

)

				 */
				foreach ($cols as $colDesc) {
					$addName = '';
					if (count($name) == 1) {
						if (strcasecmp($name[0], $colDesc[1]) == 0) {
							continue;
						}
						$addName = $name[0];
					} else {
						if ((strcasecmp($name[1], $colDesc[1]) == 0) && (strcasecmp($name[0], $colDesc[0]) == 0)) {
							continue;
						}
						$addName = (empty($name[0]) ? '' : $name[0] . '.') . $name[1];
					}
					if (!in_array($addName, $colNew)) {
						$colNew[] = $addName;
					}
					$this->log()->debug(__METHOD__ . " Added: " . Hpls_Debug::print_r($colNew));
				}
			}
		}
		if (count($orders) > 0) {
			//um folgende SQL-Exception zu vermeiden:
			//Invalid column reference: 7 FEHLER:  bei SELECT
			// DISTINCT müssen ORDER-BY-Ausdrücke in der Select-Liste erscheinen
			$this->log()->debug(Hpls_Debug::print_r($colNew, __METHOD__));
			$select->columns($colNew);
			
			return $select->order($orders);
		}
		
		return $select;
	}

	public function getSQL() {
		return $this->getSelect()->assemble();
	}

	/**
	 * @todo comment
	 * @return Hpls_Var_Clause
	 */
	public function getOriginalClause() {
		return $this->_originalClause;
	}

	/**
	 * @todo comment
	 * 
	 * @return Hpls_Var_Clause
	 */
	public function getClause() {
		return $this->_clause;
	}

	protected function hasClause() {
		return !is_null($this->_clause);
	}

	protected function getNewSelect() {
		$this->_preparedValues = array();
		return $this->_db->select();
	}

	public function getPreparedValues() {
		return $this->_preparedValues;
	}

	protected function setPreparedValues(array $values) {
		$this->_preparedValues = $values;
	}

	protected function addPreparedValues($key, $value = null) {
		if (is_array($key)) {
			foreach ($key as $k => $v) {
				$this->_preparedValues[$k] = $v;
			}
		} else {
			$this->_preparedValues[$key] = $value;
		}
	}

	/**
	 * Tranformiert eine Zeile von Zend_Db_Statement::fetch() in das eigentliche Rückgabeformat, welches von der Query zurückgeliefert werden soll.
	 * Im allgemeinen ein Objekt des Datenmodells aus Hpls_Log_*.
	 * @param array|mixed $row
	 * @return object|array
	 */
	abstract protected function transformResult($row);

	protected function correctDateFrom($date) {
		return Hmt_Log::i()->toDbTimestamp($date);
	}

	protected function quote($text) {
		return $this->getDb()->quote($text);
	}

	protected function quoteIdentifier($names) {
		return $this->getDb()->quoteIdentifier($names);
	}

	protected function rewriteClause(Hpls_Var_Clause $clause) {
		$replacing = array();
		$remove = array();
		
		
		
		foreach ($clause->get() as $k => $v) {
			
			if ($v instanceof Hpls_Var_Clause) {
				$this->rewriteClause($v);
			} elseif (array_key_exists($k, Hmt_Log::$REPLACEABLE_FIELDS)) {
				$replacing[$k] = $v;
			}
			foreach ($v as $values) {
				if(isset($values['value'])){
					if (!is_array($values['value'])) {
						if (preg_match('/^\\*+$/', $values['value']) > 0) {
							//nur '*' als value -> alle datensätze -> muss nicht in where vorhanden sein
							$remove[$k] = $values['value'];
						}
					}
				}
			}
		} //foreach
		

		foreach ($replacing as $k => $values) {
			isDebug('hmt_search_values', 'hmt_model_sql_compose') && $this->log()->debug(__METHOD__ . '##HAVE##' . Hpls_Debug::print_r($values, $k));
			$clause->remove($k);
			$newValues = array();
			//hier gibt es nur 
			foreach ($values as $v) {
				$split = explode('@', $v['value']);
				switch (count($split)) {
					case 1 : //'@'-zeichen ist nicht im suchstring -> überall suchen
						//@todo problem: *jksf*jkldk* <- mittlerer '*' könnte '@' sein
						// gleichen suchstring in allen tabellenfeldern suchen
						$c2 = new Hpls_Var_Clause(Hpls_Var_Clause::CL_OR);
						foreach (Hmt_Log::$REPLACEABLE_FIELDS[$k] as $newField) {
							$c2->addValue($newField, $split[0], $v['op']);
						}
						$clause->addValue($c2);
						break;
					case 2 : //'@'-Zeichen ist drin
						$i = 0;
						$c2 = new Hpls_Var_Clause(Hpls_Var_Clause::CL_AND);
						foreach (Hmt_Log::$REPLACEABLE_FIELDS[$k] as $newField) {
							if (preg_match('/^\s*\*+\s*$/', $split[$i]) == 0) {
								//feld nur aufnehmen, wenn es kein catch-all ('*') ist
								$c2->addValue($newField, $split[$i], $v['op']);
							}
							$i++;
						}
						$clause->addValue($c2);
						break;
					default :
						//error: email mit mehr als 1x '@'!
						require_once 'Hmt/Db/Query/Exception.php';
						throw new Hmt_Db_Query_Exception("email mit mehr als 1x '@'!", Hpls_Mailtrace_Consts::ERR_QUERY_EMAIL);
				}
			} //foreach $values
		} //foreach $replacing
		

		return $clause;
	}

	protected function generateWhere(Zend_Db_Select $select, Hpls_Var_Clause $clause = NULL) {
		if (is_null($clause)) {
			$clause = $this->getClause();
		}
		if ($clause->isEmpty())
			return null;
		
		isDebug('hmt_search_values', 'hmt_model_sql_compose') && $this->log()->debug(Hpls_Debug::print_r($clause, __CLASS__ . '::$_clause'));
		switch ($clause->getLogic()) {
			case Hpls_Var_Clause::CL_IN :
				return $this->generateWhereIn($select, $clause);
			case Hpls_Var_Clause::CL_AND :
			case Hpls_Var_Clause::CL_OR :
				return $this->generateWhereAndOr($select, $clause);
			case Hpls_Var_Clause::CL_NOT :
			default :
				break;
		}
		return null;
	}

	/**
	 * Enter description here ...
	 * 
	 * @todo hier ist ein fehler, wenn mehr als ein IN am start ist!
	 * 
	 * @param Zend_Db_Select $select
	 * @param Hpls_Var_Clause $clause
	 * @return string
	 */
	protected function generateWhereIn(Zend_Db_Select $select, Hpls_Var_Clause $clause) {
		$counter = 0;
		$values = array();
		$fieldName = null;
		foreach ($clause->get() as $field => $inValues1) {
			
			$fieldName = $field;
			if (is_array($inValues1)) {
				foreach ($inValues1 as $inValues) {
					isDebug('hmt_search_values', 'hmt_model_sql_compose') && $this->log()->debug(Hpls_Debug::print_r($inValues, __METHOD__));
					if (!is_array($inValues['value'])) {
						$inValues['value'] = explode(',', $inValues['value']);
					}
					foreach ($inValues['value'] as $value) {
						$values[":n$fieldName$counter"] = $value;
						$counter++;
					}
					$this->addPreparedValues($values);
				}
			}
		}
		isDebug('hmt_model_sql', 'hmt_model_sql_compose') && $this->log()->debug(__METHOD__ . "##" . Hpls_Debug::print_r($values, $fieldName));
		return $this->quoteIdentifier($this->sqlFieldName($fieldName)) . ' IN (' . implode(',', array_keys($values)) . ')';
	}

	protected function generateWhereAndOr(Zend_Db_Select $select, Hpls_Var_Clause $clause) {
		isDebug('hmt_model_sql', 'hmt_model_sql_compose') && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($clause, " clause to gen AndOr"));
		$ret = array();
		foreach ($clause->get() as $key => $values) {
			
			if ($values instanceof Hpls_Var_Clause) {
				$ret[] = '(' . $this->generateWhere($select, $values) . ')';
			} else {
				foreach ($values as $value) {
					$ret[] = $this->generateSqlExpression($key, $value['value'], $value['op']);
				}
			}
		}
		isDebug('hmt_model_sql', 'hmt_model_sql_compose') && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($ret, " return"));
		return implode($this->clauseLogicToSql($clause->getLogic()), $ret);
	}

	protected function generateSqlExpression($srchName, $value, $op) {
		$fieldName = $this->sqlFieldName($srchName);
		//		if ($value === null) {
		//			return array(
		//					'op' => 'IS NULL',
		//					'value' => null );
		//		}
		if (in_array($srchName, Hmt_Log::$DATE_FIELDS)) {
			$value = $this->correctDateFrom($value);
			if ($op == Hpls_Var_Clause::OP_BESTGUESS || $op == Hpls_Var_Clause::OP_LIKE) {
				$op = Hpls_Var_Clause::OP_EQ;
			}
			$op = $this->varOpToSqlOp($op);
		} else {
			isDebug('hmt_search_values', 'hmt_model_sql_compose') && $this->log()->debug("regex-1: " . $value);
			//		if (get_magic_quotes_gpc() !== 0) {
			//			$text = stripslashes($text);
			//		}
			$orig = $value;
			$value = preg_replace('/\\*+/', '*', $value);
			isDebug('hmt_search_values', 'hmt_model_sql_compose') && $this->log()->debug("regex00: " . $value);
			$value = str_replace('%', '\\%', $value);
			isDebug('hmt_search_values', 'hmt_model_sql_compose') && $this->log()->debug("regex0: " . $value);
			$value = str_replace('_', '\\_', $value);
			isDebug('hmt_search_values', 'hmt_model_sql_compose') && $this->log()->debug("regex1: " . $value);
			$value = preg_replace(array(
					'/^\\*/',
					'/([^\\\\])\\*/',
					'/^\\?/',
					'/([^\\\\])\\?/' ), array(
					'%',
					'$1%',
					'_',
					'$1_' ), $value);
			isDebug('hmt_search_values', 'hmt_model_sql_compose') && $this->log()->debug("regex2: " . $value);
			$ret = array();
			if ($op == Hpls_Var_Clause::OP_BESTGUESS || $op == Hpls_Var_Clause::OP_EQ) {
				
				if (strcmp($orig, $value) === 0) {
					//no wildcard expansion
					$op = '=';
				} else {
					$op = 'like';
				}
			} else {
				$op = $this->varOpToSqlOp($op);
			}
			//TODO replacement
			isDebug('hmt_search_values', 'hmt_model_sql_compose') && $this->log()->debug("wildcard expanded : " . print_r($value, true));
		}
		$prepValueName = $this->getPreparedValueKey($fieldName);
		$this->addPreparedValues($prepValueName, $value);
		return $this->quoteIdentifier($fieldName) . " $op " . $prepValueName;
	}

	protected function varOpToSqlOp($op) {
		switch ($op) {
			case Hpls_Var_Clause::OP_EQ :
				return '=';
			case Hpls_Var_Clause::OP_GE :
				return '>=';
			case Hpls_Var_Clause::OP_GT :
				return '>';
			case Hpls_Var_Clause::OP_LE :
				return '<=';
			case Hpls_Var_Clause::OP_LIKE :
				return 'like';
			case Hpls_Var_Clause::OP_LT :
				return '<';
		
		}
		return null;
	}

	protected function clauseLogicToSql($logicKeyword) {
		switch ($logicKeyword) {
			case Hpls_Var_Clause::CL_AND :
				return ' AND ';
			case Hpls_Var_Clause::CL_OR :
				return ' OR ';
			case Hpls_Var_Clause::CL_NOT :
				return ' NOT ';
			case Hpls_Var_Clause::CL_IN :
				return ' IN ';
		}
		return ' NOTSUPPORTED_LOGIC ';
	}

	protected function getPreparedValueKey($fieldName) {
		$this->_fieldValueCount++;
		return ':' . str_replace('.', '_', $fieldName) . '_' . $this->_fieldValueCount;
	}

	/**
	 * Enter description here ...
	 * @param string $srchName Der Name des Suchfeldes aus der Hpls_Var_Clause
	 * @return string In SQL-Feldnamen übersetzer und gequoteter $srchName.
	 */
	protected function sqlFieldName($srchName) {
		$tablePrefix = '';
		$fieldName = '';
		if (strncasecmp($srchName, 'c_', 2) == 0) {
			$tablePrefix = 'c.';
			if (!array_key_exists($srchName, Hmt_Log::$CONNECT_FIELD_MAP)) {
				require_once 'Hmt/Db/Query/Exception.php';
				throw new Hmt_Db_Query_Exception("Search field '$srchName' has no map to DB fields.", 1);
			}
			return $tablePrefix . Hmt_Log::$CONNECT_FIELD_MAP[$srchName];
		} elseif (strncasecmp($srchName, 'd_', 2) == 0) {
			$tablePrefix = 'd.';
			if (!array_key_exists($srchName, Hmt_Log::$DELIVER_FIELD_MAP)) {
				require_once 'Hmt/Db/Query/Exception.php';
				throw new Hmt_Db_Query_Exception("Search field '$srchName' has no map to DB fields.", 1);
			}
			return $tablePrefix . Hmt_Log::$DELIVER_FIELD_MAP[$srchName];
		} elseif (strncasecmp($srchName, 's_', 2) == 0) {
			$tablePrefix = 's.';
			if (!array_key_exists($srchName, Hmt_Log::$SERVER_FIELD_MAP)) {
				require_once 'Hmt/Db/Query/Exception.php';
				throw new Hmt_Db_Query_Exception("Search field '$srchName' has no map to DB fields.", 1);
			}
			return $tablePrefix . Hmt_Log::$SERVER_FIELD_MAP[$srchName];
		} else {
			require_once 'Hmt/Db/Query/Exception.php';
			throw new Hmt_Db_Query_Exception("Do not know how to map search field '$srchName' to DB field.", 1);
		
		}
		
		return null;
	}

	protected function quoteValue($value, $fieldName = null) {
		return $this->quote($value);
	}

	abstract protected function getTableIDField();

	protected function log() {
		return Hpls_Utils::log();
	}
}