<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 * 
 * created        : Oct 31, 2010
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28 Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/HMT/GUI/Core/trunk/hpls.hmt.gui.libs/Hmt/Db/Query/LogSearch.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hmt/Db/Query/PartitionedAbstract.php';
require_once 'Hmt/Db/Query/Connects.php';
require_once 'Hmt/Db/Query/Delivers.php';
require_once 'Hmt/Log/ConnectEntry.php';
require_once 'Hmt/Log/DeliverEntry.php';

/**
 * @todo comment
 * 
 * Ablauf:
 * <ul>
 * <li> Überprüfen, ob 
 * </ul>
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 9196 $
 * @since 1.0.9
 */
class Hmt_Db_Query_LogSearch extends Hmt_Db_Query_PartitionedAbstract {

	/**
	 * 
	 */
	public function execute() {
		$this->log()->debug(__METHOD__);
		//get matching dbids
		$sql = $this->getSelect()->distinct();
		$this->applyLimit($sql);
		$this->applyOrder($sql);
		$bind = $this->getPreparedValues();
		isDebug('hmt_search_values', 'hmt_model_sql') &&$this->log()->debug(__METHOD__.' ORIGINAL CLAUSE '.$this->_clause);
		isDebug('hmt_search_values', 'hmt_model_sql') &&$this->log()->debug(__METHOD__.' REWRITTEN CLAUSE '.$this->_clause);
		isDebug( 'hmt_model_sql') && $this->log()->info(Hpls_Debug::print_r($bind, get_class($this) . '::' . __FUNCTION__ . "## SQL: " . $sql->assemble()));
		
		$this->_currentStatement = $this->getDb()->query($sql, $bind);
		
		$dbids = array();
		$utime = array(
				'newest' => 0,
				'oldest' => time() * 1000 + 1 );
		while ($row = $this->_currentStatement->fetch(PDO::FETCH_NUM)) {
			$dbids[] = $row[0];
			isDebug('hmt_search_values', 'hmt_model_sql_compose') && $this->log()->debug(__METHOD__ . " date: " . $row[1]);
			if (!empty($row[1])) {
				$t = Hmt_Log::i()->toTimestamp($row[1]);
				isDebug('hmt_search_values', 'hmt_model_sql_compose') && $this->log()->debug(__METHOD__ . " transformed date: " . $t);
				if ($utime['newest'] < $t) {
					$utime['newest'] = $t;
				}
				if ($utime['oldest'] > $t) {
					$utime['oldest'] = $t;
				}
			}
		}
		
		//		$dbids = $this->_currentStatement->fetchAll();
		if (count($dbids) > 0) {
			
			//fetch connects
			$this->_currentResults = array();
			$this->_currentResults['c_ids'] = $dbids;
			$this->_currentResults['offset'] = count($dbids) + (empty($this->_resultOptions->offset) ? 0 : $this->_resultOptions->offset);
			$this->_currentResults['limit'] = (empty($this->_resultOptions->limit) ? 0 : $this->_resultOptions->limit);
			$this->_currentResults['newest'] = $utime['newest'];
			$this->_currentResults['oldest'] = $utime['oldest'];
			
			$constraint = new Hmt_Db_Query_Constraints();
			$qry = Hmt_Log::i()->query(new Hpls_Var_Clause(Hpls_Var_Clause::CL_IN, array(
					Hpls_Mailtrace_Consts::SRCH_C_ID => $dbids )), $constraint);
			$qry->setPartition($this->getPartition());
			$this->_currentResults['connect'] = $qry;
			
			//fetch delivers
			$constraint = new Hmt_Db_Query_Constraints();
			$qry = Hmt_Log::i()->query(new Hpls_Var_Clause(Hpls_Var_Clause::CL_IN, array(
					Hpls_Mailtrace_Consts::SRCH_D_DBID => $dbids )), $constraint);
			$qry->setPartition($this->getPartition());
			$this->_currentResults['deliver'] = $qry;
		} else {
			//keine Ergebnisse gefunden
			$this->_currentResults = 0;
		}
		unset($sql);
		unset($dbids);
		unset($this->_currentStatement);
	
	}

	protected function sqlJoined() {
		isDebug('hmt_model_sql_compose') && $this->log()->info(__METHOD__);
		$sql = $this->getNewSelect();
		
		//cids aus connectlog finden, die passen könnten
		//		$ret = array($this->executeConnectOnly(),$this->executeDeliverOnly());
		

		//cids aus deliverlog finden, die passen könnten
		//		$this->executeDeliverOnly();
		//intersect aus beiden bilden 
		

		$joinCond = sprintf("%s = %s", $this->quoteIdentifier("c." . self::F_C_ID), $this->quoteIdentifier("d." . self::F_D_REF_C_ID));
		$sql->from(array(
				'c' => self::TABLE_CONNECT . $this->getPartition() ), array(
				self::F_C_ID,
				self::F_C_CREATE_TIME ))->joinLeft(array(
				'd' => self::TABLE_DELIVER . $this->getPartition() ), $joinCond, array());
		if (!$this->getClause()->isEmpty()) {
			$sql->where($this->generateWhere($sql, $this->getClause()));
		}
		return $sql;
	}

	protected function sqlConnectOnly() {
		isDebug('hmt_model_sql_compose') && $this->log()->info(__METHOD__);
		//cids aus connectlog finden, die passen könnten
		//		$connectClause = $this->getConnectSearchPart();
		//		$this->log()->debug(__METHOD__ . Hpls_Debug::print_r($connectClause));
		//		
		$sql = $this->getNewSelect();
		$sql->from(array(
				'c' => self::TABLE_CONNECT . $this->getPartition() ), array(
				self::F_C_ID,
				self::F_C_CREATE_TIME ));
		if (!$this->getClause()->isEmpty()) {
			$sql->where($this->generateWhere($sql, $this->getClause()));
		}
		return $sql;
	}

	protected function sqlDeliverOnly() {
		//cids aus deliverlog finden, die passen könnten
		isDebug('hmt_model_sql_compose') && $this->log()->info(__METHOD__);
		$sql = $this->getNewSelect();
		$sql->from(array(
				'd' => self::TABLE_DELIVER  . $this->getPartition() ), array(
				self::F_D_REF_C_ID,
				self::F_D_CREATE_TIME ));
		if (!$this->getClause()->isEmpty()) {
			$sql->where($this->generateWhere($sql, $this->getClause()));
		}
		return $sql;
	}

//	/**
//	 * @todo comment
//	 * @param $dbSelect
//	 * @param $field
//	 * @param $value
//	 * @param $searchKey
//	 * @return array
//	 */
//	protected function genPreparedWhereClause(Zend_Db_Select &$dbSelect, $field, $value, $searchKey) {
//		$ret = array();
//		$sql = null;
//		$wildcard = true;
//		$op = '=';
//		//@todo value between e.g. time		
//		if (strncasecmp($field, 'd_', 2) == 0) {
//			$sql = 'd.%s %s %s';
//			//deliver table
//			if (strcasecmp($searchKey, MTConsts::SRCH_D_DATE_FROM) === 0) {
//				$op = '>=';
//				$value = $this->correctDateFrom($value);
//			} elseif (strcasecmp($searchKey, MTConsts::SRCH_D_DATE_TO) === 0) {
//				$op = '<=';
//				$value = $this->correctDateTo($value);
//			} elseif (strcasecmp($searchKey, MTConsts::SRCH_STATUS) === 0) {
//				//status === unknown
//				if (strcasecmp($value, MTConsts::SRCH_STATUS_UNKNOWN) === 0) {
//					//					$op = 'IS NULL';
//					//					$value = null;
//					$sql = 'd.%s %s'; //remove 3rd '%s', so no value placeholder is added
//					$op = 'IS NULL';
//					$value = null;
//				} else {
//					// do nothing -> use defaults from above 
//				//					$value = $this->wildCardExpansion($value);
//				}
//			} else {
//				$value = $this->wildCardExpansion($value);
//				$op = $value['op'];
//				$value = $value['value'];
//			}
//		} elseif (strncasecmp($field, 'c_', 2) == 0) {
//			$sql = 'c.%s %s %s';
//			//connect
//			if (strcasecmp($searchKey, MTConsts::SRCH_C_DATE_FROM) === 0) {
//				$op = '>=';
//				$value = $this->correctDateFrom($value);
//			} elseif (strcasecmp($searchKey, MTConsts::SRCH_C_DATE_TO) === 0) {
//				$op = '<=';
//				$value = $this->correctDateTo($value);
//			} else {
//				$value = $this->wildCardExpansion($value);
//				$op = $value['op'];
//				$value = $value['value'];
//			}
//		}
//		if (is_array($value)) { //			foreach ()
//		}
//		self::$_FieldCount++;
//		$varField = ":$field" . self::$_FieldCount; //for uniqueness if field is present more than 1 times  
//		$ret['sql'] = sprintf($sql, $this->quoteIdentifier($field), $op, $varField);
//		if (is_null($value)) {
//			$ret['value'] = array();
//		} else {
//			$ret['value'] = array(
//					$varField => $value );
//		}
//		return $ret;
//	}

	/* (non-PHPdoc)
	 * @see Hmt_Db_Query::getSelect()
	 */
	public function getSelect() {
		// TODO Auto-generated method stub
		if ($this->isConnectQuery($this->getClause())) {
			//only connect table
			return $this->sqlConnectOnly();
		} else { //if ($this->isJoinQuery($this->getClause())) {
			//both connect and deliver tables, also only deliver table, cause of getting F_C_CREATE_UTIME
			return $this->sqlJoined();
		}
	
		//		else {
	//			//only deliver table
	//			return $this->executeConnectOnly();
	//		}
	}

	public function getResults() {
		return $this->_currentResults;
	}

	/* (non-PHPdoc)
	 * @see Hmt_Db_Query::transformResult()
	 */
	protected function transformResult($row) {
		// TODO Auto-generated method stub
	

	}

	/* (non-PHPdoc)
	 * @see Hmt_Db_Query::getTableIDField()
	 */
	protected function getTableIDField() {
		// TODO Auto-generated method stub
	

	}

	protected function log() {
		return Hpls_Utils::log();
	}

	/* (non-PHPdoc)
	 * @see Hmt_Db_Query_PartitionedAbstract::getTablePrefix()
	 */
	protected function getTablePrefix() {
		// TODO Auto-generated method stub
	

	}

	protected function isJoinQuery(Hpls_Var_Clause $clause) {
		return $this->isConnectQuery($clause) && $this->isDeliverQuery($clause);
	}

	protected function isConnectQuery(Hpls_Var_Clause $clause) {
		$keys = $clause->getAllKeys();
		return count(array_intersect(array_keys(Hmt_Log::$DELIVER_FIELD_MAP), $keys)) == 0;
	}

	protected function isDeliverQuery(Hpls_Var_Clause $clause) {
		$keys = $clause->getAllKeys();
		return count(array_intersect(array_keys(Hmt_Log::$CONNECT_FIELD_MAP), $keys)) == 0;
	
	}

}