<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : Aug 14, 2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hmt/Db/Statistics.php';
require_once 'Hpls/Settings.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision$
 * @since 1.0.9
 */
class Hmt_Db_Statistics_Mysql extends Hmt_Db_Statistics {

	const SQL_DB_SIZE = "SHOW TABLE STATUS FROM %s;";	
	const SQL_STATUS_COUNTS = "SELECT SUM( IF( d_status = 'sent', 1, 0 ) ) AS sent, SUM( IF( d_status = 'rejected', 1, 0 ) ) AS rejected, SUM( IF( d_status = 'deferred', 1, 0 ) ) AS deferred, SUM( IF( d_status = 'bounced', 1, 0 ) ) AS bounced FROM %s";
	const SQL_STATUS_C_COUNTS = "SELECT COUNT(*) AS counted FROM %s";
	const SQL_SERVER_C_COUNTS = "SELECT COUNT(*) FROM %s WHERE c_server = %d;";
	const SQL_SERVER_D_COUNTS = "SELECT COUNT(*) FROM %s WHERE d_server = %d;";
	const SQL_USER_DOMAIN_COUNTS = "SELECT c_mailfrom_domain,COUNT(c_mailfrom_domain) AS counted FROM %s GROUP BY c_mailfrom_domain ORDER BY counted DESC LIMIT 10";
	const SQL_USER_COUNTS = "SELECT c_mailfrom,COUNT(c_mailfrom) AS counted FROM %s GROUP BY c_mailfrom ORDER BY counted DESC LIMIT 10";
	
	/* (non-PHPdoc)
	 * @see Hmt_Db_Statistics::getTupleCountsConnect()
	 */
	function getTupleCountsConnect() {
		// TODO performantere Variante entwickeln
		$db = $this->getDb();
		$ret = 0;
		foreach (Hmt_Log::i()->getConnectTables() as $ival) {
			$stm = $db->query('SELECT COUNT(*) FROM ' . $db->quoteIdentifier($ival));
			$ret += (float) $stm->fetchColumn();
		}
		return $ret;
	}

	/* (non-PHPdoc)
	 * @see Hmt_Db_Statistics::getTupleCountsDeliver()
	 */
	function getTupleCountsDeliver() {
		// TODO performantere Variante entwickeln
		$db = $this->getDb();
		$ret = 0;
		foreach (Hmt_Log::i()->getDeliverTables() as $ival) {
			$stm = $db->query('SELECT COUNT(*) FROM ' . $db->quoteIdentifier($ival));
			$ret += (float) $stm->fetchColumn();
		}
		return $ret;
	}

	/* (non-PHPdoc)
	 * @see Hmt_Db_Statistics::getDbSize()
	 */
	function getDbSize() {
		$size = 0;
		$dbCfg = Hpls_Settings::i()->getConfig('hmt.mtlog.storage.db', true, true);
		$dbConfig = $dbCfg->toArray();
		
		
		$ints = Hmt_Log::i()->getIntervals();
		try {
			$db = $this->getDb();
			$query = sprintf(self::SQL_DB_SIZE, $dbConfig['dbname']);
			$stm = $db->query($query);
			foreach ($stm->fetchAll() AS $set){
				$size += $set['Data_length'] + $set['Index_length'];
			}
			
		} catch (Exception $ex) {
			//ein fehler in der DB kommunikation
			Hpls_Utils::log()->err(__METHOD__ . ": $ex");
			return null;
		}
		return $size;
	}

	/* (non-PHPdoc)
	 * @see Hmt_Db_Statistics::getLastReorg()
	 */
	function getLastReorg() {
		// TODO Auto-generated method stub
		return '';
	}
	
        function getRejectTimelineStats(){
                $stats = array();
                $types = array();
                $db = $this->getDb();
                $query = "SELECT s_time, s_type, SUM(s_value) AS val FROM statistics WHERE s_range = 1 GROUP BY s_time, s_type;";
                $stm = $db->query($query);
                foreach ($stm->fetchAll() AS $set){
                        $set['s_time'] = strtotime($set['s_time']) * 1000;
			$stats['days'][$set['s_time']][$set['s_type']] = $set['val'];
                        $types[$set['s_type']] = 1;
                }
                $stats['days'][$set['s_time']] = array_reverse($stats['days'][$set['s_time']], TRUE);
                $query = "SELECT s_time, s_type, SUM(s_value) AS val FROM statistics WHERE s_range = 30 GROUP BY s_time, s_type;";
                $stm = $db->query($query);
                foreach ($stm->fetchAll() AS $set){
                        $set['s_time'] = strtotime($set['s_time']) * 1000;
			$stats['months'][$set['s_time']][$set['s_type']] = $set['val'];   
                        $types[$set['s_type']] = 1;
		}
                $query = "SELECT s_time, s_type, SUM(s_value) AS val FROM statistics WHERE s_range = 365 GROUP BY s_time, s_type;";
                $stm = $db->query($query);
                foreach ($stm->fetchAll() AS $set){
                        $set['s_time'] = strtotime($set['s_time']) * 1000;
			$stats['years'][$set['s_time']][$set['s_type']] = $set['val'];
                        $types[$set['s_type']] = 1;
		}
                $stats['types'] = array_keys($types);
                return $stats;
        }
        
        function getRejectDailyStats(){
                $stats = array();
                $total = 0;
                $db = $this->getDb();
                $date = date('Y-m-d');
                $query = "SELECT s_type, SUM(s_value) AS val FROM statistics WHERE s_range = 1 AND s_time = '$date' GROUP BY s_type;";
                $stm = $db->query($query);
                foreach ($stm->fetchAll() AS $set){
			$stats['today'][$set['s_type']] = $set['val'];
                        $total += $set['val'];
                }
                $stats['today'][9] = $total;
                $total = 0;
                $date = date('Y-m-' . '01');
                $query = "SELECT s_type, SUM(s_value) AS val FROM statistics WHERE s_range = 30 AND s_time = '$date' GROUP BY s_type;";
                $stm = $db->query($query);
                foreach ($stm->fetchAll() AS $set){
			$stats['month'][$set['s_type']] = $set['val'];
                         $total += $set['val'];
		}
                $stats['month'][9] = $total;
                $total = 0;
                $date = date('Y-' . '01-01');
                $query = "SELECT s_type, SUM(s_value) AS val FROM statistics WHERE s_range = 365 AND s_time = '$date' GROUP BY s_type;";
                $stm = $db->query($query);
                foreach ($stm->fetchAll() AS $set){
			$stats['year'][$set['s_type']] = $set['val'];
                         $total += $set['val'];
		}
                $stats['year'][9] = $total;
                return $stats;
        }
        
	function getUserDailyStats($day){
		$db = $this->getDb();
		$query = sprintf(self::SQL_USER_COUNTS, $day) . ";";
		$stm = $db->query($query);
		$dailys = array();
		foreach ($stm->fetchAll() AS $set){
			$dailys[$set['c_mailfrom']] = $set['counted'];
		}
		return $dailys;
	}
	
	function getUserDomainDailyStats($day){
		$db = $this->getDb();
		$query = sprintf(self::SQL_USER_DOMAIN_COUNTS, $day) . ";";
		$stm = $db->query($query);
		$dailys = array();
		foreach ($stm->fetchAll() AS $set){
			$dailys[$set['c_mailfrom_domain']] = $set['counted'];
		}
		return $dailys;
	}
	
	function getConnectDailyStats($day){
		$db = $this->getDb();
		$query = sprintf(self::SQL_STATUS_C_COUNTS, $day) . ";";
		$stm = $db->query($query);
		foreach ($stm->fetchAll() AS $set){
			$dailys = $set['counted'];
		}
		return $dailys;
	}
	
	function getConnectTimedStats($day){
		$db = $this->getDb();
		$times = array(
				'0' => array('00:00:00','00:59:59'),
				'1' => array('01:00:00','01:59:59'),
				'2' => array('02:00:00','02:59:59'),
				'3' => array('03:00:00','03:59:59'),
				'4' => array('04:00:00','04:59:59'),
				'5' => array('05:00:00','05:59:59'),
				'6' => array('06:00:00','06:59:59'),
				'7' => array('07:00:00','07:59:59'),
				'8' => array('08:00:00','08:59:59'),
				'9' => array('09:00:00','09:59:59'),
				'10' => array('10:00:00','10:59:59'),
				'11' => array('11:00:00','11:59:59'),
				'12' => array('12:00:00','12:59:59'),
				'13' => array('13:00:00','13:59:59'),
				'14' => array('14:00:00','14:59:59'),
				'15' => array('15:00:00','15:59:59'),
				'16' => array('16:00:00','16:59:59'),
				'17' => array('17:00:00','17:59:59'),
				'18' => array('18:00:00','18:59:59'),
				'19' => array('19:00:00','19:59:59'),
				'20' => array('20:00:00','20:59:59'),
				'21' => array('21:00:00','21:59:59'),
				'22' => array('22:00:00','22:59:59'),
				'23' => array('23:00:00','23:59:59')
		);
		$hits = array();
		
		foreach ($times as $timekey => $timesection) {
			$query = sprintf(self::SQL_STATUS_C_COUNTS, $day) . " WHERE c_time > '" . date('Y-m-d') . " " . $timesection[0] . "' AND c_time < '" . date('Y-m-d') . " " . $timesection[1] . "';";
			$stm = $db->query($query);
			foreach ($stm->fetchAll() AS $set){
				$hits[$timekey] = $set['counted'];
			}
		}
		return $hits;
	}
	
	function getConnectServerStats($serverid){
		$db = $this->getDb();
		$ret = 0;
		foreach (Hmt_Log::i()->getConnectTables() as $ival) {
			$query = sprintf(self::SQL_SERVER_C_COUNTS, $db->quoteIdentifier($ival), $serverid);
			$stm = $db->query($query);
			$ret += (int)$stm->fetchColumn();
		}
		return $ret;
	}
	
	function getDeliveryDailyStats($day) {
		$db = $this->getDb();
		$query = sprintf(self::SQL_STATUS_COUNTS, $day) . ";";
		$stm = $db->query($query);
		foreach ($stm->fetchAll() AS $set){
			$dailys = array(
						'total' => $set['sent'] + $set['rejected'] + $set['deferred'] + $set['bounced'],
						'sent' => $set['sent'], 
						'rejected' => $set['rejected'],
						'deferred' => $set['deferred'],
						'bounced' => $set['bounced'],
					);
		}
		return $dailys;
	}
	
	function getDeliveryServerStats($serverid){
		$db = $this->getDb();
		$ret = 0;
		foreach (Hmt_Log::i()->getDeliverTables() as $ival) {
			$query = sprintf(self::SQL_SERVER_D_COUNTS, $db->quoteIdentifier($ival), $serverid);
			$stm = $db->query($query);
			$ret += (int)$stm->fetchColumn();
		}
		return $ret;
	}

	function getDeliveryTimedStats($day){
		$db = $this->getDb();
		$times = array(
					'0' => array('00:00:00','00:59:59'),
					'1' => array('01:00:00','01:59:59'),
					'2' => array('02:00:00','02:59:59'),
					'3' => array('03:00:00','03:59:59'),
					'4' => array('04:00:00','04:59:59'),
					'5' => array('05:00:00','05:59:59'),
					'6' => array('06:00:00','06:59:59'),
					'7' => array('07:00:00','07:59:59'),
					'8' => array('08:00:00','08:59:59'),
					'9' => array('09:00:00','09:59:59'),
					'10' => array('10:00:00','10:59:59'),
					'11' => array('11:00:00','11:59:59'),
					'12' => array('12:00:00','12:59:59'),
					'13' => array('13:00:00','13:59:59'),
					'14' => array('14:00:00','14:59:59'),
					'15' => array('15:00:00','15:59:59'),
					'16' => array('16:00:00','16:59:59'),
					'17' => array('17:00:00','17:59:59'),
					'18' => array('18:00:00','18:59:59'),
					'19' => array('19:00:00','19:59:59'),
					'20' => array('20:00:00','20:59:59'),
					'21' => array('21:00:00','21:59:59'),
					'22' => array('22:00:00','22:59:59'),
					'23' => array('23:00:00','23:59:59')
				);
		$hits = array();
		
		foreach ($times as $timekey => $timesection) {
			$query = sprintf(self::SQL_STATUS_COUNTS, $day) . " WHERE d_time > '" . date('Y-m-d') . " " . $timesection[0] . "' AND d_time < '" . date('Y-m-d') . " " . $timesection[1] . "';";
			$stm = $db->query($query);
			foreach ($stm->fetchAll() AS $set){
				if($set['sent'] <= 0 && $set['rejected'] <= 0 && $set['deferred'] <= 0 && $set['bounced'] <= 0){
					
				} else {
					$hits[$timekey] = array(
						'total' => $set['sent'] + $set['rejected'] + $set['deferred'] + $set['bounced'],
						'sent' => $set['sent'],
						'rejected' => $set['rejected'],
						'deferred' => $set['deferred'],
						'bounced' => $set['bounced'],
					);
				}
			}
		}
		return $hits;
	}
}