<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : Aug 14, 2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hmt/Db/Statistics.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision$
 * @since 1.0.9
 */
class Hmt_Db_Statistics_Pgsql extends Hmt_Db_Statistics {
	const SQL_TUPLES_COUNT_C = "SELECT relname, reltuples FROM pg_class where (relname ilike 'connectlog_%') ORDER BY relname;";
	const SQL_TUPLES_COUNT_D = "SELECT relname, reltuples FROM pg_class where (relname ilike 'deliverlog_%') ORDER BY relname;";
	const SQL_VACUUM_TIME = "SELECT pg_stat_get_last_vacuum_time((SELECT oid FROM pg_class WHERE relname = '%s'));";
	const SQL_DB_SIZE = "SELECT pg_relation_size('%s');";
	const SQL_STATUS_COUNTS = "SELECT SUM(CASE WHEN d_status = 'sent' THEN 1 ELSE 0 END) AS sent, SUM(CASE WHEN d_status = 'rejected' THEN 1 ELSE 0 END) AS rejected, SUM(CASE WHEN d_status = 'deferred' THEN 1 ELSE 0 END) AS deferred, SUM(CASE WHEN d_status = 'bounced' THEN 1 ELSE 0 END) AS bounced FROM %s";
	const SQL_STATUS_C_COUNTS = "SELECT COUNT(*) AS counted FROM %s";
	const SQL_SERVER_C_COUNTS = "SELECT COUNT(*) FROM %s WHERE c_server = %d;";
	const SQL_SERVER_D_COUNTS = "SELECT COUNT(*) FROM %s WHERE d_server = %d;";
	const SQL_USER_DOMAIN_COUNTS = "SELECT c_mailfrom_domain,COUNT(c_mailfrom_domain) AS counted FROM %s GROUP BY c_mailfrom_domain ORDER BY counted DESC LIMIT 10";
	const SQL_USER_COUNTS = "SELECT c_mailfrom,COUNT(c_mailfrom) AS counted FROM %s GROUP BY c_mailfrom ORDER BY counted DESC LIMIT 10";
	
	
	/* (non-PHPdoc)
	 * @see Hmt_Db_Statistics::getTupleCounts()
	 */
	function getTupleCountsConnect() {
		// TODO Auto-generated method stub
		$db = $this->getDb();
		$stm = $db->query(self::SQL_TUPLES_COUNT_C);
		$ret = 0;
		foreach ($stm->fetchAll(PDO::FETCH_ASSOC) as $r) {
			$matches = array();
			if (preg_match('/^' . Hmt_Db_Consts::TABLE_CONNECT . '(\d+)$/', $r['relname'], $matches) > 0) {
				$ret += floatval($r['reltuples']);
			}
		}
		
		return $ret;
	}

	/* (non-PHPdoc)
	 * @see Hmt_Db_Statistics::getDbSize()
	 */
	function getDbSize() {
		// TODO Auto-generated method stub
		$ints = Hmt_Log::i()->getIntervals();
		$size = 0;
		try {
			foreach ($ints as $iv) {
				$db = $this->getDb();
				$stm = $db->query(sprintf(self::SQL_DB_SIZE, Hmt_Db_Consts::TABLE_CONNECT . $iv));
				$res = $stm->fetchColumn();
				if (empty($res))
					continue;
				$size += $res;
				$stm = $db->query(sprintf(self::SQL_DB_SIZE, Hmt_Db_Consts::TABLE_DELIVER . $iv));
				$res = $stm->fetchColumn();
				if (empty($res))
					continue;
				$size += $res;
			}
		} catch (Exception $ex) {
			//ein fehler in der DB kommunikation
			Hpls_Utils::log()->err(__METHOD__ . ": $ex");
			return null;
		}
		
		return $size;
	}

	/* (non-PHPdoc)
	 * @see Hmt_Db_Statistics::getTupleCountsDeliver()
	 */
	function getTupleCountsDeliver() {
		$db = $this->getDb();
		$stm = $db->query(self::SQL_TUPLES_COUNT_D);
		$ret = 0;
		foreach ($stm->fetchAll(PDO::FETCH_ASSOC) as $r) {
			$matches = array();
			if (preg_match('/^' . Hmt_Db_Consts::TABLE_DELIVER . '(\d+)$/', $r['relname'], $matches) > 0) {
				$ret += floatval($r['reltuples']);
			}
		}
		
		return $ret;
	}

	/* (non-PHPdoc)
	 * @see Hmt_Db_Statistics::getLastReorg()
	 */
	function getLastReorg() {
		// TODO Auto-generated method stub
		$ints = Hmt_Log::i()->getIntervals();
		$time = 0;
		try {
			foreach ($ints as $iv) {
				$db = $this->getDb();
				$stm = $db->query(sprintf(self::SQL_VACUUM_TIME, Hmt_Db_Consts::TABLE_CONNECT . $iv));
				$time = $stm->fetchColumn();
				if (!empty($time))
					break;
			}
		} catch (Exception $ex) {
			//ein fehler in der DB kommunikation
			Hpls_Utils::log()->err(__METHOD__ . ": $ex");
			return null;
		}
		
		return $time;
	}
	
	function getUserDailyStats($day){
		$db = $this->getDb();
		$query = sprintf(self::SQL_USER_COUNTS, $day) . ";";
		$stm = $db->query($query);
		$dailys = array();
		foreach ($stm->fetchAll() AS $set){
			$dailys[$set['c_mailfrom']] = $set['counted'];
		}
		return $dailys;
	}
	
	function getUserDomainDailyStats($day){
		$db = $this->getDb();
		$query = sprintf(self::SQL_USER_DOMAIN_COUNTS, $day) . ";";
		$stm = $db->query($query);
		$dailys = array();
		foreach ($stm->fetchAll() AS $set){
			$dailys[$set['c_mailfrom_domain']] = $set['counted'];
		}
		return $dailys;
	}
	
	function getConnectDailyStats($day){
		$db = $this->getDb();
		$query = sprintf(self::SQL_STATUS_C_COUNTS, $day) . ";";
		$stm = $db->query($query);
		foreach ($stm->fetchAll() AS $set){
			$dailys = $set['counted'];
		}
		return $dailys;
	}
	
	function getConnectTimedStats($day){
		$db = $this->getDb();
		$times = array(
				'0' => array('00:00:00','00:59:59'),
				'1' => array('01:00:00','01:59:59'),
				'2' => array('02:00:00','02:59:59'),
				'3' => array('03:00:00','03:59:59'),
				'4' => array('04:00:00','04:59:59'),
				'5' => array('05:00:00','05:59:59'),
				'6' => array('06:00:00','06:59:59'),
				'7' => array('07:00:00','07:59:59'),
				'8' => array('08:00:00','08:59:59'),
				'9' => array('09:00:00','09:59:59'),
				'10' => array('10:00:00','10:59:59'),
				'11' => array('11:00:00','11:59:59'),
				'12' => array('12:00:00','12:59:59'),
				'13' => array('13:00:00','13:59:59'),
				'14' => array('14:00:00','14:59:59'),
				'15' => array('15:00:00','15:59:59'),
				'16' => array('16:00:00','16:59:59'),
				'17' => array('17:00:00','17:59:59'),
				'18' => array('18:00:00','18:59:59'),
				'19' => array('19:00:00','19:59:59'),
				'20' => array('20:00:00','20:59:59'),
				'21' => array('21:00:00','21:59:59'),
				'22' => array('22:00:00','22:59:59'),
				'23' => array('23:00:00','23:59:59')
		);
		$hits = array();
		
		foreach ($times as $timekey => $timesection) {
			$query = sprintf(self::SQL_STATUS_C_COUNTS, $day) . " WHERE c_time > '" . date('Y-m-d') . " " . $timesection[0] . "' AND c_time < '" . date('Y-m-d') . " " . $timesection[1] . "';";
			$stm = $db->query($query);
			foreach ($stm->fetchAll() AS $set){
				$hits[$timekey] = $set['counted'];
			}
		}
		return $hits;
	}
	
	function getConnectServerStats($serverid){
		$db = $this->getDb();
		$ret = 0;
		foreach (Hmt_Log::i()->getConnectTables() as $ival) {
			$query = sprintf(self::SQL_SERVER_C_COUNTS, $db->quoteIdentifier($ival), $serverid);
			$stm = $db->query($query);
			$ret += (int)$stm->fetchColumn();
		}
		return $ret;
	}
	/* (non-PHPdoc)
	 * @see Hmt_Db_Statistics::getDeliveryDailyStats()
	 */
	function getDeliveryDailyStats($day) {
		$db = $this->getDb();
		$query = sprintf(self::SQL_STATUS_COUNTS, $day) . ";";
		$stm = $db->query($query);
		foreach ($stm->fetchAll() AS $set){
			$dailys = array(
						'total' => $set['sent'] + $set['rejected'] + $set['deferred'] + $set['bounced'],
						'sent' => $set['sent'], 
						'rejected' => $set['rejected'],
						'deferred' => $set['deferred'],
						'bounced' => $set['bounced'],
					);
		}
		return $dailys;
	}

	/* (non-PHPdoc)
	 * @see Hmt_Db_Statistics::getDeliveryTimedStats()
	 */
	function getDeliveryTimedStats($day) {
		$db = $this->getDb();
		$times = array(
					'0' => array('00:00:00','00:59:59'),
					'1' => array('01:00:00','01:59:59'),
					'2' => array('02:00:00','02:59:59'),
					'3' => array('03:00:00','03:59:59'),
					'4' => array('04:00:00','04:59:59'),
					'5' => array('05:00:00','05:59:59'),
					'6' => array('06:00:00','06:59:59'),
					'7' => array('07:00:00','07:59:59'),
					'8' => array('08:00:00','08:59:59'),
					'9' => array('09:00:00','09:59:59'),
					'10' => array('10:00:00','10:59:59'),
					'11' => array('11:00:00','11:59:59'),
					'12' => array('12:00:00','12:59:59'),
					'13' => array('13:00:00','13:59:59'),
					'14' => array('14:00:00','14:59:59'),
					'15' => array('15:00:00','15:59:59'),
					'16' => array('16:00:00','16:59:59'),
					'17' => array('17:00:00','17:59:59'),
					'18' => array('18:00:00','18:59:59'),
					'19' => array('19:00:00','19:59:59'),
					'20' => array('20:00:00','20:59:59'),
					'21' => array('21:00:00','21:59:59'),
					'22' => array('22:00:00','22:59:59'),
					'23' => array('23:00:00','23:59:59')
				);
		$hits = array();
		
		foreach ($times as $timekey => $timesection) {
			$query = sprintf(self::SQL_STATUS_COUNTS, $day) . " WHERE d_time > '" . date('Y-m-d') . " " . $timesection[0] . "' AND d_time < '" . date('Y-m-d') . " " . $timesection[1] . "';";
			$stm = $db->query($query);
			foreach ($stm->fetchAll() AS $set){
				if($set['sent'] <= 0 && $set['rejected'] <= 0 && $set['deferred'] <= 0 && $set['bounced'] <= 0){
					
				} else {
					$hits[$timekey] = array(
						'total' => $set['sent'] + $set['rejected'] + $set['deferred'] + $set['bounced'],
						'sent' => $set['sent'],
						'rejected' => $set['rejected'],
						'deferred' => $set['deferred'],
						'bounced' => $set['bounced'],
					);
				}
			}
		}
		return $hits;
	}

	/* (non-PHPdoc)
	 * @see Hmt_Db_Statistics::getDeliveryServerStats()
	 */
	function getDeliveryServerStats($serverid) {
		$db = $this->getDb();
		$ret = 0;
		foreach (Hmt_Log::i()->getDeliverTables() as $ival) {
			$query = sprintf(self::SQL_SERVER_D_COUNTS, $db->quoteIdentifier($ival), $serverid);
			$stm = $db->query($query);
			$ret += (int)$stm->fetchColumn();
		}
		return $ret;
	}
}