<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 01.02.2012
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/**
 */
require_once 'Hpls/Utils/Logusing.php';

/**
 *
 * @todo comment
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision$
 * @since 1.1.0
 */
abstract class Hmt_Log_ManageAbstract extends Hpls_Utils_Logusing {
	const PREFIX_TRIGGER_REGEX = '/^create\.trigger\./i';
	const PREFIX_DATA_BEFORE_REGEX = '/^data\.initial\.before\.trigger\./i';
	const PREFIX_DATA_AFTER_REGEX = '/^data\.initial\.after\.trigger\./i';
	const PREFIX_TABLE_REGEX = '/^create\.table\./i';
	const PREFIX_INDEX_REGEX = '/^create\.index\./i';
	const PREFIX_GRANT_REGEX = '/^grant\./i';

	static public function factory(Zend_Config $cfg) {
		switch (strtolower($cfg->type)) {
			case 'mysql' :
				require_once 'Hmt/Log/Mysql/Manage.php';
				return new Hmt_Log_Mysql_Manage($cfg);
			case 'pgsql' :
				require_once 'Hmt/Log/Pgsql/Manage.php';
				return new Hmt_Log_Pgsql_Manage($cfg);
			case 'pgsql' :
				require_once 'Hmt/Log/Sqlite/Manage.php';
				return new Hmt_Log_Sqlite_Manage($cfg);
		}
		return null;
	}
	protected $_config = null;
	protected $_db = null;

	protected function __construct($cfg) {
		$this->_config = $cfg;
	
	}

	/**
	 *
	 * @todo comment
	 */
	public function createStructure() {
		$this->createTables();
		$this->grantRights();
		$this->insertFirstDataBeforeTriggers();
		$this->createTriggers();
		$this->insertFirstDataAfterTriggers();
		$this->createIndexes();
	}

	protected function createTables() {
		// TODO Auto-generated method stub
		$this->processSqlFiles(self::PREFIX_TABLE_REGEX, __FUNCTION__, Hpls_Mailtrace_Err::ERR_MTLOG_CREATE, array(
				Hpls_Consts0::SQL_VAR_TABLEPREFIX => $this->_config->tableprefix ));
	}
	
	/*
	 * (non-PHPdoc) @see Hmt_Log_ManageAbstract::createTriggers()
	 */
	protected function createTriggers() {
		// TODO Auto-generated method stub
		$this->processSqlFiles(self::PREFIX_TRIGGER_REGEX, __FUNCTION__, Hpls_Mailtrace_Err::ERR_MTLOG_CREATE_TRIGGER, array(
				Hpls_Consts0::SQL_VAR_TABLEPREFIX => $this->_config->tableprefix ));
	}

	protected function grantRights() {
		$this->processSqlFiles(self::PREFIX_GRANT_REGEX, __FUNCTION__, Hpls_Mailtrace_Err::ERR_MTLOG_CREATE_GRANT, array(
				Hpls_Consts0::SQL_VAR_TABLEPREFIX => $this->_config->tableprefix,
				Hpls_Consts0::SQL_VAR_OWNER => (string) $this->_config->username ));
	}

	protected function insertFirstDataBeforeTriggers() {
		$this->processSqlFiles(self::PREFIX_DATA_BEFORE_REGEX, __FUNCTION__, Hpls_Mailtrace_Err::ERR_MTLOG_CREATE_INITIAL_DATA, array(
				Hpls_Consts0::SQL_VAR_TABLEPREFIX => $this->_config->tableprefix ));
	}

	protected function insertFirstDataAfterTriggers() {
		$this->processSqlFiles(self::PREFIX_DATA_AFTER_REGEX, __FUNCTION__, Hpls_Mailtrace_Err::ERR_MTLOG_CREATE_INITIAL_DATA, array(
				Hpls_Consts0::SQL_VAR_TABLEPREFIX => $this->_config->tableprefix ));
	}

	protected function createIndexes() {
		// TODO Auto-generated method stub
		$this->processSqlFiles(self::PREFIX_INDEX_REGEX, __FUNCTION__, Hpls_Mailtrace_Err::ERR_MTLOG_CREATE_INDEX, array(
				Hpls_Consts0::SQL_VAR_TABLEPREFIX => $this->_config->tableprefix ));
	}

	protected function processSqlFiles($regex, $stepName, $errCode, $replaces = null) {
		try {
			$files = $this->getSqlFiles($regex);
			foreach ($files as $file) {
				$text = file_get_contents($file);
				foreach (Hpls_Utils::extractSections($text, Hpls_Consts0::MARKER_PHPSQL_BEGIN, Hpls_Consts0::MARKER_PHPSQL_END) as $sql) {
					if (!empty($replaces)) {
						$sql = str_replace(array_keys($replaces), array_values($replaces), $sql);
					}
					isDebug('hmt_log_create') && $this->log()->debug(__METHOD__ . ' sql: ' . $sql);
					// $stm = $this->getDb()->closeConnection();
					$stm = $this->getDb()->prepare($sql);
					if (!$stm->execute()) {
						$this->log()->err(__METHOD__ . ' ERROR in SQL');
					}
				}
			}
		} catch (Exception $ex) {
			$this->log()->err($ex);
			require_once 'Hpls/Mailtrace/Exception.php';
			throw new Hpls_Mailtrace_Exception("Error in creating log-db while executing step $stepName.", $errCode, $ex);
		}
	}

	protected function getSqlFiles($regex) {
		$cfgDir = Hpls_Application0::i()->getConfigDir(Hpls_Mailtrace_Consts::COMP_NAME);
		isDebug('hmt_log_create') && $this->log()->debug(__METHOD__ . " START - cfgdir: $cfgDir - regex: $regex");
		
		$sqlFile1 = $cfgDir . DIRECTORY_SEPARATOR . 'SQL' . DIRECTORY_SEPARATOR . $this->_config->type . DIRECTORY_SEPARATOR . 'log';
		isDebug('hmt_log_create') && $this->log()->debug(Hpls_Debug::print_r($this->_config, __METHOD__ . ' CONFIG'));
		$files = scandir($sqlFile1);
		isDebug('hmt_log_create') && $this->log()->debug(Hpls_Debug::print_r($files, __METHOD__ . ' FOUND FILES'));
		$ret = array();
		foreach ($files as $file) {
			if (preg_match($regex, $file)) {
				$ret[] = $sqlFile1 . DIRECTORY_SEPARATOR . $file;
			}
		}
		sort($ret);
		isDebug('hmt_log_create') && $this->log()->debug(Hpls_Debug::print_r($ret, __METHOD__ . ' RESULT'));
		return $ret;
	}

	/**
	 *
	 * @todo comment
	 * @throws Hpls_Exception_Configuration
	 * @return Zend_Db_Adapter_Abstract
	 */
	public function getDb() {
		if (is_null($this->_db)) {
			$config = $this->_config;
			if (empty($config)) {
				require_once 'Hpls/Exception/Configuration.php';
				throw new Hpls_Exception_Configuration('No configuration for hmt.mtlog.storage.db in Settings', Hpls_Err::ERR_CONFIGURATION);
			}
			$dbConfig = $config->toArray();
			
			$this->_db = Hpls_Utils_Db::mapDriverType2ZendDb($dbConfig);
		}
		return $this->_db;
	}

}