<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 09.03.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2012-10-23 16:54:02 +0200 (Di, 23 Okt 2012) $
 * last changed by: $LastChangedBy: lazee $
 * revision       : $LastChangedRevision: 9792 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/HMT/GUI/Core/trunk/hpls.hmt.gui.libs/Hmt/Reason.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/**
 */
require_once 'Hpls/Utils/Db.php';
require_once 'Hpls/Utils/Logusing.php';
require_once 'Hmt/Reason/Cachedb.php';
require_once 'Hmt/Reason/Item.php';
require_once 'Hmt/Reason/RegEx.php';
require_once 'Hmt/Reason/Role.php';
require_once 'Hmt/Reason/Rule.php';
require_once 'Hmt/Reason/Text.php';

/**
 *
 * @todo comment
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 9792 $
 * @since 1.0.9
 */
class Hmt_Reason extends Hpls_Utils_Logusing {
	const TABLE_REGEX = Hmt_Reason_Db::TABLE_REGEX;
	const TABLE_RULES = Hmt_Reason_Db::TABLE_RULES;
	const TABLE_REASON_PREFIX = Hmt_Reason_Db::TABLE_REASON_PREFIX;
	const TABLE_ROLES = Hmt_Reason_Db::TABLE_ROLES;
	
	const ID_SEP = '-';
	
	const INACTIVESTATE_INACTIVE = Hmt_Reason_Db::INACTIVESTATE_INACTIVE;
	const INACTIVESTATE_DELETED = Hmt_Reason_Db::INACTIVESTATE_DELETED;
	const INACTIVESTATE_ACTIVE = Hmt_Reason_Db::INACTIVESTATE_ACTIVE;
	
	protected static $_instance = null;

	/**
	 *
	 * @return Hmt_Reason
	 */
	public static function i() {
		if (is_null(self::$_instance)) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}
	
	protected $_breakIfNotExact = false;
	/**
	 *
	 * @todo comment
	 * @var Hmt_Reason_Db
	 */
	private $_db = null;
	protected $_dbObjs = array();
	
	private $_cacheDB = null;
	
	protected $_regexs = null;
	protected $_explanations = null;
	protected $_roleMap = null;
	protected $_defaultLang = null;
	protected $_defaultRole = null;

	protected function __construct() {
		$this->setDefaultLanguage('de_de');
		$this->setDefaultRole(1);
	}

	public function setDefaultLanguage($lang) {
		$lang = strtolower($lang);
		//		if (!in_array($lang, $this->getAvaiableLanguages())) {
		//			require_once 'Hpls/Mailtrace/Exception/Reason.php';
		//			throw new Hpls_Mailtrace_Exception_Reason("Kann Default-Sprache nicht setzen, da Sprache '$lang' nicht verfügbar ist.");
		//		}
		$this->_defaultLang = $lang;
	}

	public function getDefaultLanguage() {
		return $this->_defaultLang;
	}

	public function setDefaultRole($role) {
		$this->_defaultRole = $role;
	}

	public function getDefaultRole() {
		return $this->_defaultRole;
	}

	public function setBreakIfNotExactly($value = true) {
		$this->_breakIfNotExact = $value;
	}

	/**
	 *
	 * @todo comment
	 * @return multitype:string
	 */
	public function getAvaiableLanguages() {
		return $this->getDbO()->getAvaiableLanguages();
	}

	/**
	 *
	 * @todo comment
	 * @param $deliver Hmt_Log_DeliverEntry
	 * @return NULL Hmt_Reason_Item
	 */
	public function matchRule(Hmt_Log_DeliverEntry $deliver) {
		$reasonToMatch = $deliver->getOriginalReason();
		$matches = array();
		$ret = null;
		isDebug('hmt_reason_matching') && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($deliver, "Matching"));
		foreach ($this->getRegEx4use() as $regExData) {
			isDebug('hmt_reason_matching') && $this->log()->debug(__METHOD__ . '## trying RegEx: ' . $regExData['regex']);
			if (preg_match($regExData['regex'], $reasonToMatch, $matches) > 0) {
				isDebug('hmt_reason_matching') && $this->log()->debug(__METHOD__ . '## found match with RegEx: ' . $regExData['regex']);
				$ret = new Hmt_Reason_Item($regExData, $matches, $deliver);
				break;
			}
		}
		return $ret;
	}

	public function getReasonTag(Hmt_Reason_Item $match, $role = 'admin', $lang = 'de_de') {
		try {
			$explain = $this->loadExplanations($role, $lang);
			isDebug('hmt_reason') && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($explain, 'EXPLAINS'));
			if (isset($explain[$match->getRuleID()])) {
				return $explain[$match->getRuleID()]['tag_desc'];
			}
		} catch (Exception $ex) {
			throw $ex;
		}
		
		return null;
	}

	public function getShortExplanation(Hmt_Reason_Item $match, $role = 'admin', $lang = 'de_de') {
		
		try {
			$explain = $this->loadExplanations($role, $lang);
			isDebug('hmt_reason') && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($explain, 'EXPLAINS'));
			if (isset($explain[$match->getRuleID()])) {
				return $explain[$match->getRuleID()]['short_desc'];
			}
		} catch (Exception $ex) {
			throw $ex;
		}
		
		return null;
	}

	/**
	 *
	 * @todo comment
	 *       <p>Aufgrund der bisweilen enormen Größe der langen Erklärung, wird diese jedes Mal aus
	 *       der DB oder vom File-Cache gelesen, statt sie im Speicher vorzuhalten. Die lange
	 *       Erklärung wird ohnehin
	 *       nur auf den extra Erklärungsseiten und damit nur eine (mit Hobs ein paar wenige) pro
	 *       Controller-Durchlauf aufgerufen.
	 * @param $ruleID string
	 * @param $role string
	 * @param $lang unknown_type
	 * @return NULL
	 */
	public function getLongExplanation($ruleID, $role = 'admin', $lang = 'de_de') {
		//		if ($ruleID instanceof Hmt_Reason_Item) {
		//			$ruleID = $ruleID->getRuleID();
		//		}
		//		//@todo caching
		//		$sql = $this->getDb()->select();
		//		$sql->from(strtolower(self::TABLE_REASON_PREFIX . $lang), array(
		//				'rule_id',
		//				'long_desc' ))->where('role_id = ?', $this->getRoleID($role))->where('rule_id = ?', $this->mapRuleID($ruleID));
		//		isDebug('hmt_reason') && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($sql->assemble(), 'SQL'));
		//		$stm = $sql->query();
		//		$res = $stm->fetchAll(PDO::FETCH_ASSOC);
		//		if (count($res) == 0) {
		//			return null;
		//		}
		//		$res = array_change_key_case($res, CASE_LOWER);
		

		return $this->getDbO()->getLongExplanation($ruleID);
	}

	public function getExplanations() {
	}

	/**
	 *
	 * @return Zend_Db_Adapter_Abstract
	 */
	public function getDb($name = null) {
		return $this->getDbO($name)->getDb();
	}

	/**
	 *
	 * @todo comment
	 * @return NULL
	 */
	public function getRegEx4use() {
		if (is_null($this->_regexs)) {
			$this->_regexs = $this->getDbO()->getRegEx4use();
		}
		isDebug('hmt_reason_matching') && $this->log()->debug(Hpls_Debug::print_r($this->_regexs, __METHOD__ . ': USED RegEx'));
		return $this->_regexs;
	
	}

	/**
	 *
	 * @todo comment
	 * @param $ruleID string
	 * @return Ambigous <multitype:multitype: , multitype:unknown >
	 */
	public function getRegExs($ruleID) {
		return $this->getDbO()->getRegExs($ruleID);
	}

	/**
	 *
	 * @todo comment
	 * @param $regexID string
	 * @return Ambigous <multitype:multitype: , multitype:unknown >
	 */
	public function getRegEx($regexID) {
		return $this->getDbO()->getRegEx($regexID);
	}

	public function getRule($id) {
		return $this->getDbO()->getRule($id);
	}

	/**
	 *
	 * @todo comment
	 * @return array:Hmt_Reason_Rule Hmt_Reason_Edit_Rule
	 */
	public function getRules() {
		return $this->getDbO()->getRules();
	}

	public function getRoles() {
		return $this->getDbO()->getRoles();
	}

	public function getReasonText($ruleID, $lang = null, $role = null) {
		return $this->getDbO()->getReasonText($ruleID, $lang);
	}

	public function getReasonTextById($lang, $textid) {
		//		if (!in_array($lang, $this->getAvaiableLanguages())) {
		//			require_once 'Hpls/Mailtrace/Exception/Reason.php';
		//			throw new Hpls_Mailtrace_Exception_Reason("Language '$lang' is not avaiable.");
		//		}
		//		
		//		$sql = $this->getDb()->select();
		//		$sql->from(self::TABLE_REASON_PREFIX . $lang)->where($this->getDb()->quoteIdentifier('id') . ' = :rid');
		//		$this->log()->debug(Hpls_Debug::print_r($sql->assemble(), __METHOD__));
		//		$stm = $this->getDb()->query($sql, array(
		//				':rid' => $textid ));
		//		$stm->execute();
		//		$ret = $this->fetch($stm, __FUNCTION__);
		//		foreach ($ret as $r) {
		//			$r->setLanguage($lang);
		//		}
		return $this->getDbO()->getReasonTextById($lang, $textid);
	}

	public function clearCache() {
		if ($this->getDbO() instanceof Hmt_Reason_Cachedb) {
			$this->getDbO()->deleteDb();
		} else {
			$dbConfig = Hpls_Settings::i()->getConfig('hmt.reason.storage.' . Hpls_Mailtrace_Consts::CONFIG_KEY_CACHEDB);
			$dbConfig = is_null($dbConfig) ? null : $dbConfig->toArray();
			$db = new Hmt_Reason_Cachedb($dbConfig, array(
					$this,
					'fetch' ));
			$db->deleteDb();
		}
	}

	/**
	 *
	 * @todo comment
	 * @param $dbName
	 * @return NULL multitype:string
	 */
	public function getVersion($dbName = null) {
		$cfg = Hpls_Settings::i()->getConfig('hmt.reason.storage');
		if (empty($cfg)) {
			return null;
		}
		if ($dbName !== null) {
			return $this->getDbO($dbName)->getVersion();
		}
		
		$ret = array();
		foreach ($cfg as $k => $cfgDb) {
			if ($k == Hpls_Mailtrace_Consts::CONFIG_KEY_CACHEDB)
				continue;
			$version = $this->getDbO($k)->getVersion();
			if (empty($version)) {
				$version = 'initial';
			}
			$ret[$k] = $version;
		}
		
		return $ret;
	}

	/**
	 * Gibt die Version des Datenbankschemas zurück.
	 * 
	 * @return NULL string
	 */
	public function getStructureVersion() {
		$cfg = Hpls_Settings::i()->getConfig('hmt.reason.storage');
		if (empty($cfg)) {
			return null;
		}
		return (string) $this->getDbO(Hpls_Mailtrace_Consts::CONFIG_KEY_VENDORDB)->getConfigValue(Hpls_Consts0::SQL_CONFIGKEY_VERSION);
	}

	/**
	 *
	 * @todo comment
	 * @return array <ul><li><b>[connection]</b></li>
	 *         <li><b>[structure]</b></li>
	 *         <li><b>[data]</b></li>
	 *         </ul>
	 */
	public function checkReadyness() {
		$this->getDbO();
		$ret = array();
		$config = Hpls_Settings::i()->getConfig('hmt.reason.storage');
		$dbConfig = $config->toArray();
		
		foreach ($dbConfig as $name => $dbObj) {
			$ret[$name] = $this->getDbO($name)->checkReadyness();
		}
		return $ret;
	}

	public function isReadyToUse() {
		return $this->getDbO(Hpls_Mailtrace_Consts::CONFIG_KEY_VENDORDB)->isReadyToUse();
	}

	public function refreshConfig() {
		$this->_db = null;
		$this->_dbObjs = array();
		$this->_cacheDB = null;
		$this->_explanations = array();
		$this->_regexs = null;
		$this->_roleMap = null;
	}

	public function getSources() {
		$config = Hpls_Settings::i()->getConfig('hmt.reason.storage');
		$dbConfig = $config->toArray();
		$ret = array_keys($dbConfig);
		unset($ret[Hpls_Mailtrace_Consts::CONFIG_KEY_CACHEDB]);
		return $ret;
	}

	/**
	 *
	 * @todo comment
	 * @param $stm unknown_type
	 * @param $inFunction unknown_type
	 * @return multitype:unknown |multitype:multitype:
	 * @deprecated do not use directly! (nicht wirklich deprecated, nur einfach anderweitig nutzen
	 *             als es jetzt schon ist!)
	 */
	public function fetch($stm, $inFunction) {
		
		$fetchClass = null;
		switch ($inFunction) {
			case 'getRules' :
			case 'getRule' :
				$fetchClass = 'Hmt_Reason_Rule';
				break;
			case 'getRoles' :
				$fetchClass = 'Hmt_Reason_Role';
				break;
			case 'getReasonText' :
			case 'getReasonTextById' :
				$fetchClass = 'Hmt_Reason_Texts';
				break;
			
			case 'getRegExs' :
			case 'getRegEx' :
				$fetchClass = 'Hmt_Reason_RegEx';
		}
		if (!empty($fetchClass)) {
			$ret = array();
			while ($row = $stm->fetchObject($fetchClass)) {
				//				$row = array_change_key_case($row, CASE_LOWER);
				$ret[] = $row;
			}
			return $ret;
		} else {
			$ret = array();
			while ($row = $stm->fetch(PDO::FETCH_ASSOC)) {
				$row = array_change_key_case($row, CASE_LOWER);
				$ret[] = $row;
			}
			return $ret;
		}
	
	}

	/**
	 *
	 * @todo comment
	 * @param $name string
	 * @throws Hpls_Exception_Configuration
	 * @return Hmt_Reason_Db
	 */
	protected function getDbO($name = null) {
		if (is_null($name))
		$name = Hpls_Mailtrace_Consts::CONFIG_KEY_CACHEDB;
		$name = strtolower($name);
		if (!isset($this->_dbObjs[$name])) {
			try {
				$config = Hpls_Settings::i()->getConfig("hmt.reason.storage.$name");
				$dbConfig = $config->toArray();
				if ($name == Hpls_Mailtrace_Consts::CONFIG_KEY_CACHEDB) {
					$this->_dbObjs[$name] = new Hmt_Reason_Cachedb($dbConfig, array(
							$this,
							'fetch' ), $name);
					$this->_db = $this->_dbObjs[$name];
				} else {
					$this->_dbObjs[$name] = new Hmt_Reason_Db($dbConfig, array(
							$this,
							'fetch' ), $name);
				}
			} catch (Hpls_Exception_KeyNotExists $ex) {
				$this->log()->err(__METHOD__ . " $ex");
				require_once 'Hpls/Exception/Configuration.php';
				throw new Hpls_Exception_Configuration("No configuration for Reason-DB '$name'");
			}
			isDebug('hmt_reason_config') && $this->log()->debug(Hpls_Debug::print_r($this->_dbObjs, __METHOD__));
		}
		isDebug('hmt_reason_cachedb') && $this->log()->info(__METHOD__ . " $name");
		return $this->_dbObjs[$name];
	}

	protected function loadExplanations($role, $lang) {
		$role = strtolower($role);
		$lang = strtolower($lang);
		if (isset($this->_explanations[$role . $lang])) {
			
			$this->log()->debug('LOADEXPLANATIONS: returning (cached) '.print_r($this->_explanations[$role . $lang], TRUE));
			return $this->_explanations[$role . $lang];
		}
		$this->_explanations[$role . $lang] = $this->getDbO()->getExplanations($role, $lang);
		$this->log()->debug('LOADEXPLANATIONS: returning (noncached) '.print_r($this->_explanations[$role . $lang], TRUE));
		return $this->_explanations[$role . $lang];
	}

	/**
	 *
	 * @todo comment
	 * @param $role unknown_type
	 * @return NULL
	 */
	protected function getRoleID($role) {
		return $this->getDbO()->getRoleID($role);
	}

	/**
	 *
	 * @todo comment
	 * @param $ruleID unknown_type
	 * @return Ambiguous NULL
	 */
	protected function mapRuleID($ruleID) {
		return $this->getDbO()->mapRuleID($ruleID);
	
	}

}