<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 30.08.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2012-04-15 16:36:08 +0200 (So, 15 Apr 2012) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9311 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/HMT/GUI/Core/trunk/hpls.hmt.gui.libs/Hmt/Reason/Exporter.php $
 * 
 * author         : Sebastian Nitzsche <s.nitzsche@heinlein-support.de>
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/**
 */
require_once 'Hmt/Reason/Edit.php';
require_once 'Hmt/Reason/Entry.php';
require_once 'Zend/Config/Writer/Ini.php';

/**
 *
 * @todo comment
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 9311 $
 * @since 1.0.9
 */
class Hmt_Reason_Exporter extends Hpls_Utils_Logusing {
	
	const TPL_ARCHIVE_FILENAME = "export_%s";
	const OPTION_KEY_CONFIG = 'config';
	
	const DDL_TEMPLATE_CREATE = "[[DBTYPE]]/create.[[DBTYPE]].tpl.sql";
	const DDL_TEMPLATE_CREATE_REASONS = "[[DBTYPE]]/create.tbl_reasons.[[DBTYPE]].tpl.sql";
	const DDL_TEMPLATE_ALTER = "[[DBTYPE]]/alter_table.[[DBTYPE]].tpl.sql";
	const DDL_TEMPLATE_ALTER_REASONS = "[[DBTYPE]]/alter_table.tbl_reasons.[[DBTYPE]].tpl.sql";
	
	protected static $_instance;
	
	protected $_entryList = array();
	protected $_langs = array();
	protected $_sqltpls = array();
	protected $_config = null;
	protected $_files = array();
	protected $_archive = null;
	protected $_createTime = 0;
	protected $_ddlTemplates = array();
	/**
	 *
	 * @var Hmt_Reason_Db
	 */
	protected $_db = null;
	protected $_dbSource = null;
	protected $_dbVersion = null;
	protected $_filename = null;
	
	protected $_tmpdir = null;
	protected $_processSuccess = true;

	/**
	 *
	 * @return Hmt_Reason_Exporter
	 */
	public static function i() {
		if (is_null(self::$_instance)) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	/**
	 * Archiv-Datei erstellen.
	 * 
	 * @param $options unknown_type
	 */
	public static function createArchive($options, $tmpdir, $files) {
		switch (strtolower($options->archivemethod)) {
			case 'zip' :
				Hmt_Reason_Exporter::createZipArchive($options, $tmpdir, $files);
				break;
		}
	}

	public static function deleteTempDir($tmpdir, $filetype) {
		self::deleteTree($tmpdir, $filetype);
	}

	public function setOptions($options) {
		if ($options['langs'][0] == 'all') {
			$this->_langs = Hmt_Reason_Edit::i()->getAvaiableLanguages();
		} else {
			$this->_langs = $options['langs'];
		}
		
		$this->_createTime = intval(strtotime(Hmt_Reason::i()->getVersion(Hpls_Mailtrace_Consts::CONFIG_KEY_VENDORDB))); //intval(microtime(true));
		$this->_dbSource = $options['database'];
		$this->_config = $options[self::OPTION_KEY_CONFIG];
		$this->_tmpdir = $options[self::OPTION_KEY_CONFIG]->dir . DIRECTORY_SEPARATOR . date('YmdHis', $this->_createTime).'_'.Hmt_Reason::i()->getStructureVersion(); // . '_' . md5('export_hmtr_');
		//        $this->_tmpdir = $options[self::OPTION_KEY_CONFIG]->dir . DIRECTORY_SEPARATOR . md5('export_hmtr_');
		

		$this->_sqltpls = explode(',', $this->_config->availablesqltpls);
		$this->_tables = array(
				'config',
				'roles',
				'rule_ids',
				'regexrule' );
		$this->_reasons = "reasons_%s";
		
		if (FALSE === is_dir($this->_tmpdir)) {
			mkdir($this->_tmpdir);
		}
	}

	public function getTempDir() {
		return $this->_tmpdir;
	}

	public function getFiles() {
		return $this->_files;
	}

	public function setTempDir($dir) {
		$this->_tmpdir = $dir;
	}

	public function hasProcessSuccess() {
		return $this->_processSuccess;
	}

	/**
	 * Extraiert ein ZipArchive an einen gewünschtes Ziel.
	 * 
	 * @param $file String
	 * @param $desitnation String
	 * @return void
	 */
	public function extractZipArchive($file, $desitnation) {
		$zip = new ZipArchive();
		$zip->open($file);
		$zip->extractTo($desitnation);
		$zip->close();
	}

	public function run() {
		$this->getDbO();
		$this->generateDataInserts();
	}

	/**
	 * Erstellt das Ziparchiv
	 * 
	 * @return void
	 */
	protected static function createZipArchive($options, $tmpdir, $files) {
		try {
			if (FALSE === class_exists('ZipArchive')) {
				throw new Exception("PHP Erweiterung Zip nicht vorhanden.");
			}
			$exportCfg = Hpls_Settings::i()->getConfig('hmt.reason.export');
			$zip = new ZipArchive();
			$archive = sprintf($exportCfg->filename, basename($tmpdir)) . Hpls_Mailtrace_Consts::FILE_EXTENSION_REASON_UPDATE;
			
			if (TRUE !== $zip->open($options->dir . DIRECTORY_SEPARATOR . $archive, ZipArchive::CM_PKWARE_IMPLODE)) {
				throw new Exception("Ziparchiv $archive konnte nicht erstellt werden.");
			}
			
			foreach ($files as $filepath) {
				if (FALSE === $zip->addFile($filepath, str_replace(dirname($filepath) . DIRECTORY_SEPARATOR, '', $filepath))) {
					throw new Exception("Datei $filepath konnte nicht dem Ziparchiv ($archive) hinzugefügt werden.");
				}
			}
			$zip->close();
		
		} catch (Exception $e) {
			Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'hmt_reason_export', $e->getMessage());
			$this->_processSuccess = false;
			return;
		}
	}

	/**
	 * Löscht ein Verzeichnisbaum, den der Apacheuser auch löschen darf.
	 * (hier für TMP-Dir)
	 * 
	 * @param $dir String
	 * @return void
	 */
	protected static function deleteTree($dir, $filetype, $isRoot = true) {
		$files = glob($dir . '*', GLOB_MARK);
		foreach ($files as $file) {
			if (substr($file, -1) == '/') {
				self::deleteTree($file, $filetype, false);
			} else if (!$isRoot && FALSE === strpos($file, '.' . $filetype)) {
				if (FALSE === unlink($file)) {
					Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_WARN, 'hmt_reason_export', "System konnte die Tmp-Datei '$file' nicht löschen.");
				}
			}
		}
		
		if (is_dir($dir)) {
			if (FALSE === rmdir($dir)) {
				Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_WARN, 'hmt_reason_export', "System konnte den Tmp-Ordner '$dir' nicht löschen.");
			}
		}
	}

	protected function __construct() {
	
	}

	protected function generateDataInserts() {
		
		$this->generateStructureDDL();
		
		$this->generateDataConfig();
		$this->generateDataRoles();
		$this->generateDataRules();
		$this->generateDataRegex();
		$this->generateDataReasons();
		
		// generate controller.ini
		$this->generateMetaData();
	
	}

	protected function generateMetaData() {
		$iniFilename = realpath(Hpls_Application0::i()->getAppsConfigPath() . DIRECTORY_SEPARATOR . $this->_config->templatepath . DIRECTORY_SEPARATOR . 'controller.ini');
		$ini = new Zend_Config_Ini($iniFilename,null,true);
		$ini->settings->languages = (string)implode(',', $this->_langs);
		$ini->settings->version = (string)$this->_dbVersion;
		$ini->settings->archive_version = Hmt_Reason::i()->getStructureVersion();
		$cfgFile = $this->_tmpdir . DIRECTORY_SEPARATOR . "controller.ini";
		
		$writer = new Zend_Config_Writer_Ini();
		$writer->write($cfgFile,$ini);
		$this->_files[] = $cfgFile;
	}

	/**
	 * Generiert bzw.
	 * kopiert die SQL-Files, die die ALTER und/oder CREATE statements zur Anpassung der DB-Struktur
	 * bereitstellen in das Archiv.
	 */
	protected function generateStructureDDL() {
		$path = realpath(Hpls_Application0::i()->getAppsConfigPath() . DIRECTORY_SEPARATOR . $this->_config->templatepath) . DIRECTORY_SEPARATOR;
		
		// 		foreach ($this->_sqltpls as $dbtype) {
		// 			$file1 = str_replace('[[DBTYPE]]',$dbtype,Hmt_Reason_Exporter::DDL_TEMPLATE_CREATE);
		// 			$file2 = str_replace('[[DBTYPE]]',$dbtype,Hmt_Reason_Exporter::DDL_TEMPLATE_CREATE_REASONS);
		// 			$file3 = str_replace('[[DBTYPE]]',$dbtype,Hmt_Reason_Exporter::DDL_TEMPLATE_ALTER);
		// 			$file4 = str_replace('[[DBTYPE]]',$dbtype,Hmt_Reason_Exporter::DDL_TEMPLATE_ALTER_REASONS);
		

		// 			$content1 = file_get_contents($path . $file1);
		// 			$content2tmp = file_get_contents($path . $file2);
		// 			$content2 = '';
		// 			foreach ($this->_langs as $locale) {
		// 				$content2 .= str_replace('[[LOCALE]]', $locale, $content2tmp) . "\n";
		// 			}
		

		// 			$content3 = file_get_contents($path . $file3);
		// 			$content4tmp = file_get_contents($path . $file4);
		// 			$content4 = '';
		// 			foreach ($this->_langs as $locale) {
		// 				$content4 .= str_replace('[[LOCALE]]', $locale, $content4tmp) . "\n";
		// 			}
		

		// 			file_put_contents($this->_tmpdir . DIRECTORY_SEPARATOR . $file1, $content1);
		// 			file_put_contents($this->_tmpdir . DIRECTORY_SEPARATOR . $file2, $content2);
		// 			file_put_contents($this->_tmpdir . DIRECTORY_SEPARATOR . $file3, $content3);
		// 			file_put_contents($this->_tmpdir . DIRECTORY_SEPARATOR . $file4, $content4);
		

		// 			$this->_files[] = $this->_tmpdir . DIRECTORY_SEPARATOR . $file1;
		// 			$this->_files[] = $this->_tmpdir . DIRECTORY_SEPARATOR . $file2;
		// 			$this->_files[] = $this->_tmpdir . DIRECTORY_SEPARATOR . $file3;
		// 			$this->_files[] = $this->_tmpdir . DIRECTORY_SEPARATOR . $file4;
		// 		}
	}

	protected function generateDataConfig() {
		$this->loadTableData(Hmt_Reason_Db::TABLE_CONFIG);
	}

	protected function generateDataRoles() {
		$this->loadTableData(Hmt_Reason_Db::TABLE_ROLES);
	}

	protected function generateDataRules() {
		$this->loadTableData(Hmt_Reason_Db::TABLE_RULES);
	}

	protected function generateDataRegex() {
		$this->loadTableData(Hmt_Reason_Db::TABLE_REGEX);
	}

	protected function generateDataReasons() {
		foreach ($this->_langs as $locale) {
			$tbl = Hmt_Reason_Db::TABLE_REASON_PREFIX . $locale;
			$this->loadTableData($tbl);
		}
	}

	/**
	 *
	 * @return Hmt_Reason_Db
	 */
	protected function getDbO() {
		$this->_db = Hmt_Reason_Edit::i()->getDbO($this->_dbSource);
		return $this->_db;
	}

	/**
	 * Gibt die maximale Dateigröße zurück, die in der Konfigdatei definirt wurde (in Bytes)
	 * 
	 * @return Integer
	 */
	protected function getMaxFileSize() {
		$max = strtoupper($this->_config->maxfilesize);
		if (substr_count($max, 'K') == 1 || substr_count($max, 'M') == 1 || substr_count($max, 'G') == 1) {
			$number = intval(substr($max, 0, strlen($max) - 1));
			if (substr_count($max, 'K') == 1) {
				$number = $number * 1024;
			} else if (substr_count($max, 'M') == 1) {
				$number = $number * 1024 * 1024;
			} else if (substr_count($max, 'G') == 1) {
				$number = $number * 1024 * 1024 * 1024;
			}
		} else {
			$number = intval($this->_options->_config->maxfilesize);
		}
		
		return $number;
	}

	protected function loadTableData($tblname) {
		$select = $this->_db->getDb()->select();
		$select->from($tblname);
		$stmt = $this->_db->getDb()->query($select);
		$result = $stmt->fetchAll();
		$list = array();
		
		$json = '';
		foreach ($result as $row) {
			$entry = new Hmt_Reason_Entry();
			
			$entry->_confTableName = $tblname;
			foreach ($row as $key => $value) {
				$entry->setValue($key, $value);
			}
			$list[] = $entry;
		}
		
		$part = 1;
		$filename = "tbl_" . $tblname;
		$tplpath = $this->_tmpdir . DIRECTORY_SEPARATOR . "%s.%s.json";
		$filepath = sprintf($tplpath, $filename, $part);
		$size = 0;
		$this->_files[] = $filepath;
		
		foreach ($list as $item) {
			if ($size > $this->getMaxFileSize()) {
				$part++;
				$size = 0;
				$filepath = sprintf($tplpath, $filename, $part);
			}
			
			if ($tblname == Hmt_Reason_Db::TABLE_CONFIG) {
				if ($item->getValue('key') == 'version') {
					$this->_dbVersion = $item->getValue('value');
				}
			}
			
			$content = Hmt_Reason_Entry::toJsonStr($item) . "\n";
			file_put_contents($filepath, $content, FILE_APPEND);
			$size += strlen($content);
		}
	}
}