<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 30.08.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2012-04-15 16:36:08 +0200 (So, 15 Apr 2012) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9311 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/HMT/GUI/Core/trunk/hpls.hmt.gui.libs/Hmt/Reason/Updater.php $
 *
 * author         : Sebastian Nitzsche <s.nitzsche@heinlein-support.de>
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/**
 */
require_once 'Hmt/Reason/Edit.php';
require_once 'Hmt/Reason/Entry.php';

/**
 *
 * @todo comment
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 9311 $
 * @since 1.0.9
 */
class Hmt_Reason_Updater {
	
	const OPTION_KEY_CONFIG = 'config';
	
	protected static $_instance;
	
	protected $_entryList = array();
	protected $_langs = array();
	protected $_config = null;
	protected $_files = array();
	protected $_archive = null;
	protected $_archiveVersion = null;
	protected $_createTime = null;
	protected $_extractMethod = null;
	protected $_availableExtracts = array(
			'zip' );
	/**
	 *
	 * @var Hmt_Reason_Db
	 */
	protected $_db = null;
	protected $_dbSource = null;
	protected $_dbVersion = null;
	protected $_filename = null;
	
	protected $_tmpdir = null;
	protected $_processSuccess = true;
	
	protected $_isInitialized = false;
	protected $_isFirstEntry = true;

	/**
	 *
	 * @return Hmt_Reason_Exporter
	 */
	public static function i() {
		if (is_null(self::$_instance)) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	/**
	 * Setzen der Options aus der GUI.
	 * Wenn Options NULL ist werden Default-Werte verwendet.
	 * Optionen:
	 * exportdir = Exportpfad (String)
	 * langs = Sprachen, der zu exportierenden TBLs (Array)
	 * database = Name des Datenbank-Objektes
	 * 
	 * @param $options Array
	 */
	public function setOptions($options) {
		$this->_config = $options[self::OPTION_KEY_CONFIG];
		$this->_config->archiveversion = Hmt_Reason::i()->getStructureVersion();
		$this->_dbSource = $options['database'];
		$this->_createTime = intval(microtime(true));
		$this->_tmpdir = $options[self::OPTION_KEY_CONFIG]->dir . DIRECTORY_SEPARATOR . $this->_createTime . '_' . md5('export_hmtr_');
		$this->_extractMethod = strtolower($options['extractmethod']);
		$this->_archive = $options['file'];
		
		$this->getDbO();
	}

	/**
	 *
	 * @return Boolean
	 */
	public function hasProcessSuccess() {
		return $this->_processSuccess;
	}

	public function getVersion() {
		return $this->_dbVersion;
	}

	/**
	 *
	 * @todo comment
	 */
	public function run() {
		try {
			$configVersion = $this->_config->archiveversion;
			$this->extractFile($this->_archive);
			$this->readConfig();

			if ($configVersion == $this->_archiveVersion) {
				$this->updateStructure();
				$this->updateData();
				Hmt_Reason_Edit::i()->clearCache();
			} else {
				Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'hmt_reason_update', "Update kann nicht erfolgen. Grund:<br />Die Archiv-Version der Updatedatei ist nicht gleich der Einstellung in der Config.<br />Archiv-Version: " . $this->_archiveVersion . "<br />Config: " . $configVersion);
				$this->_processSuccess = false;
			}
		} catch (Exception $e) {
			$this->_processSuccess = false;
			
			Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'hmt_reason_update', $e->getMessage());
		}
	}

	protected function extractFile($filename) {
		list($dirname, $type) = explode('.', $filename);
		switch ($this->_extractMethod) {
			case 'zip' :
				$this->extractZipArchive($filename, $dirname);
				break;
			
			default :
				Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'hmt_reason_update', "Ungültige Extrahierungs-Methode: '" . $this->_extractMethod . "'.");
				$this->_processSuccess = false;
				break;
		}
		$this->_tmpdir = $dirname;
	}

	protected function extractZipArchive($filename, $destination) {
		if (FALSE === class_exists('ZipArchive')) {
			throw new Exception("Dieser Webserver bietet keine Unterstützung für ZIP-Archive.");
		}
		
		$zip = new ZipArchive();
		
		if (TRUE !== $zip->open($filename)) {
			throw new Exception("Ziparchiv $filename konnte nicht geöffnet werden.");
		}
		
		if ($zip->numFiles < 1) {
			throw new Exception("Es wurden keine Dateien im Archive $filename gefunden.");
		}
		
		if (FALSE === $zip->extractTo($destination)) {
			throw new Exception("Datei $filename konnte nicht nach $destination entpackt werden.");
		}
		$zip->close();
		unlink($filename);
	
	}

	protected function readConfig() {
		$config = new Zend_Config_Ini($this->_tmpdir . '/controller.ini');
		$config = $config->settings;
		$this->_langs = explode(',', $config->languages);
		$this->_dbVersion = $config->version;
		$this->_config = $config->import;
		$this->_archiveVersion = $config->archive_version;
	}

	protected function updateStructure() {
	
	}

	protected function updateData() {
		$files = explode(',', $this->_config->insertsort);
		$filelist = scandir($this->_tmpdir);
		
		foreach ($files as $prefix) {
			foreach ($filelist as $tmpfile) {
				if (FALSE !== strpos($tmpfile, $prefix) && preg_match('/\.json$/i', $tmpfile)) {
					$filepath = $this->_tmpdir . DIRECTORY_SEPARATOR . $tmpfile;
					$this->_isFirstEntry = true;
					try {
						$lines = file($filepath);
						foreach ($lines as $json) {
//    	                    echo "<b>$filepath</b><br>$json<hr>";
							$entry = Hmt_Reason_Entry::parseJsonStr($json);
							$this->processInsert($entry);
						}
					} catch (Exception $ex) {
						Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_WARN, 'hmt_reason_update', "Meldung: " . $ex->getMessage());
						$this->_processSuccess = false;
					}
				}
			}
		}
	}

	/**
	 * BEFORE INSERT / WITHOUT INDEX (disable)
	 * 
	 * @param $entry Hmt_Reason_Entry
	 */
	protected function processPrepareInsert(Hmt_Reason_Entry $entry) {
		// TODO: CREATE
		$files = explode(',', $this->_config->createsort);
		$filelist = scandir($this->_tmpdir);
		$doPrepareInsert = true;
		
		switch (get_class($this->_db->getDb())) {
			case 'Zend_Db_Adapter_Pdo_Mysql' :
				$dbtype = 'mysql';
				break;
			
			case 'Zend_Db_Adapter_Pdo_Pgsql' :
				$dbtype = 'pgsql';
				break;
			
			default :
				Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_WARN, 'hmt_reason_update', "Der Datenbank-Adapter ist nicht definiert: " . get_class($this->_db->getDb()));
				$doPrepareInsert = false;
		}
		
		if (TRUE === $doPrepareInsert) {
			if (FALSE === $this->_isInitialized) {
				// einmalige Ausführung 
				$this->_isInitialized = true;
				
				foreach ($files as $prefix) {
					foreach ($filelist as $tmpfile) {
						if (FALSE !== strpos($tmpfile, $prefix . $dbtype)) {
							$query = trim(file_get_contents($this->_tmpdir . DIRECTORY_SEPARATOR . $tmpfile));
							
							if (!empty($query)) {
								$this->_db->getDb()->query($query);
							}
						}
					}
				}
			}
			
			if (TRUE === $this->_isFirstEntry) {
				// Ausführung befor der erste Eintrag in die Datenbank geschrieben wird
				$this->_isFirstEntry = false;
				$this->_db->getDb()->query("TRUNCATE " . $entry->getTablename() . ";");
			}
		}
	}

	protected function processInsert(Hmt_Reason_Entry $entry) {
		
		// BEFORE INSERT
		$this->processPrepareInsert($entry);
		
		$db = $this->_db->getDb();
		
		$data = array();
		$table = $entry->getTablename();
		try {
			// 			$result = $db->describeTable($table);
			foreach ($entry->getKeys() as $key) {
				
				// Ausnahmebehandlung von bool'schen Werten
				if ($key === 'deactivated') {
					if ($entry->getValue($key) == '') {
						$entry->setValue($key, 0);
					}
				}
				$data[$key] = $entry->getValue($key);
			}
			$db->insert($entry->getTablename(), $data);
		} catch (Exception $e) {
			Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_WARN, 'hmt_reason_update', "Meldung: " . $e->getMessage() . ' ('.$entry->getTablename().')');
		}
		
		// AFTER INSERT
		//	    $this->processPostInsert($entry);
	}

	/**
	 * AFTER INSERT / CREATE INDEX
	 */
	protected function processPostInsert(Hmt_Reason_Entry $entry) {
	
	}

	/**
	 * Löscht ein Verzeichnisbaum, den der Apacheuser auch löschen darf.
	 * (hier für TMP-Dir)
	 * 
	 * @param $dir String
	 * @return void
	 */
	protected function deleteTree($dir, $isRoot = true) {
		$files = glob($dir . '*', GLOB_MARK);
		foreach ($files as $file) {
			if (substr($file, -1) == '/') {
				self::deleteTree($file, false);
			} else {
				if (FALSE === unlink($file)) {
					Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_WARN, 'hmt_reason_update', "System konnte die Tmp-Datei '$file' nicht löschen.");
				}
			}
		}
		
		if (is_dir($dir)) {
			if (FALSE === rmdir($dir)) {
				Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_WARN, 'hmt_reason_update', "System konnte den Tmp-Ordner '$dir' nicht löschen.");
			}
		}
	}

	/**
	 *
	 * @return Hmt_Reason_Db
	 */
	protected function getDbO() {
		$this->_db = Hmt_Reason_Edit::i()->getDbO($this->_dbSource);
		return $this->_db;
	}
}
