<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created   : May 17, 2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2012-01-04 20:22:41 +0100 (Mi, 04 Jan 2012) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9199 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Acl.php $
 *
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/**
 */
require_once 'Zend/Acl.php';
require_once 'Hpls/Utils.php';
require_once 'Hpls/Acl/Ids.php';
require_once 'Hpls/Acl/Ops.php';
require_once 'Hpls/Acl/Resource.php';
require_once 'Hpls/Acl/Role.php';
require_once 'Hpls/Acl/Licence/Reader.php';
require_once 'Hpls/Acl/Reader/Abstract.php';
require_once 'Hpls/Acl/Reader/Xml.php';
require_once 'Hpls/Acl/ZendExtend.php';
require_once 'Hpls/Acl/Acl.php';
require_once 'Hpls/Auth.php';
require_once 'Hpls/Consts0.php';

/**
 * Alias für Hpls_Acl::isAllowed().
 * <p>
 * Hauptsächlich dafür, um es in Smarty-TPLs nutzen zu können.
 *
 * @param $resourceId unknown_type       	
 * @param $resOperation unknown_type       	
 * @param $role unknown_type       	
 */
function acl_isAllowed($resourceId, $resOperation = null, $role = null) {
	// FOR DEBUG
	// return false;
	return Hpls_Acl::i()->isAllowed($resourceId, $resOperation, $role);
}

/**
 *
 * @todo comment
 *      
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *           
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 9199 $
 * @since 1.0.9
 */
class Hpls_Acl_Data {
	/**
	 *
	 * @todo comment
	 * @var Zend_Acl_Resource
	 */
	public $rootResource = null;
	/**
	 *
	 * @var Hpls_Acl_ZendExtend
	 */
	public $ACL = null;
	/**
	 *
	 * @todo comment
	 * @var array:string
	 */
	public $moduleMapping = null;

	public function load($licenseReader, $reader, Hpls_Acl $aclObj) {
		$this->rootResource = new Hpls_Acl_Resource(Hpls_Acl_Ids::ROOT);
		
		$this->ACL = new Hpls_Acl_Acl($this->rootResource, $aclObj);
		$readers = array();
		
		if (is_null($reader))
			$readers[] = new Hpls_Acl_Reader_Xml();
		elseif (is_array($reader)) {
			$readers = array_merge($readers, $reader);
		} else {
			$readers[] = $reader;
		}
		
		if (is_null($licenseReader))
			$readers[] = new Hpls_Acl_Licence_Reader_Backend();
		elseif (is_array($licenseReader)) {
			$readers = array_merge($readers, $licenseReader);
		} else {
			$readers[] = $licenseReader;
		}
		foreach ($readers as $aclReader) {
			$aclReader->setTarget($this->ACL);
			$aclReader->loadRoles($aclObj);
			$aclReader->loadResources($aclObj);
		}
		// dieses separat, damit resourcen und rollen zu dem zeitpunkt
		// existieren
		foreach ($readers as $aclReader) {
			$aclReader->loadRules();
		}
		@file_put_contents('/tmp/ACL.dot', $this->ACL->toDot());
	}

}

/**
 *
 * @todo comment
 *      
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *           
 * @package AC
 * @subpackage Rechtesystem
 * @version $LastChangedRevision: 9199 $
 * @since 1.0ufz
 */
class Hpls_Acl {
	
	const DEFAULT_CONFIG_FILENAME = 'acl.ini';
	const KEY_RESOURCE = '__resources__';
	const KEY_ROLES = '__roles__';
	
	const ROLE_GUEST = Hpls_Consts0::ROLE_GUEST;
	const ROLE_USER = Hpls_Consts0::ROLE_USER;
	const ROLE_ADMIN = Hpls_Consts0::ROLE_ADMIN;
	const ROLE_SUPERADMIN = Hpls_Consts0::ROLE_SUPERADMIN;
	const ROLE_LOGGEDIN = Hpls_Consts0::ROLE_LOGGEDIN;
	const ROLE_HPLSADMIN = Hpls_Consts0::ROLE_HPLSADMIN;
	
	/**
	 *
	 * @var array<string> Namen der privilegien mit Booleanwertebereich.
	 */
	public static $BOOL_PRIV = array(
			'enabled',
			'read',
			'write',
			'start',
			'stop',
			'licensed' );
	/**
	 *
	 * @var array<string> Namen der privilegien mit Integerwertebereich.
	 */
	public static $INT_PRIV = array(
			'max_count',
			'max_warn_count',
			'max_size',
			'max_warn_size',
			'max_ratio',
			'max_warn_ratio' );
	
	protected static $_instance = null;
	/**
	 *
	 * @todo comment
	 * @var Hpls_Acl_Data
	 */
	protected $_data = null;
	protected $_NS = array(
			''
	/*,
			Hpls_Acl_Consts::NS_MASTERSLAVE*/ );
	
	protected $_superadminMode = NULL;
	
	protected $_licenseReader = null;
	protected $_aclReader =null;

	/**
	 * Initialisiert das Singleton-Instanz-Objekt.
	 * <p>Muss die erste Methode an diesem Objekt sein, die aufgerufen wird.
	 *
	 * @param $licenseReader Hpls_Acl_Licence_Reader|array:Hpls_Acl_Licence_Reader       	
	 * @param $reader Hpls_Acl_Reader_Abstract|array:Hpls_Acl_Reader_Abstract       	
	 */
	public static function initInstanceObject($licenseReader, $reader = null) {
		self::$_instance = new Hpls_Acl($licenseReader, $reader);
	
	}

	/**
	 *
	 * @todo comment
	 *      
	 * @return Hpls_Acl
	 */
	public static function i() {
		// if (is_null(self::$_instance)) {
		// self::initInstanceObject();
		// }
		return self::$_instance;
	}

	/**
	 *
	 * @todo comment
	 *      
	 * @param $privName string       	
	 * @return boolean
	 */
	public static function isBoolPrivilege($privName) {
		return in_array($privName, self::$BOOL_PRIV);
	}

	public static function isIntPrivilege($privName) {
		return in_array($privName, self::$INT_PRIV);
	}

	/**
	 * splits a ACL-Query-String into an array
	 * <p>
	 *
	 * [key]#[privilege1_1]+[privilege1_2];[key2]#[privilege2_1]+[privilege2_2]+[privilege2_3]
	 *
	 * @param $aNeed string
	 *       	 in form of
	 *       	 <i>[key]#[privilege1_1]+[privilege1_2];[key2]#[privilege2_1]+[privilege2_2]+[privilege2_3]</i>
	 * @return array <ul>
	 *         <li>[key] => [privileges] array:
	 *         <ul>
	 *         <li>[privilege1_1]=>val,
	 *         <li>[privilege1_2]=>val
	 *         <li>...
	 *         </ul>
	 *         <li>[key2] => ...
	 *         </ul>
	 */
	public static function splitResourceNeeds($aNeed) {
		// multiple resources
		$aNeed = explode(Hpls_Acl_Consts::ID_ID_SEPARATOR, strtolower(trim((string) $aNeed)));
		$ret = array();
		foreach ($aNeed as $need) {
			// multiple privileges
			$valSplit = explode(Hpls_Acl_Consts::ID_VAL_SEPARATOR, $need);
			$valSplit['resource'] = $valSplit[0];
			if (count($valSplit) > 2) { // mindenstens 1 privileg wurde angegeben
				$valSplit['privileges'] = explode(Hpls_Acl_Consts::ID_VAL_VAL_SEPARATOR, $valSplit[1]);
				unset($valSplit[1]);
			}
			unset($valSplit[0]);
			$ret[$valSplit['resource']] = $valSplit;
		}
		return $ret;
	}

	/**
	 *
	 * @todo comment
	 *      
	 * @param $resID string       	
	 * @return string string
	 */
	public static function getParentResource($resID) {
		$pos = strrpos($resID, Hpls_Acl_Consts::ID_SEPARATOR);
		if ($pos === false) { // Beachten sie die drei Gleichheitszeichen
		                      // nicht gefunden ...
			return Hpls_Acl_Ids::ROOT;
		}
		return substr($resID, 0, $pos);
	}

	/**
	 *
	 * @param $licenseReader Hpls_Acl_Licence_Reader|array:Hpls_Acl_Licence_Reader       	
	 * @param $reader Hpls_Acl_Reader_Abstract|array:Hpls_Acl_Reader_Abstract       	
	 */
	protected function __construct($licenseReader, $reader = null) {
		$this->_licenseReader = $licenseReader;
		$this->_aclReader = $reader;
		$this->init();
	}
	
	protected function init() {
		$cache = Hpls_Utils::getCache(null, null);
		if (!$this->_data = $cache->load('ac_acl_data')) {
			isDebug('auth') && $this->log()->debug(__METHOD__ . ": ACLs new to cache" /*. Hpls_Debug::print_r($this->_data, '')*/);
			$this->_data = new Hpls_Acl_Data();
			$this->_data->load($this->_licenseReader, $this->_aclReader, $this);
			$cache->save($this->_data, 'ac_acl_data', array(
					Hpls_Consts0::CACHE_TAG_ACL ));
		} else {
			$this->log()->debug(__METHOD__ . ": ACLs from Cache" /*. Hpls_Debug::print_r($this->_data, )*/);
		}
		if (Hpls_Auth::i()->isLoggedIn()) {
			$userrole = 'usr_' . Hpls_Auth::i()->getUsername();
			$this->addUserRole($userrole, array_diff(Hpls_Auth::i()->getUserRoles(), array(
					$userrole,
					$userrole )));
		}
		// $this->_ACL->deny(null,'ac.services');
		if (isDebug('acl', 'acl_read')) {
			$this->log()->debug(__METHOD__ . ": " . Hpls_Debug::print_r($this->_data, 'ACLs'));
		}
		
	}

	public function addUserRole($role, $parents = null) {
		isDebug('acl', 'acl_read') && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($role, ' role') . Hpls_Debug::print_r($parents, ' parents'));
		if ($this->_data->ACL->hasRole($role)) {
			return;
		}
		if (is_string($role)) {
			$role = new Hpls_Acl_Role($role);
		}
		$this->_data->ACL->addRole($role, $parents);
	}

	/**
	 *
	 * @todo comment
	 * @return NULL Ambigous
	 * @deprecated Use Hpls_Auth::i()->getUsername()
	 */
	public function getCurrentUsername() {
		return Hpls_Auth::i()->getUsername();
	}

	/**
	 * Enter description here .
	 *
	 *
	 *
	 * ..
	 *
	 * @return NULL array:string
	 * @deprecated Use Hpls_Auth::i()->getUserRoles()
	 */
	public function getCurrentUserRoles() {
		return Hpls_Auth::i()->getUserRoles();
	}

	/**
	 *
	 * @todo comment
	 * @param $username string       	
	 * @param $authSource string|null       	
	 * @param $recalculate boolean       	
	 * @return multitype:string:string
	 * @deprecated Use Hpls_Auth::i()->getUserRoles()
	 */
	public function getUserRoles($username, $authSource = null, $recalculate = false) {
		return Hpls_Auth::i()->getUserRoles($username, $authSource, $recalculate);
	}

	public function isAllowedAclQuery($aclQuery, $role = null) {
		if (!is_array($aclQuery)) {
			$aclQuery = Hpls_Acl::splitResourceNeeds($aclQuery);
		}
		
		if (is_array($aclQuery)) {
			$ret = true;
			foreach ($aclQuery as $resID => $res) {
				$privileges = null;
				if (is_array($res)) {
					if (isset($res['privileges'])) {
						$privileges = $res['privileges'];
					}
					
					$resID = $this->getFirstExistingResource($resID);
					isDebug('acl') && $this->log()->debug(__METHOD__ . "(id: " . print_r($resID, TRUE) . ",priv: " . print_r($privileges, TRUE) . ",u:	" . print_r($role, TRUE) . ")");
					
					$ret = $ret && $this->isAllowed($resID, $privileges, $role);
				} else {
					require_once 'Hpls/Exception/Acl.php';
					throw new Hpls_Exception_Acl();
				}
				if ($ret === false) { // shortcut: 1x false => alles false
					isDebug('acl') && $this->log()->debug(__METHOD__ . ": ACL-Request: " . print_r($resID, true) . " #" . Hpls_Debug::print_r($privileges) . ", *$role >> denied");
					return false;
				}
			}
			isDebug('acl') && $this->log()->debug(__METHOD__ . ": ACL-Request: $aclQuery, *$role >> " . ($ret ? "allowed" : "denied"));
			return $ret;
		}
		return false;
	}

	/**
	 * Prüft, ob die definierten ACLs erlaub, dass $resOperation auf $resourceId
	 * von $role ausgeführt werden dürfen.
	 *
	 * @todo comment
	 *      
	 * @param $resourceId string|array       	
	 * @param $resOperation string|array|NULL
	 * @param $role string|NULL       	
	 * @return bool
	 */
	public function isAllowed($resourceId, $resOperation = null, $role = null) {
		
		isDebug('acl') && $this->log()->debug(__METHOD__ . "  (id: " . Hpls_Debug::print_r($resourceId) . ",op: " . Hpls_Debug::print_r($resOperation) . ",u: " . Hpls_Debug::print_r($role) . ")");
		$ret = true;
		if (is_null($role)) {
			$role = Hpls_Auth::i()->getMainRole();
		}
		
		// mehrere resourcen?
		if (is_array($resourceId)) {
			$ret = true;
			foreach ($resourceId as $res) {
				if (is_array($res)) {
					require_once 'Hpls/Exception/Acl.php';
					throw new Exception('Use isAllowedAclQuery instead!');
				} else {
					$res = $this->getFirstExistingResource($res);
					$ret = $ret && $this->isAllowed($res, $resOperation, $role);
				}
				if ($ret === false) { // shortcut: 1x false => alles false
					isDebug('acl') && $this->log()->debug(__METHOD__ . ": ACL-Request: " . print_r($resourceId, true) . " #$resOperation, *$role >> denied");
					return false;
				}
			}
			isDebug('acl') && $this->log()->debug(__METHOD__ . ": ACL-Request: " . print_r($resourceId, true) . " #$resOperation, *$role >> " . ($ret ? "allowed" : "denied"));
			return $ret;
		}
		
		// single resource
		if ($this->hasResource($resourceId)) {
			$ret = true;
			if (!is_array($resOperation)) {
				$resOperation = array(
						$resOperation );
			}
			// TODO an dieser Stelle das Privileg 'licensed' als required
			// einfügen?
			$acl = $this->_data->ACL;
			foreach ($this->_NS as $namespace) {
				foreach ($resOperation as $priv) {
					$ret = $ret && $acl->isAllowed($role, $resourceId, $namespace . $priv);
					isDebug('acl') && $this->log()->debug(__METHOD__ . ": role: $role -> res: $resourceId op: $namespace.$priv >> " . ($acl->isAllowed($role, $resourceId, $namespace . $priv) ? "allowed" : "denied"));
					if ($ret === FALSE) // shortcut: one FALSE => everything FALSE =>
					                    // no more tests needed
						return FALSE;
				}
			}
			
			isDebug('acl') && $this->log()->debug(__METHOD__ . ": role: $role -> res: $resourceId op: $resOperation >> " . ($ret ? "allowed" : "denied"));
			return $ret;
		} else {
			return false;
		}
		// $keypath = $this->getResPath();
		// $ret = $this->isAllowedRecursive($keypath);
		
		return $ret;
	}

	public function isAllowedAction($module, $controller = null, $action = null, $params = null, $role = null) {
		isDebug('acl', 'acl_allow') && $this->log()->debug(__METHOD__ . "($module,$controller,$action) START");
		// $reqRes = $this->getRequiredResources($module, $controller, $action);
		$reqRoles = $this->getRequiredRoles($module, $controller, $action, $params);
		// $roles = null;
		// if (empty($role)) {
		// $roles = Hpls_Auth::i()->getUserRoles();
		// } else {
		// $roles = Hpls_Auth::i()->getUserRoles($role);
		// }
		
		// foreach ($reqRes as $res) {
		// $this->isAllowed($res, null, $roles);
		// }
		$ret = Hpls_Auth::i()->hasRole($reqRoles);
		isDebug('acl', 'acl_allow') && $this->log()->debug(__METHOD__ . ' required: ' . implode(',', $reqRoles) /*. '; have roles: ' . implode(',', $roles)*/ . ' RESULT: ' . ($ret ? 'true' : 'false'));
		return $ret;
	}

	/**
	 *
	 * @todo comment
	 *      
	 * @param $resourceID string       	
	 * @return boolean
	 */
	public function hasResource($resourceID) {
		return $this->_data->ACL->has($resourceID);
	}

	/**
	 *
	 * @todo comment
	 *      
	 * @param $role string|array
	 *       	 Wenn array, dann wird geprüft, ob der aktuelle Nutzer EINE
	 *       	 dieser Rolen ausfüllt und dann true zurück gegeben.
	 * @return boolean
	 * @deprecated Use Hpls_Auth::i()->hasRole()
	 */
	public function hasRole($requiredRoles, $user = null, $oneOf = true) {
		if (empty($requiredRoles)) {
			// keine Rolle benötigt -> dann hat der user auf jedenfall die
			// bedingung erfüllt
			isDebug('acl_allow') && $this->log()->debug(__METHOD__ . " NO ROLES required -> allowed");
			return true;
		}
		if ($requiredRoles instanceof Hpls_Var_Clause) {
			// hier mit AND, OR, NOT arbeiten
		} elseif (!is_array($requiredRoles)) {
			$requiredRoles = explode(',', $requiredRoles);
		}
		
		// if (is_null($username)) {
		// $username = $this->getCurrentUserRoles();
		// }
		// preparation
		$ownedRoles = null;
		if (is_null($user)) {
			$user = Hpls_Auth::i()->getUsername();
		}
		
		if (!empty($user) && $user == Hpls_Auth::i()->getUsername()) {
			$ownedRoles = array(
					'usr_' . $user );
		} else {
			$ownedRoles = Hpls_Auth::i()->getUserRoles($user);
		}
		
		isDebug('acl_allow') && $this->log()->debug(__METHOD__ . " " . Hpls_Debug::print_r($requiredRoles, 'REQUIRED ROLES') . Hpls_Debug::print_r($ownedRoles, 'OWNED ROLES'));
		
		// hat Nutzer keine Rollen, so kann er auch nichts erfüllen
		if (empty($ownedRoles)) {
			return false;
		}
		
		// eigentliche überprüfung
		foreach ($requiredRoles as $requiredSingleRole) {
			foreach ($ownedRoles as $ownRole) {
				isDebug('acl_allow') && $this->log()->debug(__METHOD__ . " MATCHING for: $requiredSingleRole, AGAINST $ownRole");
				if ($oneOf) {
					if ($this->_data->ACL->inheritsRole($ownRole, $requiredSingleRole)) {
						isDebug('acl_allow') && $this->log()->debug(__METHOD__ . " ONEOF matched: required: $requiredSingleRole, own: $ownRole");
						return true;
					}
				} else {
					// alle sind nötig
					if (!$this->_data->ACL->inheritsRole($ownRole, $requiredSingleRole)) {
						isDebug('acl_allow') && $this->log()->debug(__METHOD__ . " ALLOF 1 unmatched: required: $requiredSingleRole, own: $ownRole");
						return false;
					}
				}
			}
		}
		return false;
	}

	public function getRootResource() {
		$this->_data->_rootResource;
	}

	/**
	 *
	 * @todo comment
	 * @param $module string       	
	 * @param $controller string       	
	 * @param $action string       	
	 * @param $params string       	
	 * @return multitype:string
	 */
	public function getRequiredResources($module, $controller, $action, $params = array()) {
		isDebug('acl') && $this->log()->debug(__METHOD__ . " START");
		$ret = array(
				'hmt' );
		$map = $this->_data->moduleMapping;
		
		foreach (array(
				"$module.$controller.$action",
				"$module.$controller",
				"$module" ) as $k) {
			if (isset($map[$k])) {
				$m = $map[$k];
				if (isset($m[self::KEY_RESOURCE])) {
					$ret = array_merge($ret, $m[self::KEY_RESOURCE]->getKeys());
				}
				if (!empty($ret)) {
					$ret = array_unique($ret);
					isDebug('acl') && $this->log()->debug(__METHOD__ . "($module,$controller,$action) END: " . Hpls_Debug::print_r($ret));
					return $ret;
				}
			
			}
		}
		
		if (isset($map[self::KEY_RESOURCE])) {
			array_merge($ret, $map[self::KEY_RESOURCE]->getKeys());
		}
		
		$ret = array_unique($ret);
		isDebug('acl') && $this->log()->debug(__METHOD__ . "($module,$controller,$action) END: " . Hpls_Debug::print_r($ret));
		return $ret;
	}

	/**
	 *
	 * @todo comment
	 * @param $module string       	
	 * @param $controller string       	
	 * @param $action string       	
	 * @param $params array:string       	
	 * @param $defaultRole string       	
	 * @return multitype:string
	 */
	public function getRequiredRoles($module, $controller = null, $action = null, $params = array(), array $defaultRole = array('admin')) {
		isDebug('acl', 'acl_allow') && $this->log()->debug(__METHOD__ . "($module,$controller,$action) START");
		if ($module == 'auth' && in_array($controller, array(
				'login',
				'logout' ))) {
			return array(
					'guest' );
		}
		if (empty($module)) {
			$module = Zend_Controller_Front::getInstance()->getDefaultModule();
		}
		if (empty($controller)) {
			$controller = Zend_Controller_Front::getInstance()->getDefaultControllerName();
		}
		if (empty($action)) {
			$action = Zend_Controller_Front::getInstance()->getDefaultAction();
		}
		$ret = array();
		$map = $this->_data->moduleMapping;
		isDebug('acl') && $this->log()->debug(Hpls_Debug::print_r($map, __METHOD__ . ' MAPPING '));
		// von action zu controller zu module
		// sobald eine rolle gefunden -> return, da spezifistische Beschreibung
		// alles andere überschreibt.
		foreach (array(
				"$module.$controller.$action",
				"$module.$controller",
				"$module" ) as $k) {
			if (isset($map[$k])) {
				$m = $map[$k];
				if (isset($m[self::KEY_ROLES])) {
					$ret = array_merge($ret, $m[self::KEY_ROLES]->getKeys());
				}
				if (!empty($ret)) {
					$ret = array_unique($ret);
					isDebug('acl', 'acl_allow') && $this->log()->debug(__METHOD__ . "($module,$controller,$action) END: " . Hpls_Debug::print_r($ret));
					return $ret;
				}
			}
		}
		
		if (isset($map[self::KEY_ROLES])) {
			array_merge($ret, $map[self::KEY_ROLES]->getKeys());
		}
		$ret = array_unique(count($ret) == 0 ? $defaultRole : $ret);
		isDebug('acl', 'acl_allow') && $this->log()->debug(__METHOD__ . "($module,$controller,$action) END: " . Hpls_Debug::print_r($ret));
		return $ret;
	}

	/**
	 *
	 * @param $action string       	
	 * @param $controller string       	
	 * @param $module string       	
	 *
	 * @return null array:string
	 */
	// public function mapControllerAcl($action, $controller, $module) {
	//
	// }
	
	/**
	 *
	 * @param $resourceKey string       	
	 * @return array
	 */
	public function getDetails($resourceId) {
		if ($this->_data->ACL->has($resourceId)) {
			$resObj = $this->_data->ACL->get($resourceId);
			return $resObj->getValues();
		}
		return null;
	}

	public function setModuleMapping($map) {
		
		if ($map instanceof Zend_Config) {
			$map = $map->toArray();
		}
		// isDebug('acl','acl_read') &&
		// $this->log()->debug(Hpls_Debug::print_r($map, __METHOD__ . "
		// ResMappings"));
		$this->_data->moduleMapping = array();
		if (isset($map[self::KEY_RESOURCE])) {
			if (!($map[self::KEY_RESOURCE] instanceof Hpls_Var_Clause)) {
				throw new InvalidArgumentException("'__resources__' must be of type Hpls_Var_Clause.");
			}
			$this->_data->moduleMapping[self::KEY_RESOURCE] = $map[self::KEY_RESOURCE];
		}
		if (isset($map[self::KEY_ROLES])) {
			if (!($map[self::KEY_ROLES] instanceof Hpls_Var_Clause)) {
				throw new InvalidArgumentException("'__roles__' must be of type Hpls_Var_Clause.");
			}
			$this->_data->moduleMapping[self::KEY_ROLES] = $map[self::KEY_ROLES];
		}
		
		foreach ($map as $module => $mCfg) {
			$this->_data->moduleMapping["$module"] = array();
			if (isset($mCfg[self::KEY_RESOURCE])) {
				if (!($mCfg[self::KEY_RESOURCE] instanceof Hpls_Var_Clause)) {
					throw new InvalidArgumentException("'$module.__resources__' must be of type Hpls_Var_Clause.");
				}
				$this->_data->moduleMapping["$module"][self::KEY_RESOURCE] = $mCfg[self::KEY_RESOURCE];
			}
			if (isset($mCfg[self::KEY_ROLES])) {
				if (!($mCfg[self::KEY_ROLES] instanceof Hpls_Var_Clause)) {
					throw new InvalidArgumentException("'$module.__roles__' must be of type Hpls_Var_Clause.");
				}
				$this->_data->moduleMapping["$module"][self::KEY_ROLES] = $mCfg[self::KEY_ROLES];
			}
			foreach ($mCfg as $controller => $cCfg) {
				if (isset($cCfg[self::KEY_RESOURCE])) {
					if (!($cCfg[self::KEY_RESOURCE] instanceof Hpls_Var_Clause)) {
						throw new InvalidArgumentException("'$module.$controller.__resources__' must be of type Hpls_Var_Clause.");
					}
					$this->_data->moduleMapping["$module.$controller"] = $cCfg[self::KEY_RESOURCE];
				}
				if (isset($cCfg[self::KEY_ROLES])) {
					if (!($cCfg[self::KEY_ROLES] instanceof Hpls_Var_Clause)) {
						throw new InvalidArgumentException("'$module.$controller.__roles__' must be of type Hpls_Var_Clause.");
					}
					$this->_data->moduleMapping["$module.$controller"][self::KEY_ROLES] = $cCfg[self::KEY_ROLES];
				}
				foreach ($cCfg as $action => $aCfg) {
					if (isset($aCfg[self::KEY_RESOURCE])) {
						if (!($aCfg[self::KEY_RESOURCE] instanceof Hpls_Var_Clause)) {
							throw new InvalidArgumentException("'$module.$controller.$action.__resources__' must be of type Hpls_Var_Clause.");
						}
						
						$this->_data->moduleMapping["$module.$controller.$action"] = $aCfg[self::KEY_RESOURCE];
					}
					if (isset($aCfg[self::KEY_ROLES])) {
						if (!($aCfg[self::KEY_ROLES] instanceof Hpls_Var_Clause)) {
							throw new InvalidArgumentException("'$module.$controller.$action.__roles__' must be of type Hpls_Var_Clause.");
						}
						$this->_data->moduleMapping["$module.$controller.$action"][self::KEY_ROLES] = $aCfg[self::KEY_ROLES];
					}
				}
			}
		}
		isDebug('acl', 'acl_read') && $this->log()->debug(Hpls_Debug::print_r($this->_data->moduleMapping, __METHOD__ . " ResMappings"));
	} // function
	
	public function rereadACL() {
		$cache = Hpls_Utils::getCache();
		$cache->clean(Zend_Cache::CLEANING_MODE_MATCHING_TAG, array( Hpls_Consts0::CACHE_TAG_ACL));
		$this->init();
	}
	
	
	/**
	 *
	 * @todo comment
	 *      
	 * @param $resID unknown_type       	
	 * @return string
	 */
	protected function getFirstExistingResource($resID) {
		while (!$this->hasResource($resID)) {
			// Hpls_Utils::log()->debug(__METHOD__."(id:
			// ".print_r($resID,TRUE).")");
			$resID = Hpls_Acl::getParentResource($resID);
		}
		return $resID;
	}

	/**
	 * Enter description here .
	 *
	 *
	 *
	 * ..
	 *
	 * @return boolean
	 * @deprecated nur solange verwenden, bis es tatsächlich einen
	 *             Superadmin-Login gibt.
	 * @todo Workaround ersetzen
	 */
	protected function isSuperadminMode() {
		if (is_null($this->_superadminMode)) {
			$this->_superadminMode = false;
			// nur an, wenn...
			if (Zend_Registry::isRegistered(Hpls_Consts0::REGK_MAINTENANCE)) {
				$maintCfg = Zend_Registry::get(Hpls_Consts0::REGK_MAINTENANCE);
				if (isset($maintCfg['maintenance_on'])) {
					$this->_superadminMode = Hpls_Utils::toBool($maintCfg['maintenance_on']);
				}
			}
		}
		
		return $this->_superadminMode;
	}

	protected function getResPath($resourceId) {
		return explode(Hpls_Acl_Consts::ID_SEPARATOR, $resourceId);
	}

	protected function getRawData($resourceId) {
	
	}

	protected function saveCache() {
		// isDebug('auth') && $this->log()->debug(__METHOD__ . ": " .
		// Hpls_Debug::print_r($this->_data, 'ACLs saving to cache'));
		$cache = Hpls_Utils::getCache(null, null);
		$cache->save($this->_data, 'ac_acl_data', array(
				Hpls_Consts0::CACHE_TAG_ACL ));
	}

	/**
	 *
	 * @todo Zend_Log
	 */
	protected function log() {
		return Hpls_Utils::log();
	}

}