<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 * 
 * created   : May 26, 2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2012-01-04 16:29:57 +0100 (Mi, 04 Jan 2012) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9198 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Acl/Licence/Reader.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/**
 */
require_once 'Hpls/Acl/Reader/Interface.php';
/**
 *
 * @todo comment
 *      
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *           
 * @package AC
 * @subpackage Rechtesystem
 * @version $LastChangedRevision: 9198 $
 * @since 1.0ufz
 */
abstract class Hpls_Acl_Licence_Reader implements 
	Hpls_Acl_Reader_Interface {
	
	const OLD_EXTERNAL = 'EXTERNAL';
	
	/**
	 *
	 * @var Zend_Acl
	 */
	protected $_ACL = null;
	protected $_source = null;
	
	/**
	 *
	 * @var unknown_type
	 * @deprecated just a workaround till the licence takes care of new value
	 *             scheme
	 */
	protected static $licResMap = array(
			'customer' => 'ac.product#customer',
			'customerid' => 'ac.product#customerid',
			'contact' => 'ac.product#contact',
			'product' => 'ac.product#product',
			'cpu' => 'ac.hardware.cpu#max_count',
			'mem' => 'ac.hardware.mem#max_size',
			'nic' => 'ac.hardware.nic#max_count',
			'emails' => 'hma.functions.archive.revision.mail#max_count',
			'backup' => 'hma.functions.archive.backup.mail#max_ratio',
			'sql' => 'hma.services.db.extern#enabled', // extern/intern
			'auth' => 'ac.functions.auth.extern#enabled',
			'expiration' => 'ac.product#expiration',
			'features' => 'ac.product#features' );

	public function setTarget(Zend_Acl $ACL) {
		$this->_ACL = $ACL;
	}

	public function setSource($source) {
		$this->_source = $source;
	}

	public function loadResources(Hpls_Acl $ACL) {
		// nothing to do in license with this
	}

	public function loadRoles(Hpls_Acl $ACL) {	}

	/**
	 *
	 * @param $ACL Zend_Acl       	
	 * @param $oldKeys array       	
	 * @deprecated only till the new licence builder is avaiable
	 */
	protected function handleOld(array &$oldKeys) {
		foreach (self::$licResMap as $k => $v) {
			if (isset($oldKeys[$k])) {
				if (isDebug('acl', 'acl_read'))
					Hpls_Utils::log()->debug("found: $k use $v");
				$oldVal = $oldKeys[$k];
				if (strcasecmp(self::OLD_EXTERNAL, $oldVal) === 0 && ($k === 'sql' || $k === 'auth')) {
					$oldVal = true;
				}
				$oldKeys[$v] = $oldVal;
				unset($oldKeys[$k]);
			}
		}
		return $oldKeys;
	}

	protected function apply(array $licenceVal) {
		// if (DEBUG) Hpls_Utils::log()->debug("Old values:
		// ".print_r($licenceVal,true));
		$licenceVal = $this->handleOld($licenceVal);
		// if (DEBUG) Hpls_Utils::log()->debug("New values:
		// ".print_r($licenceVal,true));
		foreach ($licenceVal as $k => $v) {
			$split = explode(Hpls_Acl_Consts::ID_VAL_SEPARATOR, $k);
			if (count($split) !== 2) {
				Hpls_Utils::log()->warn("Licence: Key '" . $k . "' is not in a valid format.");
			}
			if ($this->_ACL->has($split[0])) {
				/*
				 * @var $resObj Hpls_Acl_Resource
				 */
				$resObj = $this->_ACL->get($split[0]);
				$resObj->setValue($split[1], $v);
				
				$this->applyPrivilege($resObj, 'licensed', true);
				$this->applyPrivilege($resObj, $split[1], $v);
				
				// Hpls_Utils::log()->debug("resObj: $resObj");
			} else {
				Hpls_Utils::log()->warn("Resource '" . $split[0] . "' not defined, but the licence has a value for it.");
			}
		}
	}

	protected function applyPrivilege($resObj, $privilege, $value) {
		// Hpls_Utils::log()->debug(__METHOD__ . " $privilege: $value");
		$resObj->setValue($privilege, $value);
		if (Hpls_Acl::isBoolPrivilege($privilege)) {
			$roles = $this->getRolesFor($resObj, $privilege, $value);
			foreach ($roles as $role) {
				isDebug('acl', 'acl_read') &&					Hpls_Utils::log()->debug(__METHOD__ . "$role: $privilege: $value");
				if ($this->isAllowRule($resObj, $privilege, $value)) {
					$this->_ACL->allow($role, $resObj, $this->getPrivilegeForRole($role, $privilege, $value));
				} else {
					$this->_ACL->deny($role, $resObj, $this->getPrivilegeForRole($role, $privilege, $value));
				}
			} // foreach
		} elseif (Hpls_Acl::isIntPrivilege($privilege)) {
		
		}
		
		// $this->_ACL->
	}

	/**
	 *
	 * @todo implement
	 *      
	 * @param $resObj unknown_type       	
	 * @param $privilege unknown_type       	
	 * @param $value unknown_type       	
	 */
	protected function isAllowRule($resObj, $privilege, $value) {
		if (Hpls_Acl::isBoolPrivilege($privilege)) {
			return Hpls_Utils::toBool($value);
		}
		return true;
	}

	protected function getPrivilegeForRole($role, $privilege, $value) {
		if (strncmp(Hpls_Acl_Consts::MULTI_ROLE_MARKER, $value, strlen(Hpls_Acl_Consts::MULTI_ROLE_MARKER)) === 0) {
			// @todo implement
		} else if ($privilege === 'enabled') {
			return null; // for allprivileges
		} else {
			return $privilege;
		}
	}

	/**
	 *
	 * @todo comment
	 * @todo implement
	 *      
	 * @param $resObj unknown_type       	
	 * @param $privilege unknown_type       	
	 * @param $value unknown_type       	
	 * @return multitype:string
	 */
	protected function getRolesFor($resObj, $privilege, $value) {
		return array(
				null );
	}

	/**
	 *
	 * @todo Zend_Log
	 */
	protected function log() {
		return Hpls_Utils::log();
	}

}