<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : Apr 20, 2010
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28 Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.libs/Hpls/Application/Config/Frontcontrollerplugins.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 */
/***/
require_once 'Hpls/Consts.php';
require_once 'Hpls/Application/ConfigAbstract.php';
require_once 'Zend/Config/Ini.php';

/**
 * @todo comment
 *
 * Initialisiert Frontcontroller-Plugins anhand des verlangten Outputtypes.
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package HPLS
 * @subpackage subpackage
 * @version $LastChangedRevision: 9196 $
 * @since alpha.1
 *
 */
class Hpls_Application_Config_Frontcontrollerplugins extends Hpls_Application_ConfigAbstract {
	protected $_dontInclude = array();

	public function init() {
		isDebug('config_plugins_init') && $this->log()->info(__METHOD__);
		
		$front = $this->getFrontController();
		$ctype = $front->getParam(Hpls_Consts::FRONTP_OUTTYPE);
		
		// Register Plugins
		if (isDebug()) {
			$this->doRegister('Hpls/Controller/Plugin/Debug.php', 'Hpls_Controller_Plugin_Debug');
		}
		//		try {
		

		//			// Authentifizierung-Plugin (Admin oder Mailaccount-Benutzer)
		if ($this->getApplication()->isStandAlone()) {
			$this->doRegister('Hpls/Controller/Plugin/Authentication.php', 'Hpls_Controller_Plugin_Authentication');
		} else {
			$this->doRegister('Hpls/Controller/Plugin/AuthenticationMailapp.php', 'Hpls_Controller_Plugin_AuthenticationMailapp');
		}
		
		//veraltete persistente Nachrichten ablöschen 
		$this->doRegister('Hpls/Controller/Plugin/Messages.php', 'Hpls_Controller_Plugin_Messages');
		
		$this->doRegister('Hpls/Controller/Plugin/Maintenance.php', 'Hpls_Controller_Plugin_Maintenance');
		
		//in ValidationUnification-Configurer wird bestimmt, ob die folgenden beiden Plugins wirklich inkludiert wird.
		//stehen in $_dontInclude
		$this->doRegister('Hpls/Controller/Plugin/Constraints.php', 'Hpls_Controller_Plugin_Constraints');
		
		//			// XML Filterkette
		if ($ctype == Hpls_Consts::FRONTP_OUTTYPE_XML) {
			$this->registerForXML($front);
		
		//				
		} elseif ($ctype == Hpls_Consts::FRONTP_OUTTYPE_JSON) {
			$this->registerForJSON($front);
		} elseif ($ctype == Hpls_Consts::FRONTP_OUTTYPE_PNG) {
			// PNG Filterkette
		//				$png = new Mailapp_Plugin_PngSerializer();
		//				$front->registerPlugin($png);
		

		} elseif ($ctype == Hpls_Consts::FRONTP_OUTTYPE_PDF) {
			// PDF Filterkette
			$this->registerForPDF($front);
		} else {
			// HTML Filterkette
			$this->registerForHTML($front);
		}
		//
		//			// Logging
		//			$logging = new Mailapp_Plugin_Logging();
		//			$front->registerPlugin($logging);
		//
		//			// Cleanup
		$this->doRegister('Hpls/Controller/Plugin/Cleanup.php', 'Hpls_Controller_Plugin_Cleanup');
	
		//		} catch (Mailapp_Exception $e) {
	//			print('Fehler bei Anwendungs-Start<br/>');
	//		}
	

	}

	public function setDontInclude($pluginClassName) {
		isDebug('config_plugins_init', 'method_trace') && $this->log()->debug(__METHOD__ . " START");
		$this->_dontInclude[$pluginClassName] = $pluginClassName;
	}

	protected function registerForXML($front) {
		//				$xml = new Mailapp_Plugin_XmlSerializer();
	//				$front->registerPlugin($xml);
	//		$this->doRegister('Hpls/Controller/Plugin/Serializer/Json.php', 'Hpls_Controller_Plugin_Serializer_Json');
	}

	protected function registerForJSON($front) {
		$this->doRegister('Hpls/Controller/Plugin/Serializer/Json.php', 'Hpls_Controller_Plugin_Serializer_Json');
		$this->getApplicationConfigurer('Hpls_Application_Config_HtmlView')->setDontRun(true);
	}

	protected function registerForHTML($front) {
		//		$this->doRegister('Hpls/Controller/Plugin/Serializer/Html.php','Hpls_Controller_Plugin_Serializer_Html');
		if (Hpls_Application::i()->isStandAlone()) {
			$this->doRegister('Hpls/Controller/Plugin/Mailtraceappliance.php', 'Hpls_Controller_Plugin_Mailtraceappliance');
		}

		$this->doRegister('Hpls/Controller/Plugin/Navigation.php', 'Hpls_Controller_Plugin_Navigation');
	
		//		$this->doRegister('Hpls/Controller/Plugin/I18n.php','Hpls_Controller_Plugin_I18n');	
	}

	protected function registerForPDF($front) {
		//				$pdf = new Mailapp_Plugin_PdfSerializer();
	//				$front->registerPlugin($pdf);
	//				
	

	}

	protected function doRegister($includeFile, $className) {
		if (in_array($className, $this->_dontInclude)) {
			//do nothing if plugin is not wanted
			return;
		}
		require_once $includeFile;
		$this->getFrontController()->registerPlugin(new $className());
	}
}