<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 20.09.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2012-02-21 15:00:14 +0100 (Di, 21 Feb 2012) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9222 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Application0.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/**
 */
require_once 'Hpls/Consts0.php';
require_once 'Hpls/Err.php';
require_once 'Hpls/User/Object.php';
require_once 'Hpls/Acl.php';
require_once 'Hpls/Utils.php';
require_once 'Hpls/Auth.php';

/**
 *
 * @todo comment
 *      
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *           
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 9222 $
 * @since 1.0.9
 */
abstract class Hpls_Application0 {
	protected static $_instance = null;

	/**
	 * Instanziiert die Singleton-Instance einer Hpls_Application.
	 *
	 * Muss als allererstes vor jeder anderen Methode an der Klasse aufgerufen
	 * werden.
	 *
	 * Jede ableitende Klass muss diese Method implementieren und darin
	 * self::$_instance eine Instanz der eigenen Klasse zuweisen.
	 */
	public abstract static function initApplicationObject();

	/**
	 *
	 * @return Hpls_Application0
	 */
	public static function i() {
		return self::$_instance;
	}

	/**
	 *
	 * @todo comment
	 *      
	 * @param $product string       	
	 * @return Zend_Session_Namespace
	 */
	public function getSession($product) {
		if (empty($product) || !is_string($product)) {
			require_once 'Hpls/Exception.php';
			throw new Hpls_Exception("A productname (string) must be given to get a session.");
		}
		
		return Zend_Session::sessionExists() ? new Zend_Session_Namespace($product) : null;
	}

	public function getSessionRefId() {
		if (!Zend_Session::sessionExists()) {
			return null;
		}
		if (Zend_Session::isStarted()) {
			$sess = new Zend_Session_Namespace('hpls_session');
			if (!isset($sess->session_ref_id)) {
				$sess->session_ref_id = uniqid('hpls_sess');
			}
			
			return $sess->session_ref_id;
		}
		return null;
	}

	/**
	 * Gibt die Sprache zurück, die als Default für diese Applikation in der
	 * application.ini eingestellt wurde.
	 *
	 * @return string
	 */
	public function getDefaultLocale() {
		return 'de_DE';
	}

	/**
	 * Gibt die Sprache zurück, die für die aktuelle Sitzung (session/Nutzer
	 * Anmeldung) eingestellt wurde.
	 *
	 * @return string
	 */
	public function getCurrentLocale() {
		return 'de_DE';
	}

	/**
	 * Führt die Initialisierung der Applikation durch und ruft dabei alle
	 * Teil-init...() Methoden in der benötigten Reihenfolge auf.
	 * <p>Die init...()-Methoden müssen "händisch" hier aufgerufen werden. Am
	 * Ende muss ein
	 * <pre>
	 * $this->_initDone = true;
	 * </pre>
	 * stehen.
	 * ACHTUNG: nur in unbedingten Notfällen überschreiben, ansonsten runInits()
	 * verwenden
	 *
	 * @return void
	 */
	public function init() {
		$this->initIncludePaths();
		$this->initAutoloader();
		$this->initApplicationConfigs();
		Hpls_Utils::getCache()->clean(Zend_Cache::CLEANING_MODE_OLD);
		$this->runInits();
		$this->log()->info(__CLASS__ . " Running in " . ($this->isStandAlone() ? ' STANDALONE mode' : 'APPLIANCE mode'));
	}

	/**
	 * Die eigentliche initialisierungsroutine für Ableitungen.
	 * Hier alle init*() Methoden in der gewünschten Reihenfolge aufrufen.
	 */
	abstract protected function runInits();

	abstract protected function initPaths();

	/**
	 *
	 * @todo comment
	 *       WICHTIG: hier noch kein logging-statements verwenden!
	 * @todo comment
	 */
	protected abstract function initIncludePaths();

	protected abstract function initFrontcontroller();

	protected abstract function initACL();

	protected abstract function initConfigs();

	/**
	 * Die aller erste Konfiguration, die gelesen wird: application.ini
	 * <ul>
	 * <li>Die Cache-Konfig muss hier initialisiert werden
	 * </ul>
	 * WICHTIG: hier noch kein logging-statements verwenden!
	 *
	 * @todo comment
	 */
	abstract protected function initApplicationConfigs();

	/**
	 *
	 * @return boolean
	 */
	abstract public function isStandAlone();

	/**
	 *
	 * @return Hpls_Acl
	 */
	public function ACL() {
		return Hpls_Acl::i();
	}

	abstract public function getVersion();

	/**
	 * Gibt die auf der Maschine aktuell installierten HPLS-Produkte als Array
	 * mit den Jira-Keys der Produkte zurück.
	 *
	 * @return array:string->string Jira-Projektkürzel der installierten
	 *         Produkte key: jira-key-lowercase, value => Verzeichnisname im
	 *         configs-dir
	 */
	abstract public function getInstalledHPLSProducts();

	/**
	 * Pfad zu ./frontend/configs
	 * @return string
	 */
	public function getAppsConfigPath() {
		return APP_CONFIGS_PATH;
	}

	abstract public function getConfigDir($product = null);

	/**
	 *
	 * @todo comment
	 * @param $product string
	 *       	 Wenn <null>, dann das Parent-Dir von allen Produkten.
	 */
	abstract public function getVariableConfigDir($product = null);

	/**
	 *
	 * @todo comment
	 * @return Zend_Log
	 */
	abstract protected function log();
}