<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 09.06.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/**
 */
require_once 'Hpls/Utils/Logusing.php';
require_once 'Hpls/User/Object.php';
require_once 'Zend/Auth.php';
require_once 'Hpls/Auth/Source.php';

function getCurrentUsername() {
	return Hpls_Auth::i()->getUsername();
}

/**
 *
 * @todo session+auth-nutzung implementieren
 *      
 * @param $roles array|string
 *       	 Required roles
 * @param $user string
 *       	 (optional)
 * @param $oneOf boolean
 *       	 (optional) <i>true</i>: eine der Rollen aus $roles genügt,
 *       	 <i>false</i> <b>alle</b> rollen aus $roles müssen erfüllt sein.
 * @return string boolean
 */
function hasRole($roles, $user = null, $oneOf = true) {
	return Hpls_Auth::i()->hasRole($roles, $oneOf, $user);
}

/**
 *
 * @todo comment
 *      
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *           
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision$
 * @since 1.0.9
 */
class Hpls_Auth extends Hpls_Utils_Logusing {
	protected static $_instance = null;

	/**
	 *
	 * @return Hpls_Auth
	 */
	public static function i() {
		if (is_null(self::$_instance)) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}
	
	/**
	 *
	 * @var Zend_Auth
	 */
	private $_auth = NULL;
	
	/**
	 *
	 * @var Zend_Auth_Result
	 */
	private $_authResult = null;
	private $_authCfgArray = null;
	protected $_readers = array();
	/**
	 *
	 * @todo comment
	 * @var array:Hpls_Auth_Source
	 */
	protected $_authSources = null;

	protected function __construct() {
		$this->_auth = Zend_Auth::getInstance();
		isDebug('auth', 'config_read') && $this->log()->debug(Hpls_Debug::print_r($this->getAuthConfig()->toArray(), __METHOD__));
	}

	public function getIdentity() {
		// if (is_array($this->_auth->getIdentity())){
			return $this->_auth->getIdentity();
		// } else {
		//	return $this->_auth->generateIdentity();
		// }
	}

	/**
	 *
	 * @todo comment
	 * @return bool
	 */
	public function hasIdentity() {
		return $this->_auth->hasIdentity();
	}

	public function getUserRoles($username = null, $authSource = null, $recalculate = false) {
		//$this->log()->debug(Hpls_Debug::print_r($this->_auth, __METHOD__));
		$ret = array();
		if (empty($username)) {			
			if (FALSE === $this->hasIdentity()) {
				return null;
			}
			$id = $this->getIdentity();

			if( isset($id['role']) && $id['role'] == 'admin'){
				$id = array(
					'source' => 'appliance',
					'username' => 'admin',
					'roles' => array(
						'admin' => 'admin',
						'loggedin' => 'loggedin',
						'usr_admin' => 'usr_admin'
							)
				);
			}

			if (is_array($id)){
				$ret = $id['roles'];
			}
			
			// guest ist immer dabei
			if (!in_array('guest', $ret)) {
				$ret['guest'] = 'guest';
			}
			// mailuser ist immer dabei
			if (!in_array('mailuser', $ret)) {
				$ret['mailuser'] = 'mailuser';
			}
			return $ret;
		} else {
			$ret = array();
			
			if ($recalculate || $username != $this->getUsername() || !Zend_Session::sessionExists() || !Zend_Session::namespaceIsset(Hpls_Consts0::SESNS_AUTH)) {
				// rollen des users zusammensuchen aus INIs und RoleSources
				// wenn username != currentuser muss auch neu kalkuliert werden
				$ret = $this->readRoles($username, $authSource);
				if (!in_array('guest', $ret)) {
					$ret['guest'] = 'guest';
				}
			} else {
				// Rollen aus der Sesssion auslesen, wenn sie schon existiert
				// und username == currentuser
				$sess = new Zend_Session_Namespace(Hpls_Consts0::SESNS_AUTH);
				
			}
			$this->log()->debug(Hpls_Debug::print_r($ret, __METHOD__));
			return $ret;
		
		}
	}

	public function hasRole($requiredRoles, $oneOf = true, $user = null) {
		return Hpls_Acl::i()->hasRole($requiredRoles, $user, $oneOf);
	}

	/**
	 *
	 * @todo comment
	 * @param $role string|Hpls_Acl_Role       	
	 * @param $parents array|string       	
	 */
	public function addUserRole($role, $parents = null) {
		Hpls_Acl::i()->addUserRole($role, $parents);
	}

	/**
	 *
	 * @todo comment
	 * @return Hpls_User_Object
	 */
	public function getUserObject() {
		$sess = Hpls_Application0::i()->getSession(Hpls_Consts0::SESSION_NS_AUTH);
		if (isset($sess->userObj)) {
			return $sess->userObj;
		}
		if ($this->isLoggedIn()) {
			if(is_dir("/opt/heinlein-elements/appliance/webfront/cache")){
				// / SPECIAL für Appliance-Integegration UserObj nochmal
				// initialisieren
				$id = $this->getIdentity();
				$id['source'] = 'appliance';
				$this->initUserSession($this->getUsername(), $id);
			
				// Userobj nochmal aus session neu laden
				$sess = Hpls_Application0::i()->getSession(Hpls_Consts0::SESSION_NS_AUTH);
				if (isset($sess->userObj))
					return $sess->userObj;
			}
		}
		return null;
	}

	/**
	 *
	 * @todo comment
	 * @return NULL string
	 */
	public function getUsername() {
		if (FALSE === $this->hasIdentity()) {
			return null;
		}
		$id = $this->getIdentity();
		if(is_array($id)){
			return $id['username'];
		} else {
			return $id;
		}
	}

	public function getAuthSource() {
		$id = $this->getIdentity();
		return empty($id) ? null : $id['source'];
	}

	public function isLoggedIn() {
		return $this->hasIdentity();
	}

	/**
	 *
	 * @todo comment
	 *       Überprüft nicht, ob der Login schon besteht.
	 * @param $username string       	
	 * @param $password string       	
	 * @throws Hpls_Exception_Configuration
	 * @throws Hpls_Exception_NotSupported
	 * @return Zend_Auth_Result boolean wenn kein Auth-Source definiert
	 */
	public function login($username, $password) {
		$auth = Zend_Auth::getInstance();
		$this->_adapter = array();
		$_authResult = false;
		foreach ($this->getActiveAuthSources() as $source) {
			isDebug('auth') && $this->log()->info(__METHOD__ . " trying auth source '" . $source->getName() . "'");
			
			/**
			 *
			 * @var Hpls_Auth_Adapter_Abstract
			 */
			$_adapter = $source->getAuthAdapter();
			$_adapter->setUsername($username);
			$_adapter->setPassword($password);
			isDebug('auth') && $this->log()->debug(__METHOD__ . ": trying to auth with USER: '$username', PASS: '$password'");
			
			/**
			 *
			 * @var Zend_Auth_Result
			 */
			$_authResult = $auth->authenticate($_adapter);
			if ($_authResult->isValid()) {
				// Userdaten initialisieren
				$sess = Hpls_Application0::i()->getSession(Hpls_Consts0::SESSION_NS_AUTH);
				$sess->unsetAll();
				$this->initUserSession($username, $_authResult->getIdentity());
				return $_authResult;
			}
			if (!$source->isContinue()) {
				return $_authResult;
			}
		} // foreach
		return $_authResult;
	}

	/**
	 * Löscht alle pro User relevanten allgemeinen Daten aus Session, Caches &
	 * Co
	 */
	public function logout() {
		$cache = Hpls_Utils::getCache();
		$cache->clean(Zend_Cache::CLEANING_MODE_MATCHING_TAG, array( Hpls_Consts0::CACHE_TAG_ACL));
		$cache->clean(Zend_Cache::CLEANING_MODE_MATCHING_TAG, array(Hpls_Consts0::CACHE_TAG_AUTH));
	
	}

	public function masquerade($asUsername) {
	
	}

	public function setMainRole($role) {
		$this->getUserObject()->setMainRole($role);
	}

	public function getMainRole() {
		$usr = $this->getUserObject();
		return is_null($usr) ? Hpls_Acl::ROLE_GUEST : $usr->getMainRole();
	}

	public function addAuthError($errorText, $code = -1, array $errorTextParams = array()) {
		Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'auth', $errorText, $code, $errorTextParams, true, true);
	}

	public function clearAuthError() {
		Hpls_Msgs::i()->clearGlobalMsgs('auth');
	}

	protected function readRoles($username, $source) {
		$this->log()->debug(__METHOD__ . "(usr: $username, src: $source)");
		if (empty($username)) {
			throw new InvalidArgumentException('Username has to be given.');
		}
		
		if (empty($source)) {
			throw new InvalidArgumentException('Authsource has to be given.');
		}
		if (!is_array($source)) {
			$source = explode(',', $source); // wenn kein ',' drin, ist es dennoch
				                                 // ein array der länge 1 ;)
		}
		$authSrcs = $this->getActiveAuthSources();
		$roles = array();
		foreach ($source as $src) {
			
			if (!isset($authSrcs[$src])) {
				require_once 'Hpls/Exception/Authentication.php';
				throw new Hpls_Exception_Authentication("Unknown or inactive auth source '$src'", Hpls_Consts0::ERR_AUTH_NOSOURCES);
			}
			$roles = array_merge($roles, $authSrcs[$src]->readRoles($username, array()));
		}
		
		// $roles = Hpls_Auth_Reader_Userdata::i()->readInRoles($usrObj);
		// "usr_$username" => "usr_$username" );
		// Hpls_Acl::i()->addUserRole("usr_$username");
		
		return $roles;
	}

	protected function getActiveAuthSources() {
		if (is_null($this->_authSources)) {
			require_once 'Hpls/Var/Matcher.php';
			$authXmlCfg = $this->getAuthConfig();
			if (!isset($authXmlCfg['auth'])) {
				require_once 'Hpls/Exception/Configuration.php';
				throw new Hpls_Exception_Configuration("There is no configuration section 'auth' in " . Hpls_Consts0::FILE_CONFIG_AUTH . "'");
			}
			
			$sources = isset($authXmlCfg['auth']['source']) ? $authXmlCfg['auth']['source'] : null;
			$loginmapping = isset($authXmlCfg['auth']['loginmapping']) ? $authXmlCfg['auth']['loginmapping'] : null;
			
			if (is_null($sources)) {
				require_once 'Hpls/Exception/Configuration.php';
				throw new Hpls_Exception_Configuration("There is no 'source' configuration in section 'auth' in " . Hpls_Consts0::FILE_CONFIG_AUTH . "'", Hpls_Consts0::ERR_AUTH_NOSOURCES);
			}
			if (!is_null($loginmapping)) {
				if (!is_array($loginmapping) || !isset($loginmapping['match'])) {
					require_once 'Hpls/Exception/Configuration.php';
					throw new Hpls_Exception_Configuration("Loginmapping in section 'auth' is not correctly specified in " . Hpls_Consts0::FILE_CONFIG_AUTH . "'", Hpls_Consts0::ERR_AUTH_LOGINMAPPING_NOTCORRECT);
				}
				/*
				 * <loginmapping match="/^([^@]+)@(.+)$/i"> <p1 value="USERNAME"
				 * /> <p2 value="DOMAIN" /> </loginmapping>
				 */
				$loginmapping = new Hpls_Var_Matcher(array(
						'match' => $loginmapping ));
			} else {
				$loginmapping = Hpls_Var_Matcher::createLoginMapper();
			}
			
			if (!is_array($sources)) {
				$sources = preg_split('/\s*,\s*/', $sources);
			}
			$this->_authSources = array();
			foreach ($sources as $source) {
				if (!isset($authXmlCfg[$source])) {
					require_once 'Hpls/Exception/Configuration.php';
					throw new Hpls_Exception_Configuration("There is no configuration section for source '$source' in '" . Hpls_Consts0::FILE_CONFIG_AUTH . "'");
				}
				$this->_authSources[$source] = new Hpls_Auth_Source($source, $authXmlCfg[$source], $loginmapping);
			}
		}
		return $this->_authSources;
	}

	/**
	 *
	 * @todo comment
	 *       Achtung: User muss schon gültig eingeloggt sein!
	 * @param $username unknown_type       	
	 * @param $source unknown_type       	
	 * @return Ambigous <NULL, string, multitype:>
	 */
	public function readRolesForLogingetUserRoles($username, $source) {
		$roles = $this->getUserRoles($username, $source, true);
		// Nutzerrole im Role-Tree setzen
		Hpls_Acl::i()->addUserRole('usr_' . $username, $roles);
		$roles['usr_' . $username] = 'usr_' . $username;
		return $roles;
	}

	protected function initUserSession($username, $identity) {
		// isDebug('auth') &&
		$this->log()->debug(__METHOD__ . "($username) START");
		
		$sess = Hpls_Application0::i()->getSession(Hpls_Consts0::SESSION_NS_AUTH);
		$userObj = new Hpls_User_Object();
		$userObj->setName($username);
		$roles = Hpls_Auth::i()->getUserRoles();
		
		$userObj->setRoles($roles);
		$userObj->setAuthedBy($identity['source']);
		
		try {
			$src = $this->getAuthSourceObj($identity['source']);
			if (!empty($src)) {
				$ret = $this->getAuthSourceObj($identity['source'])->readEmails($userObj);
				$userObj->addEmail($ret);
			}
		} catch (Hpls_Exception_NotSupported $ex) {
			$this->log()->err(__METHOD__ . $ex);
		}
		$sess->userObj = $userObj;
		// isDebug('auth') &&
		$this->log()->debug(__METHOD__ . "($username) END " . Hpls_Debug::print_r($userObj));
	}

	/**
	 *
	 * @todo comment
	 * @return Zend_Config
	 */
	protected function getAuthConfig() {
		if (is_null($this->_authCfgArray)) {
			$this->_authCfgArray = Zend_Registry::get(Hpls_Consts0::REGK_AUTH)->toArray();
		}
		return $this->_authCfgArray;
	}

	/**
	 *
	 * @todo comment
	 * @param $sourcename unknown_type       	
	 * @return null Hpls_Auth_Source
	 */
	protected function getAuthSourceObj($sourcename) {
		return isset($this->_authSources[$sourcename]) ? $this->_authSources[$sourcename] : null;
	}

}