<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 20.05.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Auth/Adapter/Abstract.php';
require_once 'Zend/Mail/Protocol/Imap.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision$
 * @since 1.0.9
 */
class Hpls_Auth_Adapter_Imap extends Hpls_Auth_Adapter_Abstract {
	
	const OPT_HOST = 'host';
	const OPT_PORT = 'port';
	const OPT_METHOD = 'method';
	const OPT_SSL = 'ssl';
	const OPT_CNCTSECURITY = 'connectsecurity';
	const OPT_AUTHMETHOD = 'authenticationmethod';
	protected static $DEFAULTS = array(
			self::OPT_AUTHMETHOD => 'none',
			self::OPT_CNCTSECURITY => 'encrypted',
			self::OPT_HOST => 'localhost',
			//			self::OPT_PORT => '143',
			self::OPT_SSL => 'no' );

	function setOptions(array $options) {
		parent::setOptions($options);
		$this->setDefaults(self::$DEFAULTS);
		//			imap2           143/tcp         imap            # Interim Mail Access P 2 and 4                                                                      
		//imap2           143/udp         imap
		//imap3           220/tcp                         # Interactive Mail Access
		//imap3           220/udp                         # Protocol v3
		//imaps           993/tcp                         # IMAP over SSL
		//imaps           993/udp
		

		switch (strtolower($this->_options[self::OPT_SSL])) {
			case 'on' :
			case 'ssl' :
			case '1' :
			case 'true' :
			case 'y' :
			case 'yes' :
			case 't' :
				//		    Secure IMAP (IMAP4-SSL) - port 585
				//    IMAP4 over SSL (IMAPS) - port 993
				$this->_options[self::OPT_SSL] = 'SSL';
				if (!isset($this->_options[self::OPT_PORT])) {
					$this->_options[self::OPT_PORT] = 993;
				}
				break;
			case 'tls' :
			case 'starttls' :
				if (!isset($this->_options[self::OPT_PORT])) {
					$this->_options[self::OPT_PORT] = 143;
				}
				$this->_options[self::OPT_SSL] = 'TLS';
				break;
			case 'off' :
			case '0' :
			case 'false' :
			case 'n' :
			case 'no' :
			case 'f' :
				if (!isset($this->_options[self::OPT_PORT])) {
					$this->_options[self::OPT_PORT] = 143;
				}
				$this->_options[self::OPT_SSL] = false;
				break;
			default :
				require_once 'Hpls/Exception/Configuration.php';
				throw new Hpls_Exception_Configuration('Option "/' . $this->getSourceName() . '/server/ssl" has an invalid value.');
		}
	}

	/* (non-PHPdoc)
	 * @see Zend/Auth/Adapter/Zend_Auth_Adapter_Interface#authenticate()
	 */
	function authenticate() {
		//resource imap_open  ( string $mailbox  , string $username  , string $password  [, int $options = NIL  [, int $n_retries = 0  ]] )
		//imap_close
		$this->log()->debug(Hpls_Debug::print_r($this->_options, __METHOD__ . ' OPTIONS '));
		$identity = null;
		$code = $this->validateLoginData();
		if ($code === Zend_Auth_Result::SUCCESS) {
			$roles = $this->getRoles();
			$identity = array(
					'source' => $this->_sourceName,
					'username' => $this->getUsername(),
					'roles' => $roles );
		} else {
			//failure in config data
		

		}
		return new Zend_Auth_Result($code, $identity);
	}

	protected function validateLoginData() {
		
		$host = $this->getOption(self::OPT_HOST);
		$password = ($this->getPassword() !== null) ? $this->getPassword() : '';
		$username = $this->getUsername();
		$port = intval($this->getOption(self::OPT_PORT));
		$ssl = $this->getOption(self::OPT_SSL);
		
		try {
			isDebug('auth') && $this->log()->debug(__METHOD__ . " LOGGIN IN with $ssl:$host:$port as " . $this->getUsername() . " PW: " . Hpls_Utils::crypt_hpls($password));
			
			$protocol = new Zend_Mail_Protocol_Imap();
			isDebug('auth') && $this->log()->debug(__METHOD__ . ' LOGIN created Zend_IMAP ');
			$protocol->connect($host, $port, $ssl);
			isDebug('auth') && $this->log()->debug(__METHOD__ . ' LOGIN connected ');
			if (!$protocol->login($this->getUsername(), $password)) {
				isDebug('auth') && $this->log()->debug(__METHOD__ . ' LOGIN INVALID ');
				return Zend_Auth_Result::FAILURE_CREDENTIAL_INVALID;
			} else {
				//succes
				isDebug('auth') && $this->log()->debug(__METHOD__ . ' LOGIN SUCCESS ');
				$protocol->logout();
				return Zend_Auth_Result::SUCCESS;
			}
		} catch (Zend_Mail_Protocol_Exception $ex) {
			$this->log()->err(__METHOD__ . ' LOGIN FAILED with Exception ' . $ex);
			return Zend_Auth_Result::FAILURE;
		}
	}

	protected function genMailboxString() {
	
	}

}