<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 26.07.2010
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28 Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Cli/Abstract.php $
 *
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Zend/Registry.php';
require_once 'Zend/Log.php';
require_once 'Zend/Config/Ini.php';
require_once 'Hpls/Consts.php';
require_once 'Hpls/Cli/Getopt.php';
require_once 'Hpls/Cli/Consts.php';
require_once 'Hpls/Utils.php';
require_once 'Hpls/Cli/Command/Abstract.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package AC
 * @subpackage CLI-UI
 * @version $LastChangedRevision: 9196 $
 * @since 1.0.0.5
 */
abstract class Hpls_Cli_Abstract implements 
	Hpls_Cli_Consts {
	
	protected static $DEFAULT_GLOBAL_ARGUMENTS = array(
			'p1' => array(
					'name' => "help",
					'optional' => true,
					'help' => "Getting help" ),
			'p1' => array(
					'name' => "config|c=1",
					'optional' => true,
					'help' => "Read config from file (ini)" ),
			'debug1' => array(
					'name' => 'debug',
					'optional' => true,
					'help' => "Enable debug log messages" ),
			'host1' => array(
					'name' => 'host|h=s',
					'optional' => true,
					'paramName' => 'IP-Address/Hostname of Host',
					'default' => 'localhost',
					'help' => "IP/Name of host to run command on" ),
			'verbose' => array(
					'name' => "verbose|v",
					'optional' => true,
					'help' => "Be verbose" ) );
	
	protected $_commandPath = array();
	protected $_cmdLine = array();
	protected $_options = array();
	
	/**
	 * @var Hpls_Cli_Getopt
	 */
	protected $_getOpt = NULL;
	/**
	 * <p>Zweite Funktion: Marker, ob parseCmdLine() schon mal ausgeführt wurde (!== NULL).
	 * @var string|NULL
	 */
	protected $_commandToRun = NULL;
	
	protected $_commandInstanceMap = array();
	protected $_commandFileMap = NULL;
	protected $_commandOptionsMap = array();
	
	protected $_hosts = array();
	
	/**
	 * @var Zend_Log
	 */
	protected $_logger = NULL;
	protected $_output = TRUE;
	protected $DEBUG = FALSE;
	protected $VERBOSE = FALSE;

	/**
	 * @param array $argv
	 * @param array $commandPath
	 * @param Zend_Log $logger
	 * @param boolean $output
	 */
	/**
	 * @param array $argv Die Kommandozeilenargument (z.B. $_SERVER['argv'])
	 * @param array $commandPath ClassPrefix => path
	 * @param array $options <dl>
	 * <dt>['logger']: Zend_Log</dt><dd>Zend_Log-Instanz in die geloggt werden soll.</dd>
	 * <dt>['echo']: boolean</dt><dd>Sollen Ausgaben nicht nur als Return-Wert zurückgeben werden sondern auch per 'echo' an den STDOUT geschickt werden?</dd>
	 * <dt>['throwExceptions']: boolean, default: false</dt><dd></dd>
	 * <dt>['dieOnError']: boolean, default: true</dt><dd>Wenn ein Fehler auftritt, Prozess beenden, statt run()-Methode normal zubeenden und Fehler in der Rückgabe mitschicken</dd>
	 * </dl>
	 *
	 */
	protected function __construct(array $argv, array $commandPath, array $options = array()) {
		if (is_null($commandPath) || (is_array($commandPath) && count($commandPath) === 0)) {
			require_once 'Hpls/Exception/Cli.php';
			throw new Hpls_Exception_Cli("Es muss mindestens ein Pfad zu Kommando-Implementierungen angegeben werden.");
		}
		$commandPath['Hpls_Cli_Command_'] = dirname(__FILE__) . '/Command';
		foreach ($commandPath as $prefix => $path) {
			if (!is_dir($path)) {
				require_once 'Hpls/Exception/Cli.php';
				throw new Hpls_Exception_Cli("Pfad '$commandPath' ist kein Verzeichnis.");
			}
			$this->_commandPath[$prefix] = realpath($path);
		}
		
		//filter --debug from command line and if set enable debug mode
		$debug = FALSE;
		$tmpArray = array();
		foreach ($argv as $arg) {
			if (strcmp('--debug', $arg) === 0) {
				$this->DEBUG = TRUE;
			
			} else if (strcmp('-v', $arg) === 0 || strcmp('--verbose', $arg) === 0) {
				$this->VERBOSE = TRUE;
			} else {
				$tmpArray[] = $arg;
			}
		}
		$argv = $tmpArray;
		unset($tmpArray);
		
		$this->_cmdLine = $argv;
		if (isset($options['echo'])) {
			$this->_output = $options['echo'];
		}
		
		$this->_commandOptionsMap[self::GLOBAL_OPTIONS_NAME] = array(
				Hpls_Cli_Command_Abstract::CFG_OPTIONS => self::$DEFAULT_GLOBAL_ARGUMENTS );
		
		//options abarbeiten und defaults setzen
		if (!isset($options['throwExceptions'])) {
			$options['throwExceptions'] = FALSE;
		}
		if (!isset($options['dieOnError'])) {
			$options['dieOnError'] = TRUE;
		}
		$this->_options = $options;
		
		//load command definitions
		$this->initAllCommandDefinition();
		$this->setCommandToRun();
		
		//cache dir für das template system setzen
		if (is_readable(Hpls_Consts::APP_BACKEND_CONFIG)) {
			$conf = new Zend_Config_Ini(Hpls_Consts::APP_BACKEND_CONFIG, 'CONF');
			Zend_Registry::set('cache_dir', $conf->cacheDir);
			$this->log()->debug("Cache dir: " . $conf->cacheDir);
		}
		
		$this->log()->info("CmdLine: " . implode(' ', $this->_cmdLine));
		$this->log()->debug("CommandPath: " . print_r($this->_commandPath, true));
	}

	public function setCommandToRun($commandName = NULL) {
		if (is_null($commandName)) {
			if (count($this->_cmdLine) < 2) {
				$this->_commandToRun = self::DEFAULT_COMMAND;
			} else {
				$this->_commandToRun = $this->_cmdLine[1];
			}
		} else {
			$this->_commandToRun = $commandName;
		}
		if ($this->_commandToRun == '--help') {
			$this->_commandToRun = 'help';
		}
		
		if ($this->isCommandAvaiable($this->_commandToRun) === FALSE) {
			throw new Hpls_Exception_Cli($this->_commandToRun . " ist kein gültiges Kommando.");
		}
	
	}

	/**
	 * @todo comment
	 *
	 * @return multitype:string
	 */
	public function getCmdLine() {
		return $this->_cmdLine;
	}

	/**
	 * @todo comment
	 *
	 * @return multitype:string
	 */
	function getAvaiableCommands() {
		return array_keys($this->_commandFileMap);
	}

	/**
	 * Überprüft, ob es eine Implementierung für das Kommando mit dem Namen $commandName gibt.
	 * @param $commandName
	 * @return boolean
	 */
	function isCommandAvaiable($commandName) {
		return in_array($commandName, $this->getAvaiableCommands());
	}

	/**
	 * Gibt den Namen des per Kommandozeile gewünschten Kommandos zurück. Ist keins angegeben, so wird self::DEFAULT_COMMAND zurückgegeben.
	 * @return string
	 */
	public function getCommandToRun() {
		return $this->_commandToRun;
	}

	/**
	 * Die Haupt-Methode. Führt alles nötige aus, um ein Kommando ausführen zu können und führt es aus.
	 *
	 * Die Fehlerbehandlung geschieht hier.
	 *
	 * @return int Return-Code
	 * @throws Hpls_Cli_Exception
	 */
	public function run() {
		$ret = array();
		try {
			$this->initAllCommandDefinition();
			$this->checkConfig();
			
			$ret = $this->runCommand($this->getCommandToRun(), $this->getCommandCmdLine())->getExecReturn();
			if ($this->_output === TRUE) {
				echo implode("\n", $ret[Hpls_Cli_Command_Abstract::RET_OUT]);
				echo "\n";
			}
		} catch (Exception $ex) {
			if (isset($this->_options['throwExceptions']) && $this->_options['throwExceptions'] === TRUE) {
				//falls kein ErrorCode gegeben ist, ist was falsch, weil der immer angegeben werden soll, da ein CLI-Tool
				//bei dem Errorcodes != 0 Fehler zeigen und == 0 fehlerfrei bedeutet.
				if ($ex->getCode() === 0) {
					require_once 'Hpls/Exception/Cli.php';
					throw new Hpls_Exception_Cli($ex->getMessage(), self::ERR_UNKNOWN_ERROR);
				}
				throw $ex;
			}
			//Fehlerbehandlung: Usage-Ausgabe und Errorcodeausgabe
			

			if ($this->_output === TRUE) {
				echo "Fehler: " . $ex->getMessage() . "\n\n";
				//bei Konfigurationsfehler auch usage anzeigen
				if ($ex instanceof Hpls_Exception_Cli_Config) {
					$out = $this->printUsage();
					echo implode("\n", $out[Hpls_Cli_Command_Abstract::RET_OUT]);
				}
				echo "\n";
			}
			
			exit($ex->getCode());
		}
		return $ret;
	}

	/**
	 * @return Hpls_Cli_Command_Abstract
	 */
	public function runCommand($command, array $commandLine = array()) {
		$cmd = $this->getCommandInstance($command, $commandLine);
		$cmd->parseCmdLine();
		$cmd->prepareAndCheckConfig();
		$cmd->exec();
		return $cmd;
	}

	protected function getCommandCmdLine() {
		if (count($this->_cmdLine) <= 2) {
			//kein kommando oder nur kommandoname, sonst nichts
			return array();
		} else {
			return array_slice($this->_cmdLine, 2);
		}
	}

	/**
	 * Nur zum Rufen während eines Fehlerfalles gedacht, um noch die usage-Meldung anzeigen zu können.
	 *
	 * Umgeht den üblichen Weg um das 'usage'-Kommando zu rufen. Dadurch ist es auch rufbar, wenn bei einem anderen Kommando ein Fehler auftrat.
	 *
	 * Achtung: Löscht die ursprüngliche $this->_cmdLine und das $this->_commandToRun
	 */
	public function printUsage() {
		$this->_cmdLine = array(
				$_SERVER['argv'][0] );
		$this->_commandToRun = 'usage';
		$cmd = $this->getCommandInstance($this->getCommandToRun(), $this->getCmdLine());
		$cmd->exec();
		return $cmd->getExecReturn();
	}

	/**
	 * @throws Hpls_Exception_Cli_Config
	 */
	protected function checkConfig() {
		if ((count($this->_cmdLine) > 1) && strncmp($this->_cmdLine[1], '-', 1) === 0) {
			require_once 'Hpls/Exception/Cli/Config.php';
			throw new Hpls_Exception_Cli_Config("Das erste Argument muss ein Kommando sein. Option '" . $this->_cmdLine[1] . "' wurde aber gefunden.", self::ERR_UNKNOWN_COMMAND);
		}
	}

	public function getGlobalsUsage() {
		$opt = $this->getCommandConfig(self::GLOBAL_OPTIONS_NAME);
		return $opt[Hpls_Cli_Command_Abstract::CFG_OPTIONS];
	}

	public function getGlobalsHelp() {
		$opt = $this->getCommandConfig(self::GLOBAL_OPTIONS_NAME);
		return $opt[Hpls_Cli_Command_Abstract::CFG_OPTIONS];
	}

	public function getCliName() {
		return basename($_SERVER['argv'][0]);
	}

	/**
	 * Gibt die zum Kommandonamen passende Instanz zurück. Existiert die Instanz nicht, wird sie erzeugt.
	 *
	 * @param unknown_type $commandName
	 * @param NULL|array $commandCmdLine Wenn NULL, dann wird getCmdLine() verwendet.
	 * @param unknown_type $config
	 * @return Hpls_Cli_Command_Abstract
	 *
	 * @deprecated Do not use this, it's only public for acces from Hpls_Cli_Command_Abstract classes!
	 */
	public function getCommandInstance($commandName, $commandCmdLine = NULL, $config = NULL) {
		if (in_array($commandName, $this->getAvaiableCommands()) === FALSE) //side effect -> loading command map
{
			require_once 'Hpls/Exception/Cli/Config.php';
			throw new Hpls_Exception_Cli_Config("Unbekanntes Kommando '$commandName'.", self::ERR_UNKNOWN_COMMAND);
		}
		
		//already instantiated?
		if (isset($this->_commandInstanceMap[$commandName])) {
			return $this->_commandInstanceMap[$commandName];
		}
		//instanz erzeugen
		$cmdInst = NULL;
		//existiert ein handler für das kommando?
		$fileCommandHandler = $this->_commandFileMap[$commandName];
		if (file_exists($fileCommandHandler)) {
			require_once $fileCommandHandler;
			$prefix = array_search(dirname($fileCommandHandler), $this->_commandPath);
			$className = $prefix . $commandName;
			$cmdInst = new $className($this, is_null($commandCmdLine) ? $this->getCmdLine() : $commandCmdLine, is_null($config) ? $this->_commandOptionsMap[$commandName] : $config);
		} else {
			require_once 'Hpls/Exception/Cli.php';
			$this->printUsage();
			throw new Hpls_Exception_Cli("Unknown command '$commandName'.", self::ERR_UNKNOWN_MODE);
		}
		
		return $cmdInst;
	
	}

	/**
	 * Gibt die gesamte Konfiguration für das Kommand $commandName zurück, die in der entsprechenden Kommando-*.ini-Datei stand.
	 *
	 * Die erste Key-Ebene des Arrays sind die Sektionsnamen der ini.
	 *
	 * @param string $commandName
	 * @return multitype:array:|multitype:
	 */
	public function getCommandConfig($commandName) {
		if (isset($this->_commandOptionsMap[$commandName])) {
			return $this->_commandOptionsMap[$commandName];
		}
		return array();
	}

	/**
	 * Generiert aus allen gefundenen Kommando-*.ini-Dateien die nötige Konfiguration der CmdLine-Optionen
	 * für den Konstruktor von Hpls_Cli_Getopt.
	 *
	 * @return Ambigous <multitype:string , unknown>
	 */
	protected function getGetoptOptions($cmd = NULL) {
		if (is_null($cmd)) {
			foreach ($this->_commandOptionsMap as $cmd => $options) {
				if (isset($options['options']) && is_array($options['options'])) {
					foreach ($options['options'] as $option) {
						//@todo vergleich nach inkompabtibler
						$name = $option['name'];
						$ret[$name] = ''; //description -> bleibt leer, da usage() anderweitig generiert
					}
				}
			}
		}
		return $ret;
	}

	/**
	 * Scant alle gegebenen Verzeichnisse für Kommandos nach den Kommandos und trägt sie in $this->_commandFileMap ein.
	 * Evtl. vorhandene Kommando-*.ini-Dateien werden gleich mit eingelesen.
	 *
	 * Erstellt noch keine Instanzen der Kommandos!
	 */
	protected function initAllCommandDefinition() {
		if (is_null($this->_commandFileMap)) {
			$this->_commandFileMap = array();
			foreach ($this->_commandPath as $cmdPath) {
				$dirlist = scandir($cmdPath);
				if ($dirlist === FALSE)
					return FALSE;
				foreach ($dirlist as $entry) {
					
					if (preg_match('/\.php$/i', $entry) > 0 && preg_match('/Abstract\.php$/i', $entry) === 0) {
						$classFile = "$cmdPath/$entry";
						$entry = substr($entry, 0, strlen($entry) - 4);
						//existiert schon eine instanz?
						if (!isset($this->_commandFileMap[$entry])) {
							$this->_commandFileMap[$entry] = $classFile;
							//							$this->log()->debug("checking: $cmdPath/$entry.ini");
							//config datei vorhanden?
							if (file_exists("$cmdPath/$entry.ini")) {
								//-> einlesen
								$optIni = new Zend_Config_Ini("$cmdPath/$entry.ini");
								$this->_commandOptionsMap[$entry] = $optIni->toArray();
							
		//								$this->log()->debug("Have config for '$entry': " . print_r($this->_commandOptionsMap[$entry], true));
							

							} else {
								//keine config-datei vorhanden dann default (leere config)
								$this->_commandOptionsMap[$entry] = array();
							}
						
						}
					}
				}
			}
		}
	}

	/**
	 * Gibt die Log-Instance zurück mit der geloggt wird.
	 *
	 * @return Zend_Log
	 */
	public function log() {
		if (is_null($this->_logger)) {
			$logger = '';
			if (isset($this->_options['logger'])) {
				$logger = $this->_['logger'];
			} else {
				if (Zend_Registry::isRegistered(Hpls_Consts::REGK_LOGGER)) {
					$logger = Zend_Registry::get(Hpls_Consts::REGK_LOGGER);
				} else if ($this->DEBUG === TRUE) {
					require_once 'Zend/Log/Writer/Stream.php';
					$writer = new Zend_Log_Writer_Stream('php://output');
					$logger = new Zend_Log($writer);
				
				} else if ($this->VERBOSE === TRUE) {
					require_once 'Zend/Log/Writer/Stream.php';
					$writer = new Zend_Log_Writer_Stream('php://output');
					$logger = new Zend_Log($writer);
					$logger->addFilter(new Zend_Log_Filter_Priority(Zend_Log::INFO));
				} else {
					//					require_once 'Zend/Log/Writer/Null.php';
					//					$writer = new Zend_Log_Writer_Null();
					require_once 'Zend/Log/Writer/Stream.php';
					$writer = new Zend_Log_Writer_Stream('php://output');
					$logger = new Zend_Log($writer);
					$logger->addFilter(new Zend_Log_Filter_Priority(Zend_Log::WARN));
				}
			}
			Zend_Registry::set(Hpls_Consts0::REGK_LOGGER, $logger);
			$this->_logger = $logger;
		
		}
		return $this->_logger;
	}
}