<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 26.07.2010
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28 Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Cli/Command/Abstract.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Utils.php';
require_once 'Hpls/Cli/Consts.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package AC
 * @subpackage CLI-UI
 * @version $LastChangedRevision: 9196 $
 * @since 1.0.0.5
 */
abstract class Hpls_Cli_Command_Abstract implements 
	Hpls_Cli_Consts {
	
	protected $_commandName = null;
	
	/**
	 * @var array:string
	 */
	protected $_cmdLine = NULL;
	protected $_cmdLineParser = null;
	protected $_config = array();
	protected $_execReturn = array(
			self::RET_CODE => 0,
			self::RET_OUT => array() );
	/**
	 * Die CLI-Mainklasse, zu der die Instanz des Kommandos gehört.
	 * @var Hpls_Cli_Abstract
	 */
	protected $_owner = NULL;
	
	protected $_cmdOptions = array();

	/**
	 * @param Hpls_Cli_Abstract $owner
	 * @param array:string $cmdLineOptions Prinzipiell die $argv, nur schon geparst als Hpls_Cli_Getopt-Objekt. 
	 * @param array $config Die Konfiguration, die eigentlich aus der gleichnamigen *.ini-Datei zum Kommando stammen sollte, aber auch aus
	 * anderer Quelle generiert werden kann (muss die selber Array-Struktur aufweisen wie die aus der *.ini-Datei).
	 */
	public function __construct(Hpls_Cli_Abstract $owner, array $cmdLine, array $config = array()) {
		$this->_owner = $owner;
		$this->_config = $config;
		$this->_cmdLine = $cmdLine;
	}

	/**
	 * Führt die Aktionen der Kommandos aus. 
	 */
	abstract public function exec();

	/**
	 * Wird vom Main-Prozess vor exec() und checkConfig() gerufen und bietet die Gelegenheit die Kommandozeile noch einmal extra zu parsen.
	 * 
	 * Ein erstes Parsen hat die Owner-Klasse bereits vor dem Aufruf dieser Methode durchgeführt, deshalb muss hier nicht mehr geparst werden,
	 * es sei denn, man wünscht eine andere Parsung.
	 */
	public function parseCmdLine() {
		if (is_null($this->_cmdLineParser) === FALSE) {
			return;
		}
		$this->log()->debug("CommandCmdLine: " . print_r($this->getCmdLine(), true));
		
		$optionSpec = $this->getCmdLineParserOptions();
		
		//		$this->log()->debug(print_r($optionSpec, true));
		try {
			$getopt = new Hpls_Cli_Getopt($optionSpec, $this->getCmdLine(), array(
					Hpls_Cli_Getopt::CONFIG_RULEMODE => Hpls_Cli_Getopt::MODE_HPLS ));
			//		$getopt->setArguments($this->getAvaiableCommands());
			$getopt->parse();
		} catch (Exception $ex) {
			throw new Hpls_Exception_Cli($ex->getMessage(), self::ERR_UNKNOWN_OPTION);
		}
		
		$this->_cmdLineParser = $getopt;
	}

	/**
	 * Überprüft die Konfiguration des Kommandos.
	 * 
	 * Sollte vor exec() und muss nach parseCmdLine() gerufen werden.
	 * 
	 * @throws Hpls_Exception_Cli_Config Wenn die per Kommandozeile übergegebene Konfiguration fehlerhaft ist.
	 */
	abstract public function prepareAndCheckConfig();

	public function getCommandName() {
		if (is_null($this->_commandName)) {
			$name = get_class($this);
			$split = explode('_', $name);
			$this->_commandName = $split[count($split) - 1];
		}
		return $this->_commandName;
	}

	/**
	 * @return string
	 */
	function getHelp() {
		if (isset($this->_config[self::CFG_OPTIONS])) {
			return $this->_config[self::CFG_OPTIONS];
		}
		return array();
	}

	/**
	 * @return string
	 */
	function getUsage() {
		if (isset($this->_config[self::CFG_OPTIONS])) {
			return $this->_config[self::CFG_OPTIONS];
		}
		return array();
	}

	/**
	 * @todo comment
	 * 
	 * @return array <dl><dt>['code'] (int)</dt><dd></dd>
	 * <dt>['output'] (array:string)</dt><dd></dd>
	 * </dl>
 
	 */
	public function getExecReturn() {
		return $this->_execReturn;
	}

	public function getOutput() {
		return $this->_execReturn['output'];
	}

	public function getExitCode() {
		return $this->_execReturn['code'];
	}

	/**
	 * @return Zend_Log
	 */
	protected function log() {
		return $this->_owner->log();
	}

	/**
	 * @todo comment
	 * 
	 * @return array:string 
	 */
	protected function getHosts() {
		$hosts = $this->getOpt()->getOption('host');
		if (is_null($hosts) || empty($hosts)) {
			return array(
					self::HOST_LOCALHOST );
		}
		if (is_array($hosts) === FALSE) {
			return array(
					$hosts );
		}
		return $hosts;
	}

	/**
	 * Überprüft ob $hostNameOrIp den lokalen Host repräsentiert.
	 * @param string $hostNameOrIp
	 */
	protected function isLocalHost($hostNameOrIp) {
		
		return $hostNameOrIp == '127.0.0.1' || strcasecmp($hostNameOrIp, 'localhost') === 0;
	}

	/**
	 * @return Hpls_Cli_Abstract
	 */
	protected function getOwner() {
		return $this->_owner;
	}

	/**
	 * Enter description here ...
	 * @return array:string
	 */
	protected function getCmdLine() {
		return $this->_cmdLine;
	}

	protected function setCmdLine(array $cmdLine) {
		$this->_cmdLine = $cmdLine;
	}

	/**
	 * Enter description here ...
	 * @return Ambigous <multitype:string, string>
	 */
	protected function getCmdLineParserOptions() {
		//		$this->log()->debug(print_r($this->_config));
		$ret = array();
		$configs = array(
				$this->getOwner()->getCommandConfig(self::GLOBAL_OPTIONS_NAME),
				$this->_config );
		//		if (is_null($cmd)) {
		foreach ($configs as $config) {
			if (isset($config['options']) && is_array($config['options'])) {
				foreach ($config['options'] as $option) {
					//@todo vergleich nach inkompabtibler
					$name = $option['name'];
					$ret[$name] = ''; //description -> bleibt leer, da usage() anderweitig generiert
				}
			}
		}
		//		}
		return $ret;
	
	}

	protected function setCmdOption($key, $value) {
		$this->_cmdOptions[$key] = $value;
	}

	protected function getCmdOption($key) {
		if (isset($this->_cmdOptions[$key])) {
			return $this->_cmdOptions[$key];
		}
		return $this->getOpt()->getOption($key);
	}

	protected function hasCmdOption($key) {
		return in_array($key, $this->getOpt()->getOptions());
	}

	/**
	 * Gibt die geparsten Kommandozeilen Optionen zurück als Hpls_Cli_Getopt-Objekt.
	 * Auf einzelne Optionswerte kann dann per $this->getCLOptions()->optionsName zugegriffen werden.
	 * @see Doku von Zend_Console_Getopt.
	 * 
	 * @return Hpls_Cli_Getopt
	 */
	protected function getOpt() {
		return $this->_cmdLineParser;
	}

	protected function addOutput($line) {
		if (is_array($line)) {
			foreach ($line as $l) {
				$this->_execReturn[self::RET_OUT][] = $l;
			}
		} else {
			$this->_execReturn[self::RET_OUT][] = $line;
		}
	}

	protected function setReturnCode($code) {
		$this->_execReturn[self::RET_CODE] = (int) $code;
	}
}