<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 * 
 * created   : 26.07.2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28 Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Cli/Command/usage.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Cli/Command/Abstract.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package AC
 * @subpackage CLI-UI
 * @version $LastChangedRevision: 9196 $
 * @since 1.0.0.5
 */
class Hpls_Cli_Command_usage extends Hpls_Cli_Command_Abstract {

	function exec() {
		$this->log()->debug(__METHOD__ . " START");
		$toolName = $this->_owner->getCliName();
		$ret = array(
				self::RET_CODE => 0,
				self::RET_OUT => array() );
		$this->addOutput("usage: $toolName <command> [{ <global options> | <command options> }]\n");
		$this->genUsageGlobalOptions();
		$this->genUsageAllCommands();
		$this->addOutput('');
		$this->log()->debug(__METHOD__ . " END");
	
		//		print_r();
	}

	/**
	 * 
	 */
	function prepareAndCheckConfig() {
	
	}

	/**
	 * Generiert usage für die globalen Optionen und fügt sie an die Kommandoausgabe an.
	 */
	protected function genUsageGlobalOptions() {
		$this->addOutput("<global options>");
		$options = $this->getOwner()->getGlobalsUsage();
		$line = "\t";
		foreach ($options as $optDescription) {
			$line .= ' ';
			$line .= $this->formatUsage($optDescription);
		}
		$this->addOutput($line);
	}

	/**
	 * Generiert usage für alle verfügbaren Kommandos und fügt sie an die Kommandoausgabe an.
	 */
	protected function genUsageAllCommands() {
		$this->addOutput("<command>:\n");
		$cmds = $this->getOwner()->getAvaiableCommands();
		$ret = array();
		foreach ($cmds as $cmd) {
			$cmdInst = $this->getOwner()->getCommandInstance($cmd, array());
			$ret[$cmd] = $cmdInst->getUsage();
		}
		$usages = $ret;
		
		foreach ($usages as $cmd => $options) {
			$line = "\n\t$cmd\t";
			foreach ($options as $optDescription) {
				$line .= ' ';
				$line .= $this->formatUsage($optDescription);
			}
			$this->addOutput($line);
		}
	}

	/**
	 * @todo comment
	 * 
	 * @param array $optDescription
	 * @return string
	 */
	protected function formatUsage(array $optDescription) {
		$line = "";
		if ($this->isRequiredOption($optDescription) === FALSE) {
			$line .= '[';
		}
		$line .= implode('|', $this->putSigns($optDescription[self::CFG_OPT_NAME]));
		if (isset($optDescription[self::CFG_OPT_PARAMNAME])) {
			$line .= " " . $this->formatParamName($optDescription);
		}
		if ($this->isRequiredOption($optDescription) === FALSE) {
			$line .= ']';
		}
		return $line;
	}

	/**
	 * Prüft ob eine Option als required definiert wurde.
	 * 
	 * @param array $optDescription
	 * @return boolean
	 */
	protected function isRequiredOption(array $optDescription) {
		if (isset($optDescription[self::CFG_OPT_REQUIRED]) && Hpls_Utils::toBool($optDescription[self::CFG_OPT_REQUIRED]) !== FALSE) {
			return TRUE;
		}
		return FALSE;
	}

	/**
	 * Formatiert den Parameternamen in abhängigkeit davon, ob er required ist. Ist ein spezieller Name für den Parameter 'paramName' vergeben
	 * worden in der *.ini, wird dieser genommen, fehlt er wird in Abhängigkeit vom Paramtype ein Name des Wertebereichs genommen.
	 * 
	 * @param array $optDescription
	 * @return string|string|string
	 */
	protected function formatParamName(array $optDescription) {
		$splitUp = Hpls_Cli_Getopt::splitUp($optDescription[self::CFG_OPT_NAME]);
		$param = Hpls_Cli_Getopt::getParamType($splitUp);
		if ($param['param'] === 'none') {
			return '';
		}
		
		if (isset($optDescription[self::CFG_OPT_PARAMNAME])) {
			$name = '<' . $optDescription[self::CFG_OPT_PARAMNAME] . '>';
		} else {
			if (isset($param['paramType'])) {
				$name = '<' . $param['paramType'] . '>';
			} else {
				$name = '<value>';
			}
		}
		
		if ($param['param'] !== 'required') {
			return "[$name]";
		}
		return $name;
	}

	/**
	 * Fügt '--' oder '-' vor den Optionsnamen ein und gibt sie als array zurück.
	 * 
	 * @param string $optName
	 * @return Ambigous <multitype:, string>
	 */
	protected function putSigns($optName) {
		$ret = Hpls_Cli_Getopt::splitUp($optName);
		$names = $ret['names'];
		$ret = array();
		foreach ($names as $name) {
			if (strlen($name) === 1) {
				$ret[] = "-$name";
			} else {
				$ret[] = "--$name";
			}
		}
		return $ret;
	
	}

}