<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 * 
 * created   : 23.08.2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28 Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Cli/Copy.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Cli/Consts.php';
require_once 'Hpls/Utils/Logusing.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package AC
 * @subpackage CLI
 * @version $LastChangedRevision: 9196 $
 * @since 1.0.1.6
 */
class Hpls_Cli_Copy extends Hpls_Utils_Logusing {
	const REMOTE_EXECUTOR = '/usr/bin/scp';
	
	protected $_srcPath;
	protected $_destPath;
	protected $_srcHost;
	protected $_srcUser = null;
	protected $_destHosts;
	protected $_stopByOneError;
	protected $_logger;
	protected $_remoteUser;

	public function __construct($srcPath, $destinationPath, $destHosts = null, $srcHost = null) {
		$this->setSource($srcPath, $srcHost);
		$this->setDestination($destinationPath, $destHosts);
	}

	/**
	 * @throws Hpls_Exception_Executable_Wrapper wenn das Kopieren nicht geklappt hat.
	 */
	public function exec() {
		foreach ($this->getDestinationHosts() as $destHost) {
			$this->copyTo($destHost);
		}
	
	}

	/**
	 * @param unknown_type $host
	 * @throws Hpls_Exception_Executable_Wrapper wenn das Kopieren nicht geklappt hat.
	 */
	public function copyTo($host) {
		$cmd = 'scp ';
		//source parameter generieren
		if ($this->isLocalhost($this->_srcHost)) {
			if ($this->isLocalhost($host)) { //lokal zu lokal
				$cmd = "cp ";
			} else {
			}
		} else {
			if (!empty($this->_srcUser)) {
				$cmd .= "$this->_srcUser@";
			}
			$cmd .= "$this->_srcHost:";
		}
		$cmd .= "'$this->_srcPath' ";
		//destination parameter generieren
		if (!$this->isLocalhost($host)) {
			if (!empty($this->_remoteUser)) {
				$cmd .= "$this->_remoteUser@";
			}
			$cmd .= "$host:";
		}
		$cmd .= "'$this->_destPath'";
		$this->log()->debug(__METHOD__ . ": $cmd");
		$retCode = 0;
		$output = array();
		exec($cmd, $output, $retCode);
		if ($retCode !== 0) {
			require_once 'Hpls/Exeception/Executable/Wrapper.php';
			throw new Hpls_Exception_Executable_Wrapper("Fehler beim Kopieren", $retCode, $cmd, $host);
		}
	
		//scp root@h:~/gg root@h2:
	}

	public function getDestinationHosts() {
		return $this->_destHosts;
	}

	public function setSource($srcPath, $srcHost = null) {
		if ($this->isLocalhost($srcHost)) {
			$srcHost = Hpls_Cli_Consts::HOST_LOCALHOST;
		}
		$this->_srcPath = $srcPath;
		
		$this->_srcHost = $srcHost;
	}

	public function setDestination($destinationPath, $destinationHosts) {
		$this->_destPath = $destinationPath;
		if (empty($destinationHosts)) {
			$destinationHosts = Hpls_Cli_Consts::HOST_LOCALHOST;
		}
		if (!is_array($destinationHosts)) {
			$destinationHosts = array(
					$destinationHosts );
		}
		//duplikate filtern
		$this->_destHosts = array();
		foreach ($destinationHosts as $host) {
			if ($this->isLocalhost($host)) {
				//unifizierung von localhost
				$host = Hpls_Cli_Consts::HOST_LOCALHOST;
			}
			if (!in_array($host, $this->_destHosts)) {
				$this->_destHosts[] = $host;
			}
		}
	
	}

	public function setRemoteUser($user = null) {
		$this->_remoteUser = $user;
	}

	public function getRemoteUser() {
		return $this->_remoteUser;
	}

	protected function getCopyCmd($destHost) {
		$host = trim($host);
		if (!$this->isLocalhost($host)) {
			$cmd = $this->getRemoteExecutor($host, $this->getRemoteUser()) . " '$cmd'";
		} else {
			$host = Hpls_Cli_Consts::HOST_LOCALHOST;
		}
	}

	protected function isLocalhost($host) {
		return empty($host) || strcasecmp($host, Hpls_Cli_Consts::HOST_LOCALHOST) === 0 || strcasecmp($host, Hpls_Cli_Consts::HOST_LOCALHOST_IP) === 0;
	}
}