<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 10.11.2010
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28 Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Config/Writer/Ini.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Zend/Config/Writer/Ini.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 9196 $
 * @since 1.0.6
 */
class Hpls_Config_Writer_Ini extends Zend_Config_Writer_Ini {
	protected $_renderQuotes = true;
	protected $_renderEqualSpaces = true;

	protected function _prepareValue($value) {
		if (is_integer($value) || is_float($value)) {
			return $value;
		} elseif (is_bool($value)) {
			return ($value ? 'true' : 'false');
		} elseif (strpos($value, '"') === false) {
			if ($this->_renderQuotes) {
				return '"' . $value . '"';
			} else {
				return $value;
			}
		} else {
			/** @see Zend_Config_Exception */
			require_once 'Zend/Config/Exception.php';
			throw new Zend_Config_Exception('Value can not contain double quotes "');
		}
	}

	/**
	 * Render a Zend_Config into a INI config string.
	 *
	 * @since 1.10
	 * @return string
	 */
	public function render() {
		$iniString = '';
		$extends = $this->_config->getExtends();
		$sectionName = $this->_config->getSectionName();
		
		if ($this->_renderWithoutSections == true) {
			$iniString .= $this->_addBranch($this->_config);
		} else if (is_string($sectionName)) {
			$iniString .= '[' . $sectionName . ']' . "\n" . $this->_addBranch($this->_config) . "\n";
		} else {
			$config = $this->_sortRootElements($this->_config);
			foreach ($config as $sectionName => $data) {
				if (!($data instanceof Zend_Config)) {
					$iniString .= $sectionName . ($this->_renderEqualSpaces ? ' = ' : '=') . $this->_prepareValue($data) . "\n";
				} else {
					if (isset($extends[$sectionName])) {
						$sectionName .= ' : ' . $extends[$sectionName];
					}
					
					$iniString .= '[' . $sectionName . ']' . "\n" . $this->_addBranch($data) . "\n";
				}
			}
		}
		
		return $iniString;
	}

	/**
	 * Add a branch to an INI string recursively
	 *
	 * @param  Zend_Config $config
	 * @return void
	 */
	protected function _addBranch(Zend_Config $config, $parents = array()) {
		$iniString = '';
		
		foreach ($config as $key => $value) {
			$group = array_merge($parents, array(
					$key ));
			
			if ($value instanceof Zend_Config) {
				$iniString .= $this->_addBranch($value, $group);
			} else {
				$iniString .= implode($this->_nestSeparator, $group) . ($this->_renderEqualSpaces ? ' = ' : '=') . $this->_prepareValue($value) . "\n";
			}
		}
		
		return $iniString;
	}

	public function setRenderquotes($value = true) {
		$this->_renderQuotes = $value;
	}

	/**
	 * @param field_type $_renderEqualSpaces
	 */
	public function setRenderEqualSpaces($value = true) {
		$this->_renderEqualSpaces = $value;
	}

}