<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created   : Jan 21, 2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28 Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.libs/Hpls/Controller/Plugin/Authentication.php $
 *
 */
/***/
require_once 'Hpls/Controller/Plugin/Abstract.php';
require_once 'Zend/Auth.php';
require_once 'Hpls/Acl.php';
require_once 'Hpls/Auth.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package HPLS
 * @subpackage subpackage
 * @version $LastChangedRevision: 9196 $
 * @since 
 *
 */

class Hpls_Controller_Plugin_Authentication extends Hpls_Controller_Plugin_Abstract {
	
	/**
	 * @var Zend_Auth
	 */
	private $_auth = NULL;
	
	/**
	 * @var Zend_Auth_Result
	 */
	private $_authResult = null;

	function __construct() {
		$this->_auth = Zend_Auth::getInstance();
	}

	function dispatchLoopStartup(Zend_Controller_Request_Abstract $request) {
		Hpls_Debug::log()->debug(__METHOD__ . " START");
		
		$module = $this->getRequest()->getModuleName();
		$controller = $this->getRequest()->getControllerName();
		$action = $this->getRequest()->getActionName();
		$params = $this->getRequest()->getParams();
		$requiredAclRes = Hpls_Acl::i()->getRequiredResources($module, $controller, $action, $params);
		$requiredRoles = Hpls_Acl::i()->getRequiredRoles($module, $controller, $action, $params);
		
		//wenn ACLs richtig funktionieren, dann auch:
		// if ( ... ||  Hpls_Acl::i()->isAllowed($requiredAclRes /*, 'read'*/))
		if (empty($requiredRoles) || in_array('guest', $requiredRoles) || ($module == 'auth' && $controller == 'login')) {
			return;
		}
		
		$id = $this->_auth->getIdentity();
		$this->log()->debug(__METHOD__ . ": identity: " . Hpls_Debug::print_r($id));
		
		if (FALSE === $this->_auth->hasIdentity()) {
			//nicht angemeldet
			$sess = Hpls_Application::i()->getSession(Hpls_Consts::SESSION_NS_AUTH);
			$sess->forwardUrl = array(
					'module' => $module,
					'controller' => $controller,
					'action' => $action,
					'params' => array_diff_key($params, array(
							'module' => 1,
							'controller' => 1,
							'action' => 1 )) );
			$redirect = new Zend_Controller_Action_Helper_Redirector();
			$redirect->gotoSimpleAndExit('index', 'login', 'auth');
			return;
		}
		//ab hier ist der User eingeloggt
		$sess = Hpls_Application::i()->getSession(Hpls_Consts0::SESSION_NS_AUTH);
		if (!isset($sess->userObj) || !is_object($sess->userObj)) {
			$sess->forwardUrl = array(
					'module' => $module,
					'controller' => $controller,
					'action' => $action,
					'params' => array_diff_key($params, array(
							'module' => 1,
							'controller' => 1,
							'action' => 1 )) );
			Hpls_Auth::i()->addAuthError(array(
					'code' => 1,
					'errorText' => 'Serverkonfiguration hat sich geändert. Bitte neu einloggen.' ));
			$redirect = new Zend_Controller_Action_Helper_Redirector();
			$redirect->gotoSimpleAndExit('serverchanged', 'logout', 'auth');
		}
		
		//wenn angemeldet, dann Usernamed-Role zum ACL-System hinzufügen, damit es dort vorhanden ist (bei jedem Aufruf machen!)
		//@todo durch ACL cachen in Session ablösen
		//		$usrrole = 'usr_'.$sess->userObj->getName();
		//		Hpls_Auth::i()->addUserRole($usrrole,array_diff($sess->userObj->getRoles(),array($usrrole)));
		

		//Gehört der User einer Rolle an, die benötigt wird für den Zugang?
		if (FALSE === Hpls_Auth::i()->hasRole($requiredRoles)) {
			$sess->forwardUrl = array(
					'module' => $module,
					'controller' => $controller,
					'action' => $action,
					'params' => array_diff_key($params, array(
							'module' => 1,
							'controller' => 1,
							'action' => 1 )) );
			Hpls_Auth::i()->addAuthError(array(
					'code' => 1,
					'errorText' => 'ac_auth_err_role_required',
					'errorTextParams' => array(
							'REQUIREDROLES' => $requiredRoles,
							'HASROLES' => Hpls_Acl::i()->getCurrentUserRoles(),
							'URL' => $_SERVER['REQUEST_URI'],
							'CONTROLLER' => $controller,
							'MODULE' => $module,
							'ACTION' => $action ) ));
			$redirect = new Zend_Controller_Action_Helper_Redirector();
			$redirect->gotoSimpleAndExit('rolerequired', 'login', 'auth');
		}
	
		//ab hier ist der User richtig  eingeloggt
	/*if (!$acl->isAllowed($role,$request->getModuleName(), $request->getControllerName(),$request->getActionName())) {
			$newAction = (Hpls_Acl::ROLE_GUEST == $role)? 'login' : 'forbidden';
			$request->setModuleName('default');
			$request->setControllerName('authentication');
			$request->setActionName($newAction);
		}*/
	}

}