<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created   : Jan 29, 2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2010-11-11 13:24:44 +0100 (Do, 11 Nov 2010) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 7685 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/HMT/GUI/Core/trunk/hpls.hmt.gui.libs/Hpls/Mailtrace/Model/Log/Storage/Factory.php $
 *
 */
/**
 *
 */
require_once 'Hpls/Utils.php';
require_once 'Hpls/Mailtrace/Consts.php';
require_once 'Hpls/Mailtrace.php';

class Hpls_Mailtrace_Model_Log_Storage_Factory {
	protected static $_instance = NULL;
	protected static $CONF_PATH = array (MTConsts::CONF_MTLOG,MTConsts::CONF_STORAGE,MTConsts::CONF_STORAGE_DB);
	protected $_config = null;
	const TYPE_DB_MYSQL = 'mysql';
	const TYPE_DB_PGSQL = 'pgsql';

	public static function getInstance() {
		if (is_null(self::$_instance)) {
			self::$_instance = new self();
		}
		return self::$_instance;

	}
	protected function __construct() {

	}
	public function getStorage($type = null) {
		if (empty($type)) {
			$type = $this->getDefaultType();
		}
		
		if (strcasecmp($type,self::TYPE_DB_MYSQL) === 0) {
			require_once 'Hpls/Mailtrace/Model/Log/Storage/Db/Mysql/Implementation.php';
			return new Hpls_Mailtrace_Model_Log_Storage_Db_Mysql_Implementation($this->getTypeConfig($type));
		} elseif (strcasecmp($type,self::TYPE_DB_PGSQL) === 0) {
			require_once 'Hpls/Mailtrace/Model/Log/Storage/Db/Pgsql/Implementation.php';
			return new Hpls_Mailtrace_Model_Log_Storage_Db_Pgsql_Implementation($this->getTypeConfig($type));
		} else {
			require_once 'Hpls/Exception/NotSupported.php';
			throw new Hpls_Exception_NotSupported("Unsupported storage type: ".$type);
		}


	}
	protected function getTypeConfig($number) {
		//assume $type has value
		/*
		 * drwlog.storage.db.username =
drwlog.storage.db.password =
drwlog.storage.db.dbname = maillog-db
drwlog.storage.db.dbhost = localhost
drwlog.storage.db.dbport =
drwlog.storage.db.table.connect = connectlog
drwlog.storage.db.table.deliver = deliverlog
*/

		$config = $this->getConfig();
//		if (!empty($config) && is_array($config)) {
//			if (strcasecmp($type,self::TYPE_DB_MYSQL) === 0 || strcasecmp($type,self::TYPE_DB_PGSQL) ===0) {
//				$dbconf = null;
//				try {
//					$dbConf = Hpls_Utils::getDeepArray($config,self::$CONF_PATH);
//				} catch (Hpls_Exception_KeyNotExists $ex) {
//					require_once 'Hpls/Exception/Configuration.php';
//					throw new Hpls_Exception_Configuration("No configuration found for storage ".$type." in component.ini for Mailtrace. KeyPath: ".implode(".",$ex->getKeyPath()));
//				}
//				return $dbConf;
//			}
//		}
		return $config;
	}

	protected function getDefaultType() {
		//drwatson.drwlog.storage.default
		$config = $this->getConfig();
		if (empty($config[MTConsts::CONF_STORAGE_DB_TYPE])) {
			
		}
		return $config[MTConsts::CONF_STORAGE_DB_TYPE];
	}
	
	protected function getConfig() {
		if (is_null($this->_config)) {
			try {
				$config = Hpls_Mailtrace::i()->getConfig(self::$CONF_PATH);
				$this->_config = $config;
			} catch (Hpls_Exception_Configuration $ex) {
				throw new Hpls_Exception_Configuration("Configuration entry missing.",null,array (MTConsts::CONF_MTLOG,MTConsts::CONF_STORAGE,MTConsts::CONF_STORAGE_DB_TYPE,MTConsts::CONF_STORAGE_DB));
			}
			
		}
		return $this->_config;
	}
}