<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created   : Mar 19, 2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2010-10-22 12:10:45 +0200 (Fr, 22 Okt 2010) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 7387 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/HMT/GUI/Core/trunk/hpls.hmt.gui.libs/Hpls/Mailtrace/Model/Reason.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 */
/***/
require_once 'Hpls/Mailtrace.php';
require_once 'Hpls/Mailtrace/Model/Reason/Entry.php';
require_once 'Hpls/Debug.php';
/**
 * @todo comment
 * 
 * Singleton.
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package Mailtrace
 * @subpackage Reason
 * @version $LastChangedRevision: 7387 $
 * @since 
 *
 */
class Hpls_Mailtrace_Model_Reason {
	protected static $_instance = null;
	/**
	 * @var array
	 */
	protected $_reasonIni = null;
	protected $_defaultRole = null;
	protected $_defaultLocale = null;
	protected $_parsedRules = array();

	protected function __construct() {
	}

	public static function getInstance() {
		if (is_null(self::$_instance)) {
			self::$_instance = new Hpls_Mailtrace_Model_Reason();
		}
		return self::$_instance;
	}

	/**
	 * @todo comment
	 * @param string $longReasonText
	 * @return Mailtrace_Model_Reason_Entry
	 */
	public function match($longReasonText, $role = null, Zend_Locale $locale = null) {
		$regexArray = $this->getAllRegEx();
		foreach ($regexArray as $ruleID => $regex) {
			if (isDebug('reason_match'))
				$this->getLogger()->debug("Matching rule $ruleID, $regex: ");
			$results = preg_match($regex, $longReasonText);
			if ($results === false) { //@todo error handling
				require_once 'Hpls/Mailtrace/Exception/Reason.php';
				throw new Hpls_Mailtrace_Exception_Reason("Error in executing regex '$regex' on:\n$longReasonText");
			}
			if ($results > 0) {
				if (is_null($locale)) {
					$locale = $this->getDefaultLocale();
				}
				if (is_null($role)) {
					$role = $this->getDefaultRole();
				}
				
				$rule = $this->getParsedRule($ruleID, $role, $locale);
				$ret = new Mailtrace_Model_Reason_Entry($ruleID, $rule, $role, $locale);
				return $ret;
			}
		}
	}

	/**
	 * @todo comment
	 * @return array<string>
	 * 
	 * @throws Hpls_Mailtrace_Exception_Reason If [rules] section in config missing.
	 */
	public function getAllRegEx() {
		$rules = $this->getRules();
		//		$this->getLogger()->debug('Reason: ' . Zend_Debug::dump($rules, "rules: ", false));
		$ret = array();
		foreach ($rules as $ruleID => $rule) {
			$ret[$ruleID] = $rule[MTConsts::REASON_RULE_REGEX];
		}
		return $ret;
	}

	/**
	 * @todo comment
	 * 
	 * @todo implement role and locale fallback
	 * 
	 * @param string $ruleID
	 * @param string $userRole
	 * @param Zend_Locale $locale
	 */
	public function getHelpText($ruleID, $userRole, Zend_Locale $locale = null) {
		$rule = $this->getRule($ruleID);
		if (is_null($rule)) {
			return null;
		}
		//first try -> text in reason.ini
		if (isset($rule[MTConsts::REASON_RULE_HELPTEXT])) {
			$ret = $this->get4RoleLocale($rule[MTConsts::REASON_RULE_HELPTEXT], $userRole, $locale);
		}
		//second try -> file
		if (is_null($ret)) {
			$file = null;
			if (isset($rule[MTConsts::REASON_RULE_HELPTEXT])) {
				$file = $this->get4RoleLocale($rule[MTConsts::REASON_RULE_HELPTEXT], $userRole, $locale);
			} // no else because return can else be NULL => $file=== null 
			//still no result? 
			if (is_null($file)) {
				$file = $ruleID;
			}
			if (!Hpls_Utils::isAbsolutePath($file))
				//contruct from reason text dir and ruleID
				try {
					$textDir = Hpls_Mailtrace::i()->getConfig(array(
							MTConsts::CONF_MTLOG,
							MTConsts::CONF_REASON,
							MTConsts::CONF_REASON_TEXTDIR ));
				} catch (Hpls_Exception_KeyNotExists $ex) {
					$this->getLogger()->warning("Mailtrace configuration does not contain " . implode('.', $ex->getKeyPath()));
					$textDir = null;
				}
			if (empty($file) || !is_dir($file)) {
				//error handling
				require_once 'Hpls/Exception/Configuration.php';
				throw new Hpls_Exception_Configuration("Default reason text dir is not existent or configured in component.ini for Mailtrace.");
			}
			$file .= "$textDir" . PATH_SEPARATOR . "$file";
			if (!file_exists($file)) {
			} else {
			}
			//TODO
		}
		return null;
	}

	/**
	 * @todo comment
	 * 
	 * @param string $ruleID
	 * @return array<array>
	 * 
	 * @throws Hpls_Mailtrace_Exception_Reason If [rules] section in config missing.
	 */
	public function getRule($ruleID) {
		$rules = $this->getRules();
		if (isset($rules[$ruleID])) {
			return $rules[$ruleID];
		} else
			return null;
	}

	public function getParsedRule($ruleID, $role = null, Zend_Locale $locale = null) {
		if (!isset($this->_parsedRules[$ruleID])) {
			$realRule = $this->getRule($ruleID);
			$ret = null;
			if (!is_null($realRule)) {
				$ret = array(
						MTConsts::REASON_RULE_REGEX => $realRule[MTConsts::REASON_RULE_REGEX],
						MTConsts::REASON_RULE_VARNAMING => $realRule[MTConsts::REASON_RULE_VARNAMING] );
				$ret[MTConsts::REASON_RULE_SHORT] = $this->get4RoleLocale($realRule[MTConsts::REASON_RULE_SHORT], $role, $locale);
				$requiredFailed = null;
				if (isset($realRule[MTConsts::REASON_RULE_HELPTEXT])) {
					try {
						$ret[MTConsts::REASON_RULE_HELPTEXT] = $this->get4RoleLocale($realRule[MTConsts::REASON_RULE_HELPTEXT], $role, $locale, true);
					} catch (Hpls_Mailtrace_Exception_Reason $ex) {
						$requiredFailed = $ex;
					}
				}
				if (isset($realRule[MTConsts::REASON_RULE_HELPLINK])) {
					try {
						$ret[MTConsts::REASON_RULE_HELPLINK] = $this->get4RoleLocale($realRule[MTConsts::REASON_RULE_HELPLINK], $role, $locale, true);
					} catch (Hpls_Mailtrace_Exception_Reason $ex) {
						if (!is_null($requiredFailed)) {
							throw new Hpls_Mailtrace_Exception_Reason("Either " . MTConsts::REASON_RULE_HELPTEXT . " nor " . MTConsts::REASON_RULE_HELPTEXT . " defined for rule '$ruleID'.\nExceptions:\n1:\n $requiredFailed\n2:\n $ex");
						}
					}
				}
				if (isset($realRule[MTConsts::REASON_RULE_HELPFILE])) {
					$ret[MTConsts::REASON_RULE_HELPFILE] = $this->get4RoleLocale($realRule[MTConsts::REASON_RULE_HELPFILE], $role, $locale, true);
				}
			}
			$this->_parsedRules[$ruleID] = $ret;
		}
		return $this->_parsedRules[$ruleID];
	}

	/**
	 * @todo comment
	 * @return Ambigous <multitype:array, multitype:>
	 */
	protected function getRules() {
		$rules = $this->getReasonIni();
		if (is_array($rules) && isset($rules[MTConsts::REASON_INI_RULES])) {
			return $rules[MTConsts::REASON_INI_RULES];
		} else {
			require_once 'Hpls/Mailtrace/Exception/Reason.php';
			throw new Hpls_Mailtrace_Exception_Reason("No [" . MTConsts::REASON_INI_RULES . "] defined in reason config file.");
		}
	}

	/**
	 * @todo comment
	 * @throws Hpls_Mailtrace_Exception_Reason If config file not existent.
	 * @return multitype:array
	 * 
	 * @throws Hpls_Exception_Configuration If the configuration is missing for Mailtrace.
	 */
	protected function getReasonIni() {
		if (is_null($this->_reasonIni)) {
			$modConfig = Hpls_Mailtrace::getConfig(); //exception is passed through to caller
			$reasonConf = null;
			try {
				//				if (DEBUG) {
				//					require_once 'Zend/Debug.php';
				//					Hpls_Mailtrace::i()->getLogger()->debug(
				//					Zend_Debug::dump($modConfig, " MT config: ", false));
				//				}
				$reasonConf = Hpls_Utils::getDeepArray($modConfig, array(
						MTConsts::CONF_MTLOG,
						MTConsts::CONF_REASON ));
			} catch (Hpls_Exception_KeyNotExists $ex) {
				require_once 'Hpls/Mailtrace/Exception/Reason.php';
				throw new Hpls_Mailtrace_Exception_Reason("Section " . implode($ex->getKeyPath(), '.') . " missing in component.ini for Mailtrace.");
			}
			$filename = $reasonConf[MTConsts::CONF_REASON_FILE];
			if (!file_exists($filename)) {
				//+++DEBUG
				if (DEBUG) {
					require_once 'Zend/Debug.php';
					Hpls_Mailtrace::i()->getLogger()->debug(Zend_Debug::dump($modConfig, " MT config: ", false));
				}
				//---DEBUG
				require_once 'Hpls/Mailtrace/Exception/Reason.php';
				throw new Hpls_Mailtrace_Exception_Reason("The reason configuration file '$filename' does not exist.");
			}
			$config = new Zend_Config_Ini($filename);
			$this->_reasonIni = $config->toArray();
			$this->checkReasonFormat($this->_reasonIni);
		}
		return $this->_reasonIni;
	}

	/**
	 * @todo comment
	 * Check the reason definition for correctnes.
	 * @param array $reasonsIni
	 */
	protected function checkReasonFormat($reasonsIni) {
		if (!is_array($reasonsIni)) {
			require_once 'Hpls/Mailtrace/Exception/Reason.php';
			throw new Hpls_Mailtrace_Exception_Reason('Reason config is not in array format.');
		}
		if (!isset($reasonsIni[MTConsts::REASON_INI_RULES])) {
			require_once 'Hpls/Mailtrace/Exception/Reason.php';
			throw new Hpls_Mailtrace_Exception_Reason("No [" . MTConsts::REASON_INI_RULES . "] defined in reason config file.");
		}
	}

	/**
	 * @todo comment
	 * @return string|NULL
	 */
	public function getDefaultRole() {
		if (is_null($this->_defaultRole)) {
			$reasonIni = $this->getReasonIni();
			if (is_array($reasonIni[MTConsts::REASON_GLOBAL_DEFAULT])) {
				$defaults = $reasonIni[MTConsts::REASON_GLOBAL_DEFAULT];
				if (isset($defaults[MTConsts::REASON_GLOBAL_DEFAULT_ROLE])) {
					$this->_defaultRole = $defaults[MTConsts::REASON_GLOBAL_DEFAULT_ROLE];
				}
			}
		}
		return $this->_defaultRole;
	}

	/**
	 * @todo comment
	 * @return string|NULL
	 */
	public function getDefaultLocale() {
		if (is_null($this->_defaultLocale)) {
			$reasonIni = $this->getReasonIni();
			if (is_array($reasonIni[MTConsts::REASON_GLOBAL_DEFAULT_ROLE])) {
				$defaults = $reasonIni[MTConsts::REASON_GLOBAL_DEFAULT];
				if (isset($defaults[MTConsts::REASON_GLOBAL_DEFAULT_LOCALE])) {
					$this->_defaultRole = $defaults[MTConsts::REASON_GLOBAL_DEFAULT_LOCALE];
				}
			}
		}
		return $this->_defaultLocale;
	}

	/**
	 * @todo comment
	 * 
	 * @todo implement role, locale fallback
	 * 
	 * @param array $valuePool
	 * @param unknown_type $role
	 * @param Zend_Locale $locale
	 */
	protected function get4RoleLocale(array $valuePool, $role = null, Zend_Locale $locale = null) {
		if (is_null($locale)) {
			$locale = $this->getDefaultLocale();
		}
		if (is_null($role)) {
			$role = $this->getDefaultRole();
		}
		$lang = strtolower($locale->getLanguage());
		$role = strtolower($role);
		$val = null;
		//role check
		if (!isset($valuePool[$role])) {
			if (DEBUG) {
				require_once 'Hpls/Mailtrace/Exception/Reason.php';
				throw new Hpls_Mailtrace_Exception_Reason("Role '$role' does not exists in given array." . Zend_Debug::dump($valuePool, '$valuePool: ', false));
			} else {
				$this->getLogger()->warning("Role '$role' does not exists in given array.");
				$role = $this->getDefaultRole();
				if (!isset($valuePool[$role])) {
					$this->getLogger()->warning("Default role '$role' does not exists in given array.");
					return null;
				}
			}
		}
		//language check
		$val = $valuePool[$role];
		if (!isset($val[$lang])) {
			if (DEBUG) {
				require_once 'Hpls/Mailtrace/Exception/Reason.php';
				throw new Hpls_Mailtrace_Exception_Reason("In role '$role' language '$lang' does not exists in given array. " . Zend_Debug::dump($valuePool, '$valuePool: ', false));
			} else {
				$this->getLogger()->warning("In role '$role' language '$lang' does not exists in given array.");
				$locale = $this->getDefaultLocale();
				$lang = strtolower($locale->getLanguage());
				if (!isset($val[$lang])) {
					$this->getLogger()->warning("In role '$role' default language '$lang' does not exists in given array.");
					return null;
				}
			}
		}
		//variable replacement
		return $val[$lang];
	}

	protected function getLogger() {
		return Hpls_Mailtrace::i()->getLogger();
	}
}