<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created   : Mar 25, 2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2010-04-07 16:42:09 +0200 (Mi, 07 Apr 2010) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 5129 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/HMT/GUI/Core/trunk/hpls.hmt.gui.libs/Hpls/Mailtrace/Model/Reason/Entry.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 */
/***/
require_once 'Hpls/Mailtrace.php';
require_once 'Hpls/Var/Parser.php';
require_once 'Hpls/Mailtrace/Utils.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package Mailtrace
 * @subpackage Reason
 * @version $LastChangedRevision: 5129 $
 * @since alpha
 *
 */
class Mailtrace_Model_Reason_Entry
{
	protected $_varParser = null;
	protected $_rule;
	protected $_ruleID;
	/**
	 * @var Mailtrace_Model_LogEntry
	 */
	protected $_logEntry;
	protected $_matchedParams = array();
	protected $_locale = null;
	protected $_role = null;

	/**
	 * @todo comment
	 * @param string $ruleID
	 * @param array $rule
	 * @param Mailtrace_Model_LogEntry $logEntry
	 * 
	 * @throws Hpls_Mailtrace_Exception_Reason
	 */
	public function __construct($ruleID, array &$rule,$role,Zend_Locale &$locale, Mailtrace_Model_LogEntry &$logEntry = null) {
		if (!is_array($rule)) {
			require_once 'Hpls/Mailtrace/Exception/Reason.php';
			throw new Hpls_Mailtrace_Exception_Reason('Rule is empty.');
		}
		if (empty($ruleID)) {
			require_once 'Hpls/Mailtrace/Exception/Reason.php';
			throw new Hpls_Mailtrace_Exception_Reason('A ruleID must be given.');
		}
		$this->_ruleID = $ruleID;
		$this->_rule = &$rule;
		$this->_logEntry = &$logEntry;
		$this->_locale = $locale;
		$this->_role = $role;
	}

	/**
	 * @todo comment
	 * @param Mailtrace_Model_LogEntry $logEntry
	 * 
	 * @throws Hpls_Mailtrace_Exception_Reason
	 */
	public function setLogEntry(Mailtrace_Model_LogEntry &$logEntry) {
		$this->_logEntry = &$logEntry;
		$parser = $this->getVarParser();
		$parser->setValue('T:C_ID', $this->handleValue($logEntry->getC_ID()));
		
		$value = $logEntry->getClient();
		$parser->setValue('T:C_CLIENT', $this->handleValue($value));
		
		if (!is_null($value)) {
			try {
			$split = Hpls_Mailtrace_Utils::splitIpName($value);
			if (is_array($split)) {
				$parser->setValue('T:C_CLIENT_NAME', $this->handleValue($split['name']));
				$parser->setValue('T:C_CLIENT_IP', $this->handleValue($split['ip']));
			}
			} catch (Hpls_Mailtrace_Exception $ex) {
				$this->getLogger()->error("Splitting C_Client($value) in NAME and IP failed: $ex");
			}
		}
		
//		$split = preg_match('/^[^\[]*\[([^[])\]/i',$logEntry->getClient());
		$parser->setValue('T:C_MAILFROM', $this->handleValue($logEntry->getMailFrom()));
		$parser->setValue('T:C_MSG_ID', $this->handleValue($logEntry->getMessageID()));
		$parser->setValue('T:C_SIZE', $this->handleValue($logEntry->getSize()));
		$parser->setValue('T:C_QID', $this->handleValue($logEntry->getConnectQueueID()));
		$parser->setValue('T:C_MXNAME', $this->handleValue($logEntry->getMxName()));
		$parser->setValue('T:C_TIME', $this->handleValue($logEntry->getConnectTime()));
		// deliverlog:
		$parser->setValue('T:D_ID', $this->handleValue($logEntry->getD_ID()));
		$parser->setValue('T:D_REF_C_ID', $this->handleValue($logEntry->getC_ID())); // TODO replace through REF_CID
		$parser->setValue('T:D_MAILTO', $this->handleValue($logEntry->getMailTo()));
		$parser->setValue('T:D_STATUS', $this->handleValue($logEntry->getStatus()));
		$parser->setValue('T:D_REASON', $this->handleValue($logEntry->getReason()));
		
		$value = $logEntry->getRelayHost();
		$parser->setValue('T:D_RELAYHOST', $this->handleValue($value));
		
		if (!is_null($value)) {
			try {
			$split = Hpls_Mailtrace_Utils::splitIpName($value);
			if (is_array($split)) {
				$parser->setValue('T:D_RELAYHOST_NAME', $this->handleValue($split['name']));
				$parser->setValue('T:D_RELAYHOST_IP', $this->handleValue($split['ip']));
			}
			} catch (Hpls_Mailtrace_Exception $ex) {
				$this->getLogger()->error("Splitting D_RELAYHOST($value) in NAME and IP failed: $ex");
			}
		}
		$parser->setValue('T:D_QID', $this->handleValue($logEntry->getDeliverQueueID()));
		$parser->setValue('T:D_TIME', $this->handleValue($logEntry->getDeliverTime()));
		$this->match();
	}

	/**
	 * @todo comment
	 * @return string the regex expression directly usable in preg_*
	 */
	public function getRegex() {
		return $this->_rule[MTConsts::REASON_RULE_REGEX];
	}

	/**
	 * @todo comment
	 * @return array:string
	 */
	public function getMatchedParams() {
		return $this->_matchedParams;
	}

	/**
	 * @todo comment
	 * @return string|NULL
	 */
	public function getHelpText() {
		$this->getLogger()->debug("In getHelpText()");
		$rule = $this->getRule();
		//first try -> text in reason.ini
		if (isset($rule[MTConsts::REASON_RULE_HELPTEXT])) {
			$ret = $rule[MTConsts::REASON_RULE_HELPTEXT];
			$this->getLogger()->debug("getHelpText(): using helpText '$ret'.");
		}
		//second try -> file
		if (empty($ret)) {
			$this->getLogger()->debug("getHelpText(): file required.");
			$file = null;
			if (isset($rule[MTConsts::REASON_RULE_HELPFILE])) {
				$file = $rule[MTConsts::REASON_RULE_HELPFILE];
			} // no else because return can else be NULL => $file=== null 
			//still no result? 
			if (is_null($file)) {
				$file = "$this->_role." . $this->getRuleID().".".$this->_locale->getLanguage() . ".html";
			}
			if (!Hpls_Utils::isAbsolutePath($file)) {
				//contruct from reason text dir and ruleID
				try {
					$textDir = Hpls_Mailtrace::i()->getConfig(
					array(MTConsts::CONF_MTLOG, MTConsts::CONF_REASON, 
					MTConsts::CONF_REASON_TEXTDIR));
				} catch (Hpls_Exception_KeyNotExists $ex) {
					$this->getLogger()->warning(
					"Mailtrace configuration does not contain " . implode('.', 
					$ex->getKeyPath()));
					$textDir = null;
				}
				if (empty($textDir) || !is_dir($textDir)) {
					//error handling
					$exText = empty($textDir) ? "Default reason text dir is not configured in component.ini for Mailtrace." : "Default reason text dir configured in component.ini for Mailtrace '$textDir' is not existent.";
					require_once 'Hpls/Exception/Configuration.php';
					throw new Hpls_Exception_Configuration($exText);
				}
				$file = $textDir . DIRECTORY_SEPARATOR . $file;
			}
			if (file_exists($file)) {
				$buffer = "";
				$handle = @fopen($file, 'r');
				if (!$handle) {
					require_once 'Hpls/Mailtrace/Exception/Reason.php';
					throw new Hpls_Mailtrace_Exception_Reason(
					"Could not open reason explanation file '$file'.");
				}
				while (!feof($handle)) {
					$buffer .= fread($handle, 1024);
				}
				fclose($handle);
				$ret = $buffer;
				unset($buffer);
				unset($handle);
			} else {
				require_once 'Hpls/Mailtrace/Exception/Reason.php';
				throw new Hpls_Mailtrace_Exception_Reason(
				"Reason explanation file '$file' for rule '" . $this->getRuleID() . "' does not exists.");
			}
			//TODO
		}
		return $this->replaceVariables($ret); //."  Short: ".$this->getShort();
	}

	/**
	 * @todo comment
	 * @return string|NULL
	 */
	public function getHelpLink() {
		$rule = $this->getRule();
		if (isset($rule[MTConsts::REASON_RULE_HELPLINK])) {
			return $this->replaceVariables($rule[MTConsts::REASON_RULE_HELPLINK]);
		}
		return null;
	}

	/**
	 * @todo comment
	 * @return string|NULL
	 */
	public function getShort() {
		$rule = $this->getRule();
		if (isset($rule[MTConsts::REASON_RULE_SHORT])) {
			return $this->replaceVariables($rule[MTConsts::REASON_RULE_SHORT]);
		}
		return null;
	}

	/**
	 * @todo comment
	 */
	public function getRule() {
		return $this->_rule;
	}

	public function getRuleID() {
		return $this->_ruleID;
	}

	/**
	 * @todo comment
	 * @return Mailtrace_Model_LogEntry
	 */
	public function getLogEntry() {
		return $this->_logEntry;
	}

	/**
	 * @todo comment
	 * @param string $text
	 * @return string
	 */
	public function replaceVariables($text) {
		$varParser = $this->getVarParser();
		$varParser->setText($text, false);
//		$this->getLogger()->debug(
//		"replaceVariables(): text: '$text'" . Zend_Debug::dump($varParser->getVarValues(), 
//		"\n Used Values:", false));
		$varParser->parse();
		$ret = $varParser->getReplaced();
//		$this->getLogger()->debug("replaceVariables(): " . Zend_Debug::dump($ret, "Replaced: ", 
//		false));
		return $ret;
	}

	public function __toString() {
		return $this->_ruleID;
	}

	/**
	 * @todo comment
	 */
	protected function match() {
		$reason = $this->getLogEntry()->getReason();
		$regex = $this->getRegex();
		$matches = array();
		$results = preg_match($this->getRegex(), $reason, $matches);
		if ($results === false || $results <= 0) {
			$ruleID = $this->getRuleID();
			throw new Hpls_Mailtrace_Exception_Reason(
			"The combination of rule '$ruleID' and reason: \n$reason\n is not possible.");
		}
		$varNames = $this->getRule();
		$varParser = $this->getVarParser();
		$this->_matchedParams = array();
		if (isset($varNames[MTConsts::REASON_RULE_VARNAMING]))
			$varNames = $varNames[MTConsts::REASON_RULE_VARNAMING];
		else
			$varNames = null;
		for($i = 1; $i < count($matches); $i ++) {
			$val = $this->handleValue($matches[$i]);
			$varParser->setValue($i, $val);
			$this->_matchedParams[$i] = $val;
			if (!is_null($varNames) && !empty($varNames[$i])) {
				$varParser->setValue($varNames[$i], $val);
				$this->_matchedParams[$varNames[$i]] = $matches[$i];
			}
		}
//		Hpls_Mailtrace::i()->getLogger()->debug(
//		"Found matches for regex '$regex' in reason: \n$reason\n " . Zend_Debug::dump($matches, 
//		"MATCHES: ", false) . Zend_Debug::dump($this->_matchedParams, "\nNAMED MATCHES: ", false) .
//		 Zend_Debug::dump($this->getVarParser()->getVarValues(), "\nNAMED VARVALUES: ", false));
	}

	/**
	 * @todo comment
	 * @return Hpls_Var_Parser
	 */
	protected function getVarParser() {
		if (is_null($this->_varParser)) {
			$this->_varParser = new Hpls_Var_Parser();
		}
		return $this->_varParser;
	}

	protected function handleValue($text) {
		if (is_null($text)) {
			$text = '<NULL>';
		}
		$text = htmlentities($text);
		return "<span class=\"reasonVarValue".(DEBUG?" DEBUG":"")."\">$text</span>";
	}

	protected function getLogger() {
		return Hpls_Mailtrace::i()->getLogger();
	}
}