<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 * 
 * created   : Sep 7, 2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28 Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Navigation/Loader.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Consts0.php';
require_once 'Hpls/Utils/Logusing.php';
require_once 'Hpls/Navigation/Page/Tab.php';
require_once 'Hpls/Navigation/Page/Uri.php';
require_once 'Hpls/Navigation/Page/Controller.php';
require_once 'Hpls/Navigation/Page/Root.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 9196 $
 * @since 1.0.3
 */
class Hpls_Navigation_Loader extends Hpls_Utils_Logusing {
	protected static $PRODUCTDIRS;
	
	protected $_result = null;
	protected $_idMap = array();
	protected $_loadRunned = FALSE;

	public function __construct() {
		$this->setLogger(Hpls_Utils::getLogger());
		self::$PRODUCTDIRS = Hpls_Application::i()->getInstalledHPLSProducts();
	
		//		$content = @file_get_contents($path)
	}

	public function getResult() {
		if (FALSE === $this->_loadRunned) {
			$this->load();
		}
		return $this->_result;
	}

	public function getIdMap() {
		if (FALSE === $this->_loadRunned) {
			$this->load();
		}
		return $this->_idMap;
	}

	/**
	 * Hauptfunktion und Einstiegspunkt.
	 * @todo comment
	 */
	public function load() {
		$ret = new Hpls_Navigation_Page_Root();
		$confDir = Hpls_Application::i()->getConfigDir();
		isDebug('config_read', 'navigation') && $this->log()->debug(__METHOD__ . " *** " . print_r($confDir, true));
		foreach (self::$PRODUCTDIRS as $product => $productdir) {
			$navConfDir = "$confDir/$productdir/navigation";
			if (false === file_exists($navConfDir) || false === is_dir($navConfDir)) {
				continue;
			}
			$dircontent = scandir($navConfDir);
			foreach ($dircontent as $file) {
				isDebug('config_read', 'navigation') && $this->log()->debug(__METHOD__ . " *** trying '$file'");
				
				if (preg_match('/' . Hpls_Consts0::MENU_CONFIG_NAME_POSTFIX . '$/', $file) > 0) {
					isDebug('config_read', 'navigation') && $this->log()->debug(__METHOD__ . " *** found '$file'");
					//					$file = "$confDir/$productdir/".;
					

					//			if (file_exists($file)) {
					//				Hpls_Utils::log()->debug("Reading menu config from: $file");
					$content = file_get_contents("$navConfDir/$file");
					$tmpContent = simplexml_load_string($content);
					$result = $this->parseContent($tmpContent);
					isDebug('config_read', 'navigation') && $this->log()->debug(__METHOD__ . " *** CONTENT " . Hpls_Debug::print_r($result));
					foreach ($result as $id => $tabMenu) {
						$ret->addPage($tabMenu);
						$this->addID($id, $tabMenu);
					}
				
		//			}
				}
			}
		}
		$this->_result = $ret;
		$this->_loadRunned = true;
	}

	protected function addID($id, Zend_Navigation_Page $page) {
		if (isset($this->_idMap[(string) $id])) {
			require_once 'Hpls/Exception/Navigation.php';
			throw new Hpls_Exception_Navigation("$id is defined twice. Correct it in *.menu.xml files and try again.");
		}
		$this->_idMap[(string) $id] = $page;
	}

	/**
	 * Einstiegspunkt für das Parsen des Inhalt einer *.menu.xml Datei.
	 * 
	 * @param SimpleXMLElement $content
	 * @return multitype:Ambigous <NULL, Hpls_Navigation_Page_Uri, Hpls_Navigation_Page_Controller> 
	 */
	protected function parseContent(SimpleXMLElement $content) {
		$ret = array();
		if (isset($content->tab)) {
			foreach ($content->tab as $tab) {
				isDebug('config_read', 'navigation') && $this->log()->debug(__METHOD__ . " have tab " . Hpls_Debug::print_r($tab));
				$tabNav = $this->parseTab($tab);
				if (!is_null($tabNav) && Hpls_Acl::i()->isAllowedAction($tabNav->getModule(), $tabNav->getController(), $tabNav->getAction())) {
					$ret[(string) $tab['id']] = $tabNav;
				}
			}
		}
		return $ret;
	}

	protected function parseTab(SimpleXMLElement $tabXml) {
		if (!$this->isInContext($tabXml)) {
			return null;
		}
		$ret = $this->createNavClass($tabXml);
		$this->parseCommonAttributes($tabXml, $ret);
		
		if (isset($tabXml->block)) {
			foreach ($tabXml->block as $block) {
				$blockNav = $this->parseBlock($block);
				if (!empty($blockNav)) {
					$ret->addPage($blockNav);
				}
			}
		}
		
		return $ret;
	}

	protected function parseBlock(SimpleXMLElement $blockXml) {
		return $this->parseMenuItem($blockXml);
	}

	protected function parseMenuItem(SimpleXMLElement $menuXml) {
		if (!$this->isInContext($menuXml)) {
			return null;
		}
		
		if (isset($menuXml->menuItem)) {
			$ret = $this->parseContainer($menuXml);
		} else {
			$ret = $this->parseSingleItem($menuXml);
		}
		return $ret;
	}

	protected function parseContainer(SimpleXMLElement $menuXml) {
		$ret = $this->createNavClass($menuXml);
		$this->parseCommonAttributes($menuXml, $ret);
		foreach ($menuXml->menuItem as $menuItem) {
			$page = $this->parseMenuItem($menuItem);
			if (!empty($page)) {
				if (Hpls_Acl::i()->isAllowedAction($page->getModule(), $page->getController(), $page->getAction(), $page->getParams())) {
					$ret->addPage($page);
				}
			}
		}
		return $ret;
	}

	protected function parseSingleItem(SimpleXMLElement $menuXml) {
		$ret = $this->createNavClass($menuXml);
		$this->parseCommonAttributes($menuXml, $ret);
		return $ret;
	}

	protected function parseCommonAttributes(SimpleXMLElement $itemXml, Zend_Navigation_Page $itemPage) {
		$itemPage->setId((string) $itemXml['id']);
		if (isset($itemXml['hidden']) && TRUE === Hpls_Utils::toBool($itemXml['hidden'])) {
			$itemPage->setVisible(false);
		}
		if (isset($itemXml['enabled'])) {
			$itemPage->setActive(Hpls_Utils::toBool($itemXml['enabled']));
		}
		if (isset($itemXml['lcID'])) {
			$itemPage->setLabel((string) $itemXml['lcID']);
		} else {
		
		}
		if (isset($itemXml['order'])) {
			$itemPage->setOrder((int) $itemXml['order']);
		}
		if (isset($itemXml->url)) {
			$itemPage->setUri((string) $itemXml->url[0]['href']);
		} elseif (isset($itemXml->mapsTo)) {
			$this->parseMapToController($itemXml->mapsTo[0], $itemPage);
		}
	
		//		$this->parseAcls($itemXml, $itemPage);
	}

	protected function createNavClass(SimpleXMLElement $menuXml) {
		if ($menuXml->getName() == 'tab') {
			return new Hpls_Navigation_Page_Tab();
		} elseif (isset($menuXml->url)) {
			return new Hpls_Navigation_Page_Uri();
		} elseif (isset($menuXml->mapsTo)) {
			return new Hpls_Navigation_Page_Controller();
		} elseif (isset($menuXml->none)) {
			return new Hpls_Navigation_Page_Uri();
		}
		require_once 'Hpls/Exception/Navigation.php';
		throw new Hpls_Exception_Navigation('<url> or <mapsTo> is missing in ' . (string) $menuXml->asXML());
		return null;
	}

	protected function parseMapToController(SimpleXMLElement $mapToXml, Hpls_Navigation_Page_Controller $itemPage) {
		isDebug('config_read', 'navigation') && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($mapToXml));
		if (!$this->isInContext($mapToXml)) {
			//wenn mapsTo nicht die benötigten bedingungen vorfindet -> nichts tun
			return;
		}
		$controller = null;
		$module = null;
		$action = null;
		if (isset($mapToXml['c'])) {
			$controller = (string) $mapToXml['c'];
		}
		if (isset($mapToXml['controller'])) {
			$controller = (string) $mapToXml['controller'];
		}
		if (isset($mapToXml['m'])) {
			$module = (string) $mapToXml['m'];
		}
		if (isset($mapToXml['module'])) {
			$module = (string) $mapToXml['module'];
		}
		if (isset($mapToXml['a'])) {
			$action = (string) $mapToXml['a'];
		}
		if (isset($mapToXml['action'])) {
			$action = (string) $mapToXml['action'];
		}
		$itemPage->setAction($action);
		$itemPage->setController($controller);
		$itemPage->setModule($module);
		isDebug('config_read', 'navigation') && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($itemPage));
	}

	/**
	 * Gibt zurück, ob dieses Eintrag im aktuellen Kontext (eingeloggter User, Lizenz) der entsprechende Eintrag gültig ist, oder nicht.
	 * @param SimpleXMLElement $xml
	 * @return boolean
	 */
	protected function isInContext(SimpleXMLElement $xml) {
		if (isset($xml['showOn'])) {
			switch (strtolower((string)$xml['showOn'])) {
				case'appliance':
				case 'a':
				case 'app':
					return !Hpls_Application0::i()->isStandAlone();
				case 'standalone':
				case 's':
					return Hpls_Application0::i()->isStandAlone();
				case 'all':
				default:
					break;
			}
		}
		if (isset($xml->acl)) {
		
		}
		return true;
	}
}