<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 18.12.2010
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 *
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision$
 * @since 1.0.7
 */
class Hpls_Protocol_Factory {
	protected static $_instance = null;
	protected $_persistenceMap = array();
	
	protected $_componentClassMap = array(
			'generic' => array(
					'class' => 'Hpls_Protocol_Entry_Generic',
					'path' => 'Hpls/Protocol/Entry/Generic.php' ) );

	/**
	 * @return Hpls_Protocol_Factory
	 */
	public static function i() {
		if (is_null(self::$_instance)) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	protected function __construct() {
	}

	/**
	 * Enter description here ...
	 * @return Hpls_Protocol_PersistenceAbstract
	 */
	public function getReader() {
		$app = Hpls_Application::i()->getApplicationType();
		switch ($app) {
			case Hpls_Consts0::APP_MAILAPP_FRONTEND :
				if (!isset($this->_persistenceMap[Hpls_Consts0::APP_MAILAPP_FRONTEND])) {
					require_once 'Hpls/Protocol/Persistence/Backend.php';
					$this->_persistenceMap[Hpls_Consts0::APP_MAILAPP_FRONTEND] = new Hpls_Protocol_Persistence_Backend();
				}
				break;
			case Hpls_Consts0::APP_MAILAPP_BACKEND :
				if (!isset($this->_persistenceMap[Hpls_Consts0::APP_MAILAPP_BACKEND])) {
					require_once 'Hpls/Protocol/Persistence/Ldap.php';
					$this->_persistenceMap[Hpls_Consts0::APP_MAILAPP_BACKEND] = new Hpls_Protocol_Persistence_Ldap();
				}
				break;
			case Hpls_Consts0::APP_MAILTRACE_FRONTEND :
				if (!isset($this->_persistenceMap[Hpls_Consts0::APP_MAILTRACE_FRONTEND])) {
					if (Hpls_Application::i()->isStandAlone()) {
						require_once 'Hpls/Protocol/Persistence/File.php';
						$this->_persistenceMap[Hpls_Consts0::APP_MAILTRACE_FRONTEND] = new Hpls_Protocol_Persistence_File();
					} else {
						//in der appliance
						$this->_persistenceMap[Hpls_Consts0::APP_MAILTRACE_FRONTEND] = new Hpls_Protocol_Persistence_Backend();
					}
				}
				break;
			default :
				$app = 'default';
				require_once 'Hpls/Protocol/Persistence/Null.php';
				return new Hpls_Protocol_Persistence_Null();
				break;
		}
		return $this->_persistenceMap[$app];
	}

	/**
	 * Enter description here ...
	 * @return Hpls_Protocol_PersistenceAbstract
	 */
	public function getWriter() {
		return $this->getReader();
	}

	public function createEntry($component, $time = null) {
		$compReg = $this->_componentClassMap[$component];
		if (!empty($compReg)) {
			require_once $compReg['path'];
			return new $comReg['class']($component, $time);
		}
		return null;
	
	}

	public function registerComponent($componentName, $className, $pathToEntryClass) {
	
	}

	protected function createBackendProxy() {
		require_once 'Hpls/Protocol/Persistence/Backend.php';
		require_once 'Mailapp/Soap/Proxy.php';
		
		$auth = Zend_Auth::getInstance();
		$id = $auth->getIdentity();
		$options = Zend_Controller_Front::getInstance()->getParam('soap');
		$principal = null;
		if ($id['role'] == 'admin') {
			$_principal = array(
					'username' => $id['username'],
					'password' => $id['password'] );
		} else {
			$mailarchiveConfig = Zend_Controller_Front::getInstance()->getParam('mailarchive');
			$principal = array(
					'username' => $mailarchiveConfig['mailarchiveUsername'],
					'password' => $mailarchiveConfig['mailarchivePassword'] );
		}
		if (is_null($options) || is_null($principal)) {
			require_once 'Hpls/Exception/Configuration.php';
			throw new Hpls_Exception_Configuration("Backend-SOAP access point is not configured.");
		}
		
		$soap = new Mailapp_Soap_Proxy($options, $principal['username'], $principal['password']);
		
		return new Hpls_Protocol_Persistence_Backend($soap);
	}
}