<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 28.04.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28 Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Settings.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Utils/Logusing.php';
require_once 'Hpls/Settings/Handler/Component.php';

/**
 * @todo comment
 * 
 * @todo Sollte lazy-loading unterstützen
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 9196 $
 * @since 1.0.9
 */
class Hpls_Settings extends Hpls_Utils_Logusing {
	const KEY_DELIMITER = '.';
	
	protected static $_instance = null;

	/**
	 * @return Hpls_Settings
	 */
	public static function i() {
		if (is_null(self::$_instance)) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

	public static function setInstance(Hpls_Settings $instance) {
		self::$_instance = $instance;
	}
	
	/**
	 * @todo comment
	 * @var unknown_type
	 * @deprecated
	 */
	protected $_children = array();
	/**
	 * @todo comment
	 * @var Hpls_Settings_Container_Component
	 * @deprecated
	 */
	protected $_compAC = null;
	
	protected $_handlerRegistry = array();

	protected function __construct() {
	}

	/**
	 * @todo comment
	 * @param string|array:string $settingsKey
	 * @param boolean $uncached Aus dem Cache laden (==false) oder direkt aus dem Persitenzspeicher (==true)?
	 * @param boolean $autoload
	 * @param boolean $create
	 * @return Zend_Config|null <null>, wenn kein Handler gefunden wurde oder wenn der $settingsKeys nicht gefunden wurde und $create == false.
	 */
	public function getConfig($settingsKey, $uncached = false, $autoload = true, $create = false) {
		$handler = $this->getHandler($settingsKey);
		if ($handler !== null) {
			isDebug('settings_read') && $this->log()->debug(Hpls_Debug::print_r($handler, __METHOD__ . " FOUND HANDLER"));
			return $handler['handler']->getConfig($handler['subkey'], $autoload, $create, $uncached);
		} else {
			$this->log()->err(Hpls_Debug::print_r($settingsKey, __METHOD__ . " NO HANDLER FOUND for"));
			return null;
		}
	
	}

	/**
	 * @todo comment
	 * @param unknown_type $settingsKey
	 * @param Zend_Config $config
	 * @return boolean
	 * @throws Zend_Config_Exception
	 */
	public function setConfig($settingsKey, Zend_Config $config, $doSave = true) {
		$handler = $this->getHandler($settingsKey);
		isDebug('settings_write') && $this->log()->debug(Hpls_Debug::print_r($handler, __METHOD__));
		if ($handler !== null) {
			$handler['handler']->setConfig($handler['subkey'], $config);
			if ($doSave)
				$handler['handler']->save();
			return $handler['handler'];
		}
		return $this;
	}

	/**
	 * @todo comment
	 * @param array:string|string $key
	 * @param Hpls_Settings_HandlerAbstract $handler
	 */
	public function registerHandler($key, Hpls_Settings_HandlerAbstract $handler, $rebind = false) {
		if (is_array($key)) {
			$key = implode(self::KEY_DELIMITER, $key);
		}
		if (!isset($this->_handlerRegistry[$key]) || $rebind) {
			$this->_handlerRegistry[$key] = $handler;
		} else {
			require_once 'Hpls/Exception/Settings.php';
			throw new Hpls_Exception_Settings("$key already registered", -1);
		}
	}

	/**
	 * @todo comment
	 * @param array:string|string $key
	 * @return null|array:[subkey]:string;[handler]:Hpls_Settings_HandlerAbstract
	 */
	public function getHandler($key) {
		(isDebug('settings_read') || isDebug('settings_write')) && $this->log()->debug(Hpls_Debug::print_r($key, __METHOD__));
		$key = strtolower($key);
		$len = strlen($key);
		$maxFoundLen = 0;
		$foundKey = null;
		foreach ($this->_handlerRegistry as $regKey => $handler) {
			if (strpos($key, $regKey) === 0) {
				//regKey ist ein substring von $key
				$regKeyLen = strlen($regKey);
				if ($regKeyLen == $len) {
					//vollständiger match
					(isDebug('settings_read') || isDebug('settings_write')) && $this->log()->debug(Hpls_Debug::print_r($regKey, __METHOD__ . " FOUND "));
					return array(
							'subkey' => '',
							'handler' => $handler );
				} elseif (($regKeyLen > $maxFoundLen) && ($key[$regKeyLen] == Hpls_Settings::KEY_DELIMITER)) {
					$maxFoundLen = $regKeyLen;
					$foundKey = $handler;
				}
			}
		}
		(isDebug('settings_read') || isDebug('settings_write')) && $this->log()->debug(Hpls_Debug::print_r($foundKey, __METHOD__ . " FOUND OR NOT "));
		
		if ($foundKey !== null) {
			return array(
					'subkey' => substr($key, $maxFoundLen + 1),
					'handler' => $foundKey );
		}
		return null;
	}

	public function write($key) {
		$handler = $this->getHandler($key);
	
	}

	/**
	 * @todo comment
	 * @deprecated
	 */
	public function save() {
		foreach ($this->_children as $child) {
			$child->save();
		}
	}

	/**
	 * @todo comment
	 * @deprecated
	 */
	public function load() {
		foreach ($this->_children as $child) {
			$child->load();
		}
	}
	
	public function clearCache() {
		foreach ($this->_children as $child) {
			$child->clearCache();
		}
	}

	public function addChild(Hpls_SettingsAbstract $settings) {
		$this->_children[$settings->getKey()] = $settings;
	}

}