<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created   : Jan 24, 2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2012-11-26 15:28:20 +0100 (Mo, 26 Nov 2012) $
 * last changed by: $LastChangedBy: d.mohn $
 * revision       : $LastChangedRevision: 9842 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Utils.php $
 *
 */
/**
 * *
 */
require_once 'Hpls/Consts0.php';
require_once 'Zend/Registry.php';
require_once 'Zend/Log.php';
require_once 'Zend/Config/Writer/Xml.php';

function array_walk_trim(&$item1, $key) {
	$item1 = trim($item1);
}

/**
 *
 * @todo comment
 *      
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *           
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 9842 $
 * @since ??
 */
abstract class Hpls_Utils {
	
	/**
	 *
	 * @var Zend_Log
	 */
	private static $_NullLogger = null;
	private static $_loggerRegged = false;

	/**
	 *
	 * @param $basepath string       	
	 * @param $fileprefix string       	
	 * @param $section string|null       	
	 * @return Zend_Config_Ini
	 */
	public static function readConfigIniWDelta($basepath, $fileprefix, $section = null) {
		if (!defined(MA_HOSTNAME) || MA_HOSTNAME === '')
			return new Zend_Config_Ini($basepath . "/$fileprefix.ini", null, true);
		$configFile = $basepath . "/$fileprefix.ini";
		$config = new Zend_Config_Ini($configFile, $section, true);
		$hostconfigfile = $basepath . "/" . MA_HOSTNAME . ".$fileprefix.delta.ini";
		if (file_exists($hostconfigfile)) {
			$config->merge(new Zend_Config_Ini($hostconfigfile));
		}
		unset($hostconfigfile);
		return $config;
	}

	/**
	 * Liest eine INI-Datei mit Namen $filename gemergt aus mehreren Verzeichnen
	 * ($paths), die Unterverzeichnisse von $basepath sind.
	 *
	 *
	 * @param $filename string       	
	 * @param $section string       	
	 * @param $paths array       	
	 * @param $basepath string       	
	 * @return Zend_Config
	 */
	public static function readConfigIniCascade($filename, $section = null, array $paths = array('.','AC','HMA','HIM','HAS','HMT'), $basepaths = null) {
		return self::readConfigCascaded($filename, $section, $paths, $basepaths, 'Zend_Config_Ini');
	}

	public static function readConfigXmlCascade($filename, $section = null, array $paths = array('.','AC','HMA','HIM','HAS','HMT'), $basepaths = null) {
		return self::readConfigCascaded($filename, $section, $paths, $basepaths, 'Zend_Config_Xml');
	}

	public static function readConfigCascaded($filename, $section = null, array $paths = array('.','AC','HMA','HIM','HAS','HMT'), $basepaths = null, $readerClass = 'Zend_Config_Ini') {
		isDebug('config_read') && self::log()->debug(__METHOD__ . "  Will read '$filename' from " . Hpls_Debug::print_r($paths, "Paths"));
		if (empty($basepaths)) {
			$basepaths = array(
					Hpls_Application0::i()->getConfigDir(),
					Hpls_Application0::i()->getVariableConfigDir() );
		}
		if (!is_array($basepaths)) {
			$basepaths = array(
					$basepaths );
		}
		if (count($paths) === 0) {
			$paths = array(
					'' );
		}
		isDebug('config_read') && self::log()->debug(__METHOD__ . Hpls_Debug::print_r($basepaths, "Basepath"));
		$ini = new Zend_Config(array(), true);
		foreach ($basepaths as $basepath) {
			foreach ($paths as $path) {
				$file = '';
				if (FALSE === empty($basepath)) {
					$file .= $basepath;
				}
				if (!empty($path)) {
					$file .= (empty($file) ? '' : DIRECTORY_SEPARATOR) . $path;
				}
				if (!empty($filename)) {
					$file .= (empty($file) ? '' : DIRECTORY_SEPARATOR) . $filename;
				}
				isDebug('config_read') && self::log()->debug(__METHOD__ . ": Trying $file");
				
				if (file_exists($file)) {
					isDebug('config_read') && self::log()->debug(__METHOD__ . ": Reading $file");
					$tmpIni = new $readerClass($file, $section, true);
					$ini->merge($tmpIni);
				}
			}
		}
		isDebug('config_read') && self::log()->debug(__METHOD__ . " END");
		return $ini;
	}

	/**
	 *
	 * @param $frontend unknown_type       	
	 * @param $backend unknown_type       	
	 * @param $frontOptions unknown_type       	
	 * @param $backOptions unknown_type       	
	 * @return Ambigous <Zend_Cache_Core, Zend_Cache_Frontend>
	 */
	public static function getCache($frontend = null, $backend = null, &$frontOptions = array(), &$backOptions = array()) {
		$config = self::getConfig();
		if (is_null($frontend)) {
			$frontend = $config['cache']['default']['frontend']['type'];
		}
		if (is_null($backend)) {
			$backend = $config['cache']['default']['backend']['type'];
		}
		if (count($frontOptions) === 0) {
			$frontOptions = $config['cache']['default']['frontend'];
			unset($frontOptions['type']);
		}
		if (count($backOptions) === 0) {
			$backOptions = $config['cache']['default']['backend'];
			unset($backOptions['type']);
		}
		return Zend_Cache::factory($frontend, $backend, $frontOptions, $backOptions);
	}

	/**
	 *
	 * @todo comment
	 *      
	 * @return unknown_type
	 */
	public static function getLogger() {
		return self::log();
	}

	/**
	 *
	 * @todo comment
	 *      
	 * @return Zend_Log
	 */
	public static function log() {
		if (self::$_loggerRegged) {
			return Zend_Registry::get(Hpls_Consts0::REGK_LOGGER);
		} elseif (Zend_Registry::isRegistered(Hpls_Consts0::REGK_LOGGER)) {
			self::$_loggerRegged = true;
			return Zend_Registry::get(Hpls_Consts0::REGK_LOGGER);
		} else {
			if (is_null(self::$_NullLogger)) {
				require_once 'Zend/Log/Writer/Null.php';
				require_once 'Hpls/Log.php';
				self::$_NullLogger = new Hpls_Log(new Zend_Log_Writer_Null());
			}
			return self::$_NullLogger;
		}
	}

	/**
	 *
	 * @todo comment
	 *      
	 * @return unknown_type
	 */
	public static function &getComponents() {
		$comps = Zend_Registry::get(Hpls_Consts0::REGK_COMPONENTS);
		return $comps;
	}

	/**
	 *
	 * @deprecated User Hpls_Application::getComponentConfig()
	 * @param $comp unknown_type       	
	 * @return NULL
	 */
	public static function &getCompConfig($comp) {
		$comps = self::getComponents();
		if (!empty($comps)) {
			return $comps[$comp][Hpls_Consts0::CFG_COMP_CONFIG];
		}
		return NULL;
	}

	public static function &getLocale() {
		return Zend_Registry::get(Hpls_Consts0::REGK_LOCALE);
	}

	public static function getI18N() {
		return Zend_Registry::get(Hpls_Consts0::REGK_I18N);
	}

	public static function getConfig() {
		return Zend_Registry::get(Hpls_Consts0::REGK_CONFIG);
	}

	public static function array_deep_value(array &$valueArray, $keyPath) {
		$ret = $valueArray;
		$path = array();
		
		if (!is_array($keyPath)) {
			$keyPath = preg_split('/(\/|\.)/', $keyPath); // explode('/',
				                                              // $keyPath);
		}
		
		foreach ($keyPath as $key) {
			$path[] = $key;
			if (isset($ret[$key])) {
				$ret = $ret[$key];
			} else {
				return null;
			}
		}
		return $ret;
	}

	/**
	 * Holt aus dem verschachtelten Array $valueArray das Array heraus, welches
	 * durch den Pfad $keyPath spezifiziert wurde.
	 * Ist es kein Array oder existiert der Pfad nicht, wird eine Exception
	 * geworfen.
	 *
	 * @param $valueArray array       	
	 * @param $keyPath string|array       	
	 * @return array:<multitype:, unknown>
	 * @throws Hpls_Exception_KeyNotExists Wenn einer der Schlüssel in $keyPath
	 *         nicht in $valueArray existiert
	 */
	public static function getDeepArray(array &$valueArray, $keyPath) {
		$ret = $valueArray;
		$path = array();
		
		if (!is_array($keyPath)) {
			$keyPath = preg_split('/(\/|\.)/', $keyPath); // explode('/',
				                                              // $keyPath);
		}
		
		foreach ($keyPath as $key) {
			$path[] = $key;
			if (!isset($ret[$key])) {
				require_once 'Hpls/Exception/KeyNotExists.php';
				throw new Hpls_Exception_KeyNotExists(implode("/", $path) . " does not exists.", $path);
			}
			$ret = $ret[$key];
			if (!is_array($ret)) {
				require_once 'Hpls/Exception/KeyNotExists.php';
				throw new Hpls_Exception_KeyNotExists(implode("/", $path) . " is not an array.", $path);
			}
		}
		return $ret;
	}

	/**
	 * Holt aus dem verschachtelten Array $valueArray das Array heraus, welches
	 * durch den Pfad $keyPath spezifiziert wurde.
	 * Existiert der Pfad nicht, wird eine Exception geworfen.
	 *
	 * @param $valueArray array       	
	 * @param $keyPath array       	
	 * @throws Hpls_Exception_KeyNotExists wenn $keyPath nicht existiert oder am
	 *         Ende kein Array ist.
	 * @return Ambigous <multitype:, unknown>
	 */
	public static function getFromDeepArray(array &$valueArray, $keyPath) {
		$ret = $valueArray;
		$path = array();
		
		if (!is_array($keyPath)) {
			$keyPath = explode('/', $keyPath);
		}
		
		$keyCount = count($keyPath); // for recognition of last key array entry in
		                             // foreach
		$i = 0;
		
		// descend into $valueArray
		foreach ($keyPath as $key) {
			$path[] = $key;
			$i++;
			$ret = $ret[$key];
			if ($keyCount == $i) { // last
				break;
			}
			if (!is_array($ret)) {
				require_once 'Hpls/Exception/KeyNotExists.php';
				throw new Hpls_Exception_KeyNotExists(implode("/", $path) . " does not exists or is not an array.", $path);
			}
		}
		return $ret;
	}

	public static function isAbsolutePath($path) {
		return strncmp($path, DIRECTORY_SEPARATOR, 1) === 0;
	}

	/**
	 * Konvertiert einen String in einen Boolean.
	 * <p>
	 *
	 * <i>false,0,'',no,n,&lt;null&gt;</i> ergeben <b>false</b> alles andere
	 * ergibt <b>true</b>
	 *
	 * @param $string unknown_type       	
	 * @return boolean
	 */
	public static function toBool($string) {
		if (is_bool($string)) {
			return $string;
		}
		if (is_numeric($string)) {
			return $string != 0;
		}
		if (empty($string))
			return false;
		if (is_string($string)) {
			$string = trim($string);
			if ((strcasecmp('false', $string) === 0) || (strcasecmp('no', $string) === 0) || (strcasecmp('n', $string) === 0)) {
				return false;
			}
		}
		return true;
	}

	/**
	 *
	 * @todo comment
	 * @param $array array       	
	 * @param $prefix string       	
	 * @return multitype:unknown
	 */
	public static function array_values_startWith(array &$array, $prefix) {
		$ret = array();
		
		if (!empty($array)) {
			$len = strlen($prefix);
			foreach ($array as $k => $value) {
				if (strncmp($prefix, $value, $len) == 0) {
					$ret[$k] = $value;
				}
			}
		}
		return $ret;
	
	}

	/**
	 *
	 * @todo comment
	 * @param $array array       	
	 * @param $prefix string       	
	 * @return multitype:unknown
	 */
	public static function array_keys_startWith(array &$array, $prefix) {
		$ret = array();
		
		if (!empty($array)) {
			$len = strlen($prefix);
			foreach ($array as $k => $value) {
				if (strncmp($prefix, $k, $len) == 0) {
					$ret[$k] = $value;
				}
			}
		}
		return $ret;
	}

	/**
	 * Rekursives array_change_key_case().
	 * Konvertiert auch keys in subarrays.
	 *
	 * @param $array array       	
	 * @param $case CASE_LOWER|CASE_UPPER       	
	 * @return array the changed $array
	 */
	public static function array_change_key_case_recursive(array &$array, $case = CASE_LOWER) {
		$array = array_change_key_case($array, $case);
		foreach ($array as $k => $v) {
			if (is_array($v)) {
				$array[$k] = array_change_key_case($v, $case);
			}
		}
		return $array;
	}

	public static function str_deqoute($str) {
		if (strpos($str, '"') == 0 && strrpos($str, '"') == strlen($str) - 1) {
			return substr($str, 1, strlen($str) - 2);
		}
		return $str;
	}

	public static function split_quoted_correct($delimiter, $str) {
		return preg_split('#(?<!\\\)\\' . $delimiter . '#', $str); // explode($delimiter,
			                                                           // $str);
	}

	/**
	 * Splitted $text immer weiter in iterativen Durchgängen mit dem nächsten
	 * Wert aus $splits auf.
	 *
	 * @todo comment
	 * @param $splits string|array       	
	 * @param $text string       	
	 * @param $regex boolean       	
	 * @return string
	 */
	public static function multi_split($splits, $text, $regex = true) {
		if (!is_array($splits)) {
			$splits = array(
					$splits );
		}
		$ret = is_array($text) ? $text : array(
				$text );
		foreach ($splits as $split) {
			$r = array(); // zwischenergebnisse können nicht gleich nach $ret, da
			              // darüber iteriert wird
			foreach ($ret as $t) {
				$result = $regex ? preg_split($split, $text) : explode($split, $text);
				$r = array_merge($r, $result);
			}
			$ret = array_merge($ret, $r);
		}
		return $ret;
	}

	public static function phpMicrotime2JSmicrotime() {
	
	}

	public static function file_exists_searchpath($filename, $searchPath = null) {
		if (empty($searchPath)) {
			$searchPath = get_include_path();
		}
		if (!is_array($searchPath)) {
			$searchPath = explode(PATH_SEPARATOR, $searchPath);
		}
		foreach ($searchPath as $path) {
			if (file_exists($path . DIRECTORY_SEPARATOR . $filename)) {
				return $path;
			}
		}
		return false;
	}

	public static function crypt_hpls($toCrypt, $key = 'hpls') {
		$xorkey = str_pad('', strlen($toCrypt), $key);
		$xor = str_rot13($toCrypt) ^ $xorkey;
		return base64_encode($xor);
	}

	public static function decrypt_hpls($crypted, $key = 'hpls') {
		$tmp = base64_decode($crypted);
		$xorkey = str_pad('', strlen($tmp), $key);
		return str_rot13($tmp ^ $xorkey);
	
	}

	public static function cryptPassword($clearPassword, $targetCrypt = 'hpls') {
		switch ($targetCrypt) {
			case 'hpls' :
				return '{hpls} ' . self::crypt_hpls($clearPassword);
			default :
				return '{plain} ' . $clearPassword;
		}
	}

	public static function getPassHashAlgorithm($text) {
		if (preg_match('/^\s*\{\s*([^}]+)\s*\}\s*(.*)\s*$/i', $text, $matches) > 0) {
			return array(
					'alg' => strtolower($matches[1]),
					'hash' => $matches[2] );
		}
		return null;
	}

	/**
	 * Dekodiert einen Passwort-String
	 *
	 * @param $passwordStr string
	 *       	 Ein Passwortstring mit Algorithmusangabe z.B '{hpls}
	 *       	 Cx4KFQISCRQ=' (ergibt 'password')
	 * @return NULL string wenn kein unterstützter Algorithmus gefunden wurde
	 *         oder nicht dekodieren konnte, das Klartextpasswort sonst.
	 */
	public static function getClearPassword($passwordStr, $defaultCrypt = '{hpls}') {
		$matches = array();
		$alg = null;
		$cryptPW = null;
		if (preg_match('/^\s*\{\s*([^}]+)\s*\}\s*(.*)\s*$/i', $passwordStr, $matches) > 0) {
			$alg = strtolower($matches[1]);
			$cryptPW = $matches[2];
		} else if (preg_match('/^\s*\{?\s*([^}]+)\s*\}?\s*$/i', $defaultCrypt, $matches) > 0) {
			$alg = strtolower($matches[1]);
			$cryptPW = $passwordStr;
		} else {
			return null;
		}
		$fct = 'decrypt_xor';
		$key = 'hpls';
		switch ($alg) {
			case 'hpls' :
				$fct = "decrypt_hpls";
				$key = 'hpls';
				break;
			case 'plain' :
			case 'clear' :
			default :
				return $cryptPW;
		}
		return self::$fct($cryptPW, $key);
	}

	public static function comparePassword($password, $cryptedPass, $defaultCrypt = '{md5}') {
		$matches = array();
		$alg = null;
		$cryptPW = null;
		if (preg_match('/^\s*\{\s*([^}]+)\s*\}\s*(.*)\s*$/i', $cryptedPass, $matches) > 0) {
			$alg = strtolower($matches[1]);
			$cryptedPass = $matches[2];
		} else if (preg_match('/^\s*\{?\s*([^}]+)\s*\}?\s*$/i', $defaultCrypt, $matches) > 0) {
			$alg = strtolower($matches[1]);
			$cryptedPass = trim($cryptedPass);
		} else {
			return null;
		}
		switch ($alg) {
			case 'hpls' :
				$password = self::crypt_hpls($password);
				break;
			// case 'xor' :
			// $password =
			// break;
			case 'md5' :
			case 'sha1' :
			case 'sha256' :
			default :
				if (in_array($alg, hash_algos())) {
					$password = hash($alg, $password);
				} else
					return null;
		}
		self::log()->debug(__METHOD__ . ": COMPARING PASSWORD '$password' WITH '$cryptedPass' in ALG '$alg'");
		
		if (strcmp($password, $cryptedPass) === 0) {
			return true;
		}
		
		return false;
	}

	/**
	 * Transformiert ein Objekt in ein Array.
	 * Es wird Deep-Convert betrieben!
	 *
	 * @param $obj object       	
	 * @return multitype:unknown
	 */
	static function toArray($obj) {
		$_arr = is_object($obj) ? get_object_vars($obj) : $obj;
		$arr = array();
		foreach ($_arr as $key => $val) {
			$val = (is_array($val) || is_object($val)) ? self::toArray($val) : $val;
			$arr[$key] = $val;
		}
		return $arr;
	}

	/**
	 *
	 * 
	 * @param $text string
	 * @param $beginMarkerRegEx string
	 * @param $endMarkerRegEx string       	
	 * @return array:string <ul><li>leeres Array: Wenn kein BEGIN-Marker gefunden wird oder $text leer ist.</li></ul>
	 */
	static function extractSections($text, $beginMarkerRegEx = '/\n\s*HPLS-BEGIN\s*\n/', $endMarkerRegEx = '/\n\s*HPLS-END\s*\n/') {
		$ret = array();
		if (empty($text))
			return $ret;
		
		$split1 = preg_split($beginMarkerRegEx, $text);
		if (count($split1) == 1) {
			// keine Marker gefunden -> leeres return
			return $ret;
		} else {
			array_shift($split1);
		}
		
		foreach ($split1 as $sp) {
			$splitEnd = preg_split($endMarkerRegEx, $sp);
			if (count($splitEnd) > 2) {
				// WARN: more than 1 $endMarker <- no matching $beginMarker
			}
			$ret[] = $splitEnd[0];
		}
		return $ret;
	}

	public static function writeConfigXml($file, $config){
		$writer = new Zend_Config_Writer_Xml();
		$writer->write($file, $config);
	}
	
}