<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created   : Mar 30, 2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28 Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Var/Clause.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 */
/***/
/**
 * @todo comment
 * 
 * <p>
 * AND { OR { v1, v2 }, AND { OR {v3, NOT { v4}}} }
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package HPLS
 * @subpackage subpackage
 * @version $LastChangedRevision: 9196 $
 * @since beta
 *
 */
class Hpls_Var_Clause implements 
	ArrayAccess, Iterator {
	const CL_OR = '_OR_';
	const CL_AND = '_AND_';
	const CL_XOR = '_XOR_';
	const CL_NOT = '_NOT_';
	const OP_LT = '_LT_';
	const OP_GT = '_GT_';
	const OP_GE = '_GE_';
	const OP_LE = '_LE_';
	const OP_LIKE = '_LIKE_';
	const OP_EQ = '_EQ_';
	const OP_IS_NULL = '__ISNULL__';
	const OP_BESTGUESS = '_BEST_';
	
	/**
	 * Der IN-Operator.
	 * <p>
	 * Werte werden in einem array zum Key hinterlegt:
	 * self::set(array('key' => array('in1','in2',...)))
	 * 
	 * @var string
	 */
	const CL_IN = '_IN_';
	
	public static $KEYWORDS = array(
			self::CL_AND,
			self::CL_NOT,
			self::CL_OR,
			self::CL_XOR,
			self::CL_IN );
	
	protected $_clause = null;
	protected $_logic = self::CL_AND;

	/**
	 * Enter description here ...
	 * @param string $logic
	 * @param array $clause Ein Array von key-value Paaren
	 */
	public function __construct($logic = self::CL_AND, array $clause = array()) {
		$this->set($clause);
		$this->setLogic($logic);
	}

	/**
	 * Make a deep copy ;)
	 */
//	public function __clone() {
//		//erstmal alle HVC-Objekte herausfiltern
//		//nicht gleich ersetzen, da das nicht in der selben foreach-schleife geht 
//		$toClone = array();
//		foreach ($this->_clause as $k => $value) {
//			if ($value instanceof Hpls_Var_Clause) {
//				$toClone[] = $k;
//			}
//		}
//		//alle Objekte in der Clause durch Klone selbiger ersetzen.
//		foreach ($toClone as $k) {
//			$this->_clause[$k] = clone $this->_clause[$k];
//		}
//	}

	/**
	 * <p><b>Möglichst wenig verwenden, ist rechen- und speicherintensiv!</b></p>
	 * @todo comment
	 * @return string
	 */
	public function __toString() {
		$ret = array();
		foreach ($this->_clause as $k => $v) {
			if ($v instanceof Hpls_Var_Clause) {
				$ret[] = $v->__toString();
			} else {
				$r = array();
				foreach ($v as $values) {
					$r[] = "(" . $values['op'] . "'" . $values['value'] . "')";
				}
				$ret[] = "'" . $k . "': " . implode(',', $r);
			}
		}
		return $this->getLogic() . '{' . implode('; ', $ret) . '}';
	}

	public function setLogic($logic) {
		$this->_logic = $logic;
	
	}

	public function getLogic() {
		return $this->_logic;
	}

	public function isLogic($logic) {
		return $this->getLogic() == $logic;
	}

	/**
	 * @todo comment
	 * @return multitype:array:string an array of clauses. all first level entries are to be interpreted as ANDed.
	 */
	public function get() {
		return $this->_clause;
	}

	public function set(array $clause) {
		$this->_clause = array();
		foreach ($clause as $k => $value) {
			$this->addValue($k, $value);
		}
	}

	public function remove($key) {
		unset($this->_clause[$key]);
	}

	public function __unset($key) {
		$this->remove($key);
	}

	public function __get($key) {
		if (isset($this->_clause[$key])) {
			return $this->_clause[$key];
		}
		return null;
	}

	/**
	 * @todo comment
	 * @param string $keyName
	 * @param boolean $deep Suche auch in SubClauses?
	 * @return boolean
	 */
	public function hasKey($keyName, $deep = true) {
		foreach ($this->_clause as $key => $value) {
			if ($deep && $value instanceof Hpls_Var_Clause) {
				if ($value->hasKey($keyName, $deep)) {
					return true;
				}
			} elseif ($key == $keyName) {
				return true;
			}
		}
		return false;
	}

	public function getAllKeys($descend = true) {
		$ret = array();
		foreach ($this->_clause as $key => $value) {
			if ($value instanceof Hpls_Var_Clause) {
				if ($descend) {
					$ret = array_merge($ret, $value->getAllKeys($descend));
				}
			} elseif (!in_array($key, $ret)) {
				$ret[] = $key;
			}
		}
		return $ret;
	}

	public function getKeys() {
		return $this->getAllKeys(false);
	}

	public function contains($value) {
		return in_array($value, $this->_clause);
	}

	/**
	 * Fügt ein Key-Value-Paar am Ende hinzu, egal, ob $key schon in der Clause existiert oder nicht. 
	 * @param string|Hpls_Var_Clause $key
	 * @param mixed $value
	 * @param string|self::OP_BESTGUESS $op
	 */
	public function addValue($key, $value = NULL, $op = self::OP_BESTGUESS) {
		if ($key instanceof Hpls_Var_Clause) {
			$this->_clause[] = $key;
		} elseif ($value instanceof Hpls_Var_Clause) {
			$this->_clause[$key] = $value;
		} else {
			if (is_array($key)) {
				if (!isset($key['key'])) {
					require_once 'Hpls/Exception/KeyNotExists.php';
					throw new Hpls_Exception_KeyNotExists('Array has no "key" index.', null);
				}
				$value = isset($key['value']) ? $key['value'] : $value;
				$op = isset($key['op']) ? $key['op'] : $op;
				$key = $key['key'];
			}
			$w = null;
			if (isset($this->_clause[$key])) {
				$w = $this->_clause[$key];
			} else {
				$w = array();
			}
			$w[] = array(
					'value' => $value,
					'op' => $op );
			$this->_clause[$key] = $w;
		}
	}

	/**
	 * Setzt das Key-Value-Paar auf die übergebenen Werte. Existiert $key schon in der Clause wird dessen Wert überschrieben.
	 * @param string|Hpls_Var_Clause $key
	 * @param mixed $value
	 * @param string $op
	 */
	public function setValue($key, $value, $op = self::OP_BESTGUESS) {
		if ($value instanceof Hpls_Var_Clause) {
			$this->_clause[$key] = $value;
		} else {
			if (is_array($value)) {
				$op = isset($value['op']) ? $value['op'] : $op;
				$value = isset($value['value']) ? $value['value'] : $value;
			}
			$this->_clause[$key] = array(
					array(
							'value' => $value,
							'op' => $op ) );
		}
	}

	/**
	 * @todo comment
	 * @param unknown_type $key
	 * @return multitype:
	 */
	public function getValue($key) {
		$ret = array();
		foreach ($this->_clause as $k => $member) {
			if ($k == $key) {
				$ret[] = array_merge($ret, $member);
			} elseif ($member instanceof Hpls_Var_Clause) {
				//subclause
				$ret = array_merge($ret, $member->getValue($key));
			}
		
		//			if ($member[])
		}
		return $ret;
	}

	public function merge($clause) {
		if ($clause instanceof Hpls_Var_Clause) {
			;
		} else {
		}
	}

	public function isEmpty() {
		return is_null($this->_clause) || count($this->_clause) == 0;
	}

	/**
	 * Enter description here ...
	 * @param unknown_type $word
	 * @return boolean
	 */
	public static function isKeyWord($word) {
		return array_search($word, self::$KEYWORDS) === false ? false : true;
	}

	/* (non-PHPdoc)
	 * @see Iterator::current()
	 */
	public function current() {
		return current($this->_clause);
	}

	/* (non-PHPdoc)
	 * @see Iterator::next()
	 */
	public function next() {
		next($this->_clause);
	}

	/* (non-PHPdoc)
	 * @see Iterator::key()
	 */
	public function key() {
		return key($this->_clause);
	}

	/* (non-PHPdoc)
	 * @see Iterator::valid()
	 */
	public function valid() {
		return key($this->_clause) !== null;
	}

	/* (non-PHPdoc)
	 * @see Iterator::rewind()
	 */
	public function rewind() {
		reset($this->_clause);
	}

	/* (non-PHPdoc)
	 * @see ArrayAccess::offsetExists()
	 */
	public function offsetExists($offset) {
		return isset($this->_clause[$offset]);
	}

	/* (non-PHPdoc)
	 * @see ArrayAccess::offsetGet()
	 */
	public function offsetGet($offset) {
		return $this->_clause[$offset];
	}

	/* (non-PHPdoc)
	 * @see ArrayAccess::offsetSet()
	 */
	public function offsetSet($offset, $value) {
		$this->_clause[$offset] = $value;
	}

	/* (non-PHPdoc)
	 * @see ArrayAccess::offsetUnset()
	 */
	public function offsetUnset($offset) {
		unset($this->_clause[$offset]);
	}

	public function toArray() {
		return clone $this->_clause;
	}

//	/* (non-PHPdoc)
//	 * @see Serializable::serialize()
//	 */
//	public function serialize() {
//		// TODO Auto-generated method stub
//		return $this->__toString();
//
//	}
//
//	/* (non-PHPdoc)
//	 * @see Serializable::unserialize()
//	 */
//	public function unserialize($serialized) {
//		// TODO Auto-generated method stub
//	}

	public function __sleep() {
		return array('_clause','_logic');
	}
}