<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 21.06.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28 Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Var/Matcher.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Var/Subster.php';

/**
 * Hilfsklasse, die per RegEx einen String in Bestandteile zerlegen kann und die so
 * erhaltenen Bestandteile können mit Namen benannt werden.
 * 
 * Mit getSubstituted() kann ein Target-Text, der Platzhalter mit den Bestandteilsnamen enthält, mit
 * den entsprechenden Werten für die Bestandteile ersetzt werden.
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 9196 $
 * @since 1.0.9
 */
class Hpls_Var_Matcher {
	protected $_regex = null;
	protected $_textToMatch = null;
	protected $_paramMap = array();
	protected $_paramMapRevers = array();
	protected $_targetPattern = null;
	protected $_isMatched = false;
	protected $_extraVariables = array();
	/**
	 * @todo comment
	 * @var Hpls_Var_Subster
	 */
	protected $_replacer = null;

	public static function createLoginMapper() {
		return new self(array(
				'match' => array(
						'regex' => "/^([^@]+)@(.+)$/i",
						'p1' => 'USERNAME',
						'p2' => 'DOMAIN' ) ));
	}

	public function __construct($options) {
		isDebug('var_matcher') && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($options, ' OPTIONS '));
		$this->_replacer = new Hpls_Var_Subster();
		if (is_array($options)) {
			if (isset($options['match'])) {
				foreach ($options['match'] as $k => $v) {
					if (strncasecmp($k, 'p', 1) === 0) {
						$this->mapParam((int) substr($k, 1), strtoupper($v));
					} elseif ($k === 'regex' || $k === 'match') {
						$this->_regex = $v;
					}
				}
			}
			if (isset($options['target'])) {
				if (isset($options['target']['pattern'])) {
					$this->_targetPattern = (string) $options['target']['pattern'];
				}
				if (isset($options['target']['case'])) {
					switch (strtolower($options['target']['case'])) {
						case 'lower' :
						case 'insensitive' :
							$this->_replacer->setChangeCase(CASE_LOWER);
							break;
						case 'upper' :
							$this->_replacer->setChangeCase(CASE_UPPER);
							break;
						default :
							$this->_replacer->setChangeCase(false);
					}
				} // if target.case
			} //if target
		}
	}

	/**
	 * @todo comment
	 * @param string $text
	 * @return boolean
	 */
	public function match($text) {
		$this->_textToMatch = $text;
		if (empty($this->_regex)) {
			return false;
		}
		$matches = array();
		isDebug('var_matcher') && $this->log()->debug(__METHOD__ . " Matching: {$this->_regex}: {$this->_textToMatch}");
		if (preg_match($this->_regex, $this->_textToMatch, $matches) > 0) {
			isDebug('var_matcher') && $this->log()->debug(__METHOD__ . " MATCHED {$this->_regex}");
			$this->_isMatched = true;
			$this->setVariables($matches);
			foreach ($this->_extraVariables as $k => $v) {
				$this->_replacer->setVariable($k, $v);
			}
		} else {
			isDebug('var_matcher') && $this->log()->debug(__METHOD__ . " UNmatched {$this->_regex}");
			$this->_isMatched = false;
			$this->setVariables($this->_extraVariables);
		}
		return $this->_isMatched;
	}

	public function replace($text,$targetPattern = null) {
		if (empty($text)) {
			return '';
		}
		if (!empty($targetPattern)) {
			$this->_targetPattern = $targetPattern;
		}
		
		$this->match($text); //setzt bzw. unsets gefunden Variablen
		return $this->_replacer->getSubstituted($this->getTargetPattern());
	}

	/**
	 * @todo comment
	 * @param string $targetPattern Das Ziel in das die gefundenen und vordefinierten Werte (setVariable()) eingefügt werden sollen.
	 * @param string|NULL $text Text, aus dem Variablenwerte mit Hilfe von getRex() geparsed werden. Wenn <i>null</i> dann wird der per setText() genommen. 
	 * @return string|NULL
	 */
	public function getSubstituted($targetPattern = null, $text = null) {
		//		isDebug('var_matcher') &&
		$this->log()->debug(__METHOD__ . "($targetPattern,$text) - START");
		if (empty($text)) {
			$text = $this->_textToMatch;
		}
		if (!empty($targetPattern)) {
			$this->_targetPattern = $targetPattern;
		}
		$this->match($text); //setzt bzw. unsets gefunden Variablen
		return $this->_replacer->getSubstituted($this->getTargetPattern());
	}

	public function mapParam($pos, $name) {
		$this->_paramMap[$pos] = $name;
		$this->_paramMapRevers[$name] = $pos;
	}

	/**
	 * @return the $_regex
	 */
	public function getRegex() {
		return $this->_regex;
	}

	public function getParam($name) {
		if (isset($this->_paramMapRevers[$name])) {
			return $this->_replacer->getVariable($this->_paramMapRevers[$name]);
		}
		return $this->_replacer->getVariable($name);
	}

	/**
	 * @return the $_paramMap
	 */
	public function getParamMap() {
		return $this->_paramMap;
	}

	/**
	 * @return the $_targetPattern
	 */
	public function getTargetPattern() {
		return $this->_targetPattern;
	}

	/**
	 * @param field_type $_regex
	 */
	public function setRegex($_regex) {
		$this->_regex = $_regex;
	}

	/**
	 * @param array $_paramMap
	 */
	public function setParamMap(array $_paramMap) {
		$this->_paramMap = $_paramMap;
		$this->_paramMapRevers = array_flip($_paramMap);
	}

	/**
	 * @param field_type $_targetPattern
	 */
	public function setTargetPattern($_targetPattern) {
		$this->_targetPattern = $_targetPattern;
	}

	public function setVariable($name, $value) {
		$this->_extraVariables[$name] = $value;
		$this->_replacer->setVariable($name, $value);
	}

	public function unsetVariable($name) {
		unset($this->_extraVariables[$name]);
		$this->_replacer->unsetVariable($name);
	}

	protected function setVariables(array $matches) {
		isDebug('var_matcher') && $this->log()->debug(Hpls_Debug::print_r($matches, __METHOD__));
		$this->_replacer->setVariables(array()); //alles löschen
		//und neues setzen:
		foreach ($matches as $pos => $value) {
			$this->_replacer->setVariable($pos, $value);
			if (isset($this->_paramMap[$pos])) {
				$this->_replacer->setVariable($this->_paramMap[$pos], $value);
			}
		}
	}

	/**
	 * @todo comment
	 * @return Zend_Log
	 */
	protected function log() {
		return Hpls_Utils::log();
	}

}