<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created   : Mar 26, 2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28 Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Var/Parser.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 */
/***/
require_once 'Hpls/Utils.php';
require_once 'Zend/Debug.php';

class Hpls_Var_Parser {
	const LDELIM = '[[';
	const RDELIM = ']]';
	
	/**
	 * @var string function separator
	 */
	const FUNC_SEP = ':';
	const MOD_SEP = '|';
	
	protected static $_instance = null;
	
	protected $_text = null;
	protected $_containedVars = null;
	protected $_varValues = null;
	protected $_replaceOnlySet = true;
	protected $_ignoreCase = true;

	/**
	 * @todo comment
	 * @return Hpls_Var_Parser
	 */
	public static function getInstance() {
		if (self::$_instance === null) {
			self::$_instance = new Hpls_Var_Parser();
		}
		return self::$_instance;
	}

	public function __construct() {
	
	}

	/**
	 * @todo comment
	 * @param unknown_type $text
	 */
	public function setText($text, $clearVarAssigns = true) {
		$this->clear();
		if ($clearVarAssigns !== false) {
			$this->clearVarAssigns();
		}
		$this->_text = $text;
	}

	/**
	 * @todo comment
	 * @return NULL
	 */
	public function getText() {
		return $this->_text;
	}

	/**
	 * @todo comment
	 * @return string
	 */
	public function isReplaceOnlySet() {
		return $this->_replaceOnlySet;
	}

	/**
	 * @todo comment
	 * @param unknown_type $bool
	 */
	public function setReplaceOnlySet($bool = true) {
		$this->_replaceOnlySet = $bool;
	}

	/**
	 * @todo comment
	 * @return array:string
	 */
	public function getContainedVars() {
		return $this->_containedVars;
	}

	public function getVarValues() {
		return $this->_varValues;
	}

	/**
	 * @todo comment
	 * @param string $varName
	 * @param mixed $value
	 */
	public function setValue($varName, $value) {
		if (is_null($this->_varValues)) {
			$this->_varValues = array();
		}
		$this->_varValues[$varName] = $value;
	}

	/**
	 * @todo comment
	 * @param string $text
	 */
	public function parse($text = null) {
		if (!is_null($text)) {
			$this->setText($text);
		}
		//nothing to parse? -> no contained variables
		if (empty($this->_text)) {
			$this->clear();
			return;
		}
		
		if (!is_array($this->_containedVars)) {
			$this->_containedVars = array();
		}
		$matches = array();
		preg_match_all("/\\[\\[[^]]*\\]\\]/", $this->_text, $matches);
		$matches = $matches[0];
		foreach ($matches as $match) {
			$match = $this->getVarNameFromPlaceholder($match);
			if (!array_key_exists($match, $this->_containedVars)) {
				$this->_containedVars[$match] = $match;
			}
		}
	}

	/**
	 * @todo comment
	 * @param unknown_type $placeholder
	 * @return mixed
	 */
	public function getVarNameFromPlaceholder($placeholder) {
		return preg_replace(array(
				'/^\[\[/',
				'/\]\]$/' ), array(
				'',
				'' ), $this->isIgnoreCase() ? strtoupper($placeholder) : $placeholder);
	}

	/**
	 * @todo comment
	 * @param string $textToReplaceIn
	 * @param bool $parse
	 * @return string
	 */
	public function getReplaced($textToReplaceIn = null, $parse = false) {
		if ($textToReplaceIn == null) {
			$textToReplaceIn = $this->_text;
		}
		if ($parse) {
			$this->parse($textToReplaceIn);
		}
		
		$searchPattern = $this->getSearchPattern();
		$replacePattern = $this->getReplacePattern();
		//		Hpls_Utils::getLogger()->debug("VarParser::Replace: ".Zend_Debug::dump($searchPattern,"\nSearchPattern:\n",false)
		//		.Zend_Debug::dump($replacePattern,"\nReplacePattern:\n",false).Zend_Debug::dump($textToReplaceIn,"\n In text: ",false));
		return preg_replace($searchPattern, $replacePattern, $textToReplaceIn);
	}

	/**
	 * @todo comment
	 *
	 * @return multitype: string
	 */
	public function getSearchPattern() {
		$ret = array();
		if (is_array($this->getContainedVars())) {
			foreach ($this->getContainedVars() as $varName => $param) {
				// replace only variables if they are set explicitly?
				if ($this->isReplaceOnlySet() && !$this->hasValue($varName))
					continue;
				
				$varName = self::LDELIM . $varName . self::RDELIM;
				$varName = $this->escape($varName);
				$varName = '/' . $varName . '/';
				$ret[] = $this->isIgnoreCase() ? $varName . 'i' : $varName;
			}
		}
		return $ret;
	}

	/**
	 * @todo comment
	 *
	 * @return multitype:string
	 */
	public function getReplacePattern() {
		$ret = array();
		if (is_array($this->getContainedVars())) {
			foreach ($this->getContainedVars() as $varName => $param) {
				// replace only variables if they are set explicitly?
				if ($this->isReplaceOnlySet() && !$this->hasValue($varName))
					continue;
				$value = $this->getValue($varName);
				if (is_null($value)) {
					$ret[] = '';
				} else {
					$ret[] = $value;
				}
			}
		}
		return $ret;
	}

	/**
	 * @todo comment
	 *
	 * @param unknown_type $string
	 * @return string
	 */
	public function escape($string) {
		return preg_replace(array(
				'/\-/',
				'/\[/',
				'/\]/' ), array(
				'\\-',
				'\\[',
				'\\]' ), $string);
	}

	/**
	 * @todo comment
	 *
	 * @param string $varName
	 * @return mixed
	 */
	public function getValue($varName) {
		$ret = null;
		if ($this->hasValue($varName)) {
			$ret = $this->_varValues[$varName];
		}
		return $ret;
	}

	/**
	 * @todo comment
	 * @param string $varName
	 * @return boolean
	 */
	public function hasValue($varName) {
		return !is_null($this->_varValues) && array_key_exists($varName, $this->_varValues);
	}

	/**
	 * @todo comment
	 * @param array $valArray
	 */
	public function setValues(array $valArray) {
		$this->_varValues = $valArray;
	}

	/**
	 * @todo comment
	 * Clears contained vars and text to parse.
	 */
	public function clear() {
		$this->_containedVars = array();
		$this->_text = null;
	}

	public function clearVarAssigns() {
		$this->_varValues = null;
	}

	public function setIgnoreCase($value = true) {
		$this->_ignoreCase = $value;
	}

	public function isIgnoreCase() {
		return $this->_ignoreCase;
	}

}